// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        14/03/2003                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Utilitaires divers.                                       *
 *             1) transformation de maillage, quadratique incomplet     *
 *                en quadratique complet.                               *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef ALGOUTILS_T
#define ALGOUTILS_T


#include "Algori.h"
#include "MatBand.h"
#include "Assemblage.h"


/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:   Utilitaires divers.
///             ex: transformation de maillage, quadratique incomplet
///                en quadratique complet.

class AlgoUtils : public Algori
{
  public :
    // CONSTRUCTEURS :
    AlgoUtils () ; // par defaut
    
    // constructeur en fonction du type de calcul
    // du sous type (pour les erreurs, remaillage etc...)
    // il y a ici lecture des parametres attaches au type
    AlgoUtils (const bool avec_typeDeCal
               ,const list <EnumSousTypeCalcul>& soustype
               ,const list <bool>& avec_soustypeDeCal
               ,UtilLecture& entreePrinc);
           
    // constructeur de copie
    AlgoUtils (const AlgoUtils& algo);
    
    // constructeur de copie à partie d'une instance indifférenciée
    Algori * New_idem(const Algori* algo) const
     {// on vérifie qu'il s'agit bien d'une instance
      if (algo->TypeDeCalcul() != UTILITAIRES)
        { cout << "\n *** erreur lors de la creation par copie d'un algo UTILITAIRES "
               << " l'algo passe en parametre est en fait : " << Nom_TypeCalcul(algo->TypeDeCalcul())
               << " arret !! " << flush;
          Sortie(1);
        }
      else
        { AlgoUtils* inter = (AlgoUtils*) algo;
          return ((Algori *) new AlgoUtils(*inter));
        };
     };

    // DESTRUCTEUR :
     ~AlgoUtils () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'algorithme 
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* varExpor,LesLoisDeComp*
                  ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    //------- décomposition en 3 du calcul d'équilibre -------------
    // a priori   : InitAlgorithme  et FinCalcul ne s'appellent qu'une fois,
    // par contre : CalEquilibre peut s'appeler plusieurs fois, le résultat sera différent si entre deux calculs
    //              certaines variables ont-été changés
    
    // pas opérationnel pour l'instant !
      
    // initialisation
    void InitAlgorithme(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                        ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                        ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* ) {};
    // mise à jour
    void MiseAJourAlgo(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                       ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                       ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* )
            {cout << "\n MiseAJourAlgo(.. pas encore operationnelle ";
             Sortie(1);
            };            ;
    // calcul de l'équilibre
    // si tb_combiner est non null -> un tableau de 2 fonctions
    //  - la première fct dit si on doit valider ou non le calcul à convergence ok,
    //  - la seconde dit si on doit sortir de la boucle ou non à convergence ok
    //
    // si la validation est effectuée, la sauvegarde pour le post-traitement est également effectuée
    //                                 en fonction de la demande de sauvegard,
    // sinon pas de sauvegarde pour le post-traitement à moins que l'on a demandé un mode debug
    //   qui lui fonctionne indépendamment
    void CalEquilibre(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                      ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                      ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats*
                      ,Tableau < Fonction_nD* > * tb_combiner){};
    // dernière passe
    void FinCalcul(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                   ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                   ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* ) {};
    // sortie du schemaXML: en fonction de enu
    void SchemaXML_Algori(ofstream& ,const Enum_IO_XML ) const {};
 
  private :  
    
    // METHODES PROTEGEES :
    // --- venant du virtuel ---
    // écriture des paramètres dans la base info (ici rien)
    void Ecrit_Base_info_Parametre(UtilLecture& ,const int& ) {};
    // lecture des paramètres dans la base info (ici rien)
    // choix = true  : fonctionnememt normal
    // choix = false : la méthode ne doit pas lire mais initialiser les données à leurs valeurs par défaut
    //                 car la lecture est impossible
    void Lecture_Base_info_Parametre(UtilLecture& ,const int& ,bool) {};
    // création d'un fichier de commande: cas des paramètres spécifiques
    void Info_commande_parametres(UtilLecture& ) {Algori::Info_com_parametres(*entreePrinc);};    
 };
 /// @}  // end of group

#endif  
