// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Algorithme de calcul de la remontée des contraintes aux  *
 *     noeuds, puis de calcul d'erreur, après un calcul de mécanique.   *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef AGORIREMONTERREUR_T
#define AGORIREMONTERREUR_T


#include "Algori.h"
#include "MatBand.h"
#include "Assemblage.h"

/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:    Algorithme de calcul de la remontée des contraintes aux
///     noeuds, puis de calcul d'erreur, après un calcul de mécanique.

class AlgoriRemontErreur : public Algori
{
  public :
    // CONSTRUCTEURS :
     AlgoriRemontErreur () ; // par defaut
     
     // constructeur en fonction du type de calcul
     // il y a ici lecture des parametres attaches au type
     AlgoriRemontErreur (EnumTypeCalcul type,UtilLecture& entreePrinc);
    // DESTRUCTEUR :
     ~AlgoriRemontErreur () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'algorithme dans le cas non dynamique
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

 
  private :  
    // VARIABLES PROTEGEES :
    
    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
