
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/05/2003                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Cette classe est dédiée au passage d'informations protégées *
 *             à d'autre classes. Cependant, la gestion des datas est   *
 *             effectuée directement par la classe ParaAlgoControle.    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef VARIABLES_TEMPS_H
#define VARIABLES_TEMPS_H

#include <fstream>
//#include "Debug.h"

#ifndef ENLINUX_STREAM
  #include <sstream> // pour le flot en memoire centrale
#else  
  #include <strstream> // pour le flot en memoire centrale
#endif
#include <string.h>
#include <string>
#include "Sortie.h"


class ParaAlgoControle; 

/// @addtogroup Groupe_concernant_le_chargement
///  @{
///

class VariablesTemps
{
  public :
    friend class ParaAlgoControle;
    friend class Loi_comp_abstraite;
    // CONSTRUCTEURS :
    // par défauts
    VariablesTemps();
    
    // constructeur de copies : 
    VariablesTemps(const VariablesTemps& t);
    
    // DESTRUCTEUR :
    ~VariablesTemps() {};
    
    // METHODES PUBLIQUES :
    // ramène le temps courant
    const double & TempsCourant() const {return temps;};
    // ramène l'incrément du temps en cours
    const double & IncreTempsCourant() const { return deltat;};
    // ramène l'incrément du temps maxi
    const double & IncreTempsMaxi() const { return deltatmaxi;};
    // ramène l'incrément du temps mini
    const double & IncreTempsMini() const { return deltatmini;};
    // ramène le temps fin
    const double & Tempsfin() const {return tempsfin;};
    // affichage des parametres liés au temps
    void Affiche()  const ;
    // mise à jour des paramètres bornes
    void Mise_a_jour_bornes(const VariablesTemps& t)
     {deltatmaxi = t.deltatmaxi;
      deltatmini = t.deltatmini;
      tempsfin = t.tempsfin;
      prectemps = t.prectemps;
     };
	 
	//----- lecture écriture dans base info -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
    // = 3 : on met à jour uniquement les données variables (supposées comme telles)
    //       dans ce cas il y a modification des grandeurs, mais pas redimentionnement
	   void Lecture_base_info_temps(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info_temps(ofstream& sort,const int cas) const ;
	    
  protected :  
    // VARIABLES PROTEGEES :
    double temps; // le temps global
    // la sauvegarde des paramètres d'un pas sur l'autre
    double deltat;        // increment de temps
    // les bornes
    double deltatmaxi; // increment de temps maxi
    double deltatmini; // increment de temps mini
    double tempsfin;  // temps de fin de calcul
    double prectemps;  // precision sur le temps final
            
    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
