
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Element geometrique generique pour une discrétisation C1   *
 *     et semi C1.                                                      *
 *     Ici le nombre de fonction d'interpolation est supérieur au       *
 *     nombre de noeud.                                                 *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef ELEMGEOMC1_H
#define ELEMGEOMC1_H

#include <bool.h>
#include "Mat_pleine.h"
#include "Tableau_T.h"
#include "Vecteur.h"
#include "Coordonnee.h"
#include "ElemGeomC0.h"
         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class ElemGeomC1 : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
     ElemGeomC1(); // pardefaut 
     
     ElemGeomC1(int dim,int nbi,int nbne,int nbfe,int nbse,Tableau<int> nddNoeud); 
     // cas ou l'on connait la dimension = dim, 
     // le nombre de point d'integration nbi et le nombre
     // de noeud de l'element nbne
     // le nombre de face nbfe, le nombre de segment nbse
     
     // IMPORTANT : par defaut le nombre de pt d'integ pour les faces ou segents
     // s'il y en a, est calcule a	 partir du nombre total de point d'integ
     // se referer a la description des elements
     
     //  nddNoeud : nombre de ddl par noeud, qui sont utilisé pour
     // définir la géométrie
     
     
     ElemGeomC1(ElemGeomC1& a);  // de copie
    // DESTRUCTEUR :
     ~ElemGeomC1();
    // METHODES PUBLIQUES :       
    
  protected :  
    // VARIABLES PROTEGEES  en plus de celles définies dans ElemGeomC0:
    Tableau<int> nddNoeud; // nombre de ddl par noeud, qui sont utilisé pour
                           // définir la géométrie
  // pour l'element
  //1) les tableaux définis dans ElemGeomC0
    // ---> Tableau <Vecteur> tabPhi ; // tabPhi(ni) = phi = fonctions d'interpolation
    // au point d'interpolation ni, phi a la dimension de la somme des valeurs de 
    //  nddNoeud, et non le nombre de noeud comme dans le cas de ElemGeomC0
    // ---> Tableau < Mat_pleine > tabDPhi; // tabDPhi(ni) = Dphi tel que, Dphi(i,r) =
    // valeur de la derivee de la fonction phi(r) par rapport a la coordonnee`
    // locale i (= 1  ou 2 ou 3, ceci dependant de la dimension de l'element) 
    // ici r varie de 1 à somme des valeurs de nddNoeud 
                                                      
    // METHODES PROTEGEES :
 };
 /// @}  // end of group

#endif  
