************************************************************************
*     UNIVERSITE DE BRETAGNE OCCIDENTALE   ---   I.U.T. DE LORIENT     *
************************************************************************
*           LABORATOIRE DE GENIE MECANIQUE ET PRODUCTIQUE              *
*                        Tel 97.05.94.11                               *
*          Centre de Genie Industriel  56520 GUIDEL-PLAGES             *
************************************************************************


************************************************************************
*                                                                      *
*     DATE:        Thu Feb  4 12:58:37 WET 1993                        *
*                                                                $     *
*     AUTEUR:      J-P HILY                                            *
*                                                                $     *
*     PROJET:        EF3                                               *
*                                                                $     *
************************************************************************


************************************************************************
*                                                                      *
*     VERIFICATION:                                                    *
*                                                                      *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*     MODIFICATIONS:                                                   *
*                                                                      *
*     !  date  !   auteur   !       but                          !     *
*     ------------------------------------------------------------     *
*     !        !            !                                    !     *
*     !        !            !                                    !     *
*                                                                      *
*                                                                $     *
*                                                                      *
************************************************************************


************************************************************************
*                                                                      *
      SUBROUTINE HEXAQUA (NBNE,NBI,PHI,DPHI,WI,
     &                    NBFE,NBNF,NONF,NBIS,PHIS,DPHIS,WIS)
*                                                                $     *
      implicit none
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*     BUT:        Valeurs des fonctions d'interpolations et de leurs   *
*                 derivees aux points d'integration dans le cas        *
*                 quadratique.                                         *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*     VARIABLES:                                                       *
*                  ..................................... en entree     *
*                                                                      *
      integer    NBNE      ! nombre de noeuds par element
      integer    NBFE      ! nombre de faces par element
      integer    NBNF      ! nombre de noeuds par face
      integer    NONF(NBNF,NBFE) ! connexion des noeuds des faces par
                           ! rapport a ceux des elements
      integer    NBI  ! nombre de point d'integration pour les elements
      integer    NBIS ! nombre de point d'integration pour les faces
*                                                                      *
*                  ..................................... en sortie     *
*                                                                      *
c                pour les elements
      real*8     PHI(NBNE,NBI)     ! fonctions d'interpolation
      real*8     DPHI(3,NBNE,NBI)  ! leurs derivees
      real*8     WI(NBI)   ! poids d'integration
c                pour les faces
      real*8     PHIS(NBNF,NBIS)   ! fonctions d'interpolation
      real*8     DPHIS(2,NBNF,NBIS)! leurs derivees
      real*8     WIS(NBIS)  ! poids d'integration
*                                                               $$     *
*                  ....................................... interne     *
*                                                                      *
      integer    NI
*                                                                      *
************************************************************************

************************************************************************
*                                                                      *
*     ELEMENT DE REFERENCE , POINTS D'INTEGRATION:                     *
*                                                                      *
*     Nicolas COUTY   04/12/95                                         *
*     Source : Pour l'element : Modulef Guide No 2, p 139              *
*     Source : Pour les fonctions d'interpolation :                    *
*              Dhatt et Touzot p 135                                   *
*            + donnee suivante des valeurs des fonctions               *
*              d'interpolation aux points d'integration                *
*              (qui a permis la verification)                          *
*                                                                      *
*----------------------------------------------------------------------*
*
*                         |zeta
*                         |
*        5____________20__|________8
*        |\               |       |\
*        | \              |       | \
*        |  17            |       |  19
*        |   \            |       |   \
*       13    \           |       16   \
*        |    6\___________18___________\7
*        |     |          |       |     |
*        |     |          |       |     |
*        |     |           -------------|----eta
*       1|_____|______12___\______|     |
*        \     14           \     \4    15 
*         \    |             \     \    |
*          9   |              \     11  |
*           \  |               \     \  |
*            \ |                \     \ |
*            2\|___________10____\_____\|3
*                                 \
*                                  \xi
*
* Points d'integration (cf hexalin.f)
* a=1/racine(3)
* Pt1 (a,a,a)  ; Pt2 (a,a,-a)  ; Pt3 (a,-a,a)  ; Pt4 (a,-a,-a)
* Pt5 (-a,a,a) ; Pt6 (-a,a,-a) ; Pt7 (-a,-a,a) ; Pt8 (-a,-a,-a)
*

************************************************************************
*                                                                      *
*     CORPS DU PROGRAMME:                                              *
*                                                                      *
*----------------------------------------------------------------------*

      CALL DINVEC(PHI,NBNE*NBI)
      CALL DINVEC(DPHI,3*NBNE*NBI)
      DO NI= 1,NBI
        WI(NI)= 1.d0
      ENDDO
      CALL DINVEC(PHIS,NBNF*NBIS)
      CALL DINVEC(DPHIS,2*NBNF*NBIS)
      DO NI= 1,NBIS
        WIS(NI)= 1.d0
      ENDDO

c     ---------- face 1 ----------
      NONF(1,1)= 1
      NONF(2,1)= 4
      NONF(3,1)= 3
      NONF(4,1)= 2
      NONF(5,1)= 12
      NONF(6,1)= 11
      NONF(7,1)= 10
      NONF(8,1)= 9
c     ---------- face 2 ----------
      NONF(1,2)= 1
      NONF(2,2)= 5
      NONF(3,2)= 8
      NONF(4,2)= 4
      NONF(5,2)= 13
      NONF(6,2)= 20
      NONF(7,2)= 16
      NONF(8,2)= 12
c     ---------- face 3 ----------
      NONF(1,3)= 1
      NONF(2,3)= 2
      NONF(3,3)= 6
      NONF(4,3)= 5
      NONF(5,3)= 9
      NONF(6,3)= 14
      NONF(7,3)= 17
      NONF(8,3)= 13
c     ---------- face 4 ----------
      NONF(1,4)= 5
      NONF(2,4)= 6
      NONF(3,4)= 7
      NONF(4,4)= 8
      NONF(5,4)= 17
      NONF(6,4)= 18
      NONF(7,4)= 19
      NONF(8,4)= 20
c     ---------- face 5 ----------
      NONF(1,5)= 2
      NONF(2,5)= 3
      NONF(3,5)= 7
      NONF(4,5)= 6
      NONF(5,5)= 10
      NONF(6,5)= 15
      NONF(7,5)= 18
      NONF(8,5)= 14
c     ---------- face 6 ----------
      NONF(1,6)= 3
      NONF(2,6)= 4
      NONF(3,6)= 8
      NONF(4,6)= 7
      NONF(5,6)= 11
      NONF(6,6)= 16
      NONF(7,6)= 19
      NONF(8,6)= 15

      PHI(7,3) = -0.1870014113213579D0
      PHI(3,6) = -0.1870014113213579D0
      PHI(20,7) = 0.4146723119520977D0
      PHI(19,6) = 0.1111111111111112D0
      PHI(15,3) = 0.1111111111111112D0
      PHI(19,1) = 0.4146723119520977D0
      PHI(10,6) = 0.1111111111111112D0
      PHI(3,4) = -0.1870014113213579D0
      PHI(7,6) = -0.9077636645642013D-1
      PHI(14,3) = 0.4146723119520977D0
      PHI(16,5) = 0.4146723119520977D0
      PHI(15,4) = 0.1111111111111112D0
      PHI(12,4) = 0.1111111111111112D0
      PHI(4,8) = -0.1870014113213579D0
      PHI(16,1) = 0.1111111111111112D0
      PHI(14,6) = 0.2977213249234699D-1
      PHI(9,8) = 0.4146723119520977D0
      PHI(20,4) = 0.2977213249234699D-1
      PHI(18,4) = 0.1111111111111112D0
      PHI(9,5) = 0.2977213249234699D-1
      PHI(11,5) = 0.1111111111111112D0
      PHI(5,5) = -0.1870014113213579D0
      PHI(12,3) = 0.2977213249234699D-1
      PHI(5,1) = -0.9077636645642013D-1
      PHI(12,6) = 0.4146723119520977D0
      PHI(17,5) = 0.1111111111111112D0
      PHI(5,2) = -0.3522081090086456D-1
      PHI(16,6) = 0.4146723119520977D0
      PHI(7,8) = -0.3522081090086456D-1
      PHI(12,7) = 0.1111111111111112D0
      PHI(10,5) = 0.2977213249234699D-1
      PHI(17,6) = 0.2977213249234699D-1
      PHI(8,8) = -0.9077636645642013D-1
      PHI(9,7) = 0.1111111111111112D0
      PHI(16,7) = 0.1111111111111112D0
      PHI(8,7) = -0.1870014113213579D0
      PHI(5,3) = -0.1870014113213579D0
      PHI(13,8) = 0.4146723119520977D0
      PHI(2,2) = -0.1870014113213579D0
      PHI(9,4) = 0.4146723119520977D0
      PHI(12,5) = 0.1111111111111112D0
      PHI(5,4) = -0.9077636645642013D-1
      PHI(2,5) = -0.3522081090086456D-1
      PHI(3,7) = -0.3522081090086456D-1
      PHI(18,7) = 0.1111111111111112D0
      PHI(15,8) = 0.2977213249234699D-1
      PHI(3,1) = -0.1870014113213579D0
      PHI(5,7) = -0.1314458557658024D0
      PHI(18,3) = 0.4146723119520977D0
      PHI(12,2) = 0.1111111111111112D0
      PHI(2,8) = -0.1870014113213579D0
      PHI(6,6) = -0.3522081090086456D-1
      PHI(4,7) = -0.9077636645642013D-1
      PHI(19,8) = 0.2977213249234699D-1
      PHI(3,3) = -0.9077636645642013D-1
      PHI(4,4) = -0.9077636645642013D-1
      PHI(2,6) = -0.9077636645642013D-1
      PHI(17,4) = 0.1111111111111112D0
      PHI(13,5) = 0.1111111111111112D0
      PHI(9,1) = 0.2977213249234699D-1
      PHI(9,6) = 0.1111111111111112D0
      PHI(10,4) = 0.4146723119520977D0
      PHI(20,3) = 0.1111111111111112D0
      PHI(1,7) = -0.1870014113213579D0
      PHI(10,7) = 0.2977213249234699D-1
      PHI(14,1) = 0.1111111111111112D0
      PHI(19,2) = 0.1111111111111112D0
      PHI(17,3) = 0.4146723119520977D0
      PHI(8,5) = -0.1314458557658024D0
      PHI(5,6) = -0.9077636645642013D-1
      PHI(4,6) = -0.1314458557658024D0
      PHI(13,6) = 0.1111111111111112D0
      PHI(18,8) = 0.2977213249234699D-1
      PHI(4,1) = -0.9077636645642013D-1
      PHI(18,1) = 0.4146723119520977D0
      PHI(20,1) = 0.1111111111111112D0
      PHI(8,4) = -0.3522081090086456D-1
      PHI(1,3) = -0.9077636645642013D-1
      PHI(7,4) = -0.9077636645642013D-1
      PHI(3,5) = -0.9077636645642013D-1
      PHI(19,4) = 0.2977213249234699D-1
      PHI(13,7) = 0.4146723119520977D0
      PHI(17,2) = 0.2977213249234699D-1
      PHI(13,3) = 0.1111111111111112D0
      PHI(6,5) = -0.9077636645642013D-1
      PHI(8,3) = -0.9077636645642013D-1
      PHI(16,4) = 0.2977213249234699D-1
      PHI(15,7) = 0.2977213249234699D-1
      PHI(10,1) = 0.1111111111111112D0
      PHI(7,2) = -0.1870014113213579D0
      PHI(6,7) = -0.1870014113213579D0
      PHI(15,1) = 0.4146723119520977D0
      PHI(2,4) = -0.1314458557658024D0
      PHI(18,5) = 0.1111111111111112D0
      PHI(20,2) = 0.2977213249234699D-1
      PHI(14,4) = 0.4146723119520977D0
      PHI(15,2) = 0.4146723119520977D0
      PHI(10,2) = 0.4146723119520977D0
      PHI(14,5) = 0.2977213249234699D-1
      PHI(11,2) = 0.4146723119520977D0
      PHI(8,6) = -0.1870014113213579D0
      PHI(16,8) = 0.1111111111111112D0
      PHI(1,5) = -0.9077636645642013D-1
      PHI(2,1) = -0.9077636645642013D-1
      PHI(13,1) = 0.2977213249234699D-1
      PHI(20,5) = 0.4146723119520977D0
      PHI(6,8) = -0.9077636645642013D-1
      PHI(12,8) = 0.4146723119520977D0
      PHI(4,5) = -0.1870014113213579D0
      PHI(20,8) = 0.1111111111111112D0
      PHI(13,4) = 0.1111111111111112D0
      PHI(1,8) = -0.1314458557658024D0
      PHI(6,3) = -0.1314458557658024D0
      PHI(19,3) = 0.1111111111111112D0
      PHI(11,3) = 0.2977213249234699D-1
      PHI(15,6) = 0.1111111111111112D0
      PHI(7,7) = -0.9077636645642013D-1
      PHI(17,7) = 0.4146723119520977D0
      PHI(19,7) = 0.1111111111111112D0
      PHI(1,1) = -0.3522081090086456D-1
      PHI(5,8) = -0.1870014113213579D0
      PHI(3,8) = -0.9077636645642013D-1
      PHI(8,2) = -0.9077636645642013D-1
      PHI(14,2) = 0.1111111111111112D0
      PHI(6,4) = -0.1870014113213579D0
      PHI(12,1) = 0.2977213249234699D-1
      PHI(16,2) = 0.1111111111111112D0
      PHI(6,2) = -0.9077636645642013D-1
      PHI(16,3) = 0.2977213249234699D-1
      PHI(10,8) = 0.1111111111111112D0
      PHI(1,2) = -0.9077636645642013D-1
      PHI(11,6) = 0.4146723119520977D0
      PHI(9,2) = 0.1111111111111112D0
      PHI(10,3) = 0.1111111111111112D0
      PHI(11,7) = 0.2977213249234699D-1
      PHI(20,6) = 0.1111111111111112D0
      PHI(2,7) = -0.9077636645642013D-1
      PHI(13,2) = 0.2977213249234699D-1
      PHI(9,3) = 0.1111111111111112D0
      PHI(6,1) = -0.1870014113213579D0
      PHI(18,2) = 0.1111111111111112D0
      PHI(3,2) = -0.1314458557658024D0
      PHI(14,8) = 0.1111111111111112D0
      PHI(14,7) = 0.1111111111111112D0
      PHI(15,5) = 0.1111111111111112D0
      PHI(17,8) = 0.1111111111111112D0
      PHI(11,1) = 0.1111111111111112D0
      PHI(11,4) = 0.1111111111111112D0
      PHI(19,5) = 0.4146723119520977D0
      PHI(4,2) = -0.1870014113213579D0
      PHI(2,3) = -0.1870014113213579D0
      PHI(1,6) = -0.1870014113213579D0
      PHI(7,1) = -0.1314458557658024D0
      PHI(17,1) = 0.1111111111111112D0
      PHI(7,5) = -0.1870014113213579D0
      PHI(8,1) = -0.1870014113213579D0
      PHI(4,3) = -0.3522081090086456D-1
      PHI(18,6) = 0.2977213249234699D-1
      PHI(11,8) = 0.1111111111111112D0
      PHI(1,4) = -0.1870014113213579D0
      DPHI(1,6,6) = -0.7389594549567744D-1
      DPHI(3,10,7) = -0.7044162180172908D-1
      DPHI(1,10,4) = 0.2628917115316043D0
      DPHI(2,12,5) = -0.1924500897298754D0
      DPHI(3,3,6) = -0.128917115316044D-1
      DPHI(2,1,1) = 0.7389594549567744D-1
      DPHI(2,11,5) = 0.7044162180172908D-1
      DPHI(1,16,2) = -0.2628917115316043D0
      DPHI(1,13,8) = -0.2628917115316043D0
      DPHI(2,17,8) = -0.7044162180172908D-1
      DPHI(3,16,8) = 0.1924500897298754D0
      DPHI(3,17,2) = 0.7044162180172908D-1
      DPHI(1,14,3) = 0.2628917115316043D0
      DPHI(1,4,7) = 0.2232909936926024D-1
      DPHI(3,16,2) = 0.1924500897298754D0
      DPHI(3,2,8) = -0.128917115316044D-1
      DPHI(1,14,7) = 0.2628917115316043D0
      DPHI(2,13,1) = -0.7044162180172908D-1
      DPHI(1,19,5) = 0.7182335127930841D0
      DPHI(3,13,8) = 0.7182335127930841D0
      DPHI(3,14,3) = -0.7182335127930841D0
      DPHI(1,12,6) = -0.2628917115316043D0
      DPHI(2,19,7) = 0.2628917115316043D0
      DPHI(3,4,1) = 0.1795583781982711D0
      DPHI(2,7,1) = 0.4072292788290108D0
      DPHI(3,18,1) = 0.2628917115316043D0
      DPHI(3,3,4) = -0.128917115316044D-1
      DPHI(3,7,7) = -0.2232909936926024D-1
      DPHI(3,14,7) = -0.1924500897298754D0
      DPHI(2,8,8) = -0.1795583781982711D0
      DPHI(1,16,3) = -0.7044162180172908D-1
      DPHI(1,20,3) = -0.2628917115316043D0
      DPHI(2,5,4) = 0.2232909936926024D-1
      DPHI(1,19,3) = -0.1924500897298754D0
      DPHI(1,20,4) = -0.7044162180172908D-1
      DPHI(1,2,3) = 0.128917115316044D-1
      DPHI(3,16,3) = -0.5156684612641721D-1
      DPHI(1,7,8) = -0.7389594549567744D-1
      DPHI(2,14,2) = -0.2628917115316043D0
      DPHI(3,17,5) = 0.7044162180172908D-1
      DPHI(1,6,2) = -0.2232909936926024D-1
      DPHI(3,11,6) = -0.2628917115316043D0
      DPHI(3,9,4) = -0.2628917115316043D0
      DPHI(1,2,8) = -0.3110042339640731D0
      DPHI(1,5,6) = 0.2232909936926024D-1
      DPHI(1,14,4) = 0.2628917115316043D0
      DPHI(2,7,5) = 0.128917115316044D-1
      DPHI(3,18,6) = 0.7044162180172908D-1
      DPHI(3,20,6) = 0.2628917115316043D0
      DPHI(1,5,2) = 0.7389594549567744D-1
      DPHI(3,19,4) = 0.7044162180172908D-1
      DPHI(2,20,5) = -0.7182335127930841D0
      DPHI(3,20,1) = 0.7044162180172908D-1
      DPHI(2,10,6) = -0.1924500897298754D0
      DPHI(1,4,5) = -0.128917115316044D-1
      DPHI(1,10,7) = 0.7044162180172908D-1
      DPHI(2,8,4) = -0.7389594549567744D-1
      DPHI(1,15,6) = 0.2628917115316043D0
      DPHI(1,10,1) = 0.7044162180172908D-1
      DPHI(2,20,1) = -0.1924500897298754D0
      DPHI(1,18,2) = 0.7044162180172908D-1
      DPHI(3,14,4) = 0.7182335127930841D0
      DPHI(1,17,4) = -0.1924500897298754D0
      DPHI(3,12,5) = -0.2628917115316043D0
      DPHI(1,8,8) = 0.2232909936926024D-1
      DPHI(2,7,4) = -0.1795583781982711D0
      DPHI(1,14,1) = 0.7044162180172908D-1
      DPHI(1,11,6) = 0.7182335127930841D0
      DPHI(3,6,4) = -0.3110042339640731D0
      DPHI(2,13,5) = -0.2628917115316043D0
      DPHI(1,3,8) = -0.1795583781982711D0
      DPHI(1,4,3) = 0.7389594549567744D-1
      DPHI(3,6,3) = 0.4072292788290108D0
      DPHI(3,12,4) = -0.7044162180172908D-1
      DPHI(1,18,4) = 0.7044162180172908D-1
      DPHI(3,7,8) = -0.7389594549567744D-1
      DPHI(3,9,7) = -0.2628917115316043D0
      DPHI(2,17,3) = -0.2628917115316043D0
      DPHI(1,18,7) = 0.2628917115316043D0
      DPHI(2,2,8) = -0.128917115316044D-1
      DPHI(3,4,2) = -0.128917115316044D-1
      DPHI(2,11,7) = 0.7044162180172908D-1
      DPHI(3,7,4) = -0.1795583781982711D0
      DPHI(1,11,1) = -0.1924500897298754D0
      DPHI(3,3,2) = -0.4072292788290108D0
      DPHI(1,19,2) = -0.1924500897298754D0
      DPHI(1,15,1) = 0.2628917115316043D0
      DPHI(2,19,5) = 0.2628917115316043D0
      DPHI(1,18,6) = 0.7044162180172908D-1
      DPHI(2,4,2) = 0.128917115316044D-1
      DPHI(1,3,4) = 0.128917115316044D-1
      DPHI(3,6,8) = -0.1795583781982711D0
      DPHI(1,6,7) = -0.3110042339640731D0
      DPHI(1,12,2) = -0.2628917115316043D0
      DPHI(3,15,1) = -0.7182335127930841D0
      DPHI(1,9,6) = 0.1924500897298754D0
      DPHI(3,3,7) = 0.7389594549567744D-1
      DPHI(2,15,7) = 0.7044162180172908D-1
      DPHI(2,18,1) = -0.7182335127930841D0
      DPHI(3,5,3) = 0.128917115316044D-1
      DPHI(2,4,5) = 0.128917115316044D-1
      DPHI(3,8,5) = 0.4072292788290108D0
      DPHI(2,12,8) = 0.7182335127930841D0
      DPHI(1,12,1) = -0.7044162180172908D-1
      DPHI(2,10,3) = 0.1924500897298754D0
      DPHI(2,10,5) = -0.5156684612641721D-1
      DPHI(1,10,8) = 0.2628917115316043D0
      DPHI(1,5,7) = -0.4072292788290108D0
      DPHI(2,12,3) = 0.5156684612641721D-1
      DPHI(2,17,1) = -0.2628917115316043D0
      DPHI(2,5,3) = -0.128917115316044D-1
      DPHI(3,17,6) = 0.7044162180172908D-1
      DPHI(2,11,8) = 0.2628917115316043D0
      DPHI(3,8,1) = 0.128917115316044D-1
      DPHI(1,6,4) = 0.128917115316044D-1
      DPHI(1,14,5) = 0.7044162180172908D-1
      DPHI(2,11,3) = 0.7044162180172908D-1
      DPHI(1,4,6) = -0.4072292788290108D0
      DPHI(3,2,7) = 0.1795583781982711D0
      DPHI(3,11,1) = -0.2628917115316043D0
      DPHI(1,6,5) = -0.1795583781982711D0
      DPHI(3,14,5) = -0.5156684612641721D-1
      DPHI(3,9,5) = -0.7044162180172908D-1
      DPHI(2,12,7) = 0.1924500897298754D0
      DPHI(3,19,1) = 0.2628917115316043D0
      DPHI(2,19,3) = 0.2628917115316043D0
      DPHI(1,9,5) = 0.5156684612641721D-1
      DPHI(2,13,6) = -0.2628917115316043D0
      DPHI(2,15,4) = 0.2628917115316043D0
      DPHI(2,18,7) = 0.1924500897298754D0
      DPHI(1,12,5) = -0.7044162180172908D-1
      DPHI(2,15,6) = 0.7044162180172908D-1
      DPHI(1,11,8) = 0.1924500897298754D0
      DPHI(2,6,6) = 0.7389594549567744D-1
      DPHI(1,15,8) = 0.7044162180172908D-1
      DPHI(3,1,1) = 0.7389594549567744D-1
      DPHI(3,7,3) = 0.128917115316044D-1
      DPHI(3,11,4) = -0.7044162180172908D-1
      DPHI(1,3,3) = -0.2232909936926024D-1
      DPHI(1,2,7) = -0.1795583781982711D0
      DPHI(3,4,4) = 0.2232909936926024D-1
      DPHI(3,10,4) = -0.2628917115316043D0
      DPHI(2,3,5) = -0.2232909936926024D-1
      DPHI(1,16,7) = -0.7044162180172908D-1
      DPHI(3,7,1) = 0.4072292788290108D0
      DPHI(3,6,5) = -0.2232909936926024D-1
      DPHI(3,8,8) = -0.1795583781982711D0
      DPHI(3,15,8) = 0.5156684612641721D-1
      DPHI(1,16,8) = -0.7044162180172908D-1
      DPHI(1,15,2) = 0.2628917115316043D0
      DPHI(1,11,5) = 0.1924500897298754D0
      DPHI(2,17,7) = -0.2628917115316043D0
      DPHI(1,11,3) = -0.5156684612641721D-1
      DPHI(2,5,1) = 0.1795583781982711D0
      DPHI(2,8,1) = 0.128917115316044D-1
      DPHI(2,4,4) = -0.1795583781982711D0
      DPHI(2,17,4) = -0.7044162180172908D-1
      DPHI(3,15,6) = 0.1924500897298754D0
      DPHI(2,20,6) = -0.1924500897298754D0
      DPHI(3,16,7) = -0.1924500897298754D0
      DPHI(2,9,1) = -0.7044162180172908D-1
      DPHI(1,13,3) = -0.2628917115316043D0
      DPHI(3,15,2) = 0.7182335127930841D0
      DPHI(3,3,3) = 0.1795583781982711D0
      DPHI(1,17,3) = -0.7182335127930841D0
      DPHI(2,7,8) = -0.7389594549567744D-1
      DPHI(1,11,7) = 0.5156684612641721D-1
      DPHI(1,17,5) = 0.1924500897298754D0
      DPHI(1,5,1) = 0.1795583781982711D0
      DPHI(2,11,2) = 0.2628917115316043D0
      DPHI(2,4,3) = -0.7389594549567744D-1
      DPHI(3,13,3) = -0.1924500897298754D0
      DPHI(1,4,4) = 0.1795583781982711D0
      DPHI(3,2,6) = 0.2232909936926024D-1
      DPHI(2,9,6) = -0.2628917115316043D0
      DPHI(2,16,5) = 0.2628917115316043D0
      DPHI(2,3,3) = -0.1795583781982711D0
      DPHI(2,3,4) = -0.3110042339640731D0
      DPHI(2,20,2) = -0.5156684612641721D-1
      DPHI(1,15,3) = 0.7044162180172908D-1
      DPHI(2,15,5) = 0.7044162180172908D-1
      DPHI(3,9,3) = -0.2628917115316043D0
      DPHI(2,2,4) = -0.4072292788290108D0
      DPHI(3,5,5) = 0.128917115316044D-1
      DPHI(1,13,7) = -0.2628917115316043D0
      DPHI(2,16,1) = 0.7044162180172908D-1
      DPHI(1,20,7) = -0.2628917115316043D0
      DPHI(2,5,2) = 0.7389594549567744D-1
      DPHI(1,18,3) = 0.2628917115316043D0
      DPHI(1,1,4) = 0.3110042339640731D0
      DPHI(3,15,3) = -0.1924500897298754D0
      DPHI(2,13,2) = -0.7044162180172908D-1
      DPHI(2,19,2) = 0.7044162180172908D-1
      DPHI(3,8,4) = -0.7389594549567744D-1
      DPHI(3,13,7) = -0.7182335127930841D0
      DPHI(2,4,8) = -0.3110042339640731D0
      DPHI(1,10,2) = 0.2628917115316043D0
      DPHI(3,20,4) = 0.7044162180172908D-1
      DPHI(3,19,6) = 0.2628917115316043D0
      DPHI(1,6,8) = -0.1795583781982711D0
      DPHI(1,9,8) = 0.7182335127930841D0
      DPHI(3,18,5) = 0.7044162180172908D-1
      DPHI(1,16,5) = -0.2628917115316043D0
      DPHI(1,18,8) = 0.7044162180172908D-1
      DPHI(1,17,1) = -0.1924500897298754D0
      DPHI(1,18,5) = 0.2628917115316043D0
      DPHI(2,6,8) = 0.2232909936926024D-1
      DPHI(3,11,7) = -0.7044162180172908D-1
      DPHI(3,6,2) = -0.1795583781982711D0
      DPHI(1,12,4) = -0.2628917115316043D0
      DPHI(3,10,3) = -0.2628917115316043D0
      DPHI(2,16,6) = 0.2628917115316043D0
      DPHI(1,9,7) = 0.1924500897298754D0
      DPHI(3,9,2) = -0.7044162180172908D-1
      DPHI(1,14,6) = 0.7044162180172908D-1
      DPHI(2,9,8) = -0.2628917115316043D0
      DPHI(3,13,4) = 0.1924500897298754D0
      DPHI(1,16,4) = -0.7044162180172908D-1
      DPHI(2,6,2) = 0.1795583781982711D0
      DPHI(2,1,8) = -0.4072292788290108D0
      DPHI(3,12,7) = -0.2628917115316043D0
      DPHI(3,12,3) = -0.7044162180172908D-1
      DPHI(3,11,5) = -0.2628917115316043D0
      DPHI(3,14,6) = 0.5156684612641721D-1
      DPHI(2,13,4) = -0.7044162180172908D-1
      DPHI(3,16,4) = 0.5156684612641721D-1
      DPHI(2,6,5) = 0.1795583781982711D0
      DPHI(3,20,8) = 0.2628917115316043D0
      DPHI(1,1,1) = 0.7389594549567744D-1
      DPHI(2,2,7) = 0.2232909936926024D-1
      DPHI(3,12,8) = -0.2628917115316043D0
      DPHI(2,10,4) = 0.7182335127930841D0
      DPHI(2,11,6) = 0.2628917115316043D0
      DPHI(3,19,7) = 0.7044162180172908D-1
      DPHI(1,13,4) = -0.2628917115316043D0
      DPHI(3,1,6) = -0.128917115316044D-1
      DPHI(3,12,6) = -0.2628917115316043D0
      DPHI(3,18,7) = 0.7044162180172908D-1
      DPHI(1,17,7) = 0.7182335127930841D0
      DPHI(3,2,1) = 0.1795583781982711D0
      DPHI(2,17,2) = -0.7044162180172908D-1
      DPHI(2,1,5) = 0.1795583781982711D0
      DPHI(2,5,7) = -0.4072292788290108D0
      DPHI(1,9,1) = -0.5156684612641721D-1
      DPHI(2,16,2) = 0.7044162180172908D-1
      DPHI(1,13,1) = -0.7044162180172908D-1
      DPHI(1,8,5) = -0.4072292788290108D0
      DPHI(2,13,8) = -0.2628917115316043D0
      DPHI(3,17,8) = 0.2628917115316043D0
      DPHI(2,2,1) = 0.1795583781982711D0
      DPHI(2,7,2) = 0.128917115316044D-1
      DPHI(2,14,3) = -0.2628917115316043D0
      DPHI(2,20,8) = 0.1924500897298754D0
      DPHI(3,5,6) = -0.1795583781982711D0
      DPHI(3,14,1) = -0.1924500897298754D0
      DPHI(1,3,7) = -0.7389594549567744D-1
      DPHI(2,14,7) = -0.7044162180172908D-1
      DPHI(3,5,8) = -0.3110042339640731D0
      DPHI(2,19,8) = 0.7044162180172908D-1
      DPHI(3,13,1) = -0.5156684612641721D-1
      DPHI(3,1,3) = 0.1795583781982711D0
      DPHI(2,4,7) = -0.1795583781982711D0
      DPHI(1,7,2) = 0.128917115316044D-1
      DPHI(3,19,5) = 0.2628917115316043D0
      DPHI(2,16,3) = 0.7044162180172908D-1
      DPHI(3,12,1) = -0.7044162180172908D-1
      DPHI(1,14,2) = 0.7044162180172908D-1
      DPHI(2,17,5) = -0.2628917115316043D0
      DPHI(3,8,3) = -0.2232909936926024D-1
      DPHI(2,20,3) = 0.1924500897298754D0
      DPHI(2,9,4) = -0.2628917115316043D0
      DPHI(2,19,4) = 0.7044162180172908D-1
      DPHI(2,20,4) = 0.5156684612641721D-1
      DPHI(2,2,6) = 0.1795583781982711D0
      DPHI(3,7,6) = -0.1795583781982711D0
      DPHI(3,1,5) = 0.1795583781982711D0
      DPHI(3,14,2) = 0.1924500897298754D0
      DPHI(1,1,3) = 0.1795583781982711D0
      DPHI(2,8,3) = -0.1795583781982711D0
      DPHI(1,6,3) = 0.4072292788290108D0
      DPHI(1,20,5) = -0.2628917115316043D0
      DPHI(1,10,6) = 0.2628917115316043D0
      DPHI(1,4,2) = 0.3110042339640731D0
      DPHI(3,8,7) = 0.128917115316044D-1
      DPHI(1,20,1) = -0.2628917115316043D0
      DPHI(2,14,4) = -0.2628917115316043D0
      DPHI(3,20,7) = 0.2628917115316043D0
      DPHI(1,8,3) = 0.1795583781982711D0
      DPHI(1,3,1) = 0.128917115316044D-1
      DPHI(3,4,7) = 0.1795583781982711D0
      DPHI(2,10,7) = 0.5156684612641721D-1
      DPHI(3,2,3) = 0.3110042339640731D0
      DPHI(1,2,5) = -0.7389594549567744D-1
      DPHI(2,4,6) = 0.4072292788290108D0
      DPHI(2,10,1) = -0.1924500897298754D0
      DPHI(2,18,2) = -0.1924500897298754D0
      DPHI(3,4,3) = 0.7389594549567744D-1
      DPHI(1,2,4) = 0.4072292788290108D0
      DPHI(1,13,5) = -0.7044162180172908D-1
      DPHI(3,4,8) = -0.128917115316044D-1
      DPHI(2,14,1) = -0.2628917115316043D0
      DPHI(3,14,8) = 0.1924500897298754D0
      DPHI(3,2,4) = -0.4072292788290108D0
      DPHI(2,9,7) = -0.7044162180172908D-1
      DPHI(2,2,3) = -0.128917115316044D-1
      DPHI(1,9,3) = -0.1924500897298754D0
      DPHI(1,11,4) = -0.1924500897298754D0
      DPHI(1,1,5) = 0.2232909936926024D-1
      DPHI(3,13,5) = -0.1924500897298754D0
      DPHI(3,1,2) = 0.2232909936926024D-1
      DPHI(3,5,2) = -0.7389594549567744D-1
      DPHI(3,8,2) = -0.1795583781982711D0
      DPHI(2,12,6) = -0.7182335127930841D0
      DPHI(2,18,4) = 0.1924500897298754D0
      DPHI(2,5,5) = 0.3110042339640731D0
      DPHI(1,3,2) = 0.4072292788290108D0
      DPHI(1,17,2) = -0.5156684612641721D-1
      DPHI(1,3,6) = -0.3110042339640731D0
      DPHI(2,1,2) = 0.1795583781982711D0
      DPHI(3,2,2) = -0.128917115316044D-1
      DPHI(3,17,3) = 0.2628917115316043D0
      DPHI(2,5,6) = 0.1795583781982711D0
      DPHI(1,19,6) = 0.1924500897298754D0
      DPHI(1,20,8) = -0.7044162180172908D-1
      DPHI(1,14,8) = 0.2628917115316043D0
      DPHI(3,6,6) = -0.7389594549567744D-1
      DPHI(1,7,1) = 0.4072292788290108D0
      DPHI(1,11,2) = -0.7182335127930841D0
      DPHI(3,6,1) = 0.128917115316044D-1
      DPHI(1,7,3) = 0.128917115316044D-1
      DPHI(1,4,1) = 0.1795583781982711D0
      DPHI(2,15,1) = 0.2628917115316043D0
      DPHI(1,15,7) = 0.7044162180172908D-1
      DPHI(1,18,1) = 0.2628917115316043D0
      DPHI(1,1,2) = 0.1795583781982711D0
      DPHI(2,18,6) = -0.5156684612641721D-1
      DPHI(1,19,8) = 0.5156684612641721D-1
      DPHI(1,8,2) = 0.1795583781982711D0
      DPHI(1,5,3) = 0.3110042339640731D0
      DPHI(1,12,8) = -0.2628917115316043D0
      DPHI(1,10,3) = 0.7044162180172908D-1
      DPHI(1,10,5) = 0.7044162180172908D-1
      DPHI(3,1,7) = 0.3110042339640731D0
      DPHI(2,12,2) = -0.1924500897298754D0
      DPHI(1,9,2) = -0.1924500897298754D0
      DPHI(1,1,6) = -0.128917115316044D-1
      DPHI(3,15,7) = -0.5156684612641721D-1
      DPHI(1,12,3) = -0.7044162180172908D-1
      DPHI(3,18,3) = 0.2628917115316043D0
      DPHI(3,1,8) = -0.4072292788290108D0
      DPHI(2,2,5) = 0.7389594549567744D-1
      DPHI(2,17,6) = -0.7044162180172908D-1
      DPHI(2,3,8) = -0.1795583781982711D0
      DPHI(2,12,1) = -0.5156684612641721D-1
      DPHI(2,8,5) = 0.4072292788290108D0
      DPHI(3,10,2) = -0.2628917115316043D0
      DPHI(3,20,2) = 0.7044162180172908D-1
      DPHI(2,1,7) = -0.128917115316044D-1
      DPHI(2,10,8) = 0.1924500897298754D0
      DPHI(1,8,1) = 0.3110042339640731D0
      DPHI(3,19,8) = 0.7044162180172908D-1
      DPHI(1,9,4) = -0.7182335127930841D0
      DPHI(1,7,5) = -0.3110042339640731D0
      DPHI(3,4,6) = -0.4072292788290108D0
      DPHI(2,3,7) = -0.7389594549567744D-1
      DPHI(3,17,1) = 0.7044162180172908D-1
      DPHI(3,18,8) = 0.7044162180172908D-1
      DPHI(2,1,3) = 0.2232909936926024D-1
      DPHI(3,11,8) = -0.7044162180172908D-1
      DPHI(2,11,1) = 0.7044162180172908D-1
      DPHI(2,14,5) = -0.7044162180172908D-1
      DPHI(2,9,5) = -0.7044162180172908D-1
      DPHI(3,16,5) = -0.7182335127930841D0
      DPHI(1,8,6) = -0.128917115316044D-1
      DPHI(1,1,7) = -0.128917115316044D-1
      DPHI(1,12,7) = -0.7044162180172908D-1
      DPHI(2,19,1) = 0.2628917115316043D0
      DPHI(3,4,5) = 0.3110042339640731D0
      DPHI(3,10,6) = -0.7044162180172908D-1
      DPHI(1,13,6) = -0.7044162180172908D-1
      DPHI(3,3,1) = 0.3110042339640731D0
      DPHI(3,11,3) = -0.7044162180172908D-1
      DPHI(1,15,4) = 0.7044162180172908D-1
      DPHI(3,5,1) = -0.2232909936926024D-1
      DPHI(3,7,2) = -0.3110042339640731D0
      DPHI(3,20,3) = 0.7044162180172908D-1
      DPHI(3,5,7) = 0.4072292788290108D0
      DPHI(3,19,3) = 0.7044162180172908D-1
      DPHI(3,13,6) = 0.1924500897298754D0
      DPHI(3,15,4) = 0.1924500897298754D0
      DPHI(2,7,6) = -0.2232909936926024D-1
      DPHI(3,8,6) = -0.3110042339640731D0
      DPHI(1,2,2) = 0.128917115316044D-1
      DPHI(1,6,1) = 0.128917115316044D-1
      DPHI(1,19,1) = -0.7182335127930841D0
      DPHI(2,15,8) = 0.7044162180172908D-1
      DPHI(1,7,4) = -0.2232909936926024D-1
      DPHI(1,7,6) = -0.1795583781982711D0
      DPHI(2,8,6) = 0.128917115316044D-1
      DPHI(3,17,4) = 0.2628917115316043D0
      DPHI(1,4,8) = -0.128917115316044D-1
      DPHI(2,1,6) = 0.3110042339640731D0
      DPHI(1,20,6) = -0.7044162180172908D-1
      DPHI(2,16,7) = 0.2628917115316043D0
      DPHI(1,19,7) = 0.1924500897298754D0
      DPHI(2,16,8) = 0.2628917115316043D0
      DPHI(2,15,2) = 0.2628917115316043D0
      DPHI(1,19,4) = -0.5156684612641721D-1
      DPHI(2,7,7) = -0.1795583781982711D0
      DPHI(3,17,7) = 0.2628917115316043D0
      DPHI(1,3,5) = -0.1795583781982711D0
      DPHI(3,10,1) = -0.2628917115316043D0
      DPHI(3,18,2) = 0.2628917115316043D0
      DPHI(3,5,4) = -0.1795583781982711D0
      DPHI(1,8,4) = 0.7389594549567744D-1
      DPHI(3,3,5) = 0.1795583781982711D0
      DPHI(3,9,1) = -0.7044162180172908D-1
      DPHI(2,13,3) = -0.7044162180172908D-1
      DPHI(2,3,1) = 0.128917115316044D-1
      DPHI(1,20,2) = -0.7044162180172908D-1
      DPHI(1,15,5) = 0.2628917115316043D0
      DPHI(3,6,7) = 0.128917115316044D-1
      DPHI(2,7,3) = -0.3110042339640731D0
      DPHI(2,9,3) = -0.7044162180172908D-1
      DPHI(2,11,4) = 0.2628917115316043D0
      DPHI(2,3,6) = 0.128917115316044D-1
      DPHI(2,4,1) = -0.2232909936926024D-1
      DPHI(1,16,1) = -0.2628917115316043D0
      DPHI(3,11,2) = -0.2628917115316043D0
      DPHI(1,2,6) = -0.1795583781982711D0
      DPHI(3,9,6) = -0.7044162180172908D-1
      DPHI(2,6,3) = -0.4072292788290108D0
      DPHI(2,8,2) = -0.2232909936926024D-1
      DPHI(3,18,4) = 0.2628917115316043D0
      DPHI(3,2,5) = 0.7389594549567744D-1
      DPHI(2,15,3) = 0.2628917115316043D0
      DPHI(3,15,5) = -0.1924500897298754D0
      DPHI(1,13,2) = -0.7044162180172908D-1
      DPHI(2,6,4) = -0.128917115316044D-1
      DPHI(2,6,7) = -0.128917115316044D-1
      DPHI(2,13,7) = -0.2628917115316043D0
      DPHI(3,16,1) = -0.1924500897298754D0
      DPHI(2,20,7) = 0.7182335127930841D0
      DPHI(3,1,4) = -0.128917115316044D-1
      DPHI(1,5,4) = 0.1795583781982711D0
      DPHI(3,7,5) = 0.128917115316044D-1
      DPHI(2,19,6) = 0.7044162180172908D-1
      DPHI(2,2,2) = 0.3110042339640731D0
      DPHI(2,18,3) = 0.7182335127930841D0
      DPHI(1,17,6) = 0.5156684612641721D-1
      DPHI(3,13,2) = 0.5156684612641721D-1
      DPHI(3,19,2) = 0.2628917115316043D0
      DPHI(2,3,2) = 0.4072292788290108D0
      DPHI(2,10,2) = -0.7182335127930841D0
      DPHI(2,6,1) = 0.3110042339640731D0
      DPHI(1,8,7) = -0.128917115316044D-1
      DPHI(2,14,8) = -0.7044162180172908D-1
      DPHI(2,1,4) = -0.128917115316044D-1
      DPHI(1,7,7) = -0.1795583781982711D0
      DPHI(2,18,8) = 0.5156684612641721D-1
      DPHI(2,18,5) = -0.1924500897298754D0
      DPHI(2,5,8) = -0.128917115316044D-1
      DPHI(1,17,8) = 0.1924500897298754D0
      DPHI(1,2,1) = -0.2232909936926024D-1
      DPHI(3,12,2) = -0.7044162180172908D-1
      DPHI(1,16,6) = -0.2628917115316043D0
      DPHI(1,1,8) = -0.4072292788290108D0
      DPHI(2,9,2) = -0.2628917115316043D0
      DPHI(3,10,5) = -0.7044162180172908D-1
      DPHI(3,20,5) = 0.2628917115316043D0
      DPHI(2,12,4) = 0.1924500897298754D0
      DPHI(3,16,6) = 0.7182335127930841D0
      DPHI(3,3,8) = 0.2232909936926024D-1
      DPHI(1,5,8) = -0.128917115316044D-1
      DPHI(3,10,8) = -0.7044162180172908D-1
      DPHI(1,5,5) = -0.128917115316044D-1
      DPHI(2,14,6) = -0.7044162180172908D-1
      DPHI(2,8,7) = -0.3110042339640731D0
      DPHI(3,9,8) = -0.2628917115316043D0
      DPHI(2,16,4) = 0.7044162180172908D-1
      PHIS(7,3) = 0.5257834230632085D0
      PHIS(3,4) = -0.9622504486493765D-1
      PHIS(5,1) = 0.1408832436034581D0
      PHIS(5,2) = 0.5257834230632085D0
      PHIS(5,3) = 0.1408832436034581D0
      PHIS(2,2) = 0.9622504486493735D-1
      PHIS(5,4) = 0.5257834230632085D0
      PHIS(3,1) = 0.9622504486493735D-1
      PHIS(3,3) = -0.1666666666666668D0
      PHIS(4,4) = -0.1666666666666668D0
      PHIS(4,1) = -0.1666666666666668D0
      PHIS(8,4) = 0.5257834230632085D0
      PHIS(1,3) = -0.1666666666666668D0
      PHIS(7,4) = 0.1408832436034581D0
      PHIS(8,3) = 0.5257834230632085D0
      PHIS(7,2) = 0.1408832436034581D0
      PHIS(2,4) = -0.1666666666666668D0
      PHIS(2,1) = -0.1666666666666668D0
      PHIS(6,3) = 0.1408832436034581D0
      PHIS(1,1) = -0.9622504486493765D-1
      PHIS(8,2) = 0.1408832436034581D0
      PHIS(6,4) = 0.1408832436034581D0
      PHIS(6,2) = 0.5257834230632085D0
      PHIS(1,2) = -0.1666666666666668D0
      PHIS(6,1) = 0.5257834230632085D0
      PHIS(3,2) = -0.1666666666666668D0
      PHIS(4,2) = -0.9622504486493765D-1
      PHIS(2,3) = -0.9622504486493765D-1
      PHIS(7,1) = 0.5257834230632085D0
      PHIS(8,1) = 0.1408832436034581D0
      PHIS(4,3) = 0.9622504486493735D-1
      PHIS(1,4) = 0.9622504486493735D-1
      DPHIS(2,1,1) = 0.1830127018922193D0
      DPHIS(2,7,1) = 0.3333333333333333D0
      DPHIS(2,5,4) = -0.3333333333333333D0
      DPHIS(1,2,3) = -0.1830127018922193D0
      DPHIS(1,6,2) = 0.3333333333333333D0
      DPHIS(1,5,2) = -0.9106836025229589D0
      DPHIS(2,8,4) = 0.9106836025229589D0
      DPHIS(2,7,4) = 0.3333333333333333D0
      DPHIS(1,4,3) = -0.6830127018922194D0
      DPHIS(2,4,2) = -0.1830127018922193D0
      DPHIS(1,3,4) = -0.1830127018922193D0
      DPHIS(2,5,3) = -0.3333333333333333D0
      DPHIS(1,6,4) = 0.3333333333333333D0
      DPHIS(1,3,3) = -0.2276709006307396D0
      DPHIS(2,5,1) = -0.3333333333333333D0
      DPHIS(2,8,1) = -0.2440169358562925D0
      DPHIS(2,4,4) = -0.2276709006307396D0
      DPHIS(1,5,1) = -0.2440169358562925D0
      DPHIS(2,4,3) = 0.6830127018922194D0
      DPHIS(1,4,4) = -0.610042339640732D-1
      DPHIS(2,3,3) = 0.610042339640732D-1
      DPHIS(2,3,4) = -0.1830127018922193D0
      DPHIS(2,2,4) = -0.610042339640732D-1
      DPHIS(2,5,2) = -0.3333333333333333D0
      DPHIS(1,1,4) = -0.6830127018922194D0
      DPHIS(2,6,2) = 0.9106836025229589D0
      DPHIS(1,1,1) = 0.1830127018922193D0
      DPHIS(2,2,1) = 0.2276709006307396D0
      DPHIS(2,7,2) = 0.3333333333333333D0
      DPHIS(1,7,2) = -0.2440169358562925D0
      DPHIS(1,1,3) = -0.610042339640732D-1
      DPHIS(2,8,3) = -0.9106836025229589D0
      DPHIS(1,6,3) = 0.3333333333333333D0
      DPHIS(1,4,2) = 0.1830127018922193D0
      DPHIS(1,8,3) = -0.3333333333333333D0
      DPHIS(1,3,1) = 0.6830127018922194D0
      DPHIS(1,2,4) = -0.2276709006307396D0
      DPHIS(2,2,3) = 0.1830127018922193D0
      DPHIS(1,3,2) = 0.610042339640732D-1
      DPHIS(2,1,2) = -0.610042339640732D-1
      DPHIS(1,7,1) = -0.9106836025229589D0
      DPHIS(1,7,3) = 0.9106836025229589D0
      DPHIS(1,4,1) = 0.2276709006307396D0
      DPHIS(1,1,2) = 0.2276709006307396D0
      DPHIS(1,8,2) = -0.3333333333333333D0
      DPHIS(1,5,3) = 0.2440169358562925D0
      DPHIS(1,8,1) = -0.3333333333333333D0
      DPHIS(2,1,3) = 0.2276709006307396D0
      DPHIS(1,2,2) = 0.6830127018922194D0
      DPHIS(1,6,1) = 0.3333333333333333D0
      DPHIS(1,7,4) = 0.2440169358562925D0
      DPHIS(1,8,4) = -0.3333333333333333D0
      DPHIS(2,3,1) = 0.6830127018922194D0
      DPHIS(2,7,3) = 0.3333333333333333D0
      DPHIS(2,4,1) = 0.610042339640732D-1
      DPHIS(2,6,3) = -0.2440169358562925D0
      DPHIS(2,8,2) = 0.2440169358562925D0
      DPHIS(2,6,4) = 0.2440169358562925D0
      DPHIS(1,5,4) = 0.9106836025229589D0
      DPHIS(2,2,2) = -0.6830127018922194D0
      DPHIS(2,3,2) = -0.2276709006307396D0
      DPHIS(2,6,1) = -0.9106836025229589D0
      DPHIS(2,1,4) = -0.6830127018922194D0
      DPHIS(1,2,1) = 0.610042339640732D-1

      END
