// FICHIER : Met_biellette.h
// CLASSE : Met_biellette

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

 
 /************************************************************************
 *                                                                      *
 *     DATE:        15/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 * La classe Met_biellette est une classe derivee de la classe Met_abstraite
 * et permet de realiser tous les calculs lies a la metrique d'une biellette.
 * N.B. : La dimension de l'espace est 1.
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                      *
 *                                                                $     *
 *                                                                      *
 ************************************************************************/


// La classe Met_biellette est une classe derivee de la classe Met_abstraite
// et permet de realiser tous les calculs lies a la metrique d'une biellette.
// N.B. : La dimension de l'espace est 1.


#ifndef MET_BIELLETTE_H
#define MET_BIELLETTE_H

#include "Met_abstraite.h"

/// @addtogroup groupe_des_metrique
///  @{
///



class Met_biellette : public Met_abstraite
{


	public :


		// CONSTRUCTEUR :
		
		// Constructeur par defaut
		Met_biellette ();	
	    // constructeur permettant de dimensionner unique ment certaine variables
	    // dim = dimension de l'espace, tab = liste
	    // des variables a initialiser
	    Met_biellette (int dim_base,const  DdlElement& tabddl,
	                   const  Tableau<Enum_variable_metrique> & tab,int nomb_noeud);
		// constructeur de copie
		Met_biellette (const  Met_biellette&);
		// DESTRUCTEUR :
		
		~Met_biellette ();
     // Surcharge de l'operateur = : realise l'affectation
     // fonction virtuelle
     inline Met_abstraite& operator= (const Met_abstraite& met)
        { return (Met_abstraite::operator=(met));};
		
	protected :				
	// METHODES protegees:
    //==calcul des points, identique a Met_abstraite
		   
	    	    	         
	//== le nombre de vecteur de base pour la biellette est 1
	//== les fonctions de calcul de base sont donc redefini
		// calcul de la base naturel a t0
		 void Calcul_giB_0
		   ( const  Tableau<Noeud *>& tab_noeud,const   Mat_pleine& dphi, int nombre_noeud,const Vecteur& phi);
		// calcul de la base naturel a t
		 void Calcul_giB_t
		   ( const  Tableau<Noeud *>& tab_noeud, const  Mat_pleine& dphi, int nombre_noeud,const Vecteur& phi); 
		// calcul de la base naturel a t+dt
		 void Calcul_giB_tdt
		   ( const  Tableau<Noeud *>& tab_noeud, const  Mat_pleine& dphi, int nombre_noeud,const Vecteur& phi);
   //==  calcul de la variation des jacobiens
        void Djacobien_t(); // avant calcul de :  d_giB_t et giB_t
        void Djacobien_tdt(); // avant calcul de :  d_giB_tdt et giB_tdt   
		
};
/// @}  // end of group


#endif
		
		
