
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        4/06/98                                             *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Met_Pout2D constitue une boite a outil comme met_abstraite, *
 *   mais ici c'est particulièrement dédié aux poutres 2D.              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef MET_POUT2D_H
#define MET_POUT2D_H

#include "Met_PiPoCo.h"


/// @addtogroup groupe_des_metrique
///  @{
///


class  Met_Pout2D : public Met_PiPoCo
{
  public :
		// CONSTRUCTEUR :
		
		// Constructeur par defaut
		Met_Pout2D ();	
	    // constructeur permettant de dimensionner unique ment certaine variables
	    // dim = dimension de l'espace, tab = liste
	    // des variables a initialiser
	    Met_Pout2D (int dim_base,int nbvec,const  DdlElement& tabddl,
	                   const  Tableau<Enum_variable_metrique>& tabb,int nomb_noeud);
		// constructeur de copie
		Met_Pout2D (const  Met_Pout2D&);
		// DESTRUCTEUR :
		~Met_Pout2D ();
		
	// METHODES PUBLIQUES :
        // Surcharge de l'operateur = : realise l'affectation
        // dérivant de virtuel, a ne pas employer -> message d'erreur
	    Met_abstraite& operator= (const Met_abstraite& met);
	    Met_PiPoCo& operator= (const Met_PiPoCo& met);
	    // normale
        virtual Met_Pout2D& operator= (const Met_Pout2D& met);
		        		        
	  // passage de la dérivée seconde
      void DeriveeSeconde(Vecteur  const & tabD2phi);  
	
		
	// ============================ protege ===========================
	
	protected :				
     // calcul des normales a la poutre
        void Calcul_N_0 ();
        void Calcul_N_t ();
        void Calcul_N_tdt ();
	//== les fonctions de calcul de base donc redefini
		// calcul de la base naturel a t0
        // pendant le traitement il y a calcul de la base aiB et aiH de l'axe médian
        // la fonction contiend un argument de plus que la routine dans met_abstraite
		 void Calcul_giB_0 ( const   Tableau<Noeud *>& ,const   Mat_pleine& ,
		                       int, const  Vecteur& phi);
		// calcul de la base naturel a t
		 void Calcul_giB_t ( const   Tableau<Noeud *>& ,const   Mat_pleine& ,
		                       int, const   Vecteur& phi); 
		// calcul de la base naturel a t+dt
		 void Calcul_giB_tdt ( const   Tableau<Noeud *>& ,const   Mat_pleine& ,
		                       int, const   Vecteur& phi);
   //==  calcul de la variation des bases
         void D_giB_t( const  Mat_pleine& , int , const   Vecteur & phi); 
         void D_giB_tdt(const   Mat_pleine& , int , const   Vecteur & phi);
    //-----------// calcul du tenseur de courbure  dans la base naturelle
    // plusieurs cas sont etudies suivant l'instant considere
    // a l'instant t = 0
    Vecteur& courbure_0 (const  Tableau<Noeud *>& tab_noeud);
    // a l'instant t 
    Vecteur& courbure_t (const  Tableau<Noeud *>& tab_noeud);
    // a l'instant t+dt
    Vecteur& courbure_tdt (const  Tableau<Noeud *>& tab_noeud);
    // routine generale de calcul de la courbure
    double courbure(const  Tableau<Coordonnee>& tab_coor,
                       const  CoordonneeB & aiB1,const  CoordonneeH & );
    //--------------// calcul du tenseur de courbure et de sa variation 
    // plusieurs cas sont etudies suivant l'instant considere
    // a l'instant t 
    void Dcourbure_t(const  Tableau<Noeud *>& tab_noeud,  Vecteur& curb,
                        TabOper<Vecteur>& dcurb);
    // a l'instant tdt 
    void Dcourbure_tdt(const  Tableau<Noeud *>& tab_noeud,  
                        Vecteur& curb,TabOper<Vecteur>& dcurb);
    // routine generale de calcul de la courbure et de sa variation
    // en sortie : curb , la courbure  b11,b12,b22
    //            dcurb , la variation de courbure.
    void Dcourbure (const  Tableau<Coordonnee>& tab_coor,const CoordonneeB & aiB1,
                    const CoordonneeH & ,Tableau <BaseB>& DaiB,
                        Vecteur& curb,TabOper <Vecteur>& dcurb);
		   
     // DONNEES PROTEGEES
    Vecteur  const * tabD2phi; // pointeur sur les dérivées secondes courantes
        
    // fonctions privees
	                   
		
};
/// @}  // end of group


#endif
		
		
