// FICHIER : ElemPoint.h
// CLASSE : ElemPoint

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        24/02/2005                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  La classe ElemPoint_CP permet de declarer des elements     *
 *     ElemPoint associé à des contraintes plane.                       *
 *     L'élément comporte un noeud qui est identique au point d'inté-   *
 *     gration.                                                         *
 *                                                                      *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ELEM_POINT_CP_H
#define ELEM_POINT_CP_H

#include "ElemPoint.h"


/// @addtogroup groupe_des_elements_finis
///  @{
///


class ElemPoint_CP : public ElemPoint
{
		
	public :
	
		// CONSTRUCTEURS :
		// les tenseurs on par défaut la dimension 2
		// Constructeur par defaut
		ElemPoint_CP () : ElemPoint(unefois_CP,POINT_CP,2)
    { };
		// Constructeur fonction  d'un numero de maillage et d'identification  
		ElemPoint_CP (int num_mail,int num_id):
   ElemPoint(unefois_CP,POINT_CP,num_mail,num_id,2)
    { };

		// Constructeur de copie
		ElemPoint_CP (const ElemPoint_CP& elem):
   ElemPoint(elem)
    { };

		
		// DESTRUCTEUR :
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
		Element* Nevez_copie() const { Element * el= new ElemPoint_CP(*this); return el;};

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de ElemPoint_CP
		ElemPoint_CP& operator= (ElemPoint_CP& biel);
 
  // initialisation éventuelle: ajout de point d'intégration si nécessaire
  // les tenseurs ont la dimension 2
  virtual void InitialisationUmatAbaqus()
     {InitialisationUmatAbaqus_interne(2);};

  // lecture des grandeurs umat transmises par abaqus sur le pipe
  static const ElemPoint_CP::inNeNpti& Lecture_Abaqus(bool utilisation_umat_interne)
     {unefois_CP.doCoMemb->umatAbaqus.LectureDonneesUmat
        (utilisation_umat_interne,ParaGlob::NiveauImpression());return unefois_CP.doCoMemb->inne;};
  // récup du inNeNpti en cours
  static const ElemPoint_CP::inNeNpti& IncreElemPtint_encours() {return unefois_CP.doCoMemb->inne;};

  // 3) methodes propres a l'element
 
 protected:
    		
	  // ==== >>>> methodes virtuelles dérivant d'ElemMeca ============  
		// ramene la dimension des tenseurs contraintes et déformations de l'élément
		int Dim_sig_eps() const {return 2;};

 // -------------------- calcul de frontières en protected -------------------
 
	private :

      // VARIABLES PRIVEES :
      	

    // ------------------------------------------------------------------------------------   
        
	protected :
      // VARIABLES PROTÉGÉES :
 
  // place memoire commune a tous les elements ElemPoint_CP
  static ElemPoint::DonneeCommune * doCo_CP;
  // idem mais pour les indicateurs qui servent pour l'initialisation
  static ElemPoint::UneFois  unefois_CP;

  // les nombres propres à l'élément: idem ceux de Point
  //  static NombresConstruire nombre_V;

  // pour l'ajout d'element dans la liste : listTypeElemen, geree par la class Element
  class ConstrucElemPoint_CP : public ConstrucElement
       { public :  ConstrucElemPoint_CP ()
               { NouvelleTypeElement nouv(POINT_CP,CONSTANT,MECA_SOLIDE_DEFORMABLE,this);
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation ElemPoint_CP " << endl;
                 Element::listTypeElement.push_back(nouv);
               };
          Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new ElemPoint_CP (num_maill,num) ;
                return pt;};
             // ramene true si la construction de l'element est possible en fonction
             // des variables globales actuelles: ex en fonction de la dimension	
          bool Element_possible() {return true;};
       }; 
  static ConstrucElemPoint_CP construcElemPoint_CP;
 

};
#endif
	
	
		

