// FICHIER : Hexa.cc
// CLASSE : Hexa

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "GeomHexalin.h"

#include "Hexa.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

HexaMemb::DonnComHexa * Hexa::doCoHexa = NULL;  
HexaMemb::UneFois  Hexa::uneFois; 
Hexa::NombresConstruireHexa Hexa::nombre_V; 
Hexa::ConsHexa Hexa::consHexa;

// fonction définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
HexaMemb::NombresConstruire Hexa::ConstruireLesNombres() 
 { Hexa::NombresConstruireHexa nombre;
   return (HexaMemb::NombresConstruire) nombre;
  };
  
// constructeur des nombres spécifiques à la classe   
Hexa::NombresConstruireHexa::NombresConstruireHexa() 
  { nbne  = 8; // le nombre de noeud de l'élément
    nbneS = 4; // le nombre de noeud des facettes
    nbneA = 2; // le nombre de noeud des aretes
    nbi   = 8; // le nombre de point d'intégration pour le calcul mécanique
    nbiEr = 8; // le nombre de point d'intégration pour le calcul d'erreur
    nbiV  = 8; // le nombre de point d'intégration pour le calcul de second membre volumique
    nbiS  = 4; // le nombre de point d'intégration pour le calcul de second membre surfacique
    nbiA  = 2; // le nombre de point d'intégration pour le calcul de second membre linéique
    nbiMas = 8; // le nombre de point d'intégration pour le calcul de la matrice masse 
	 nbiHour = 0; // éventuellement, le nombre de point d'intégration un blocage d'hourglass
   };
	      

// =========================== constructeurs ==================


// Constructeur par defaut, le seul accepte en dimension different de 3
Hexa::Hexa () :
  HexaMemb(-3,LINEAIRE,HEXAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 8 noeuds,8 pt d'integration
   // calcul de doCoHexa egalement si c'est le premier passage
   // après hexa, le nombre de point d'intégration de surface pour le second membre
   // ici 4 et 4 noeuds pour les éléments de surface
   ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
   if ( doCoHexa == NULL)  
      {hexa = new GeomHexalin(nombre->nbi);
       hexaEr = new GeomHexalin(nombre->nbiEr);
       hexaMas = new GeomHexalin(nombre->nbiMas);}
   int dim = ParaGlob::Dimension();
   if (dim != 3)  // cas d'une dimension autre que trois
	 { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\nATTENTION -> dimension " << dim 
	      <<", pas de definition d\'elements hexaedriques lineaires "<< endl;
	   delete hexa;delete hexaEr;delete hexaMas;
	   unefois = NULL;
	  }	  
   else 
    { unefois = & uneFois; // affectation du pointeur de la classe générique 
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMashexaMas,NULL);
      unefois->nbelem_in_Prog++;
     } 
   };   
};

// Constructeur  fonction  d'un numero
// d'identification 	
Hexa::Hexa (int num_id) :
  HexaMemb(num_id,LINEAIRE,HEXAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 8 noeuds,8 pt d'integration
   // calcul de doCoHexa egalement si c'est le premier passage
   // après hexa, le nombre de point d'intégration de surface pour le second membre
   // ici 4 et 4 noeuds pour les éléments de surface
   ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
   if ( doCoHexa == NULL)  
      {hexa = new GeomHexalin(nombre->nbi);
       hexaEr = new GeomHexalin(nombre->nbiEr);
       hexaMas = new GeomHexalin(nombre->nbiMas);}
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      {if (ParaGlob::NiveauImpression() >= 2)
	      cout << "\n erreur de dimension dans Hexa, dim = " << ParaGlob::Dimension()
               << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,NULL,masse_volumique_defaut);
      unefois->nbelem_in_Prog++;
     } 
   };   
};

// Constructeur utile si  le numero de l'element et
// le tableau des noeuds sont connus
Hexa::Hexa (int num_id,const Tableau<Noeud *>& tab):
    HexaMemb(num_id,LINEAIRE,HEXAEDRE,tab) 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " Hexa::Hexa (double epaiss,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   // 8 noeuds,8 pt d'integration
   // calcul de doCoHexa egalement si c'est le premier passage
   // après hexa, le nombre de point d'intégration de surface pour le second membre
   // ici 4 et 4 noeuds pour les éléments de surface
   ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
   if ( doCoHexa == NULL)  
      {hexa = new GeomHexalin(nombre->nbi);
       hexaEr = new GeomHexalin(nombre->nbiEr);
       hexaMas = new GeomHexalin(nombre->nbiMas);}
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans Hexa, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,NULL);
      unefois->nbelem_in_Prog++;
      // construction du tableau de ddl spécifique à l'élément pour ses 
      ConstTabDdl(); 
    }  
   };   
};

Hexa::Hexa (const Hexa& HexaraM) :
 HexaMemb (HexaraM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFois.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element HexaLMemb, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
  {  ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL;ElemGeomC0* hexaMas=NULL; 
     unefois = & uneFois; // affectation du pointeur de la classe générique
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et HexaMemb
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,hexaMas,NULL);
     unefois->nbelem_in_Prog++;
   };	
};

Hexa::~Hexa ()
// Destruction effectuee dans HexaMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};


// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void Hexa::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n******************************************************************";
    sort << "\n Element Hexa (hexaedre trilineaire "<<nombre->nbi<<" pt d'integration) ";
    sort << "\n******************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFois.dualSortHexa) && (uneFois.CalimpPrem))
        { VarDualSort(sort,nom,tabSigHH_t,tabEpsBB,nombre->nbi,1) ;
          uneFois.dualSortHexa += 1;
         } 
    else if (uneFois.dualSortHexa && (uneFois.CalimpPrem))       
         VarDualSort(sort,nom,tabSigHH_t,tabEpsBB,nombre->nbi,11) ;
    else if (!(uneFois.dualSortHexa) && (uneFois.CalResPrem_tdt))       
        { VarDualSort(sort,nom,tabSigHH_t,tabEpsBB,nombre->nbi,2);
          uneFois.dualSortHexa += 1;
         }         
    else if ((uneFois.dualSortHexa) && (uneFois.CalResPrem_tdt))       
         VarDualSort(sort,nom,tabSigHH_t,tabEpsBB,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	





