// FICHIER : Hexa.h
// CLASSE : Hexa

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        09/02/2003                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe HexaLMemb est une classe générique.             *
 * Elle permet de declarer des elements                                 *
 * Hexaedriques Trilineaires  et de realiser                            *
 * le calcul du residu local et de la raideur locale pour une loi de    *
 * comportement donnee. La dimension de l'espace pour un tel element    *
 * est 3.                                                               *
 *                                                                      *
 * l'interpolation est trilineaire a 8 noeuds, le nombre de             *
 * point d'integration est  défini dans les classes dérivées.           *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------



#ifndef HEXALMEMB_H
#define HEXALMEMB_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomQuadrangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "HexaMemb.h"
#include "FrontQuadLine.h"
#include "FrontSegLine.h"


/// @addtogroup groupe_des_elements_finis
///  @{
///


class HexaLMemb : public HexaMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut 
		HexaLMemb ();
		
		// Constructeur  fonction  d'un numero
		// d'identification 
		HexaLMemb (int num_id);
		
		// Constructeur fonction  d'un numero d'identification et
		// du tableau de connexite des noeuds 
		HexaLMemb (int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		HexaLMemb (const HexaLMemb& hexa);
		
		// DESTRUCTEUR :
		virtual ~HexaLMemb ();
				
		// Surcharge de l'operateur = : realise l'egalite entre deux instances de Hexa
		Hexa& operator= (Hexa& hexa);
		
		// METHODES :
// 1) derivant des virtuelles pures

        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	

// 2) derivant des virtuelles 
// 3) methodes propres a l'element
                
	protected :
	  
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegLine(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem) 
          { return ((ElFrontiere*) (new FrontQuadLine(tab,ddelem)));};

      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaMembL1 
        static HexaMemb::DonnComHexa * doCoHexa;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static HexaMemb::UneFois  uneFois;

	    // fonction définissant les nombres (de noeud, de point d'integ ..)
	    // utilisé dans la construction des éléments
	    NombresConstruire ConstruireLesNombres() ; 
	    class  NombresConstruireHexa : public NombresConstruire
	     {  public: NombresConstruireHexa(); 
	      };
	    static NombresConstruireHexa nombre_V; // les nombres propres à l'élément
	     		
};
/// @}  // end of group
#endif
	
	
		

