// FICHIER : HexaQ_cm1pti.cc
// CLASSE : HexaQ_cm1pti

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "GeomHexaQuad.h"

#include "HexaQ_cm1pti.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

HexaMemb::DonnComHexa * HexaQ_cm1pti::doCoHexa = NULL; 
HexaMemb::UneFois  HexaQ_cm1pti::uneFois; 
HexaQ_cm1pti::NombresConstruireHexaQ_cm1pti HexaQ_cm1pti::nombre_V; 
HexaQ_cm1pti::ConsHexaQ_cm1pti HexaQ_cm1pti::consHexaQ_cm1pti;

// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
HexaQ_cm1pti::NombresConstruireHexaQ_cm1pti::NombresConstruireHexaQ_cm1pti() 
 {  nbne  = 20; // le nombre de noeud de l'élément
    nbneS = 8; // le nombre de noeud des facettes
    nbneA = 3; // le nombre de noeud des aretes
    nbi   = 1; // le nombre de point d'intégration pour le calcul mécanique
    nbiEr = 27; // le nombre de point d'intégration pour le calcul d'erreur
    nbiV  = 1; // le nombre de point d'intégration pour le calcul de second membre volumique
    nbiS  = 1; // le nombre de point d'intégration pour le calcul de second membre surfacique
    nbiA  = 1; // le nombre de point d'intégration pour le calcul de second membre linéique
    nbiMas = 27; // le nombre de point d'intégration pour le calcul de la matrice masse 
	 nbiHour = 27; // éventuellement, le nombre de point d'intégration un blocage d'hourglass
  }; 

// =========================== constructeurs ==================


// Constructeur par defaut, le seul accepte en dimension different de 3
HexaQ_cm1pti::HexaQ_cm1pti () :
  HexaMemb(0,-3,QUADRATIQUE,HEXAEDRE,"_cm1pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
   }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 20 noeuds,8 pt d'integration
   // calcul de doCoHexaQ_cm1pti egalement si c'est le premier passage
   // après hexa, le nombre de point d'intégration de surface pour le second membre
   // ici 4 et 8 noeuds pour les éléments de surface
   ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
	  ElemGeomC0* hexaeHourg=NULL; // pour le blocage d'hourglass
   if ( doCoHexa == NULL)  
     {hexa = new GeomHexaQuad(nombre->nbi);
      // pour le calcul d'erreur il faut plus de pt d'intégration pour éviter la singularité
      // de la matrice de raideur -> 27
      hexaEr = new GeomHexaQuad(nombre->nbiEr);
      // idem pour les calculs de matrices masses consitstantes
      hexaMas = new GeomHexaQuad(nombre->nbiMas);
		    hexaeHourg = new GeomHexaQuad(nombre->nbiHour);
     }
   int dim = ParaGlob::Dimension();
   if (dim != 3)  // cas d'une dimension autre que trois
    { if (ParaGlob::NiveauImpression() >= 7)
        cout << "\nATTENTION -> dimension " << dim 
             <<", pas de definition d\'elements hexaedriques quadratiques "<< endl;
      delete hexa;delete hexaEr;delete hexaMas;
      unefois = NULL;
    }
   else
    { unefois = & uneFois; // affectation du pointeur de la classe générique
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,hexaeHourg);
      unefois->nbelem_in_Prog++;
    } 
  };
};  

// Constructeur  fonction  d'un numero
// d'identification 	
HexaQ_cm1pti::HexaQ_cm1pti (int num_mail,int num_id) :
  HexaMemb(num_mail,num_id,QUADRATIQUE,HEXAEDRE,"_cm1pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 20 noeuds,8 pt d'integration
   // calcul de doCoHexa egalement si c'est le premier passage
   // après hexa, le nombre de point d'intégration de surface pour le second membre
   // ici 4 et 4 noeuds pour les éléments de surface
   ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
	ElemGeomC0* hexaeHourg=NULL; // pour le blocage d'hourglass
   if ( doCoHexa == NULL)  
      {hexa = new GeomHexaQuad(nombre->nbi);
      // pour le calcul d'erreur il faut plus de pt d'intégration pour éviter la singularité
      // de la matrice de raideur -> 27
      hexaEr = new GeomHexaQuad(nombre->nbiEr);}
      // idem pour les calculs de matrices masses consitstantes
      hexaMas = new GeomHexaQuad(nombre->nbiMas);
		hexaeHourg = new GeomHexaQuad(nombre->nbiHour);
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans HexaQ_cm1pti, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
    { unefois = & uneFois; // affectation du pointeur de la classe générique
      doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,hexaeHourg);
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur utile si  le numero de l'element et
// le tableau des noeuds sont connus
HexaQ_cm1pti::HexaQ_cm1pti (int num_mail,int num_id,const Tableau<Noeud *>& tab):
    HexaMemb(num_mail,num_id,QUADRATIQUE,HEXAEDRE,tab,"_cm1pti") 
  {// on intervient seulement à partir du deuxième élément,
   if (uneFois.nbelem_in_Prog == 0)
     { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
     }
   else 	// sinon on construit     
    {nombre = & nombre_V; 
     if (tab_noeud.Taille() != nombre->nbne)
     { cout << "\n erreur de dimensionnement du tableau de noeud \n";
      cout << " HexaQ_cm1pti::HexaQ_cm1pti (int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
      Sortie (1); }
     // 20 noeuds,8 pt d'integration
     // calcul de doCoHexa egalement si c'est le premier passage
     // après hexa, le nombre de point d'intégration de surface pour le second membre
     // ici 4 et 4 noeuds pour les éléments de surface
     ElemGeomC0* hexa=NULL;ElemGeomC0* hexaEr=NULL; ElemGeomC0* hexaMas=NULL;
   ElemGeomC0* hexaeHourg=NULL; // pour le blocage d'hourglass
     if ( doCoHexa == NULL)  
        {hexa = new GeomHexaQuad(nombre->nbi);
        // pour le calcul d'erreur il faut plus de pt d'intégration pour éviter la singularité
        // de la matrice de raideur -> nombre->nbiEr
        hexaEr = new GeomHexaQuad(nombre->nbiEr);}
        // idem pour les calculs de matrices masses consitstantes
        hexaMas = new GeomHexaQuad(nombre->nbiMas);
    hexaeHourg = new GeomHexaQuad(nombre->nbiHour);
     #ifdef MISE_AU_POINT	 	 
       if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
        { if (ParaGlob::NiveauImpression() >= 2)
        cout << "\n erreur de dimension dans HexaQ_cm1pti, dim = " << ParaGlob::Dimension()
                << "\n alors que l'on doit avoir  3 !! " << endl;
         Sortie (1);
        }        
     #endif 
      { unefois = & uneFois; // affectation du pointeur de la classe générique
        bool sans_init_noeud = true;
        doCoHexa = HexaMemb::Init (hexa,hexaEr,hexaMas,hexaeHourg,sans_init_noeud);
        // construction du tableau de ddl spécifique à l'élément pour ses 
        ConstTabDdl(); 
        unefois->nbelem_in_Prog++;
       } 
    };
  };  


// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
HexaQ_cm1pti::HexaQ_cm1pti (const HexaQ_cm1pti& HexaQraM) :
 HexaMemb (HexaQraM)
  { if (uneFois.nbelem_in_Prog == 1)
     { cout << "\n **** erreur pour l'element HexaQ_cm1pti, le constructeur de copie ne doit pas etre utilise"
          << " pour le premier element !! " << endl;
     Sortie (1);
      }
   else 	    
    {  unefois = & uneFois; // affectation du pointeur de la classe générique 
       // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et HexaMemb
       unefois->nbelem_in_Prog++;
     };	
  };

// Destruction effectuee dans HexaMemb
HexaQ_cm1pti::~HexaQ_cm1pti ()
  { if (unefois != NULL)
      {unefois->nbelem_in_Prog--;
       Destruction();
      }
  };

// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void HexaQ_cm1pti::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n******************************************************************";
    sort << "\n Element HexaQ_cm1pti (hexaedre triquadratique "<<nombre->nbi<<"  pts d'integration) ";
    sort << "\n******************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFois.dualSortHexa) && (uneFois.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFois.dualSortHexa += 1;
         } 
    else if ((uneFois.dualSortHexa) && (uneFois.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
    else if (!(uneFois.dualSortHexa) && (uneFois.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFois.dualSortHexa += 1;
         }         
    else if ((uneFois.dualSortHexa) && (uneFois.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	
