// FICHIER : Pental.cc
// CLASSE : Pental

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "FrontSegLine.h"
#include "FrontQuadLine.h"
#include "FrontTriaLine.h"
#include "GeomPentaL.h"

#include "PentaL.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

PentaMemb::DonnComPenta * PentaL::doCoPentaL = NULL;                                     
PentaMemb::UneFois  PentaL::uneFois; 
PentaL::NombresConstruirePentaL PentaL::nombre_V; 
PentaL::ConsPentaL PentaL::consPentaL;
int  PentaL::bidon = 0;

// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
PentaL::NombresConstruirePentaL::NombresConstruirePentaL() 
 { nbne    = 6; // le nombre de noeud de l'élément
   nbneSQ  = 4; // le nombre de noeud des facettes quadrangulaires
   nbneST  = 3; // le nombre de noeud des facettes triangulaires
   nbneAQ  = 2; // le nombre de noeud des aretes entres les faces triangulaires
   nbneAT  = 2; // le nombre de noeud des aretes des facettes triangulaires
   nbI     = 2; // nombre point d'intég pour le calcul méca pour l'élément
   nbiQ    = 1; // nombre point d'intég pour le calcul méca pour les triangles
   nbiT    = 4; // nombre point d'intég pour le calcul méca pour les quadrangles
   nbiEr   = 6; // le nombre de point d'intégration pour le calcul d'erreur
   nbiV    = 2; // le nombre de point d'intégration pour le calcul de second membre volumique
   nbiSQ   = 4; // nombre point d'intég pour le calcul de second membre surfacique quadrangulaire
   nbiST   = 1; // nombre point d'intég pour le calcul de second membre surfacique triangulaire
   nbiAQ   = 1; // nB pt integ pour calcul second membre linéique des arête entre faces triangles
   nbiAT   = 1; // nB pt integ pour calcul second membre linéique des arête des triangles
   nbiMas  = 6; // le nombre de point d'intégration pour le calcul de la matrice masse 
   nbiHour = 6; // éventuellement, le nombre de point d'intégration un blocage d'hourglass
  }; 

// =========================== constructeurs ==================


// Constructeur par defaut, le seul accepte en dimension different de 3
PentaL::PentaL () :
  PentaMemb(0,-3,LINEAIRE,PENTAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
   }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // calcul de doCoPentaL si c'est le premier passage
   ElemGeomC0* penta=NULL;ElemGeomC0* pentaEr=NULL; ElemGeomC0* pentaMas=NULL;
	  ElemGeomC0* pentaeHourg=NULL; // pour le blocage d'hourglass
   if ( doCoPentaL == NULL)  
      {penta = new GeomPentaL(nombre->nbI);
       // pour le calcul d'erreur il faut au moins autant de points d'intégration que de noeuds
       pentaEr = new GeomPentaL(nombre->nbiEr);
       // idem pour le calcul de la matrice masse consistante
       pentaMas = new GeomPentaL(nombre->nbiMas);
       pentaeHourg = new GeomPentaL(nombre->nbiHour);
      };
   int dim = ParaGlob::Dimension();
   if (dim != 3)  // cas d'une dimension autre que trois
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition d\'elements pentaedriques lineaires "<< endl;
	   delete penta;delete pentaEr;delete pentaMas;delete pentaeHourg;
	   unefois = NULL;
	  }	  
   else 
     // après penta on défini les données relatives aux surfaces externes (frontières) 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoPentaL = PentaMemb::Init (penta,pentaEr,pentaMas,pentaeHourg);
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur  fonction  d'un numero
// d'identification 	
PentaL::PentaL (int num_maill,int num_id) :
  PentaMemb(num_maill,num_id,LINEAIRE,PENTAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // calcul de doCoPentaL si c'est le premier passage
   ElemGeomC0* penta=NULL;ElemGeomC0* pentaEr=NULL; ElemGeomC0* pentaMas=NULL;
	ElemGeomC0* pentaeHourg=NULL; // pour le blocage d'hourglass
   if ( doCoPentaL == NULL)  
      {penta = new GeomPentaL(nombre->nbI);
      // pour le calcul d'erreur il faut au moins autant de points d'intégration que de noeuds
      pentaEr = new GeomPentaL(nombre->nbiEr);
      // idem pour le calcul de la matrice masse consistante
      pentaMas = new GeomPentaL(nombre->nbiMas);
		pentaeHourg = new GeomPentaL(nombre->nbiHour);
      }
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans PentaL, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
   // après penta on défini les données relatives aux surfaces externes (frontières) 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoPentaL = PentaMemb::Init (penta,pentaEr,pentaMas,pentaeHourg);
      unefois->nbelem_in_Prog++;
     } 
  };
};

// Constructeur utile si  le numero de l'element et
// le tableau des noeuds sont connus
PentaL::PentaL (int num_maill,int num_id,const Tableau<Noeud *>& tab):
    PentaMemb(num_maill,num_id,LINEAIRE,PENTAEDRE,tab) 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " PentaL::PentaL (double epaiss,int num_maill, int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   // calcul de doCoPentaL si c'est le premier passage
   ElemGeomC0* penta=NULL;ElemGeomC0* pentaEr=NULL; ElemGeomC0* pentaMas=NULL;
	ElemGeomC0* pentaeHourg=NULL; // pour le blocage d'hourglass
   if ( doCoPentaL == NULL)  
      {penta = new GeomPentaL(nombre->nbI);
      // pour le calcul d'erreur il faut au moins autant de points d'intégration que de noeuds
      pentaEr = new GeomPentaL(nombre->nbiEr);
      // idem pour le calcul de la matrice masse consistante
      pentaMas = new GeomPentaL(nombre->nbiMas);
		pentaeHourg = new GeomPentaL(nombre->nbiHour);
      }
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans PentaL, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
   // après penta on défini les données relatives aux surfaces externes (frontières) 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      bool sans_init_noeud = true;
      doCoPentaL = PentaMemb::Init (penta,pentaEr,pentaMas,pentaeHourg,sans_init_noeud);
      // construction du tableau de ddl spécifique à l'élément pour ses 
      ConstTabDdl(); 
      unefois->nbelem_in_Prog++;
    }  
  };
};

PentaL::PentaL (const PentaL& PentaLraM) :
 PentaMemb (PentaLraM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFois.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element PentaL, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
 {   unefois = & uneFois; // affectation du pointeur de la classe générique penta
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et PentaMemb
     unefois->nbelem_in_Prog++;
   };	
};

PentaL::~PentaL ()
// Destruction effectuee dans PentaMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};
                         
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void PentaL::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  {
    // affichage de l'entête de l'element
    sort << "\n******************************************************************";
    sort << "\n Element PentaL (pentaedre trilineaire " << nombre->nbI << " pt d'integration) ";
    sort << "\n******************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFois.dualSortPenta) && (uneFois.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbI,1);
          uneFois.dualSortPenta += 1;
         } 
    else if ((uneFois.dualSortPenta) && (uneFois.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbI,11);
    else if (!(uneFois.dualSortPenta) && (uneFois.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbI,2);
          uneFois.dualSortPenta += 1;
         }         
    else if ((uneFois.dualSortPenta) && (uneFois.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbI,12);
      // sinon on ne fait rien     
  };               	
      

