// FICHIER : QuadAxiQComp_cm4pti.cp
// CLASSE : QuadAxiQComp_cm4pti

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"


#include "QuadAxiQComp_cm4pti.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------
QuadAxiMemb::DonnComQuad * QuadAxiQComp_cm4pti::doCoMembQ3 = NULL;                                     
QuadAxiMemb::UneFois  QuadAxiQComp_cm4pti::uneFoisQ3; 
QuadAxiQComp_cm4pti::NombresConstruireQuadAxiQComp_cm4pti QuadAxiQComp_cm4pti::nombre_V; 
QuadAxiQComp_cm4pti::ConsQuadAxiQComp_cm4pti QuadAxiQComp_cm4pti::consQuadAxiQComp_cm4pti;

// fonction définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
QuadAxiQComp_cm4pti::NombresConstruireQuadAxiQComp_cm4pti::NombresConstruireQuadAxiQComp_cm4pti() 
 { nbne = 9; // le nombre de noeud de l'élément
   nbneA = 3;// le nombre de noeud des aretes
   nbi = 4;  // le nombre de point d'intégration pour le calcul mécanique
   nbiEr = 9;// le nombre de point d'intégration pour le calcul d'erreur
   nbiS = 4; // le nombre de point d'intégration pour le calcul de second membre surfacique
   nbiA = 2; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbiMas = 9; // le nombre de point d'intégration pour le calcul de la matrice masse 
	nbiHour = 9; // le nombre de point d'intégration un blocage d'hourglass
  }; 
// =========================== constructeurs ==================

QuadAxiQComp_cm4pti::QuadAxiQComp_cm4pti () :
  QuadAxiMemb()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = QUADRACOMPL;
   id_geom = QUAD_AXI;
   infos_annexes="_cm4pti";
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements quadrangle QuadAxiQComp_cm4pti "<< endl;
	   unefois = NULL;
	  }	  
   else 
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique quadrangle
        doCoMembQ3 = QuadAxiMemb::Init (); // initialisation par defaut
        unefois->nbelem_in_Prog++;
       } 
  };
};  
	

// Constructeur fonction  d'un numero de maillage et d'identification  
QuadAxiQComp_cm4pti::QuadAxiQComp_cm4pti (int num_mail,int num_id) :
  QuadAxiMemb(num_mail,num_id,QUADRACOMPL,QUAD_AXI,"_cm4pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements quadrangle QuadAxiQComp_cm4pti "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique quadrangle
        doCoMembQ3 = QuadAxiMemb::Init (); // initialisation par defaut
        unefois->nbelem_in_Prog++;
       } 
  };
};  

// Constructeur utile si le numero de l'element et
// le tableau des noeuds sont connus
QuadAxiQComp_cm4pti::QuadAxiQComp_cm4pti (int num_mail,int num_id,const Tableau<Noeud *>& tab):
    QuadAxiMemb(num_mail,num_id,QUADRACOMPL,QUAD_AXI,tab,"_cm4pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " QuadAxiQComp_cm4pti::QuadAxiQComp_cm4pti (int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements quadrangle QuadAxiQComp_cm4pti "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique quadrangle
        bool sans_init_noeud = true;
        doCoMembQ3 = QuadAxiMemb::Init (Donnee_specif(),sans_init_noeud);
        // construction du tableau de ddl spécifique à l'élément pour ses 
        ConstTabDdl(); 
        unefois->nbelem_in_Prog++;
      } 
  };
};  

QuadAxiQComp_cm4pti::QuadAxiQComp_cm4pti (const QuadAxiQComp_cm4pti& QuadM) :
 QuadAxiMemb (QuadM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisQ3.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element QuadAxiQComp_cm4pti, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
   { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique quadrangle
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et QuadAxiMemb
     unefois->nbelem_in_Prog++;
    } ;
};

QuadAxiQComp_cm4pti::~QuadAxiQComp_cm4pti ()
// Destruction effectuee dans QuadAxiMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};
                           
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void QuadAxiQComp_cm4pti::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n***********************************************************************************************";
    sort << "\n Element QuadAxiQComp_cm4pti (quadrangle axisymetrique quadratique complet 4 pt d'integration) ";
    sort << "\n***********************************************************************************************";
      // appel de la procedure de elem meca
      if (!(uneFoisQ3.dualSortQuad) && (uneFoisQ3.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFoisQ3.dualSortQuad += 1;
         } 
      else if ((uneFoisQ3.dualSortQuad) && (uneFoisQ3.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
      else if (!(uneFoisQ3.dualSortQuad) && (uneFoisQ3.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFoisQ3.dualSortQuad += 1;
         }         
      else if ((uneFoisQ3.dualSortQuad) && (uneFoisQ3.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	
   
