// FICHIER : QuadAxiQComp_cm4pti.h
// CLASSE : QuadAxiQComp_cm4pti

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        3/05/2011                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:         Element quadrangulaire, axi-symétrique,             *
 *     quadratique 4 pt d'integ.   9 noeuds -> quadratique complet.     *
 *     gestion des modes d'hourglass.                                   *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                      *
 *                                                                $     *
 *                                                                      *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------



#ifndef QUAD_AXI_Q_COMP_CM4PTI_H
#define QUAD_AXI_Q_COMP_CM4PTI_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomQuadrangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "QuadAxiMemb.h"
#include "FrontSegQuad.h"
#include "FrontQuadQC.h"

/// @addtogroup groupe_des_elements_finis
///  @{
///


class QuadAxiQComp_cm4pti : public QuadAxiMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		QuadAxiQComp_cm4pti ();
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		QuadAxiQComp_cm4pti (int num_mail,int num_id);
		
		// Constructeur fonction d'un numero d'identification,
		// du tableau de connexite des noeuds 
		QuadAxiQComp_cm4pti (int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		QuadAxiQComp_cm4pti (const QuadAxiQComp_cm4pti& quad);
		
		
		// DESTRUCTEUR :
		~QuadAxiQComp_cm4pti ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new QuadAxiQComp_cm4pti(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de QuadAxiQComp_cm4pti
		QuadAxiQComp_cm4pti& operator= (QuadAxiQComp_cm4pti& quad);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegQuad(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontQuadQC(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements QuadAxiQComp_cm4pti      
        static QuadAxiMemb::DonnComQuad * doCoMembQ3;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static QuadAxiMemb::UneFois  uneFoisQ3;
        
	    class  NombresConstruireQuadAxiQComp_cm4pti : public NombresConstruire
	     {  public: NombresConstruireQuadAxiQComp_cm4pti(); 
	      };
	    static NombresConstruireQuadAxiQComp_cm4pti nombre_V; // les nombres propres à l'élément

        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsQuadAxiQComp_cm4pti : public ConstrucElement
          { public :  ConsQuadAxiQComp_cm4pti () 
               { NouvelleTypeElement nouv(QUAD_AXI,QUADRACOMPL,MECA_SOLIDE_DEFORMABLE,this,"_cm4pti");
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation QuadAxiQComp_cm4pti" << endl;
                  Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new QuadAxiQComp_cm4pti (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() >= 2) return true; else return false;};	
           }; 
        static ConsQuadAxiQComp_cm4pti consQuadAxiQComp_cm4pti;
};
/// @}  // end of group
#endif
	
	
		

