// FICHIER : PoutSfe3.h
// CLASSE : PoutSfe3
/************************************************************************
 *  UNIVERSITE DE BRETAGNE SUD (UBS) --- ENSIBS DE LORIENT              *
 ************************************************************************
 *           IRDL - Equipe DE GENIE MECANIQUE ET MATERIAUX (EG2M)       *
 * Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
 * tel. 02.97.87.45.70 fax. 02.97.87.45.72 http://irdl.fr               *
 ************************************************************************
 *     DATE:        24/03/2018                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  phone  0297874573, fax : 0297874588                 *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:         Element poutre SFE3. Le calcul de la normale        *
 *       s'effectue à partir de l'interpolation des theta3 relativement *
 *      à la poutre centrale, ceci via un polynome quadratique complet. *
 *       Le jacobien est celui de la poutre linéaire centrale.          *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *  
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
/*
//
// l'interpolation est SFE, 
//                                (4)----(1)----(2)----(3)
*/


#ifndef POUTSFE3_H
#define POUTSFE3_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomSeg.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "SfeMembT.h"
#include "ElFrontiere.h"
#include "FrontSegLine.h"


class PoutSfe3 : public SfeMembT
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		PoutSfe3 ();
		
		// Constructeur fonction d'une epaisseur et
  // eventuellement d'un numero
		// d'identification 
		PoutSfe3 (double epaiss,int num_mail=0,int num_id=-3);
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		PoutSfe3 (int num_mail,int num_id);
		
		// Constructeur fonction d'une epaisseur, d'un numero d'identification,
		// du tableau de connexite des noeuds 
		PoutSfe3 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		PoutSfe3 (const PoutSfe3& Pout);
		
		
		// DESTRUCTEUR :
		~PoutSfe3 ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new PoutSfe3(*this); return el;};

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de PoutSfe3
		PoutSfe3& operator= (PoutSfe3& Pout);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
// 2) derivant des virtuelles 
// 3) methodes propres a l'element

		// les coordonnees des points d'integration dans l'epaisseur
		inline double KSI(int i) { return doCoSfe3->segment.KSI(i);};
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegLine(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontTriaLine(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements PoutMembL1
        static SfeMembT::DonnComSfe * doCoSfe3;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static SfeMembT::UneFois  uneFoisSfe3;

	    class  NombresConstruirePoutSfe3 : public NombresConstruire
	     {  public: NombresConstruirePoutSfe3();
	      };
	    static NombresConstruirePoutSfe3 nombre_V; // les nombres propres à l'élément

        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsPoutSfe3 : public ConstrucElement
          { public :  ConsPoutSfe3 ()
               { NouvelleTypeElement nouv(POUT,SFE3,MECA_SOLIDE_DEFORMABLE,this);
                 if (ParaGlob::NiveauImpression() >= 4)
                     cout << "\n initialisation PoutSfe3" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new PoutSfe3 (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() == 3) return true; else return false;};	
           }; 
        static ConsPoutSfe3 consPoutSfe3;
};
#endif
	
	
		

