// FICHIER : TriaSfe3C.cc
// CLASSE : TriaSfe3C

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "FrontSegLine.h"
#include "FrontTriaLine.h"


#include "TriaSfe3C.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------
SfeMembT::DonnComSfe * TriaSfe3C::doCoSfe3C = NULL;                                     
SfeMembT::UneFois  TriaSfe3C::uneFoisSfe3C; 
TriaSfe3C::NombresConstruireTriaSfe3C TriaSfe3C::nombre_V; 
TriaSfe3C::ConsTriaSfe3C TriaSfe3C::consTriaSfe3C;

// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
TriaSfe3C::NombresConstruireTriaSfe3C::NombresConstruireTriaSfe3C() 
 { nbnce = 3; // nb de noeud de l'element central
   nbnte = 6; // nombre total de noeud
   nbneA = 2; // le nombre de noeud des aretes de l'élément central
   nbis  = 1;  // le nombre de point d'intégration de surface pour le calcul mécanique
   nbie  = 2;  // nombre de pt d'integ d'epaisseur pour le calcul mécanique
   nbisEr= 3; // le nombre de point d'intégration de surface pour le calcul d'erreur
   nbieEr= 2; // le nombre de point d'intégration d'épaisseur pour le calcul d'erreur
   nbiSur= 3; // le nombre de point d'intégration pour le calcul de second membre surfacique
   nbiA  = 1; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbisMas = 3; // le nombre de point d'intégration de surface pour le calcul de la matrice masse 
   nbieMas = 2; // le nombre de point d'intégration d'épaisseur pour le calcul de la matrice masse 
  }; 
// =========================== constructeurs ==================

TriaSfe3C::TriaSfe3C () :
  SfeMembT()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisSfe3C.nbelem_in_Prog == 0)
   { uneFoisSfe3C.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = SFE3C;
   id_geom = TRIANGLE;
   tab_noeud.Change_taille(nombre->nbnte);
   int dim = ParaGlob::Dimension();
   ElemGeomC0* eleCentre=NULL,* eleEr=NULL,* eleS=NULL,* eleMas=NULL;
   if ( doCoSfe3C == NULL)  
      {eleCentre = new GeomTriangle (nombre->nbis,nombre->nbnce);
       eleEr = new GeomTriangle(nombre->nbisEr,nombre->nbnce);
       eleS = new GeomTriangle(nombre->nbiSur,nombre->nbnce);
       eleMas = new GeomTriangle(nombre->nbisMas,nombre->nbnce);
       };
   
   if (dim != 3)  // cas d'une dimension autre que trois
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition Triangle Sfe3C "<< endl;
       delete eleCentre; delete eleEr; delete eleS; delete eleMas;
	   unefois = NULL;
	  }	  
   else 
     // calcul de doCoSfe3C egalement si c'est le premier passage
    { unefois = & uneFoisSfe3C; // affectation du pointeur de la classe générique triangle
      int type_calcul_jacobien = 2; // facette courbe
      doCoSfe3C = SfeMembT::Init(eleCentre,eleEr,eleS,eleMas
                                ,type_calcul_jacobien); // initialisation par défaut
      unefois->nbelem_in_Prog++;
     } 
  };
};  
	
TriaSfe3C::TriaSfe3C (double epaiss,int num_mail,int num_id):
// Constructeur utile si la section de l'element et
// le numero de l'element  sont connus
       SfeMembT(num_mail,num_id,SFE3C,TRIANGLE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisSfe3C.nbelem_in_Prog == 0)
   { uneFoisSfe3C.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbnte);
   ElemGeomC0* eleCentre=NULL,* eleEr=NULL,* eleS=NULL,* eleMas=NULL;
   if ( doCoSfe3C == NULL)  
      {eleCentre = new GeomTriangle (nombre->nbis,nombre->nbnce);
       eleEr = new GeomTriangle(nombre->nbisEr,nombre->nbnce);
       eleS = new GeomTriangle(nombre->nbiSur,nombre->nbnce);
       eleMas = new GeomTriangle(nombre->nbisMas,nombre->nbnce);
       };
   
   if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      {cout << "\n erreur de dimension dans TriaSfe3C , dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       delete eleCentre; delete eleEr; delete eleS; delete eleMas;
       Sortie (1);
      };        
     // calcul de doCoSfe3C egalement si c'est le premier passage
    { unefois = & uneFoisSfe3C; // affectation du pointeur de la classe générique triangle
      int type_calcul_jacobien = 2; // facette courbe
      doCoSfe3C = SfeMembT::Init(eleCentre,eleEr,eleS,eleMas
                                ,type_calcul_jacobien,Donnee_specif(epaiss));
      unefois->nbelem_in_Prog++;
    } 	
  };
};  

// Constructeur fonction  d'un numero de maillage et d'identification  
TriaSfe3C::TriaSfe3C (int num_mail,int num_id) :
  SfeMembT(num_mail,num_id,SFE3C,TRIANGLE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisSfe3C.nbelem_in_Prog == 0)
   { uneFoisSfe3C.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbnte);
   ElemGeomC0* eleCentre=NULL,* eleEr=NULL,* eleS=NULL,* eleMas=NULL;
   if ( doCoSfe3C == NULL)  
      {eleCentre = new GeomTriangle (nombre->nbis,nombre->nbnce);
       eleEr = new GeomTriangle(nombre->nbisEr,nombre->nbnce);
       eleS = new GeomTriangle(nombre->nbiSur,nombre->nbnce);
       eleMas = new GeomTriangle(nombre->nbisMas,nombre->nbnce);
       };
   
   if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      {cout << "\n erreur de dimension dans TriaSfe3C , dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       delete eleCentre; delete eleEr; delete eleS; delete eleMas;
       Sortie (1);
      };        
     // calcul de doCoSfe3C egalement si c'est le premier passage
    { unefois = & uneFoisSfe3C; // affectation du pointeur de la classe générique triangle
      int type_calcul_jacobien = 2; // facette courbe
      doCoSfe3C = SfeMembT::Init(eleCentre,eleEr,eleS,eleMas
                                ,type_calcul_jacobien); // intialisation par defaut
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur utile si l'epaisseur de l'element, le numero de l'element et
// le tableau des noeuds sont connus
TriaSfe3C::TriaSfe3C (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab):
    SfeMembT(num_mail,num_id,SFE3C,TRIANGLE,tab) 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisSfe3C.nbelem_in_Prog == 0)
   { uneFoisSfe3C.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbnte)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " TriaSfe3C::TriaSfe3C (double epaiss,int num_mail,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   ElemGeomC0* eleCentre=NULL,* eleEr=NULL,* eleS=NULL,* eleMas=NULL;
   if ( doCoSfe3C == NULL)  
      {eleCentre = new GeomTriangle (nombre->nbis,nombre->nbnce);
       eleEr = new GeomTriangle(nombre->nbisEr,nombre->nbnce);
       eleS = new GeomTriangle(nombre->nbiSur,nombre->nbnce);
       eleMas = new GeomTriangle(nombre->nbisMas,nombre->nbnce);
       };
   
   if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      {cout << "\n erreur de dimension dans TriaSfe3C , dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       delete eleCentre; delete eleEr; delete eleS; delete eleMas;
       Sortie (1);
      };        
     // calcul de doCoSfe3C egalement si c'est le premier passage
    { unefois = & uneFoisSfe3C; // affectation du pointeur de la classe générique triangle
      bool sans_init_noeud = true;
      int type_calcul_jacobien = 2; // facette courbe
      doCoSfe3C = SfeMembT::Init(eleCentre,eleEr,eleS,eleMas
                                ,type_calcul_jacobien,Donnee_specif(epaiss),sans_init_noeud);
      // construction du tableau de ddl spécifique à l'élément pour ses 
      ConstTabDdl(); 
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur de copie
TriaSfe3C::TriaSfe3C (const TriaSfe3C& TriaM) :
 SfeMembT (TriaM)
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisSfe3C.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element TriaSfe3C, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
  {
   if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      {cout << "\n erreur de dimension dans TriaSfe3C , dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      };        
   unefois = & uneFoisSfe3C; // affectation du pointeur de la classe générique triangle
   // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et SfeMembT
   unefois->nbelem_in_Prog++;
	};
};	

TriaSfe3C::~TriaSfe3C ()
// Destruction effectuee dans SfeMembT
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};
                         
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void TriaSfe3C::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n**************************************************************************************";
    sort << "\n Element TriaSfe3C (triangle Sfe3C 2 pt d'integration dans l'epaisseur et 1 de surface) ";
    sort << "\n**************************************************************************************";
    // appel de la procedure de elem meca
    if (!(unefois->dualSortSfe) && (unefois->CalimpPrem))
        { VarDualSort(sort,nom,1,1);
          unefois->dualSortSfe += 1;
         } 
    else if ((unefois->dualSortSfe) && (unefois->CalimpPrem))       
         VarDualSort(sort,nom,3,11);
    else if (!(unefois->dualSortSfe) && (unefois->CalResPrem_tdt))       
        { VarDualSort(sort,nom,1,2);
          unefois->dualSortSfe += 1;
         }         
    else if ((unefois->dualSortSfe) && (unefois->CalResPrem_tdt))       
         VarDualSort(sort,nom,1,12);
      // sinon on ne fait rien     
  };               	
   
