// FICHIER : TriaSfe3_cm6pti.h
// CLASSE : TriaSfe3_cm6pti

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        04/07/2007                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:         Element triangulaire,Sfe3C. Le calcul de la normale *
 *       s'effectue à partir de l'interpolation des theta3 relativement *
 *       au triangle central, ceci via un polynome quadratique complet. *
 *       Le jacobien est celui de la facette plane centrale.            *
 *       Ici le nombre de point d'intégration est de 2 en épaisseur     *
 *       et 3 en surface.                                               *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
//
// l'interpolation est SFE, 
/*                                       (4)
//                                      /   \
//                                     /     \
//                                   (3)-----(2)
//                                   / \     / \
//                                  /   \   /   \
//                                (5)----(1)----(6)   
*/


#ifndef TRIASFE3_CM6PTI_H
#define TRIASFE3_CM6PTI_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomTriangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "SfeMembT.h"
#include "ElFrontiere.h"
#include "FrontSegLine.h"
#include "FrontTriaLine.h"


/// @addtogroup groupe_des_elements_finis
///  @{
///


class TriaSfe3_cm6pti : public SfeMembT
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		TriaSfe3_cm6pti ();
		
		// Constructeur fonction d'une epaisseur et eventuellement d'un numero
		// d'identification 
		TriaSfe3_cm6pti (double epaiss,int num_mail=0,int num_id=-3);
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		TriaSfe3_cm6pti (int num_mail,int num_id);
		
		// Constructeur fonction d'une epaisseur, d'un numero d'identification,
		// du tableau de connexite des noeuds 
		TriaSfe3_cm6pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		TriaSfe3_cm6pti (const TriaSfe3_cm6pti& tria);
		
		
		// DESTRUCTEUR :
		~TriaSfe3_cm6pti ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new TriaSfe3_cm6pti(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de TriaSfe3_cm6pti
		TriaSfe3_cm6pti& operator= (TriaSfe3_cm6pti& tria);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
// 2) derivant des virtuelles 
// 3) methodes propres a l'element

		// les coordonnees des points d'integration dans l'epaisseur
		inline double KSI(int i) { return doCoSfe3->segment.KSI(i);};
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegLine(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontTriaLine(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaMembL1 
        static SfeMembT::DonnComSfe * doCoSfe3;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static SfeMembT::UneFois  uneFoisSfe3;

	    class  NombresConstruireTriaSfe3_cm6pti : public NombresConstruire
	     {  public: NombresConstruireTriaSfe3_cm6pti(); 
	      };
	    static NombresConstruireTriaSfe3_cm6pti nombre_V; // les nombres propres à l'élément

        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsTriaSfe3_cm6pti : public ConstrucElement
          { public :  ConsTriaSfe3_cm6pti () 
               { NouvelleTypeElement nouv(TRIANGLE,SFE3,MECA_SOLIDE_DEFORMABLE,this,"_cm6pti");
                 if (ParaGlob::NiveauImpression() >= 4)
                     cout << "\n initialisation TriaSfe3_cm6pti" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new TriaSfe3_cm6pti (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() == 3) return true; else return false;};	
           }; 
        static ConsTriaSfe3_cm6pti consTriaSfe3_cm6pti;
};
/// @}  // end of group
#endif
	
	
		

