// FICHIER : Tetra.cc
// CLASSE : Tetra

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "FrontSegLine.h"
#include "FrontTriaLine.h"
#include "GeomTetraL.h"

#include "Tetra.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

TetraMemb::DonnComTetra * Tetra::doCoTetra = NULL;                                     
TetraMemb::UneFois  Tetra::uneFois; 
Tetra::NombresConstruireTetra Tetra::nombre_V; 
Tetra::ConsTetra Tetra::consTetra;

// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
Tetra::NombresConstruireTetra::NombresConstruireTetra() 
 {  nbne  = 4; // le nombre de noeud de l'élément
    nbneS = 3; // le nombre de noeud des facettes
    nbneA = 2; // le nombre de noeud des aretes
    nbi   = 1; // le nombre de point d'intégration pour le calcul mécanique
    nbiEr = 4; // le nombre de point d'intégration pour le calcul d'erreur
    nbiV  = 1; // le nombre de point d'intégration pour le calcul de second membre volumique
    nbiS  = 1; // le nombre de point d'intégration pour le calcul de second membre surfacique
    nbiA  = 1; // le nombre de point d'intégration pour le calcul de second membre linéique
    nbiMas = 4; // le nombre de point d'intégration pour le calcul de la matrice masse 
	 nbiHour = 0; // le nombre de point d'intégration un blocage d'hourglass
  }; 


// =========================== constructeurs ==================


// Constructeur par defaut, le seul qui accepte en dimension une valeur different de 3
Tetra::Tetra () :
  TetraMemb(0,-3,LINEAIRE,TETRAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 4 noeuds,1 pt d'integration
   // calcul de doCoTetra egalement si c'est le premier passage
   ElemGeomC0* tetra;ElemGeomC0* tetraEr;ElemGeomC0* tetraMas; 
   if ( doCoTetra == NULL)  
      {tetra = new GeomTetraL(nombre->nbi);
      // dans le cas du calcul d'erreur il faut un nombre de points d'intégration
      // qui soit au moins identique à celui des noeuds
      tetraEr = new GeomTetraL(nombre->nbiEr);
      // idem pour les calculs relatifs à la matrice de masse constistante
      tetraMas = new GeomTetraL(nombre->nbiMas);
      }
   int dim = ParaGlob::Dimension();
   if (dim != 3)  // cas d'une dimension autre que trois
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition d\'elements tetraedriques lineaires "<< endl;
	   delete tetra;delete tetraEr;delete tetraMas;
	   unefois = NULL;
	  }	  
   else 
     // après tetra on défini les données relatives aux surfaces externes (frontières) 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoTetra = TetraMemb::Init (tetra,tetraEr,tetraMas,NULL);
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur  fonction  d'un numero
// d'identification 	
Tetra::Tetra (int num_mail,int num_id) :
  TetraMemb(num_mail,num_id,LINEAIRE,TETRAEDRE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // 4 noeuds,1 pt d'integration
   // calcul de doCoTetra egalement si c'est le premier passage
   ElemGeomC0* tetra;ElemGeomC0* tetraEr;ElemGeomC0* tetraMas; 
   if ( doCoTetra == NULL)  
      {tetra = new GeomTetraL(nombre->nbi);
      // dans le cas du calcul d'erreur il faut un nombre de points d'intégration
      // qui soit au moins identique à celui des noeuds
      tetraEr = new GeomTetraL(nombre->nbiEr);
      // idem pour les calculs relatifs à la matrice de masse constistante
      tetraMas = new GeomTetraL(nombre->nbiMas);
      }
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans Tetra, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
   // après tetra on défini les données relatives aux surfaces externes (frontières) 
   // c'est-à-dire le nombre de point d'intégration et le nombre de noeud :
   // int nbiS,int nbeS
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      doCoTetra = TetraMemb::Init (tetra,tetraEr,tetraMas,NULL);
      unefois->nbelem_in_Prog++;
    }  
  };
};  

// Constructeur utile si  le numero de l'element et
// le tableau des noeuds sont connus
Tetra::Tetra (int num_mail,int num_id,const Tableau<Noeud *>& tab):
    TetraMemb(num_mail,num_id,LINEAIRE,TETRAEDRE,tab) 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFois.nbelem_in_Prog == 0)
   { uneFois.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " Tetra::Tetra (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   // 4 noeuds,1 pt d'integration
   // calcul de doCoTetra egalement si c'est le premier passage
   ElemGeomC0* tetra;ElemGeomC0* tetraEr;ElemGeomC0* tetraMas; 
   if ( doCoTetra == NULL)  
      {tetra = new GeomTetraL(nombre->nbi);
      // dans le cas du calcul d'erreur il faut un nombre de points d'intégration
      // qui soit au moins identique à celui des noeuds
      tetraEr = new GeomTetraL(nombre->nbiEr);
      // idem pour les calculs relatifs à la matrice de masse constistante
      tetraMas = new GeomTetraL(nombre->nbiMas);
      }
   #ifdef MISE_AU_POINT	 	 
     if (ParaGlob::Dimension() != 3) // cas d'une dimension autre que trois
      { if (ParaGlob::NiveauImpression() >= 2)
	     cout << "\n erreur de dimension dans Tetra, dim = " << ParaGlob::Dimension()
              << "\n alors que l'on doit avoir  3 !! " << endl;
       Sortie (1);
      }        
   #endif 
   // après tetra on défini les données relatives aux surfaces externes (frontières) 
    { unefois = & uneFois; // affectation du pointeur de la classe générique triangle
      bool sans_init_noeud = true;
      doCoTetra = TetraMemb::Init (tetra,tetraEr,tetraMas,NULL,sans_init_noeud);
      // construction du tableau de ddl spécifique à l'élément pour ses 
      ConstTabDdl(); 
      unefois->nbelem_in_Prog++;
     } 
  };
};  

Tetra::Tetra (const Tetra& TetraraM) :
 TetraMemb (TetraraM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFois.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element Tetra, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
   { unefois = & uneFois; // affectation du pointeur de la classe générique tétraèdre
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et TetraMemb
     unefois->nbelem_in_Prog++;
   };	
};

Tetra::~Tetra ()
// Destruction effectuee dans TetraMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};
                         
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void Tetra::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n******************************************************************";
    sort << "\n Element Tetra (tetraedre trilineaire "<<nombre->nbi<<" pt d'integration) ";
    sort << "\n******************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFois.dualSortTetra) && (uneFois.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFois.dualSortTetra += 1;
         } 
    else if ((uneFois.dualSortTetra) && (uneFois.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
    else if (!(uneFois.dualSortTetra) && (uneFois.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFois.dualSortTetra += 1;
         }         
    else if ((uneFois.dualSortTetra) && (uneFois.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	

