// FICHIER : TriaAxiL1.h
// CLASSE : TriaAxiL1

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

 
 /************************************************************************
 *                                                                      *
 *     DATE:        15/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *  But: définition d'un élément triangulaire axisymétrique linéaire    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                      *
 *                                                                $     *
 *                                                                      *
 ************************************************************************/

// -----------classe pour un calcul de mecanique---------



#ifndef TRIA_AXI_L1_H
#define TRIA_AXI_L1_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomTriangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "TriaAxiMemb.h"
#include "ElFrontiere.h"
#include "FrontSegLine.h"
#include "FrontTriaLine.h"

/// @addtogroup groupe_des_elements_finis
///  @{
///


class TriaAxiL1 : public TriaAxiMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		TriaAxiL1 ();
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		TriaAxiL1 (int num_mail,int num_id);
		
		// Constructeur fonction  d'un numero de maillage et d'identification,
		// du tableau de connexite des noeuds 
		TriaAxiL1 (int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		TriaAxiL1 (const TriaAxiL1& tria);
		
		
		// DESTRUCTEUR :
		~TriaAxiL1 ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new TriaAxiL1(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de TriaAxiL1
		TriaAxiL1& operator= (TriaAxiL1& tria);
		
		// METHODES :
// 1) derivant des virtuelles pures

        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
// 2) derivant des virtuelles 
// 3) methodes propres a l'element

	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegLine(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontTriaLine(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaAxiL1 
        static TriaAxiMemb::DonnComTria * doCoMembL1;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static TriaAxiMemb::UneFois  uneFoisL1;

	    class  NombresConstruireTriaAxiL1 : public NombresConstruire
	     {  public: NombresConstruireTriaAxiL1(); 
	      };
	    static NombresConstruireTriaAxiL1 nombre_V; // les nombres propres à l'élément
	     		
        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsTriaAxiL1 : public ConstrucElement
          { public :  ConsTriaAxiL1 () 
               { NouvelleTypeElement nouv(TRIA_AXI,LINEAIRE,MECA_SOLIDE_DEFORMABLE,this);
                 if (ParaGlob::NiveauImpression() >= 4)
                   cout << "\n initialisation TriaAxiL1" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new TriaAxiL1 (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() == 3) return true; else return false;};	
           }; 
        static ConsTriaAxiL1 consTriaAxiL1;
};
/// @}  // end of group
#endif
	
	
		

