// FICHIER : TriaMembL1L1.cp
// CLASSE : TriaMembL1

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"


#include "TriaMembL1.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------
TriaMemb::DonnComTria * TriaMembL1::doCoMembL1 = NULL; 
TriaMemb::UneFois  TriaMembL1::uneFoisL1; 
TriaMembL1::NombresConstruireTriaMembL1 TriaMembL1::nombre_V; 
TriaMembL1::ConsTriaMembL1 TriaMembL1::consTriaMembL1;

// fonction définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
TriaMembL1::NombresConstruireTriaMembL1::NombresConstruireTriaMembL1() 
 { nbne = 3; // le nombre de noeud de l'élément
   nbneA = 2;// le nombre de noeud des aretes
   nbi = 1;  // le nombre de point d'intégration pour le calcul mécanique
   nbiEr = 3;// le nombre de point d'intégration pour le calcul d'erreur
   nbiS = 1; // le nombre de point d'intégration pour le calcul de second membre surfacique
   nbiA = 1; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbiMas = 3; // le nombre de point d'intégration pour le calcul de la matrice masse 
	  nbiHour = 0; // le nombre de point d'intégration un blocage d'hourglass
 };
// =========================== constructeurs ==================

TriaMembL1::TriaMembL1 () :
  TriaMemb()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = LINEAIRE;
   id_geom = TRIANGLE;
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
    { if (ParaGlob::NiveauImpression() >= 7)
        cout << "\nATTENTION -> dimension " << 1 
             <<", pas de definition d\'elements triangle TriaMembL1 "<< endl;
      unefois = NULL;
    }
   else
    // clacul de doCoMembL1 egalement si c'est le premier passage
      { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
        doCoMembL1 = TriaMemb::Init(); // init par défaut
        unefois->nbelem_in_Prog++;
      };
  };
};  
	
TriaMembL1::TriaMembL1 (double epaiss,int num_mail,int num_id):
// Constructeur utile si la section de l'element et
// le numero de l'element  sont connus
       TriaMemb(num_mail,num_id,LINEAIRE,TRIANGLE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
     { if (ParaGlob::NiveauImpression() >= 7)
         cout << "\nATTENTION -> dimension " << 1 
              <<", pas de definition d\'elements triangle TriaMembL1 "<< endl;
       unefois = NULL;
       Sortie (1);
     }
   else 
    // calcul de doCoMembL1 egalement si c'est le premier passage
      { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
        doCoMembL1 = TriaMemb::Init (Donnee_specif(epaiss,nombre->nbi));
        unefois->nbelem_in_Prog++;
       } 	
  };
};  

// Constructeur fonction  d'un numero de maillage et d'identification  
TriaMembL1::TriaMembL1 (int num_mail,int num_id) :
  TriaMemb(num_mail,num_id,LINEAIRE,TRIANGLE)
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1
	          <<", pas de definition d\'elements triangle TriaMembL1 "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
    // clacul de doCoMembL1 egalement si c'est le premier passage
      { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
        doCoMembL1 = TriaMemb::Init (); // initialisation par defaut
        unefois->nbelem_in_Prog++;
       } 
  };
};  

// Constructeur utile si l'epaisseur de l'element, le numero de l'element et
// le tableau des noeuds sont connus
TriaMembL1::TriaMembL1 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab):
    TriaMemb(num_mail,num_id,LINEAIRE,TRIANGLE,tab) 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " TriaMembL1::TriaMembL1 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements triangle TriaMembL1 "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
    // clacul de doCoMembL1 egalement si c'est le premier passage
      { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
        bool sans_init_noeud = true;
        doCoMembL1 = TriaMemb::Init(Donnee_specif(epaiss,nombre->nbi),sans_init_noeud);
        // construction du tableau de ddl spécifique à l'élément pour ses 
        ConstTabDdl(); 
        unefois->nbelem_in_Prog++;
       } 
  };
};  

TriaMembL1::TriaMembL1 (const TriaMembL1& TriaM) :
 TriaMemb (TriaM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisL1.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element TriaMembL1, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
  {  unefois = & uneFoisL1; // affectation du pointeur de la classe générique triangle
     // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et TriaMemb
     unefois->nbelem_in_Prog++;
   };	
};

TriaMembL1::~TriaMembL1 ()
// Destruction effectuee dans TriaMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};
      	
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
 void TriaMembL1::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
     {// affichage de l'entête de l'element
      sort << "\n******************************************************************";
      sort << "\n Element TriaMembL1 (triangle lineaire 1 pt d'integration) ";
      sort << "\n******************************************************************";
      // appel de la procedure de elem meca
      if (!(uneFoisL1.dualSortTria) && (uneFoisL1.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFoisL1.dualSortTria += 1;
         } 
      else if ((uneFoisL1.dualSortTria) && (uneFoisL1.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
      else if (!(uneFoisL1.dualSortTria) && (uneFoisL1.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFoisL1.dualSortTria += 1;
         }         
      else if ((uneFoisL1.dualSortTria) && (uneFoisL1.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
      };
	 
   
