// FICHIER : TriaQ3_cmpti1003.cc
// CLASSE : TriaQ3_cmpti1003

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"


#include "TriaQ3_cmpti1003.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------
TriaMemb::DonnComTria * TriaQ3_cmpti1003::doCoMembQ3 = NULL;                                     
TriaMemb::UneFois  TriaQ3_cmpti1003::uneFoisQ3; 
TriaQ3_cmpti1003::NombresConstruireTriaQ3_cmpti1003 TriaQ3_cmpti1003::nombre_V; 
TriaQ3_cmpti1003::ConsTriaQ3_cmpti1003 TriaQ3_cmpti1003::consTriaQ3_cmpti1003;

// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
TriaQ3_cmpti1003::NombresConstruireTriaQ3_cmpti1003::NombresConstruireTriaQ3_cmpti1003() 
 { nbne = 6; // le nombre de noeud de l'élément
   nbneA = 3;// le nombre de noeud des aretes
   nbi = 3;  // le nombre de point d'intégration pour le calcul mécanique: 3 internes
   nbiEr = 6;// le nombre de point d'intégration pour le calcul d'erreur: 6 normaux
   nbiS = 3; // le nombre de point d'intégration pour le calcul de second membre surfacique: 3 internes
   nbiA = 2; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbiMas = 6; // le nombre de point d'intégration pour le calcul de la matrice masse: 6 normaux
	  nbiHour = 0; // le nombre de point d'intégration un blocage d'hourglass
  }; 
// =========================== constructeurs ==================

TriaQ3_cmpti1003::TriaQ3_cmpti1003 () :
  TriaMemb()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = QUADRACOMPL;
   id_geom = TRIANGLE;
   infos_annexes="_cm3pti";
	  // ici acces aux données de Element ... pas terrible, à mon avis ce constructeur ne devrait pas servir !!
  	Element::infos_annexes = "_cm3pti";
	
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
    { if (ParaGlob::NiveauImpression() >= 7)
        cout << "\nATTENTION -> dimension " << 1 
             <<", pas de definition d\'elements triangle TriaQ3_cmpti1003 "<< endl;
      unefois = NULL;
    }
   else
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique triangle
        doCoMembQ3 = TriaMemb::Init(Donnee_specif(1,0,1,0,nombre->nbi)); // initialisation avec indication
        // que l'on utilise le cas ou les 3 pt sont internes
        unefois->nbelem_in_Prog++;
      };
  };
};  
	
TriaQ3_cmpti1003::TriaQ3_cmpti1003 (double epaiss,int num_mail,int num_id):
// Constructeur utile si la section de l'element et
// le numero de l'element  sont connus
       TriaMemb(num_mail,num_id,QUADRACOMPL,TRIANGLE,"_cm3pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
   }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
    { if (ParaGlob::NiveauImpression() >= 7)
        cout << "\nATTENTION -> dimension " << 1 
             <<", pas de definition d\'elements triangle TriaQ3_cmpti1003 "<< endl;
      unefois = NULL;
      Sortie (1);
    }
   else
    // clacul de doCoMembQ3 egalement si c'est le premier passage
    { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique triangle
      doCoMembQ3 = TriaMemb::Init (Donnee_specif(epaiss,epaiss,epaiss,1,0,1,0,nombre->nbi)); // 1 indique que les 3pti sont internes
      unefois->nbelem_in_Prog++;
    };
  };
};  

// Constructeur fonction  d'un numero de maillage et d'identification  
TriaQ3_cmpti1003::TriaQ3_cmpti1003 (int num_mail,int num_id) :
  TriaMemb(num_mail,num_id,QUADRACOMPL,TRIANGLE,"_cm3pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements triangle TriaQ3_cmpti1003 "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique triangle
        doCoMembQ3 = TriaMemb::Init(Donnee_specif(1,0,1,0,nombre->nbi)); // initialisation avec indication
        // que l'on utilise le cas ou les 3 pt sont internes
        unefois->nbelem_in_Prog++;
       } 
  };
};  

// Constructeur utile si l'epaisseur de l'element, le numero de l'element et
// le tableau des noeuds sont connus
TriaQ3_cmpti1003::TriaQ3_cmpti1003 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab):
    TriaMemb(num_mail,num_id,QUADRACOMPL,TRIANGLE,tab,"_cm3pti") 
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQ3.nbelem_in_Prog == 0)
   { uneFoisQ3.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " TriaQ3_cmpti1003::TriaQ3_cmpti1003 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   if ( ParaGlob::Dimension() == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << 1 
	          <<", pas de definition d\'elements triangle TriaQ3_cmpti1003 "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else 
   // clacul de doCoMembQ3 egalement si c'est le premier passage
      { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique triangle
        bool sans_init_noeud = true;
        doCoMembQ3 = TriaMemb::Init (Donnee_specif(epaiss,epaiss,epaiss,1,0,1,0,nombre->nbi),sans_init_noeud); // 1 indique que les 3pti sont internes
        // construction du tableau de ddl spécifique à l'élément pour ses 
        ConstTabDdl(); 
        unefois->nbelem_in_Prog++;
      } 
  };
};  

TriaQ3_cmpti1003::TriaQ3_cmpti1003 (const TriaQ3_cmpti1003& TriaM) :
 TriaMemb (TriaM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisQ3.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element TriaQ3_cmpti1003, le constructeur de copie ne doit pas etre utilise"
	         << " pour le premier element !! " << endl;
	    Sortie (1);
   }
  else
   { unefois = &uneFoisQ3; // affectation du pointeur de la classe générique triangle
     // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et TriaMemb
     unefois->nbelem_in_Prog++;
   } ;
};

TriaQ3_cmpti1003::~TriaQ3_cmpti1003 ()
// Destruction effectuee dans TriaMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
    }
};
                           
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void TriaQ3_cmpti1003::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n*********************************************************************************";
    sort << "\n Element TriaQ3_cmpti1003 (triangle quadratique 3 pt d'integration hors arretes) ";
    sort << "\n*********************************************************************************";
      // appel de la procedure de elem meca
      if (!(uneFoisQ3.dualSortTria) && (uneFoisQ3.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFoisQ3.dualSortTria += 1;
         } 
      else if ((uneFoisQ3.dualSortTria) && (uneFoisQ3.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
      else if (!(uneFoisQ3.dualSortTria) && (uneFoisQ3.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFoisQ3.dualSortTria += 1;
         }         
      else if ((uneFoisQ3.dualSortTria) && (uneFoisQ3.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	
   
