// FICHIER : TriaQ3_cmpti1003.h
// CLASSE : TriaQ3_cmpti1003

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *                                                                      *
 *     DATE:        15/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Element triangulaire, quadratique 3 pt d'integ              *
 *          avec la particularité d'être interne à l'élément et non     *
 *          sur les arrêtes.                                            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                      *
 *                                                                $     *
 *                                                                      *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------

// La classe TriaMemb permet de declarer des elements triangulaire membranne et de realiser
// le calcul du residu local et de la raideur locale pour une loi de comportement
// donnee. La dimension de l'espace pour un tel element est 2
//
// l'interpolation est quadratique, le nombre de point d'integration est  de 3


#ifndef TRIAQ3_CMPTI1003H
#define TRIAQ3_CMPTI1003H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomTriangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "TriaMemb.h"
#include "FrontSegQuad.h"
#include "FrontTriaQuad.h"

/// @addtogroup groupe_des_elements_finis
///  @{
///


class TriaQ3_cmpti1003 : public TriaMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		TriaQ3_cmpti1003 ();
		
		// Constructeur fonction d'une epaisseur et eventuellement d'un numero
		// d'identification 
		TriaQ3_cmpti1003 (double epaiss,int num_maill=0,int num_id=-3);
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		TriaQ3_cmpti1003 (int num_mail,int num_id);
		
		// Constructeur fonction d'une epaisseur, d'un numero d'identification,
		// du tableau de connexite des noeuds 
		TriaQ3_cmpti1003 (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		TriaQ3_cmpti1003 (const TriaQ3_cmpti1003& tria);
		
		
		// DESTRUCTEUR :
		~TriaQ3_cmpti1003 ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new TriaQ3_cmpti1003(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de TriaQ3_cmpti1003
		TriaQ3_cmpti1003& operator= (TriaQ3_cmpti1003& tria);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegQuad(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontTriaQuad(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaQ3_cmpti1003      
        static TriaMemb::DonnComTria * doCoMembQ3;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static TriaMemb::UneFois  uneFoisQ3;
        
	    class  NombresConstruireTriaQ3_cmpti1003 : public NombresConstruire
	     {  public: NombresConstruireTriaQ3_cmpti1003(); 
	      };
	    static NombresConstruireTriaQ3_cmpti1003 nombre_V; // les nombres propres à l'élément

        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsTriaQ3_cmpti1003 : public ConstrucElement
          { public :  ConsTriaQ3_cmpti1003 () 
               { NouvelleTypeElement nouv(TRIANGLE,QUADRACOMPL,MECA_SOLIDE_DEFORMABLE,this,"_cm3pti");
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation TriaQ3_cmpti1003" << endl;
                  Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new TriaQ3_cmpti1003 (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() >= 2) return true; else return false;};	
           }; 
        static ConsTriaQ3_cmpti1003 consTriaQ3_cmpti1003;
};
/// @}  // end of group
#endif
	
	
		

