// FICHIER : QuadCCom_cm9pti.h
// CLASSE : QuadCCom

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        3/05/2011                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe QuadQCom_cm9pti permet de declarer des elements *
 * Quadrangulaire membranne et de realiser                              *
 * le calcul du residu local et de la raideur locale pour une loi de    *
 * comportement donnee. La dimension de l'espace pour un tel element    *
 * est 2.                                                               *
 *                                                                      *
 * l'interpolation est cubique complet 16 noeuds, le nombre de          *
 * point d'integration est  de 9 avec gestion d'hourglass.              *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------



#ifndef QUADCCOM_CM9PTI_H
#define QUADCCOM_CM9PTI_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomQuadrangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "QuadraMemb.h"
#include "FrontQuadCC.h"
#include "FrontSegCub.h"

/// @addtogroup groupe_des_elements_finis
///  @{
///


class QuadCCom_cm9pti : public QuadraMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		QuadCCom_cm9pti ();
		
		// Constructeur fonction d'une epaisseur et eventuellement d'un numero
		// d'identification 
		QuadCCom_cm9pti (double epaiss,int num_mail=0,int num_id=-3);
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		QuadCCom_cm9pti (int num_mail,int num_id);
		
		// Constructeur fonction d'une epaisseur, d'un numero d'identification,
		// du tableau de connexite des noeuds 
		QuadCCom_cm9pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		QuadCCom_cm9pti (const QuadCCom_cm9pti& quadra);
		
		
		// DESTRUCTEUR :
		~QuadCCom_cm9pti ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new QuadCCom_cm9pti(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de QuadCCom_cm9pti
		QuadCCom_cm9pti& operator= (QuadCCom_cm9pti& quadra);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
// 2) derivant des virtuelles 
// 3) methodes propres a l'element
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegCub(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontQuadCC(tab,ddelem)));};
	 
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaMembL1 
        static QuadraMemb::DonnComQuad * doCoQuadCCom;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static QuadraMemb::UneFois  uneFoisQCom;

	    class  NombresConstruireQuadCCom_cm9pti : public NombresConstruire
	     {  public: NombresConstruireQuadCCom_cm9pti(); 
	      };
	    static NombresConstruireQuadCCom_cm9pti nombre_V; // les nombres propres à l'élément
	     		
        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsQuadCCom_cm9pti : public ConstrucElement
          { public :  ConsQuadCCom_cm9pti () 
               { NouvelleTypeElement nouv(QUADRANGLE,CUBIQUE,MECA_SOLIDE_DEFORMABLE,this,"_cm9pti");
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation QuadCCom_cm9pti" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new QuadCCom_cm9pti (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() >= 2) return true; else return false;};	
           }; 
        static ConsQuadCCom_cm9pti consQuadCCom_cm9pti;
};
/// @}  // end of group
#endif
	
	
		

