// FICHIER : QuadQCom_cm4pti.cc
// CLASSE : QuadQCom_cm4pti

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "FrontSegQuad.h"
#include "FrontQuadQuad.h"


#include "QuadQCom_cm4pti.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

QuadraMemb::DonnComQuad * QuadQCom_cm4pti::doCoQuadQCom = NULL;                                     
QuadraMemb::UneFois  QuadQCom_cm4pti::uneFoisQCom; 
QuadQCom_cm4pti::NombresConstruireQuadQCom_cm4pti QuadQCom_cm4pti::nombre_V; 
QuadQCom_cm4pti::ConsQuadQCom_cm4pti QuadQCom_cm4pti::consQuadQCom_cm4pti;

// fonction définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
QuadQCom_cm4pti::NombresConstruireQuadQCom_cm4pti::NombresConstruireQuadQCom_cm4pti() 
 { nbne = 9; // le nombre de noeud de l'élément
   nbneA = 3;// le nombre de noeud des aretes
   nbi = 4;  // le nombre de point d'intégration pour le calcul mécanique
   nbiEr = 9;// le nombre de point d'intégration pour le calcul d'erreur
   nbiS = 4; // le nombre de point d'intégration pour le calcul de second membre surfacique
   nbiA = 2; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbiMas = 9; // le nombre de point d'intégration pour le calcul de la matrice masse 
	nbiHour = 9; // le nombre de point d'intégration un blocage d'hourglass
  }; 
// =========================== constructeurs ==================

QuadQCom_cm4pti::QuadQCom_cm4pti () :
  QuadraMemb()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQCom.nbelem_in_Prog == 0)
   { uneFoisQCom.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = QUADRACOMPL;
   id_geom = QUADRANGLE;
   infos_annexes="_cm4pti";
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     {cout << "\nATTENTION -> dimension " << dim 
	      <<", pas de definition d\'elements quadrangle QUADRATIQUE complet"<< endl;
	      }
	   unefois = NULL;
	  }	  
   else
    { unefois = & uneFoisQCom; // affectation du pointeur de la classe générique triangle
      doCoQuadQCom = QuadraMemb::Init (); // initialisation par defaut
      unefois->nbelem_in_Prog++;
     } 
  };
};  
	
QuadQCom_cm4pti::QuadQCom_cm4pti (double epaiss,int num_mail,int num_id):
// Constructeur utile si la section de l'element et
// le numero de l'element  sont connus
       QuadraMemb(num_mail,num_id,QUADRACOMPL,QUADRANGLE,"_cm4pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQCom.nbelem_in_Prog == 0)
   { uneFoisQCom.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     {cout << "\nATTENTION -> dimension " << dim 
	      <<", pas de definition d\'elements quadrangle QUADRATIQUE complet"<< endl;
	      }
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else
    { unefois = & uneFoisQCom; // affectation du pointeur de la classe générique triangle
      doCoQuadQCom = QuadraMemb::Init (Donnee_specif(epaiss,nombre->nbi));
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur fonction  d'un numero de maillage et d'identification  
QuadQCom_cm4pti::QuadQCom_cm4pti (int num_mail,int num_id) :
  QuadraMemb(num_mail,num_id,QUADRACOMPL,QUADRANGLE,"_cm4pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQCom.nbelem_in_Prog == 0)
   { uneFoisQCom.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     {cout << "\nATTENTION -> dimension " << dim 
	      <<", pas de definition d\'elements quadrangle QUADRATIQUE complet"<< endl;
	      }
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else
    { unefois = & uneFoisQCom; // affectation du pointeur de la classe générique triangle
      doCoQuadQCom = QuadraMemb::Init (); // initialisation par defaut
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur utile si l'epaisseur de l'element, le numero de l'element et
// le tableau des noeuds sont connus
QuadQCom_cm4pti::QuadQCom_cm4pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab):
    QuadraMemb(num_mail,num_id,QUADRACOMPL,QUADRANGLE,tab,"_cm4pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisQCom.nbelem_in_Prog == 0)
   { uneFoisQCom.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " QuadQCom_cm4pti::QuadQCom_cm4pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     {cout << "\nATTENTION -> dimension " << dim 
	      <<", pas de definition d\'elements quadrangle QUADRATIQUE complet"<< endl;
	      }
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else
    { unefois = & uneFoisQCom; // affectation du pointeur de la classe générique triangle
      bool sans_init_noeud = true;
      doCoQuadQCom = QuadraMemb::Init (Donnee_specif(epaiss,nombre->nbi),sans_init_noeud);
      // construction du tableau de ddl spécifique à l'élément pour ses 
      ConstTabDdl(); 
      unefois->nbelem_in_Prog++;
     } 
  };
};  

QuadQCom_cm4pti::QuadQCom_cm4pti (const QuadQCom_cm4pti& QuadraM) :
 QuadraMemb (QuadraM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisQCom.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element QuadQCom_cm4pti, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
    { unefois = & uneFoisQCom; // affectation du pointeur de la classe générique triangle
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et QuadraMemb
      unefois->nbelem_in_Prog++;
     } ;
};

QuadQCom_cm4pti::~QuadQCom_cm4pti ()
// Destruction effectuee dans QuadraMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};

                         
// renseignement d'un élément complet à partir d'un élément incomplet de même type
// retourne les nouveaux noeuds construit à partir de l'interpolation incomplète.
// dans le cas l'élément n'est pas concerné, retourne une liste vide
// ramène également une liste de même dimension contenant les bornes en numéros de noeuds
// entre lesquelles il faut définir les nouveaux numéros de noeuds si l'on veut conserver
// une largeur de bande optimisée du même type
// nbnt+1: est le premier numéro de noeud utilisable pour les nouveaux noeuds
list <Noeud *> QuadQCom_cm4pti::Construct_from_imcomplet(const Element & elem,list <DeuxEntiers> & li_bornes,int nbnt)
 { list <Noeud *> li_ret; // la liste de retour
   // ----- construction de l'élément,
   // 1) définition des noeuds venant de l'élément incomplet
   const Tableau<Noeud *>& e_tab_noeud = elem.Tab_noeud_const(); 
   int nbne = nombre-> nbne; int etaille=e_tab_noeud.Taille();
   tab_noeud.Change_taille(nbne);
   // les premiers noeuds sont ceux de l'incomplet 
   for (int i=1;i<=etaille;i++)
     tab_noeud(i)=e_tab_noeud(i);
   // 2) définition du noeud central
   int idmail = tab_noeud(1)->Num_Mail(); // récup du numéro de maillage associé au noeud
   int dim =  ParaGlob::Dimension();
   // déf d'un noeud  initialisée aux coordonnées du premier noeud pour l'instant 		  
   Noeud * ptr = new Noeud(*tab_noeud(1)); 
   ptr->Change_num_noeud(nbnt+1); // attribution d'un bon numéro
   tab_noeud(9)=ptr; // enregistrement dans les noeuds de l'élément
   // ajout du tableau de ddl des noeuds, ce qui permet d'activer les ddl XI à t si ce n'est 
   // pas encore fait 
   QuadraMemb::ConstTabDdl();
   // def des coordonnées il s'agit du noeud central donc 0 0
   Coordonnee co(dim); // les coordonnées physique
   Tableau <Coordonnee> t_co; // les coordonnées physique
   const Coordonnee co_int(0.,0.); // coordonnées locales
   // test pour savoir si le calcul à tdt est activé ou pas
   if (tab_noeud(1)->Tdt())
    // cas où l'on travailles à 0 t et tdt
    { t_co.Change_taille(3);
      ((QuadraMemb&)elem).Point_physique(co_int,t_co);
      ptr->Change_coord0(t_co(1));ptr->Change_coord1(t_co(2));ptr->Change_coord2(t_co(3));
     }
   else
    // on regarde si on a les coordonnées à t qui sont définis
    {if (tab_noeud(1)->En_service(X1))
      // cas où les coordonnees à t sont définis
      { t_co.Change_taille(2);
        ((QuadraMemb&)elem).Point_physique(co_int,t_co);
        ptr->Change_coord0(t_co(1));ptr->Change_coord1(t_co(2));
       }
     else 
      // cas où seules les coordonnées à 0 sont définis 
      // en fait ici n'arrive jamais !! mais mis pour pas oublier
      { ((QuadraMemb&)elem).Point_physique(co_int,co,TEMPS_0);
        ptr->Change_coord0(t_co(1));
       }
     }   
   // 3) enregistrement du noeud
   li_ret.push_back(ptr); 
   // 4) remplissage de li_bornes   
   li_bornes.erase(li_bornes.begin(),li_bornes.end());
   // on cherche le maxi et le mini des numéros de noeud
   //  tout d'abord initialisation
   int inf=e_tab_noeud(1)->Num_noeud(); int sup = inf; 
   for (int i=2;i<=etaille;i++)
     { int num= tab_noeud(i)->Num_noeud();
       if (inf > num) inf = num; if (sup < num) sup = num;
      }
   li_bornes.push_back(DeuxEntiers(inf,sup));
   // 5) retour
   return li_ret;
  };

                         
// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void QuadQCom_cm4pti::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n***********************************************************************";
    sort << "\n Element QuadQCom_cm4pti (quadrangle quadratique complet 4 pt d'integration) ";
    sort << "\n***********************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFoisQCom.dualSortQuad) && (uneFoisQCom.CalimpPrem))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFoisQCom.dualSortQuad += 1;
         } 
    else if ((uneFoisQCom.dualSortQuad) && (uneFoisQCom.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
    else if (!(uneFoisQCom.dualSortQuad) && (uneFoisQCom.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFoisQCom.dualSortQuad += 1;
         }         
    else if ((uneFoisQCom.dualSortQuad) && (uneFoisQCom.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	
   
