// FICHIER : Quad.cc
// CLASSE : Quad

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"


#include "Quad_cm1pti.h"
//----------------------------------------------------------------
// def des donnees commune a tous les elements
// la taille n'est pas defini ici car elle depend de la lecture 
//----------------------------------------------------------------

QuadraMemb::DonnComQuad * Quad_cm1pti::doCoQuad = NULL; 
QuadraMemb::UneFois  Quad_cm1pti::uneFoisL1; 
Quad_cm1pti::NombresConstruireQuad_cm1pti Quad_cm1pti::nombre_V; 
Quad_cm1pti::ConsQuad_cm1pti Quad_cm1pti::consQuad_cm1pti;


// constructeur définissant les nombres (de noeud, de point d'integ ..)
// utilisé dans la construction des éléments
Quad_cm1pti::NombresConstruireQuad_cm1pti::NombresConstruireQuad_cm1pti() 
 { nbne = 4; // le nombre de noeud de l'élément
   nbneA = 2;// le nombre de noeud des aretes
   nbi = 1;  // le nombre de point d'intégration pour le calcul mécanique
   nbiEr = 4;// le nombre de point d'intégration pour le calcul d'erreur
   nbiS = 4; // le nombre de point d'intégration pour le calcul de second membre surfacique
   nbiA = 1; // le nombre de point d'intégration pour le calcul de second membre linéique
   nbiMas = 4; // le nombre de point d'intégration pour le calcul de la matrice masse 
	  nbiHour = 4; // le nombre de point d'intégration un blocage d'hourglass
 }; 
// =========================== constructeurs ==================

Quad_cm1pti::Quad_cm1pti () :
  QuadraMemb()
// Constructeur par defaut
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   id_interpol = LINEAIRE;
   id_geom = QUADRANGLE;
   infos_annexes="_cm1pti";
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	   { if (ParaGlob::NiveauImpression() >= 7)
	       cout << "\nATTENTION -> dimension " << dim 
	            <<", pas de definition d\'elements quadrangle LINEAIRE "<< endl;
      unefois = NULL;
	   }	  
   else 
    { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
      doCoQuad = QuadraMemb::Init (); // initialisation par defaut
      unefois->nbelem_in_Prog++;
    };
  };
};  
	
Quad_cm1pti::Quad_cm1pti (double epaiss,int num_mail,int num_id):
// Constructeur utile si la section de l'element et
// le numero de l'element  sont connus
       QuadraMemb(num_mail,num_id,LINEAIRE,QUADRANGLE,"_cm1pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	     cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition d\'elements quadrangle LINEAIRE "<< endl;
	    unefois = NULL;
	    Sortie (1);
	  }	  
   else
    { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
      doCoQuad = QuadraMemb::Init (Donnee_specif(epaiss,nombre->nbi));
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur fonction  d'un numero de maillage et d'identification  
Quad_cm1pti::Quad_cm1pti (int num_mail,int num_id) :
  QuadraMemb(num_mail,num_id,LINEAIRE,QUADRANGLE,"_cm1pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   tab_noeud.Change_taille(nombre->nbne);
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	      cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition d\'elements quadrangle LINEAIRE "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else
    { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
      doCoQuad = QuadraMemb::Init (); // initialisation par defaut
      unefois->nbelem_in_Prog++;
     } 
  };
};  

// Constructeur utile si l'epaisseur de l'element, le numero de l'element et
// le tableau des noeuds sont connus
Quad_cm1pti::Quad_cm1pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab):
    QuadraMemb(num_mail,num_id,LINEAIRE,QUADRANGLE,tab,"_cm1pti")
{// on intervient seulement à partir du deuxième élément, 
 if (uneFoisL1.nbelem_in_Prog == 0)
   { uneFoisL1.nbelem_in_Prog++; // au premier passage on se contente d'incrémenter
    }
 else 	// sinon on construit     
  {nombre = & nombre_V; 
   if (tab_noeud.Taille() != nombre->nbne)
	  { cout << "\n erreur de dimensionnement du tableau de noeud \n";
	   cout << " Quad_cm1pti::Quad_cm1pti (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab)\n";
	   Sortie (1); }
   // vérif des dimensions
   int dim = ParaGlob::Dimension();
   if ( dim == 1)  // cas d'une dimension ou l'élément est impossible
	 { if (ParaGlob::NiveauImpression() >= 7)
	    cout << "\nATTENTION -> dimension " << dim 
	          <<", pas de definition d\'elements quadrangle LINEAIRE "<< endl;
	   unefois = NULL;
	   Sortie (1);
	  }	  
   else
    { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
      bool sans_init_noeud = true;
      doCoQuad = QuadraMemb::Init (Donnee_specif(epaiss,nombre->nbi),sans_init_noeud);
      // construction du tableau de ddl spécifique à l'élément  
      ConstTabDdl(); 
      unefois->nbelem_in_Prog++;
     } 
  };
};  

Quad_cm1pti::Quad_cm1pti (const Quad_cm1pti& QuadraM) :
 QuadraMemb (QuadraM)
 
// Constructeur de copie
// a priori si on utilise le constructeur de copie, donc il y a déjà un élément
// par contre a priori on ne doit pas faire une copie du premier élément 
{ if (uneFoisL1.nbelem_in_Prog == 1)
   { cout << "\n **** erreur pour l'element Quad_cm1pti, le constructeur de copie ne doit pas etre utilise"
	       << " pour le premier element !! " << endl;
	  Sortie (1);
    }
 else 	    
    { unefois = &uneFoisL1; // affectation du pointeur de la classe générique triangle
      // ce qui est relatif à l'initialisation est déjà effectué dans elem_meca et QuadraMemb
      unefois->nbelem_in_Prog++;
     }; 
};

Quad_cm1pti::~Quad_cm1pti ()
// Destruction effectuee dans QuadraMemb
{ if (unefois != NULL) 
    {unefois->nbelem_in_Prog--;
     Destruction();
     }
};

// affichage dans la sortie transmise, des variables duales "nom"
// aux differents points d'integration
// dans le cas ou nom est vide, affichage de "toute" les variables
void Quad_cm1pti::AfficheVarDual(ofstream& sort, Tableau<string>& nom)
  { // affichage de l'entête de l'element
    sort << "\n******************************************************************";
    sort << "\n Element Quad_cm1pti (quadrangle lineaire 1 pt d'integration) ";
    sort << "\n******************************************************************";
    // appel de la procedure de elem meca
    if (!(uneFoisL1.dualSortQuad) && (uneFoisL1.CalimpPrem ))
        { VarDualSort(sort,nom,nombre->nbi,1);
          uneFoisL1.dualSortQuad += 1;
         } 
    else if ((uneFoisL1.dualSortQuad) && (uneFoisL1.CalimpPrem))       
         VarDualSort(sort,nom,nombre->nbi,11);
    else if (!(uneFoisL1.dualSortQuad) && (uneFoisL1.CalResPrem_tdt))       
        { VarDualSort(sort,nom,nombre->nbi,2);
          uneFoisL1.dualSortQuad += 1;
         }         
    else if ((uneFoisL1.dualSortQuad) && (uneFoisL1.CalResPrem_tdt))       
         VarDualSort(sort,nom,nombre->nbi,12);
      // sinon on ne fait rien     
  };               	
		  
