// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Classe pour stocker l'ensemble des informations aux points  *
 *          d'intégration thermique                                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef LESPTINTEGTHERMIINTERNE_H
#define LESPTINTEGTHERMIINTERNE_H

#include "PtIntegThermiInterne.h"

/// @addtogroup Groupe_concernant_les_points_integration
///  @{
///


class  LesPtIntegThermiInterne

{   // surcharge de l'operator de lecture
    friend istream & operator >> (istream &, LesPtIntegThermiInterne &);
    // surcharge de l'operator d'ecriture
    friend ostream & operator << (ostream &, const LesPtIntegThermiInterne &);

  public :
    // CONSTRUCTEURS :
    // contructeur par défaut
    LesPtIntegThermiInterne();
    // contructeur fonction du nombre de points d'intégration et de la dimension des coordonnees
    LesPtIntegThermiInterne(int nbpti, int dimcoor);
    // contructeur de copie
    LesPtIntegThermiInterne(const LesPtIntegThermiInterne& lespti);
    
    // DESTRUCTEUR :
    ~LesPtIntegThermiInterne();
    
    // METHODES PUBLIQUES :
	   // Surcharge de l'operateur =
    LesPtIntegThermiInterne& operator= ( const LesPtIntegThermiInterne& lespti);
    
    // le tableau des grandeurs aux points d'intégration
    // en lecture écriture
    Tableau <PtIntegThermiInterne>& TabPtIntThermi()  {return tabPtInt;};
    // l'élément PtIntegThermiInterne de numéro i
    PtIntegThermiInterne& operator () (int i)  {return tabPtInt(i);};
    
    
    // les tableaux des densité de flux
    Tableau <CoordonneeH *>& TabfluxH() {return tabfluxH;}; // densité flux finale
    Tableau <CoordonneeH *>& TabfluxH_t() {return tabfluxH_t;}; // densité flux au début de l'incrément
    // nombre de points d'intégration
    int NbPti() const {return tabPtInt.Taille();};
    // changement de taille donc de nombre de points d'intégration
    // fonction du nombre de points d'intégration et de la dimension des coordonnees
    void Change_taille_PtIntegThermi(int nbpti, int dimtens);
    // idem, mais les instances ajoutées ou retirer ont la même dimension de coordonnees que celles
    // qui existent déjà
    void Change_taille_PtIntegThermi(int nbpti);
    // retour la dimension des vecteurs gérés
    int DimCoord() const;
    // actualisation des grandeurs actives de t+dt vers t, pour celles qui existent
    // sous ces deux formes    
    void TdtversT();
    // actualisation des grandeurs actives de t vers tdt, pour celles qui existent 
    // sous ces deux formes          
    void TversTdt();
 
	//============= lecture écriture dans base info ==========
    // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	void Lecture_base_info (ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	void Ecriture_base_info(ofstream& sort,const int cas);
    
  protected:
    // données protégées
    // grandeurs aux points d'intégration
    Tableau <PtIntegThermiInterne> tabPtInt;
    // densité de flux groupées sous forme de tableau, qui pointent sur celles de tabPtThermiInt
    Tableau <CoordonneeH *> tabfluxH; // densité flux finale
    Tableau <CoordonneeH *> tabfluxH_t; // densité flux au début de l'incrément

 };
 /// @}  // end of group

#endif  
