/*! \file EnuTypeCL.h
    \brief Type énuméré pour définir les différentes sous-classes de conditions limites.
* \date      04/10/2007
*/


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        04/10/2007                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Type énuméré pour définir les différentes sous-classes    *
 *            de conditions limites.                                    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


// Afin de realiser un gain en place memoire, les noms des types de grandeurs sont
// stockes a l'aide d'un type enumere. 


#ifndef ENUTYPECL_H
#define ENUTYPECL_H
#include <iostream>
using namespace std;
#include <map>
#include "ParaGlob.h"
#include "EnumTypeGrandeur.h"
#include "Enum_dure.h"
#include "Tableau_T.h"

/// @addtogroup Group_types_enumeres
///  @{

/// Type énuméré pour définir les différentes sous-classes de conditions limites.
enum EnuTypeCL {  TANGENTE_CL, RIEN_TYPE_CL}; 
/// @}  // end of group

/// @addtogroup Group_types_enumeres
///  @{

/// def de la map qui fait la liaison entre les string et les énumérés

class ClassPourEnuTypeCL
{ public:
  /// def de la map qui fait la liaison entre les string et les énumérés
  static map < string, EnuTypeCL , std::less < string> > map_EnuTypeCL;
  // def de la grandeur statique qui permet de remplir la map
  static ClassPourEnuTypeCL remplir_map;
  // le constructeur qui rempli effectivement la map
  ClassPourEnuTypeCL();
  };
/// @}  // end of group

// Retourne le nom du type de grandeur a partir de son identificateur de
// type enumere id_TypeCL correspondant
char* NomTypeCL (EnuTypeCL id_TypeCL);

// indique si le string correspond à un type CL reconnu ou non
bool Existe_typeCL(string& nom);

// Retourne l'identificateur de type enumere associe au nom du type de grandeur
// nom_TypeCL
EnuTypeCL Id_nomTypeCL (char* nom_TypeCL);
EnuTypeCL Id_nomTypeCL (string& nom_TypeCL);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnuTypeCL& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnuTypeCL& a);

#endif
