// FICHIER : EnuTypeQuelParticulier.cc


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "EnuTypeQuelParticulier.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>

// tout d'abord on définit la map qui permet de relier les chaines et les types énumérés
  // 1) def de la map 
  map < string,  EnuTypeQuelParticulier , std::less < string> > 
           ClassPourEnuTypeQuelParticulier::map_EnuTypeQuelParticulier;
         
  // 2) def de la grandeur statique qui permet de remplir la map
  ClassPourEnuTypeQuelParticulier ClassPourEnuTypeQuelParticulier::remplir_map;
  // le constructeur qui rempli effectivement la map
  ClassPourEnuTypeQuelParticulier::ClassPourEnuTypeQuelParticulier()
    { // remplissage de la map
      map_EnuTypeQuelParticulier["RIEN_TYPE_QUELCONQUE_PARTICULIER"]=RIEN_TYPE_QUELCONQUE_PARTICULIER; 
      map_EnuTypeQuelParticulier["PARTICULIER_TENSEURHH"]=PARTICULIER_TENSEURHH; 
      map_EnuTypeQuelParticulier["PARTICULIER_TENSEURBB"]=PARTICULIER_TENSEURBB;
      map_EnuTypeQuelParticulier["PARTICULIER_COORDONNEE"]=PARTICULIER_COORDONNEE;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_TENSEURHH"]=PARTICULIER_TABLEAU_TENSEURHH; 
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU2_TENSEURHH"]=PARTICULIER_TABLEAU2_TENSEURHH; 
      map_EnuTypeQuelParticulier["PARTICULIER_TENSEURBH"]=PARTICULIER_TENSEURBH; 
      map_EnuTypeQuelParticulier["PARTICULIER_SCALAIRE_DOUBLE"]=PARTICULIER_SCALAIRE_DOUBLE; 
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_SCALAIRE_DOUBLE"]=PARTICULIER_TABLEAU_SCALAIRE_DOUBLE; 
      map_EnuTypeQuelParticulier["PARTICULIER_SCALAIRE_ENTIER"]=PARTICULIER_SCALAIRE_ENTIER;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_SCALAIRE_ENTIER"]=PARTICULIER_TABLEAU_SCALAIRE_ENTIER;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_QUELCONQUE"]=PARTICULIER_TABLEAU_QUELCONQUE;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_COORDONNEE"]=PARTICULIER_TABLEAU_COORDONNEE; 
      map_EnuTypeQuelParticulier["PARTICULIER_VECTEUR"]=PARTICULIER_VECTEUR;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_VECTEUR"]=PARTICULIER_TABLEAU_VECTEUR;
      map_EnuTypeQuelParticulier["PARTICULIER_BASE_H"]=PARTICULIER_BASE_H;
      map_EnuTypeQuelParticulier["PARTICULIER_BASE_B"]=PARTICULIER_BASE_B;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_BASE_H"]=PARTICULIER_TABLEAU_BASE_H;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_BASE_B"]=PARTICULIER_TABLEAU_BASE_B;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_TENSEURBH"]=PARTICULIER_TABLEAU_TENSEURBH;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_TENSEURBB"]=PARTICULIER_TABLEAU_TENSEURBB; 
      map_EnuTypeQuelParticulier["PARTICULIER_TENSEURHB"]=PARTICULIER_TENSEURHB; 
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_TENSEURHB"]=PARTICULIER_TABLEAU_TENSEURHB; 
      map_EnuTypeQuelParticulier["PARTICULIER_DDL_ETENDU"]=PARTICULIER_DDL_ETENDU;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_DDL_ETENDU"]=PARTICULIER_TABLEAU_DDL_ETENDU;
      map_EnuTypeQuelParticulier["PARTICULIER_VECTEUR_NOMMER"]=PARTICULIER_VECTEUR_NOMMER;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_VECTEUR_NOMMER"]=PARTICULIER_TABLEAU_VECTEUR_NOMMER;
      map_EnuTypeQuelParticulier["PARTICULIER_TABLEAU_VECTEUR_NOMMER"]=PARTICULIER_TABLEAU_VECTEUR_NOMMER;
      map_EnuTypeQuelParticulier["PARTICULIER_SCALAIRE_DOUBLE_NOMMER_INDICER"]=PARTICULIER_SCALAIRE_DOUBLE_NOMMER_INDICER;
     };

string NomTypeQuelParticulier(EnuTypeQuelParticulier id_TypeQuelParticulier)
// Retourne le nom du type de grandeur correspondant a l'identificateur 
// de type enumere id_TypeQuelParticulier
{   string result;
	switch (id_TypeQuelParticulier)
	{   case RIEN_TYPE_QUELCONQUE_PARTICULIER : result="RIEN_TYPE_QUELCONQUE_PARTICULIER"; break;
		case PARTICULIER_TENSEURHH : result="PARTICULIER_TENSEURHH"; break;
		case PARTICULIER_TENSEURBB : result="PARTICULIER_TENSEURBB"; break;
		case PARTICULIER_COORDONNEE : result="PARTICULIER_COORDONNEE"; break;
		case PARTICULIER_TABLEAU_TENSEURHH : result="PARTICULIER_TABLEAU_TENSEURHH"; break;
		case PARTICULIER_TABLEAU2_TENSEURHH : result="PARTICULIER_TABLEAU2_TENSEURHH"; break;
		case PARTICULIER_TENSEURBH : result="PARTICULIER_TENSEURBH"; break;
		case PARTICULIER_SCALAIRE_DOUBLE : result="PARTICULIER_SCALAIRE_DOUBLE"; break;
		case PARTICULIER_TABLEAU_SCALAIRE_DOUBLE : result="PARTICULIER_TABLEAU_SCALAIRE_DOUBLE"; break;
		case PARTICULIER_SCALAIRE_ENTIER : result="PARTICULIER_SCALAIRE_ENTIER"; break;
		case PARTICULIER_TABLEAU_SCALAIRE_ENTIER : result="PARTICULIER_TABLEAU_SCALAIRE_ENTIER"; break;
		case PARTICULIER_TABLEAU_QUELCONQUE : result="PARTICULIER_TABLEAU_QUELCONQUE"; break;
		case PARTICULIER_TABLEAU_COORDONNEE : result="PARTICULIER_TABLEAU_COORDONNEE"; break;
  case PARTICULIER_VECTEUR : result="PARTICULIER_VECTEUR"; break;
  case PARTICULIER_TABLEAU_VECTEUR : result="PARTICULIER_TABLEAU_VECTEUR"; break;
  case PARTICULIER_BASE_H : result="PARTICULIER_BASE_H"; break;
  case PARTICULIER_BASE_B : result="PARTICULIER_BASE_B"; break;
  case PARTICULIER_TABLEAU_BASE_H : result="PARTICULIER_TABLEAU_BASE_H"; break;
  case PARTICULIER_TABLEAU_BASE_B : result="PARTICULIER_TABLEAU_BASE_B"; break;
		case PARTICULIER_TABLEAU_TENSEURBH : result="PARTICULIER_TABLEAU_TENSEURBH"; break;
		case PARTICULIER_TABLEAU_TENSEURBB : result="PARTICULIER_TABLEAU_TENSEURBB"; break;
		case PARTICULIER_TENSEURHB : result="PARTICULIER_TENSEURHB"; break;
		case PARTICULIER_TABLEAU_TENSEURHB : result="PARTICULIER_TABLEAU_TENSEURHB"; break;
		case PARTICULIER_DDL_ETENDU : result="PARTICULIER_DDL_ETENDU"; break;
		case PARTICULIER_TABLEAU_DDL_ETENDU : result="PARTICULIER_TABLEAU_DDL_ETENDU"; break;
		case PARTICULIER_VECTEUR_NOMMER : result="PARTICULIER_VECTEUR_NOMMER"; break;
  case PARTICULIER_TABLEAU_VECTEUR_NOMMER : result="PARTICULIER_TABLEAU_VECTEUR_NOMMER"; break;
  case PARTICULIER_SCALAIRE_DOUBLE_NOMMER_INDICER : result="PARTICULIER_SCALAIRE_DOUBLE_NOMMER_INDICER"; break;
		default :
			cout << "\nErreur : valeur incorrecte du type EnuTypeQuelParticulier !\n";
			cout << "NomTypeQuelParticulier(EnuTypeQuelParticulier ) \n";
			Sortie(1);
	};
	return result;	
};
						
EnuTypeQuelParticulier Id_nomTypeQuelParticulier (char* nom_TypeQuelParticulier)
// Retourne la variable de type enumere associe au nom nom_TypeQuelParticulier
{	// on vérifie si la variable de type enumere existe
	 map < string, EnuTypeQuelParticulier , std::less < string> >& maa=ClassPourEnuTypeQuelParticulier::map_EnuTypeQuelParticulier;
  map < string, EnuTypeQuelParticulier , std::less < string> >::iterator il,ilfin= maa.end();
  il = maa.find(nom_TypeQuelParticulier);
  if (il == ilfin)
    {// on vérifie si la variable de type enumere existe
     cout << "\nErreur : nom du type de grandeur  '" <<nom_TypeQuelParticulier <<  " 'inconnu !\n";
     cout << "Id_nomTypeQuelParticulier(char* ) \n";
     Sortie(1);
    };
	 // retour de la grandeur
	 return (*il).second;  
};

EnuTypeQuelParticulier Id_nomTypeQuelParticulier (string& nom_TypeQuelParticulier)
// Retourne la variable de type enumere associe au nom nom_TypeQuelParticulier
{	// on vérifie si la variable de type enumere existe
	map < string, EnuTypeQuelParticulier , std::less < string> >& maa=ClassPourEnuTypeQuelParticulier::map_EnuTypeQuelParticulier;
 map < string, EnuTypeQuelParticulier , std::less < string> >::iterator il,ilfin= maa.end();
 il = maa.find(nom_TypeQuelParticulier);
 if (il == ilfin)
	   {// on vérifie si la variable de type enumere existe
     cout << "\nErreur : nom du type de grandeur  '" <<nom_TypeQuelParticulier <<  " 'inconnu !\n";
     cout << "Id_nomTypeQuelParticulier(string& ) \n";
     Sortie(1);
	   };
	 // retour de la grandeur 
	 return (*il).second;  
};

// indique si le string correspond à un type quelParticulier reconnu ou non
bool Existe_typeQuelParticulier(string& nom)
{   return ( ClassPourEnuTypeQuelParticulier::map_EnuTypeQuelParticulier.find(nom) 
            != ClassPourEnuTypeQuelParticulier::map_EnuTypeQuelParticulier.end());};
	
// surcharge de la lecture
istream & operator >> (istream & entree, EnuTypeQuelParticulier & result)
 { char nom_TypeQuelParticulier[80];
   entree >> nom_TypeQuelParticulier;
   result = Id_nomTypeQuelParticulier ( nom_TypeQuelParticulier);
   return entree;
 };  
   	   
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnuTypeQuelParticulier& a)
    { // on ecrit la forme caractère
       sort << NomTypeQuelParticulier(a) << " ";
       return sort;      
     }; 

