/*! \file EnumElemTypeProblem.h
    \brief def de l'enuméré concernant les types de problème
*/
// FICHIER : EnumElemTypeProblem.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// liste des différentes types de problème que l'on peut trouver au niveau d'un élément
// fini


#ifndef ENUMELEMTYPEPROBLEM_H
#define ENUMELEMTYPEPROBLEM_H

//#include "Debug.h"
// #include <bool.h>
#include <iostream>
using namespace std;



/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de problème

enum EnumElemTypeProblem { MECA_SOLIDE_DEFORMABLE = 1, MECA_SOLIDE_INDEFORMABLE, 
                           MECA_FLUIDE, THERMIQUE, ELECTROMAGNETIQUE , RIEN_PROBLEM};
/// @}  // end of group


int Nombre_de_EnumElemTypeProblem() ; // nombre maxi d'enum
// Retourne le nom  a partir de son identificateur de
// type enumere id_ElemTypeProbleme correspondant
string NomElemTypeProblem (const EnumElemTypeProblem id_ElemTypeProblem);

// Retourne l'identificateur de type enumere associe au nom
EnumElemTypeProblem Id_nom_ElemTypeProblem (const string& nom_ElemTypeProblem);

// retourne true si l'identificateur existe, false sinon
bool ExisteEnum_ElemTypeProblem(const string& nom_ElemTypeProblem);

// retourne le nombre maxi de Type de problème apréhendé
int NbEnum_ElemTypeProblem();
// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnumElemTypeProblem& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnumElemTypeProblem& a);

// pour faire de l'inline
#ifndef MISE_AU_POINT
  #include "EnumElemTypeProblem.cc"
  #define  EnumElemTypeProblem_deja_inclus
#endif


#endif
