/*! \file EnumFonction_nD.h
    \brief Enumeration des différentes fonctions nD existantes
* \date      06/03/2023
*/


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Enumeration des différentes fonctions nD existantes      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUMFONCTION_ND_H
#define ENUMFONCTION_ND_H
#include <iostream>
using namespace std;
#include <string.h>
#include <string>
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes fonctions nD existantes

enum EnumFonction_nD { FONCTION_EXPRESSION_LITTERALE_nD=1
                      ,FONCTION_COURBE1D,FONC_SCAL_COMBINEES_ND
                      ,FONCTION_EXTERNE_ND
                      ,AUCUNE_FONCTION_nD};
/// @}  // end of group



// Retourne le nom d'une Fonction_nD a partir de son identificateur de
// type enumere id_Fonction_nD correspondant
string Nom_Fonction_nD (EnumFonction_nD id_Fonction_nD);

// Retourne l'identificateur de type enumere associe au nom d'une Fonction_nD
EnumFonction_nD Id_Nom_Fonction_nD (const string& nom_Fonction_nD) ;

// Retourne vrai si le nom passé en argument représente un type de courbe reconnu
// sinon false
bool Type_EnumFonction_nD_existe(const std::string& nom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnumFonction_nD& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnumFonction_nD& a);

#endif
