/*! \file Enum_Critere_loi.h
    \brief Enumeration des différentes méthodes permettant  d'utiliser des critères sur les lois de comportement
* \date      11/06/2014
*/

// FICHIER : Enum_Critere_loi.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        11/06/2014                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 *     BUT:    Enumeration des différentes méthodes permettant          *
 *             d'utiliser des critères sur les lois de comportement     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUM_CRITERE_LOI_H
#define ENUM_CRITERE_LOI_H
#include <iostream>
#include <string.h>
#include <string>
using namespace std;  //introduces namespace std
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes méthodes permettant  d'utiliser des critères sur les lois de comportement

enum Enum_Critere_Loi { AUCUN_CRITERE = 0, PLISSEMENT_MEMBRANE,PLISSEMENT_BIEL
                        ,RUPTURE_SIGMA_PRINC, RUPTURE_EPS_PRINC };
/// @}  // end of group



// Retourne le nom a partir de son identificateur de type enumere 
string Nom_Critere_Loi(Enum_Critere_Loi id_Critere_Loi);

// Retourne l'identificateur de type enumere associe au nom d'un Critere_Loi
Enum_Critere_Loi Id_Nom_Critere_Loi(const string& nom_Critere_Loi) ;

// Retourne vrai si le nom passé en argument représente un type de Critere_Loi reconnu
// sinon false
bool Type_Enum_Critere_Loi_existe(const string& nom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_Critere_Loi& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_Critere_Loi& a);

#endif
