/*! \file Enum_GrandeurGlobale.h
    \brief Définition de l'enuméré concernant les grandeurs globales
*/
// FICHIER : Enum_GrandeurGlobale.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des grandeurs globales
// sont stockes a l'aide d'un type enumere. Les fonctions Nom_GrandeurGlobale et
// Id_nom_GrandeurGlobale rendent possible le lien entre les noms des types d'interpolation
// et les identificateurs de type enumere correspondants.


#ifndef ENUM_GRANDEUR_GLOBALE_H
#define ENUM_GRANDEUR_GLOBALE_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les grandeurs globales

enum Enum_GrandeurGlobale { ENERGIE_CINETIQUE = 0,ENERGIE_INTERNE
                 ,ENERGIE_EXTERNE, ENERGIE_BILAN,QUANTITE_MOUVEMENT
                 ,PUISSANCE_ACCELERATION,PUISSANCE_INTERNE,PUISSANCE_EXTERNE
                 ,PUISSANCE_BILAN,ENERGIE_ELASTIQUE,ENERGIE_PLASTIQUE
                 ,ENERGIE_VISQUEUSE,ENERGIE_HOURGLASS_,ENERGIE_PENALISATION
                 ,ENERGIE_FROT_ELAST,ENERGIE_FROT_PLAST,ENERGIE_FROT_VISQ
                 ,ENERGIE_VISCO_NUMERIQUE,ENERGIE_BULK_VISCOSITY
                 ,PUISSANCE_BULK_VISCOSITY,VOLUME_TOTAL_MATIERE
                 ,ENERGIE_STABILISATION_MEMB_BIEL
                 ,VOL_TOTAL2D_AVEC_PLAN_YZ
                 ,VOL_TOTAL2D_AVEC_PLAN_XZ,VOL_TOTAL2D_AVEC_PLAN_XY
                 ,NORME_CONVERGENCE,COMPTEUR_ITERATION_ALGO_GLOBAL
                 ,MAXPUISSEXT,MAXPUISSINT,MAXREACTION,MAXRESIDUGLOBAL
                 ,MAXdeltaX,MAXvarDeltaX,MAXvarDdl
                 ,COMPTEUR_INCREMENT_CHARGE_ALGO_GLOBAL
                 ,AMOR_CINET_VISQUEUX
                 ,TEMPS_COURANT
                 ,ALGO_GLOBAL_ACTUEL
                           };
/// @}  // end of group

//***** ne pas oublier de changer la taille maxi -> taille_Enum_GrandeurGlobale
const int taille_Enum_GrandeurGlobale = 38;

// Retourne un nom a partir de son identificateur de
// type enumere id_GrandeurGlobale correspondant
string Nom_GrandeurGlobale (const Enum_GrandeurGlobale id_GrandeurGlobale) ;

// Retourne l'identificateur de type enumere associe au nom du type
//  nom_GrandeurGlobale
Enum_GrandeurGlobale Id_nom_GrandeurGlobale (const string& nom_GrandeurGlobale);

// test si le string est une grandeur globale
bool EstUneGrandeurGlobale(const string& nom_GrandeurGlobale);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_GrandeurGlobale& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_GrandeurGlobale& a);


#endif


