// FICHIER : Enum_Suite.cc


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "Enum_Suite.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>



char* Nom_Suite (Enum_Suite id_Suite)
// Retourne le nom de la  suite correspondant a l'identificateur 
// de type enumere id_Suite
{
	
   char* result;
	switch (id_Suite)
	{
		case SUITE_EQUIDISTANTE :result="SUITE_EQUIDISTANTE";break;
		case SUITE_ARITHMETIQUE :result="SUITE_ARITHMETIQUE";break;
		case SUITE_GEOMETRIQUE :result="SUITE_GEOMETRIQUE";break;
		case SUITE_NON_DEFINIE :result="SUITE_NON_DEFINIE";break;
		default :
			cout << "\nErreur : valeur incorrecte du type Enum_Suite !\n";
			cout << "Nom_Suite(Enum_Suite ) \n";
			Sortie(1);
	};
	return result;
	
};
						
Enum_Suite Id_Nom_Suite (const char* nom_Suite)
// Retourne la variable de type enumere associee au nom de la Suite
{
	
	Enum_Suite result;
	if ( strcmp(nom_Suite,"SUITE_EQUIDISTANTE")==0 )
		result=SUITE_EQUIDISTANTE;
	else if ( strcmp(nom_Suite,"SUITE_ARITHMETIQUE")==0 )
		result=SUITE_ARITHMETIQUE;
	else if ( strcmp(nom_Suite,"SUITE_GEOMETRIQUE")==0 )
		result=SUITE_GEOMETRIQUE;
	else if ( strcmp(nom_Suite,"SUITE_NON_DEFINIE")==0 )
		result=SUITE_NON_DEFINIE;
	else
	{
		cout << "\nErreur : nom de la Suite inconnu !: " << nom_Suite << "\n";
		cout << "Id_Nom_Suite (char* nom_Suite) \n";
		Sortie(1);
	};
	return result;
	
};

// Retourne vrai si le nom passé en argument représente un type de suite reconnu
// sinon false
bool Type_Enum_Suite_existe(const string& nom)
{ bool result;
	if ( nom == "SUITE_EQUIDISTANTE") result=true;
	else if (  nom == "SUITE_ARITHMETIQUE") result=true;
	else if (  nom == "SUITE_GEOMETRIQUE") result=true;
	else if (  nom == "SUITE_NON_DEFINIE") result=true;
	else result = false;
	return result;
};
   	
   // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_Suite& a)
 { char nom_Enum_Suite[50];
   entree >> nom_Enum_Suite;
   a = Id_Nom_Suite ( nom_Enum_Suite);
   return entree;
 };
   
   // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_Suite& a)
    { // on ecrit la forme caractère
       sort << Nom_Suite(a) << " ";
       return sort;      
     }; 

	
