// FICHIER : EnumTypeQuelconque.cc


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "Enum_TypeQuelconque.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>

// tout d'abord on définit la map qui permet de relier les chaines et les types énumérés
  // 1) def de la map et des tableaux
  map < string,  EnumTypeQuelconque , std::less < string> > 
           ClassPourEnumTypeQuelconque::map_EnumTypeQuelconque;
  Tableau < int > ClassPourEnumTypeQuelconque::tt_GLOB;  
  Tableau < Enum_dure > ClassPourEnumTypeQuelconque::tt_TQ_temps;
         
  // 2) def de la grandeur statique qui permet de remplir la map
  ClassPourEnumTypeQuelconque ClassPourEnumTypeQuelconque::remplir_map;
  // le constructeur qui rempli effectivement la map
  ClassPourEnumTypeQuelconque::ClassPourEnumTypeQuelconque()
    { // remplissage de la map
      map_EnumTypeQuelconque["RIEN_TYPEQUELCONQUE"]=RIEN_TYPEQUELCONQUE; 
      map_EnumTypeQuelconque["sigma_barre_BH_t"]=SIGMA_BARRE_BH_T; 
      map_EnumTypeQuelconque["contrainte_individuelle_a_t_sans_proportion"]=CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION; 
      map_EnumTypeQuelconque["contrainte_individuelle_a_chaque_loi_a_t"]=CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T; 
      map_EnumTypeQuelconque["CONTRAINTE_COURANTE"]=CONTRAINTE_COURANTE; 
      map_EnumTypeQuelconque["DEFORMATION_COURANTE"]=DEFORMATION_COURANTE; 
      map_EnumTypeQuelconque["VITESSE_DEFORMATION_COURANTE"]=VITESSE_DEFORMATION_COURANTE;       
      map_EnumTypeQuelconque["ALMANSI"]=ALMANSI; 
      map_EnumTypeQuelconque["GREEN_LAGRANGE"]=GREEN_LAGRANGE; 
      map_EnumTypeQuelconque["LOGARITHMIQUE"]=LOGARITHMIQUE; 
      map_EnumTypeQuelconque["DELTA_DEF"]=DELTA_DEF; 
      map_EnumTypeQuelconque["ALMANSI_TOTAL"]=ALMANSI_TOTAL; 
      map_EnumTypeQuelconque["GREEN_LAGRANGE_TOTAL"]=GREEN_LAGRANGE_TOTAL; 
      map_EnumTypeQuelconque["LOGARITHMIQUE_TOTALE"]=LOGARITHMIQUE_TOTALE; 
      map_EnumTypeQuelconque["DEF_PRINCIPALES"]=DEF_PRINCIPALES; 
      map_EnumTypeQuelconque["SIGMA_PRINCIPALES"]=SIGMA_PRINCIPALES; 
      map_EnumTypeQuelconque["VIT_PRINCIPALES"]=VIT_PRINCIPALES;
      map_EnumTypeQuelconque["DEF_DUALE_MISES"]=DEF_DUALE_MISES;
      map_EnumTypeQuelconque["DEF_DUALE_MISES_MAXI"]=DEF_DUALE_MISES_MAXI;
      map_EnumTypeQuelconque["CONTRAINTE_MISES"]=CONTRAINTE_MISES;
      map_EnumTypeQuelconque["CONTRAINTE_MISES_T"]=CONTRAINTE_MISES_T;
      map_EnumTypeQuelconque["CONTRAINTE_TRESCA"]=CONTRAINTE_TRESCA;
      map_EnumTypeQuelconque["CONTRAINTE_TRESCA_T"]=CONTRAINTE_TRESCA_T;
      map_EnumTypeQuelconque["ERREUR_Q"]=ERREUR_Q;
      map_EnumTypeQuelconque["DEF_PLASTIQUE_CUMULEE"]=DEF_PLASTIQUE_CUMULEE;
      map_EnumTypeQuelconque["ERREUR_SIG_RELATIVE"]=ERREUR_SIG_RELATIVE;

      map_EnumTypeQuelconque["TEMPERATURE_LOI_THERMO_PHYSIQUE"]=TEMPERATURE_LOI_THERMO_PHYSIQUE;
      map_EnumTypeQuelconque["PRESSION_LOI_THERMO_PHYSIQUE"]=PRESSION_LOI_THERMO_PHYSIQUE; 
      map_EnumTypeQuelconque["TEMPERATURE_TRANSITION"]=TEMPERATURE_TRANSITION; 
      map_EnumTypeQuelconque["VOLUME_SPECIFIQUE"]=VOLUME_SPECIFIQUE;
      map_EnumTypeQuelconque["FLUXD"]=FLUXD;
      map_EnumTypeQuelconque["GRADT"]=GRADT;
      map_EnumTypeQuelconque["DGRADT"]=DGRADT;
      map_EnumTypeQuelconque["DELTAGRADT"]=DELTAGRADT;
      map_EnumTypeQuelconque["COEFF_DILATATION_LINEAIRE"]=COEFF_DILATATION_LINEAIRE;
      map_EnumTypeQuelconque["CONDUCTIVITE"]=CONDUCTIVITE; 
      map_EnumTypeQuelconque["CAPACITE_CALORIFIQUE"]=CAPACITE_CALORIFIQUE; 
      map_EnumTypeQuelconque["MODULE_COMPRESSIBILITE"]=MODULE_COMPRESSIBILITE; 
      map_EnumTypeQuelconque["COEFF_COMPRESSIBILITE"]=COEFF_COMPRESSIBILITE; 
      map_EnumTypeQuelconque["MODULE_CISAILLEMENT"]=MODULE_CISAILLEMENT; 
      map_EnumTypeQuelconque["MODULE_COMPRESSIBILITE_TOTAL"]=MODULE_COMPRESSIBILITE_TOTAL; 
      map_EnumTypeQuelconque["MODULE_CISAILLEMENT_TOTAL"]=MODULE_CISAILLEMENT_TOTAL; 
      map_EnumTypeQuelconque["E_YOUNG"]=E_YOUNG;
      map_EnumTypeQuelconque["NU_YOUNG"]=NU_YOUNG;
      map_EnumTypeQuelconque["MU_VISCO"]=MU_VISCO;
      map_EnumTypeQuelconque["MU_VISCO_SPHERIQUE"]=MU_VISCO_SPHERIQUE;
      map_EnumTypeQuelconque["MODULE_TANGENT_1D"]=MODULE_TANGENT_1D;
      map_EnumTypeQuelconque["COMPRESSIBILITE_TANGENTE"]=COMPRESSIBILITE_TANGENTE;
      map_EnumTypeQuelconque["NB_INVERSION"]=NB_INVERSION;
      map_EnumTypeQuelconque["HYPER_CENTRE_HYSTERESIS"]=HYPER_CENTRE_HYSTERESIS; 
      map_EnumTypeQuelconque["SIGMA_REF"]=SIGMA_REF; 
      map_EnumTypeQuelconque["Q_SIG_HYST_Oi_A_R"]=Q_SIG_HYST_Oi_A_R; 
      map_EnumTypeQuelconque["Q_SIG_HYST_R_A_T"]=Q_SIG_HYST_R_A_T; 
      map_EnumTypeQuelconque["Q_DELTA_SIG_HYST"]=Q_DELTA_SIG_HYST; 
      map_EnumTypeQuelconque["COS_ALPHA_HYSTERESIS"]=COS_ALPHA_HYSTERESIS; 
      map_EnumTypeQuelconque["COS3PHI_SIG_HYSTERESIS"]=COS3PHI_SIG_HYSTERESIS; 
      map_EnumTypeQuelconque["COS3PHI_DELTA_SIG_HYSTERESIS"]=COS3PHI_DELTA_SIG_HYSTERESIS; 
      map_EnumTypeQuelconque["FCT_AIDE"]=FCT_AIDE;
      map_EnumTypeQuelconque["NB_ITER_TOTAL_RESIDU"]=NB_ITER_TOTAL_RESIDU;
      map_EnumTypeQuelconque["NB_INCRE_TOTAL_RESIDU"]=NB_INCRE_TOTAL_RESIDU;
      map_EnumTypeQuelconque["NB_APPEL_FCT"]=NB_APPEL_FCT;
      map_EnumTypeQuelconque["NB_STEP"]=NB_STEP;
      map_EnumTypeQuelconque["ERREUR_RK"]=ERREUR_RK;
      map_EnumTypeQuelconque["PRESSION_HYST_REF"]=PRESSION_HYST_REF;
      map_EnumTypeQuelconque["PRESSION_HYST_REF_M1"]=PRESSION_HYST_REF_M1;
      map_EnumTypeQuelconque["PRESSION_HYST_T"]=PRESSION_HYST_T;
      map_EnumTypeQuelconque["PRESSION_HYST"]=PRESSION_HYST;
      map_EnumTypeQuelconque["UN_DDL_ENUM_ETENDUE"]=UN_DDL_ENUM_ETENDUE;
      map_EnumTypeQuelconque["ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"]
                  =ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T;
      map_EnumTypeQuelconque["ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"]
                  =ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T;
      map_EnumTypeQuelconque["ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"]
                  =ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T;
      map_EnumTypeQuelconque["PROPORTION_LOI_MELANGE"]=PROPORTION_LOI_MELANGE; 
      map_EnumTypeQuelconque["FONC_PONDERATION"]=FONC_PONDERATION; 
      map_EnumTypeQuelconque["POSITION_GEOMETRIQUE"]=POSITION_GEOMETRIQUE; 
      map_EnumTypeQuelconque["POSITION_GEOMETRIQUE_t"]=POSITION_GEOMETRIQUE_t;
      map_EnumTypeQuelconque["POSITION_GEOMETRIQUE_t0"]=POSITION_GEOMETRIQUE_t0;
      map_EnumTypeQuelconque["CRISTALINITE"]=CRISTALINITE;
      map_EnumTypeQuelconque["VOLUME_ELEMENT"]=VOLUME_ELEMENT; 
      map_EnumTypeQuelconque["VOLUME_PTI"]=VOLUME_PTI;
      map_EnumTypeQuelconque["EPAISSEUR_MOY_INITIALE"]=EPAISSEUR_MOY_INITIALE;
      map_EnumTypeQuelconque["EPAISSEUR_MOY_FINALE"]=EPAISSEUR_MOY_FINALE; 
      map_EnumTypeQuelconque["SECTION_MOY_INITIALE"]=SECTION_MOY_INITIALE;
      map_EnumTypeQuelconque["SECTION_MOY_FINALE"]=SECTION_MOY_FINALE;
      map_EnumTypeQuelconque["EPAISSEUR_INITIALE"]=EPAISSEUR_INITIALE;
      map_EnumTypeQuelconque["EPAISSEUR_FINALE"]=EPAISSEUR_FINALE;
      map_EnumTypeQuelconque["SECTION_INITIALE"]=SECTION_INITIALE;
      map_EnumTypeQuelconque["SECTION_FINALE"]=SECTION_FINALE;
      map_EnumTypeQuelconque["VOL_ELEM_AVEC_PLAN_REF"]=VOL_ELEM_AVEC_PLAN_REF;
      map_EnumTypeQuelconque["INTEG_SUR_VOLUME"]=INTEG_SUR_VOLUME;
      map_EnumTypeQuelconque["INTEG_SUR_VOLUME_ET_TEMPS"]=INTEG_SUR_VOLUME_ET_TEMPS;
      map_EnumTypeQuelconque["STATISTIQUE"]=STATISTIQUE;
      map_EnumTypeQuelconque["STATISTIQUE_ET_TEMPS"]=STATISTIQUE_ET_TEMPS;
      map_EnumTypeQuelconque["ENERGIE_HOURGLASS"]=ENERGIE_HOURGLASS;
      map_EnumTypeQuelconque["PUISSANCE_BULK"]=PUISSANCE_BULK;
      map_EnumTypeQuelconque["ENERGIE_BULK"]=ENERGIE_BULK;
      map_EnumTypeQuelconque["ENERGIE_STABMEMB_BIEL"]=ENERGIE_STABMEMB_BIEL;
      map_EnumTypeQuelconque["FORCE_STABMEMB_BIEL"]=FORCE_STABMEMB_BIEL;
      map_EnumTypeQuelconque["TENSEUR_COURBURE"]=TENSEUR_COURBURE;
      map_EnumTypeQuelconque["COURBURES_PRINCIPALES"]=COURBURES_PRINCIPALES; 
      map_EnumTypeQuelconque["DIRECTIONS_PRINC_COURBURE"]=DIRECTIONS_PRINC_COURBURE; 
      map_EnumTypeQuelconque["DIRECTIONS_PRINC_SIGMA"]=DIRECTIONS_PRINC_SIGMA;
      map_EnumTypeQuelconque["DIRECTIONS_PRINC_DEF"]=DIRECTIONS_PRINC_DEF;
      map_EnumTypeQuelconque["DIRECTIONS_PRINC_D"]=DIRECTIONS_PRINC_D;
      map_EnumTypeQuelconque["REPERE_LOCAL_ORTHO"]=REPERE_LOCAL_ORTHO;
      map_EnumTypeQuelconque["REPERE_LOCAL_H"]=REPERE_LOCAL_H; 
      map_EnumTypeQuelconque["REPERE_LOCAL_B"]=REPERE_LOCAL_B; 		
      map_EnumTypeQuelconque["REPERE_D_ANISOTROPIE"]=REPERE_D_ANISOTROPIE;
      map_EnumTypeQuelconque["EPS_TRANSPORTEE_ANISO"]=EPS_TRANSPORTEE_ANISO;
      map_EnumTypeQuelconque["SIGMA_DANS_ANISO"]=SIGMA_DANS_ANISO;
      map_EnumTypeQuelconque["DELTA_EPS_TRANSPORTEE_ANISO"]=DELTA_EPS_TRANSPORTEE_ANISO;
      map_EnumTypeQuelconque["DELTA_SIGMA_DANS_ANISO"]=DELTA_SIGMA_DANS_ANISO;
      map_EnumTypeQuelconque["PARA_ORTHO"]=PARA_ORTHO;
      map_EnumTypeQuelconque["SPHERIQUE_EPS"]=SPHERIQUE_EPS;
      map_EnumTypeQuelconque["Q_EPS"]=Q_EPS;
      map_EnumTypeQuelconque["COS3PHI_EPS"]=COS3PHI_EPS;
      map_EnumTypeQuelconque["SPHERIQUE_SIG"]=SPHERIQUE_SIG;
      map_EnumTypeQuelconque["Q_SIG"]=Q_SIG;
      map_EnumTypeQuelconque["COS3PHI_SIG"]=COS3PHI_SIG;
      map_EnumTypeQuelconque["SPHERIQUE_DEPS"]=SPHERIQUE_DEPS;
      map_EnumTypeQuelconque["V_vol"]=V_vol;
      map_EnumTypeQuelconque["Q_DEPS"]=Q_DEPS;
      map_EnumTypeQuelconque["COS3PHI_DEPS"]=COS3PHI_DEPS;
      map_EnumTypeQuelconque["POTENTIEL"]=POTENTIEL;
      map_EnumTypeQuelconque["FCT_POTENTIEL_ND"]=FCT_POTENTIEL_ND;
      map_EnumTypeQuelconque["INVAR_B1"]=INVAR_B1;
      map_EnumTypeQuelconque["INVAR_B2"]=INVAR_B2;
      map_EnumTypeQuelconque["INVAR_B3"]=INVAR_B3;
      map_EnumTypeQuelconque["INVAR_J1"]=INVAR_J1;
      map_EnumTypeQuelconque["INVAR_J2"]=INVAR_J2;
      map_EnumTypeQuelconque["INVAR_J3"]=INVAR_J3;
      map_EnumTypeQuelconque["DEF_EQUIVALENTE"]=DEF_EQUIVALENTE;
      map_EnumTypeQuelconque["DEF_EPAISSEUR"]=DEF_EPAISSEUR;
      map_EnumTypeQuelconque["D_EPAISSEUR"]=D_EPAISSEUR;
      map_EnumTypeQuelconque["DEF_LARGEUR"]=DEF_LARGEUR;
      map_EnumTypeQuelconque["D_LARGEUR"]=D_LARGEUR;
      map_EnumTypeQuelconque["DEF_MECANIQUE"]=DEF_MECANIQUE;
      map_EnumTypeQuelconque["DEF_ASSO_LOI"]=DEF_ASSO_LOI;
      map_EnumTypeQuelconque["DEF_P_DANS_V_A"]=DEF_P_DANS_V_A;
      map_EnumTypeQuelconque["SIG_EPAISSEUR"]=SIG_EPAISSEUR;
      map_EnumTypeQuelconque["SIG_LARGEUR"]=SIG_LARGEUR;
      map_EnumTypeQuelconque["FORCE_GENE_EXT"]=FORCE_GENE_EXT;
      map_EnumTypeQuelconque["FORCE_GENE_INT"]=FORCE_GENE_INT;
      map_EnumTypeQuelconque["FORCE_GENE_EXT_t"]=FORCE_GENE_EXT_t;
      map_EnumTypeQuelconque["FORCE_GENE_INT_t"]=FORCE_GENE_INT_t;
      map_EnumTypeQuelconque["FORCE_GENE_TOT"]=FORCE_GENE_TOT;
      map_EnumTypeQuelconque["RESIDU_GLOBAL"]=RESIDU_GLOBAL;
      map_EnumTypeQuelconque["VECT_PRESSION"]=VECT_PRESSION;
      map_EnumTypeQuelconque["PRESSION_SCALAIRE"]=PRESSION_SCALAIRE;
      map_EnumTypeQuelconque["VECT_FORCE_VOLUM"]=VECT_FORCE_VOLUM;
      map_EnumTypeQuelconque["VECT_DIR_FIXE"]=VECT_DIR_FIXE;
      map_EnumTypeQuelconque["VECT_SURF_SUIV"]=VECT_SURF_SUIV;
      map_EnumTypeQuelconque["VECT_HYDRODYNA_Fn"]=VECT_HYDRODYNA_Fn;
      map_EnumTypeQuelconque["VECT_HYDRODYNA_Ft"]=VECT_HYDRODYNA_Ft;
      map_EnumTypeQuelconque["VECT_HYDRODYNA_T"]=VECT_HYDRODYNA_T;
      map_EnumTypeQuelconque["VECT_LINE"]=VECT_LINE;
      map_EnumTypeQuelconque["VECT_LINE_SUIV"]=VECT_LINE_SUIV;
      map_EnumTypeQuelconque["VECT_REAC"]=VECT_REAC;
      map_EnumTypeQuelconque["VECT_REAC_N"]=VECT_REAC_N;
      map_EnumTypeQuelconque["NN_11"]=NN_11;
      map_EnumTypeQuelconque["NN_22"]=NN_22;
      map_EnumTypeQuelconque["NN_33"]=NN_33;
      map_EnumTypeQuelconque["NN_12"]=NN_12;
      map_EnumTypeQuelconque["NN_13"]=NN_13;
      map_EnumTypeQuelconque["NN_23"]=NN_23;
      map_EnumTypeQuelconque["MM_11"]=MM_11;
      map_EnumTypeQuelconque["MM_22"]=MM_22;
      map_EnumTypeQuelconque["MM_33"]=MM_33;
      map_EnumTypeQuelconque["MM_12"]=MM_12;
      map_EnumTypeQuelconque["MM_13"]=MM_13;
      map_EnumTypeQuelconque["MM_23"]=MM_23;
      map_EnumTypeQuelconque["DIRECTION_PLI"]=DIRECTION_PLI;
      map_EnumTypeQuelconque["DIRECTION_PLI_NORMEE"]=DIRECTION_PLI_NORMEE;
      map_EnumTypeQuelconque["INDIC_CAL_PLIS"]=INDIC_CAL_PLIS;
      map_EnumTypeQuelconque["NN_SURF"]=NN_SURF;
      map_EnumTypeQuelconque["NN_SURF_t"]=NN_SURF_t;
      map_EnumTypeQuelconque["NN_SURF_t0"]=NN_SURF_t0;
      map_EnumTypeQuelconque["NOEUD_PROJECTILE_EN_CONTACT"]=NOEUD_PROJECTILE_EN_CONTACT;
      map_EnumTypeQuelconque["NOEUD_FACETTE_EN_CONTACT"]=NOEUD_FACETTE_EN_CONTACT;
      map_EnumTypeQuelconque["GLISSEMENT_CONTACT"]=GLISSEMENT_CONTACT;
      map_EnumTypeQuelconque["PENETRATION_CONTACT"]=PENETRATION_CONTACT;
      map_EnumTypeQuelconque["GLISSEMENT_CONTACT_T"]=GLISSEMENT_CONTACT_T;
      map_EnumTypeQuelconque["PENETRATION_CONTACT_T"]=PENETRATION_CONTACT_T;
      map_EnumTypeQuelconque["FORCE_CONTACT"]=FORCE_CONTACT;
      map_EnumTypeQuelconque["FORCE_CONTACT_T"]=FORCE_CONTACT_T;
      map_EnumTypeQuelconque["CONTACT_NB_PENET"]=CONTACT_NB_PENET;
      map_EnumTypeQuelconque["CONTACT_NB_DECOL"]=CONTACT_NB_DECOL;
      map_EnumTypeQuelconque["CONTACT_CAS_SOLIDE"]=CONTACT_CAS_SOLIDE;
      map_EnumTypeQuelconque["CONTACT_ENERG_PENAL"]=CONTACT_ENERG_PENAL;
      map_EnumTypeQuelconque["CONTACT_COLLANT"]=CONTACT_COLLANT;
      map_EnumTypeQuelconque["NUM_ZONE_CONTACT"]=NUM_ZONE_CONTACT;
      map_EnumTypeQuelconque["CONTACT_ENERG_GLISSE_ELAS"]=CONTACT_ENERG_GLISSE_ELAS;
      map_EnumTypeQuelconque["CONTACT_ENERG_GLISSE_PLAS"]=CONTACT_ENERG_GLISSE_PLAS;
      map_EnumTypeQuelconque["CONTACT_ENERG_GLISSE_VISQ"]=CONTACT_ENERG_GLISSE_VISQ;
      map_EnumTypeQuelconque["CONTACT_PENALISATION_N"]=CONTACT_PENALISATION_N;
      map_EnumTypeQuelconque["CONTACT_PENALISATION_T"]=CONTACT_PENALISATION_T;
      map_EnumTypeQuelconque["NORMALE_CONTACT"]=NORMALE_CONTACT;
      map_EnumTypeQuelconque["TEMPS_CPU_USER"]=TEMPS_CPU_USER;
      map_EnumTypeQuelconque["TEMPS_CPU_LOI_COMP"]=TEMPS_CPU_LOI_COMP;
      map_EnumTypeQuelconque["TEMPS_CPU_METRIQUE"]=TEMPS_CPU_METRIQUE;
      map_EnumTypeQuelconque["GENERIQUE_UNE_GRANDEUR_GLOBALE"]=GENERIQUE_UNE_GRANDEUR_GLOBALE;
      map_EnumTypeQuelconque["GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR"]=GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR;
      map_EnumTypeQuelconque["GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR"]=GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR;
      map_EnumTypeQuelconque["GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0"]=GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0;
      map_EnumTypeQuelconque["GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0"]=GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0;

      map_EnumTypeQuelconque["GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T"]=GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T;
      map_EnumTypeQuelconque["GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT"]=GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT;
      map_EnumTypeQuelconque["DEPLACEMENT"]=DEPLACEMENT;
      map_EnumTypeQuelconque["VITESSE"]=VITESSE;
      map_EnumTypeQuelconque["DELTA_XI"]=DELTA_XI;
      map_EnumTypeQuelconque["XI_ITER_0"]=XI_ITER_0;
      map_EnumTypeQuelconque["MASSE_RELAX_DYN"]=MASSE_RELAX_DYN;
      map_EnumTypeQuelconque["COMP_TORSEUR_REACTION"]=COMP_TORSEUR_REACTION;
      map_EnumTypeQuelconque["NUM_NOEUD"]=NUM_NOEUD;
      map_EnumTypeQuelconque["NUM_MAIL_NOEUD"]=NUM_MAIL_NOEUD;
      map_EnumTypeQuelconque["NUM_ELEMENT"]=NUM_ELEMENT;
      map_EnumTypeQuelconque["NUM_MAIL_ELEM"]=NUM_MAIL_ELEM;
      map_EnumTypeQuelconque["NUM_PTI"]=NUM_PTI;
      map_EnumTypeQuelconque["NUM_FACE"]=NUM_FACE;
      map_EnumTypeQuelconque["NUM_ARETE"]=NUM_ARETE;

      // remplissage de tt_GLOB
      tt_GLOB.Change_taille(218); // ****** ne pas oublier de redéfinir la taille si l'on ajoute des termes enum
      tt_GLOB(RIEN_TYPEQUELCONQUE)=1;
      tt_GLOB(SIGMA_BARRE_BH_T)=0;
      tt_GLOB(CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION)=0;
      tt_GLOB(CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=0;
      tt_GLOB(CONTRAINTE_COURANTE)=1;
      tt_GLOB(DEFORMATION_COURANTE)=1;
      tt_GLOB(VITESSE_DEFORMATION_COURANTE)=1;
      tt_GLOB(ALMANSI)=1; 
      tt_GLOB(GREEN_LAGRANGE)=1; 
      tt_GLOB(LOGARITHMIQUE)=1; 
      tt_GLOB(DELTA_DEF)=1; 
      tt_GLOB(ALMANSI_TOTAL)=1; 
      tt_GLOB(GREEN_LAGRANGE_TOTAL)=1; 
      tt_GLOB(LOGARITHMIQUE_TOTALE)=1; 
      tt_GLOB(DEF_PRINCIPALES)=1; 
      tt_GLOB(SIGMA_PRINCIPALES)=1; 
      tt_GLOB(VIT_PRINCIPALES)=1; 
      tt_GLOB(DEF_DUALE_MISES)=1; 
      tt_GLOB(DEF_DUALE_MISES_MAXI)=1; 
      tt_GLOB(CONTRAINTE_MISES)=1; 
      tt_GLOB(CONTRAINTE_MISES_T)=1;
      tt_GLOB(CONTRAINTE_TRESCA)=1;
      tt_GLOB(CONTRAINTE_TRESCA_T)=1;
      tt_GLOB(ERREUR_Q)=1;
      tt_GLOB(DEF_PLASTIQUE_CUMULEE)=1; 
      tt_GLOB(ERREUR_SIG_RELATIVE)=1;
      tt_GLOB(TEMPERATURE_LOI_THERMO_PHYSIQUE)=1;
      tt_GLOB(PRESSION_LOI_THERMO_PHYSIQUE)=1;
      tt_GLOB(TEMPERATURE_TRANSITION)=1;
      tt_GLOB(VOLUME_SPECIFIQUE)=1; 
      tt_GLOB(FLUXD)=1;
      tt_GLOB(GRADT)=1;
      tt_GLOB(DGRADT)=1;
      tt_GLOB(DELTAGRADT)=1;
      tt_GLOB(COEFF_DILATATION_LINEAIRE)=1;
      tt_GLOB(CONDUCTIVITE)=1;
      tt_GLOB(CAPACITE_CALORIFIQUE)=1; 
      tt_GLOB(MODULE_COMPRESSIBILITE)=1;
      tt_GLOB(COEFF_COMPRESSIBILITE)=1;
      tt_GLOB(MODULE_CISAILLEMENT)=1;
      tt_GLOB(MODULE_COMPRESSIBILITE_TOTAL)=1;
      tt_GLOB(MODULE_CISAILLEMENT_TOTAL)=1;
      tt_GLOB(E_YOUNG)=1;
      tt_GLOB(NU_YOUNG)=1;
      tt_GLOB(MU_VISCO)=1;
      tt_GLOB(MU_VISCO_SPHERIQUE)=1;
      tt_GLOB(MODULE_TANGENT_1D)=1;
      tt_GLOB(COMPRESSIBILITE_TANGENTE)=1;
      tt_GLOB(NB_INVERSION)=1;
      tt_GLOB(HYPER_CENTRE_HYSTERESIS)=0;
      tt_GLOB(SIGMA_REF)=0;
      tt_GLOB(Q_SIG_HYST_Oi_A_R)=1;
      tt_GLOB(Q_SIG_HYST_R_A_T)=1;
      tt_GLOB(Q_DELTA_SIG_HYST)=1;
      tt_GLOB(COS_ALPHA_HYSTERESIS)=1;
      tt_GLOB(COS3PHI_SIG_HYSTERESIS)=1;
      tt_GLOB(COS3PHI_DELTA_SIG_HYSTERESIS)=1;
      tt_GLOB(FCT_AIDE)=1;
      tt_GLOB(NB_ITER_TOTAL_RESIDU)=1;
      tt_GLOB(NB_INCRE_TOTAL_RESIDU)=1;
      tt_GLOB(NB_APPEL_FCT)=1;
      tt_GLOB(NB_STEP)=1;
      tt_GLOB(ERREUR_RK)=1;
      tt_GLOB(PRESSION_HYST_REF)=1;
      tt_GLOB(PRESSION_HYST_REF_M1)=1;
      tt_GLOB(PRESSION_HYST_T)=1;
      tt_GLOB(PRESSION_HYST)=1;
      tt_GLOB(UN_DDL_ENUM_ETENDUE)=1;
      tt_GLOB(ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=1;
      tt_GLOB(ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=1;
      tt_GLOB(ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=1;
      tt_GLOB(PROPORTION_LOI_MELANGE)=1;
      tt_GLOB(FONC_PONDERATION)=1;
      tt_GLOB(POSITION_GEOMETRIQUE)=1;
      tt_GLOB(POSITION_GEOMETRIQUE_t)=1;
      tt_GLOB(POSITION_GEOMETRIQUE_t0)=1;
      tt_GLOB(CRISTALINITE)=1;
      tt_GLOB(VOLUME_ELEMENT)=1;
      tt_GLOB(VOLUME_PTI)=1;
      tt_GLOB(EPAISSEUR_MOY_INITIALE)=1;
      tt_GLOB(EPAISSEUR_MOY_FINALE)=1;
      tt_GLOB(SECTION_MOY_INITIALE)=1;
      tt_GLOB(SECTION_MOY_FINALE)=1;
      tt_GLOB(EPAISSEUR_INITIALE)=1;
      tt_GLOB(EPAISSEUR_FINALE)=1;
      tt_GLOB(SECTION_INITIALE)=1;
      tt_GLOB(SECTION_FINALE)=1;
      tt_GLOB(VOL_ELEM_AVEC_PLAN_REF)=1;
      tt_GLOB(INTEG_SUR_VOLUME)=1;
      tt_GLOB(INTEG_SUR_VOLUME_ET_TEMPS)=1;
      tt_GLOB(STATISTIQUE)=1;
      tt_GLOB(STATISTIQUE_ET_TEMPS)=1;
      tt_GLOB(ENERGIE_HOURGLASS)=1;
      tt_GLOB(PUISSANCE_BULK)=1;
      tt_GLOB(ENERGIE_BULK)=1;
      tt_GLOB(ENERGIE_STABMEMB_BIEL)=1;
      tt_GLOB(FORCE_STABMEMB_BIEL)=1;
      tt_GLOB(TENSEUR_COURBURE)=0;
      tt_GLOB(COURBURES_PRINCIPALES)=1;
      tt_GLOB(DIRECTIONS_PRINC_COURBURE)=0;
      tt_GLOB(DIRECTIONS_PRINC_SIGMA)=1;
      tt_GLOB(DIRECTIONS_PRINC_DEF)=1;
      tt_GLOB(DIRECTIONS_PRINC_D)=1;
      tt_GLOB(REPERE_LOCAL_ORTHO)=1;
      tt_GLOB(REPERE_LOCAL_H)=1;
      tt_GLOB(REPERE_LOCAL_B)=1;
      tt_GLOB(REPERE_D_ANISOTROPIE)=0;
      tt_GLOB(EPS_TRANSPORTEE_ANISO)=1;
      tt_GLOB(SIGMA_DANS_ANISO)=1;
      tt_GLOB(DELTA_EPS_TRANSPORTEE_ANISO)=1;
      tt_GLOB(DELTA_SIGMA_DANS_ANISO)=1;
      tt_GLOB(PARA_ORTHO)=1;
      tt_GLOB(SPHERIQUE_EPS)=1;
      tt_GLOB(Q_EPS)=1;
      tt_GLOB(COS3PHI_EPS)=1;
      tt_GLOB(SPHERIQUE_SIG)=1;
      tt_GLOB(Q_SIG)=1;
      tt_GLOB(COS3PHI_SIG)=1;
      tt_GLOB(SPHERIQUE_DEPS)=1;
      tt_GLOB(V_vol)=1;
      tt_GLOB(Q_DEPS)=1;
      tt_GLOB(COS3PHI_DEPS)=1;
      tt_GLOB(POTENTIEL)=1;
      tt_GLOB(FCT_POTENTIEL_ND)=1;
      tt_GLOB(INVAR_B1)=1;
      tt_GLOB(INVAR_B2)=1;
      tt_GLOB(INVAR_B3)=1;
      tt_GLOB(INVAR_J1)=1;
      tt_GLOB(INVAR_J2)=1;
      tt_GLOB(INVAR_J3)=1;
      tt_GLOB(DEF_EQUIVALENTE)=1;
      tt_GLOB(DEF_EPAISSEUR)=1;
      tt_GLOB(D_EPAISSEUR)=1;
      tt_GLOB(DEF_LARGEUR)=1;
      tt_GLOB(D_LARGEUR)=1;
      tt_GLOB(DEF_MECANIQUE)=1;
      tt_GLOB(DEF_ASSO_LOI)=0;
      tt_GLOB(DEF_P_DANS_V_A)=1;
      tt_GLOB(SIG_EPAISSEUR)=1;
      tt_GLOB(SIG_LARGEUR)=1;
      tt_GLOB(FORCE_GENE_EXT)=1;
      tt_GLOB(FORCE_GENE_INT)=1;
      tt_GLOB(FORCE_GENE_EXT_t)=1;
      tt_GLOB(FORCE_GENE_INT_t)=1;
      tt_GLOB(FORCE_GENE_TOT)=1;
      tt_GLOB(RESIDU_GLOBAL)=1;
      tt_GLOB(VECT_PRESSION)=1;
      tt_GLOB(PRESSION_SCALAIRE)=1;
      tt_GLOB(VECT_FORCE_VOLUM)=1;
      tt_GLOB(VECT_DIR_FIXE)=1;
      tt_GLOB(VECT_SURF_SUIV)=1;
      tt_GLOB(VECT_HYDRODYNA_Fn)=1;
      tt_GLOB(VECT_HYDRODYNA_Ft)=1;
      tt_GLOB(VECT_HYDRODYNA_T)=1;
      tt_GLOB(VECT_LINE)=1;
      tt_GLOB(VECT_LINE_SUIV)=1;
      tt_GLOB(VECT_REAC)=1;
      tt_GLOB(VECT_REAC_N)=1;
      tt_GLOB(NN_11)=1;
      tt_GLOB(NN_22)=1;
      tt_GLOB(NN_33)=1;
      tt_GLOB(NN_12)=1;
      tt_GLOB(NN_13)=1;
      tt_GLOB(NN_23)=1;
      tt_GLOB(MM_11)=1;
      tt_GLOB(MM_22)=1;
      tt_GLOB(MM_33)=1;
      tt_GLOB(MM_12)=1;
      tt_GLOB(MM_13)=1;
      tt_GLOB(MM_23)=1;
      tt_GLOB(DIRECTION_PLI)=1;
      tt_GLOB(DIRECTION_PLI_NORMEE)=1;
      tt_GLOB(INDIC_CAL_PLIS)=1;
      tt_GLOB(NN_SURF)=1;
      tt_GLOB(NN_SURF_t)=1;
      tt_GLOB(NN_SURF_t0)=1;
      tt_GLOB(NOEUD_PROJECTILE_EN_CONTACT)=1;
      tt_GLOB(NOEUD_FACETTE_EN_CONTACT)=1;
      tt_GLOB(GLISSEMENT_CONTACT)=1;
      tt_GLOB(PENETRATION_CONTACT)=1;
      tt_GLOB(GLISSEMENT_CONTACT_T)=1;
      tt_GLOB(PENETRATION_CONTACT_T)=1;
      tt_GLOB(FORCE_CONTACT)=1;
      tt_GLOB(FORCE_CONTACT_T)=1;
      tt_GLOB(CONTACT_NB_PENET)=1;
      tt_GLOB(CONTACT_NB_DECOL)=1;
      tt_GLOB(CONTACT_CAS_SOLIDE)=1;
      tt_GLOB(CONTACT_ENERG_PENAL)=1;
      tt_GLOB(CONTACT_COLLANT)=1;
      tt_GLOB(NUM_ZONE_CONTACT)=1;
      tt_GLOB(CONTACT_ENERG_GLISSE_ELAS)=1;
      tt_GLOB(CONTACT_ENERG_GLISSE_PLAS)=1;
      tt_GLOB(CONTACT_ENERG_GLISSE_VISQ)=1;
      tt_GLOB(CONTACT_PENALISATION_N)=1;
      tt_GLOB(CONTACT_PENALISATION_T)=1;
      tt_GLOB(NORMALE_CONTACT)=1;
      tt_GLOB(TEMPS_CPU_USER)=1;
      tt_GLOB(TEMPS_CPU_LOI_COMP)=1;
      tt_GLOB(TEMPS_CPU_METRIQUE)=1;
      tt_GLOB(GENERIQUE_UNE_GRANDEUR_GLOBALE)=1;
      tt_GLOB(GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR)=1;
      tt_GLOB(GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR)=1;
      tt_GLOB(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0)=1;
      tt_GLOB(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T)=1;
      tt_GLOB(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT)=1;
      tt_GLOB(DEPLACEMENT)=1;
      tt_GLOB(VITESSE)=1;
      tt_GLOB(DELTA_XI)=1;
      tt_GLOB(XI_ITER_0)=1;
      tt_GLOB(MASSE_RELAX_DYN)=1;
      tt_GLOB(COMP_TORSEUR_REACTION)=1;
      tt_GLOB(NUM_NOEUD)=1;
      tt_GLOB(NUM_MAIL_NOEUD)=1;
      tt_GLOB(NUM_ELEMENT)=1;
      tt_GLOB(NUM_MAIL_ELEM)=1;
      tt_GLOB(NUM_PTI)=1;
      tt_GLOB(NUM_FACE)=1;
      tt_GLOB(NUM_ARETE)=1;

	  // remplissage de tt_TQ_temps
      tt_TQ_temps.Change_taille(218); // ****** ne pas oublier de redéfinir la taille si l'on ajoute des termes enum
      tt_TQ_temps(RIEN_TYPEQUELCONQUE)=TEMPS_0;
      tt_TQ_temps(SIGMA_BARRE_BH_T)=TEMPS_t;
      tt_TQ_temps(CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION)=TEMPS_tdt;
      tt_TQ_temps(CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=TEMPS_tdt;
      tt_TQ_temps(CONTRAINTE_COURANTE)=TEMPS_tdt;
      tt_TQ_temps(DEFORMATION_COURANTE)=TEMPS_tdt;
      tt_TQ_temps(VITESSE_DEFORMATION_COURANTE)=TEMPS_tdt;
      tt_TQ_temps(ALMANSI)=TEMPS_tdt;
      tt_TQ_temps(GREEN_LAGRANGE)=TEMPS_tdt; 
      tt_TQ_temps(LOGARITHMIQUE)=TEMPS_tdt;
      tt_TQ_temps(DELTA_DEF)=TEMPS_tdt;
      tt_TQ_temps(ALMANSI_TOTAL)=TEMPS_tdt;
      tt_TQ_temps(GREEN_LAGRANGE_TOTAL)=TEMPS_tdt;
      tt_TQ_temps(LOGARITHMIQUE_TOTALE)=TEMPS_tdt;
      tt_TQ_temps(DEF_PRINCIPALES)=TEMPS_tdt;
      tt_TQ_temps(SIGMA_PRINCIPALES)=TEMPS_tdt;
      tt_TQ_temps(VIT_PRINCIPALES)=TEMPS_tdt;
      tt_TQ_temps(DEF_DUALE_MISES)=TEMPS_tdt;
      tt_TQ_temps(DEF_DUALE_MISES_MAXI)=TEMPS_tdt;
      tt_TQ_temps(CONTRAINTE_MISES)=TEMPS_tdt;
      tt_TQ_temps(CONTRAINTE_MISES_T)=TEMPS_t;
      tt_TQ_temps(CONTRAINTE_TRESCA)=TEMPS_tdt;
      tt_TQ_temps(CONTRAINTE_TRESCA_T)=TEMPS_t;
      tt_TQ_temps(ERREUR_Q)=TEMPS_tdt;
      tt_TQ_temps(DEF_PLASTIQUE_CUMULEE)=TEMPS_tdt;
      tt_TQ_temps(ERREUR_SIG_RELATIVE)=TEMPS_tdt;
      tt_TQ_temps(TEMPERATURE_LOI_THERMO_PHYSIQUE)=TEMPS_tdt;
      tt_TQ_temps(PRESSION_LOI_THERMO_PHYSIQUE)=TEMPS_tdt;
      tt_TQ_temps(TEMPERATURE_TRANSITION)=TEMPS_tdt;
      tt_TQ_temps(VOLUME_SPECIFIQUE)=TEMPS_tdt; 
      tt_TQ_temps(FLUXD)=TEMPS_tdt;
      tt_TQ_temps(GRADT)=TEMPS_tdt;
      tt_TQ_temps(DGRADT)=TEMPS_tdt;
      tt_TQ_temps(DELTAGRADT)=TEMPS_tdt;
      tt_TQ_temps(COEFF_DILATATION_LINEAIRE)=TEMPS_tdt;
      tt_TQ_temps(CONDUCTIVITE)=TEMPS_tdt;
      tt_TQ_temps(CAPACITE_CALORIFIQUE)=TEMPS_tdt; 
      tt_TQ_temps(MODULE_COMPRESSIBILITE)=TEMPS_tdt;
      tt_TQ_temps(COEFF_COMPRESSIBILITE)=TEMPS_tdt;
      tt_TQ_temps(MODULE_CISAILLEMENT)=TEMPS_tdt;
      tt_TQ_temps(MODULE_COMPRESSIBILITE_TOTAL)=TEMPS_tdt;
      tt_TQ_temps(MODULE_CISAILLEMENT_TOTAL)=TEMPS_tdt;
      tt_TQ_temps(E_YOUNG)=TEMPS_tdt;
      tt_TQ_temps(NU_YOUNG)=TEMPS_tdt;
      tt_TQ_temps(MU_VISCO)=TEMPS_tdt;
      tt_TQ_temps(MU_VISCO_SPHERIQUE)=TEMPS_tdt;
      tt_TQ_temps(MODULE_TANGENT_1D)=TEMPS_tdt;
      tt_TQ_temps(COMPRESSIBILITE_TANGENTE)=TEMPS_tdt;
      tt_TQ_temps(NB_INVERSION)=TEMPS_tdt;
      tt_TQ_temps(HYPER_CENTRE_HYSTERESIS)=TEMPS_tdt;
      tt_TQ_temps(SIGMA_REF)=TEMPS_tdt;
      tt_TQ_temps(Q_SIG_HYST_Oi_A_R)=TEMPS_tdt;
      tt_TQ_temps(Q_SIG_HYST_R_A_T)=TEMPS_tdt;
      tt_TQ_temps(Q_DELTA_SIG_HYST)=TEMPS_tdt;
      tt_TQ_temps(COS_ALPHA_HYSTERESIS)=TEMPS_tdt;
      tt_TQ_temps(COS3PHI_SIG_HYSTERESIS)=TEMPS_tdt;
      tt_TQ_temps(COS3PHI_DELTA_SIG_HYSTERESIS)=TEMPS_tdt;
      tt_TQ_temps(FCT_AIDE)=TEMPS_tdt;
      tt_TQ_temps(NB_ITER_TOTAL_RESIDU)=TEMPS_tdt;
      tt_TQ_temps(NB_INCRE_TOTAL_RESIDU)=TEMPS_tdt;
      tt_TQ_temps(NB_APPEL_FCT)=TEMPS_tdt;
      tt_TQ_temps(NB_STEP)=TEMPS_tdt;
      tt_TQ_temps(ERREUR_RK)=TEMPS_tdt;
      tt_TQ_temps(PRESSION_HYST_REF)=TEMPS_tdt;
      tt_TQ_temps(PRESSION_HYST_REF_M1)=TEMPS_t;
      tt_TQ_temps(PRESSION_HYST_T)=TEMPS_t;
      tt_TQ_temps(PRESSION_HYST)=TEMPS_tdt;
      tt_TQ_temps(UN_DDL_ENUM_ETENDUE)=TEMPS_tdt;
      tt_TQ_temps(ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=TEMPS_t;
      tt_TQ_temps(ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=TEMPS_t;
      tt_TQ_temps(ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T)=TEMPS_t;
      tt_TQ_temps(PROPORTION_LOI_MELANGE)=TEMPS_tdt;
      tt_TQ_temps(FONC_PONDERATION)=TEMPS_tdt;
      tt_TQ_temps(POSITION_GEOMETRIQUE)=TEMPS_tdt;
      tt_TQ_temps(POSITION_GEOMETRIQUE_t)=TEMPS_t;
      tt_TQ_temps(POSITION_GEOMETRIQUE_t0)=TEMPS_0;
      tt_TQ_temps(CRISTALINITE)=TEMPS_tdt;
      tt_TQ_temps(VOLUME_ELEMENT)=TEMPS_tdt;
      tt_TQ_temps(VOLUME_PTI)=TEMPS_tdt;
      tt_TQ_temps(EPAISSEUR_MOY_INITIALE)=TEMPS_0;
      tt_TQ_temps(EPAISSEUR_MOY_FINALE)=TEMPS_tdt;
      tt_TQ_temps(SECTION_MOY_INITIALE)=TEMPS_0;
      tt_TQ_temps(SECTION_MOY_FINALE)=TEMPS_tdt;
      tt_TQ_temps(EPAISSEUR_INITIALE)=TEMPS_0;
      tt_TQ_temps(EPAISSEUR_FINALE)=TEMPS_tdt;
      tt_TQ_temps(SECTION_INITIALE)=TEMPS_0;
      tt_TQ_temps(SECTION_FINALE)=TEMPS_tdt;
      tt_TQ_temps(VOL_ELEM_AVEC_PLAN_REF)=TEMPS_tdt;
      tt_TQ_temps(INTEG_SUR_VOLUME)=TEMPS_tdt;
      tt_TQ_temps(INTEG_SUR_VOLUME_ET_TEMPS)=TEMPS_tdt;
      tt_TQ_temps(STATISTIQUE)=TEMPS_tdt;
      tt_TQ_temps(STATISTIQUE_ET_TEMPS)=TEMPS_tdt;
      tt_TQ_temps(ENERGIE_HOURGLASS)=TEMPS_tdt;
      tt_TQ_temps(PUISSANCE_BULK)=TEMPS_tdt;
      tt_TQ_temps(ENERGIE_BULK)=TEMPS_tdt;
      tt_TQ_temps(ENERGIE_STABMEMB_BIEL)=TEMPS_tdt;
      tt_TQ_temps(FORCE_STABMEMB_BIEL)=TEMPS_tdt;
      tt_TQ_temps(TENSEUR_COURBURE)=TEMPS_tdt;
      tt_TQ_temps(COURBURES_PRINCIPALES)=TEMPS_tdt;
      tt_TQ_temps(DIRECTIONS_PRINC_COURBURE)=TEMPS_tdt;
      tt_TQ_temps(DIRECTIONS_PRINC_SIGMA)=TEMPS_tdt;
      tt_TQ_temps(DIRECTIONS_PRINC_DEF)=TEMPS_tdt;
      tt_TQ_temps(DIRECTIONS_PRINC_D)=TEMPS_tdt;
      tt_TQ_temps(REPERE_LOCAL_ORTHO)=TEMPS_tdt;
      tt_TQ_temps(REPERE_LOCAL_H)=TEMPS_tdt;
      tt_TQ_temps(REPERE_LOCAL_B)=TEMPS_tdt;
      tt_TQ_temps(REPERE_D_ANISOTROPIE)=TEMPS_tdt;
      tt_TQ_temps(EPS_TRANSPORTEE_ANISO)=TEMPS_tdt;
      tt_TQ_temps(SIGMA_DANS_ANISO)=TEMPS_tdt;
      tt_TQ_temps(DELTA_EPS_TRANSPORTEE_ANISO)=TEMPS_tdt;
      tt_TQ_temps(DELTA_SIGMA_DANS_ANISO)=TEMPS_tdt;
      tt_TQ_temps(PARA_ORTHO)=TEMPS_tdt;
      tt_TQ_temps(SPHERIQUE_EPS)=TEMPS_tdt;
  	   tt_TQ_temps(Q_EPS)=TEMPS_tdt;
      tt_TQ_temps(COS3PHI_EPS)=TEMPS_tdt;
      tt_TQ_temps(SPHERIQUE_SIG)=TEMPS_tdt;
      tt_TQ_temps(Q_SIG)=TEMPS_tdt;
      tt_TQ_temps(COS3PHI_SIG)=TEMPS_tdt;
      tt_TQ_temps(SPHERIQUE_DEPS)=TEMPS_tdt;
      tt_TQ_temps(V_vol)=TEMPS_tdt;
      tt_TQ_temps(Q_DEPS)=TEMPS_tdt;
      tt_TQ_temps(COS3PHI_DEPS)=TEMPS_tdt;
      tt_TQ_temps(POTENTIEL)=TEMPS_tdt;
      tt_TQ_temps(FCT_POTENTIEL_ND)=TEMPS_tdt;
      tt_TQ_temps(INVAR_B1)=TEMPS_tdt;
      tt_TQ_temps(INVAR_B2)=TEMPS_tdt;
      tt_TQ_temps(INVAR_B3)=TEMPS_tdt;
      tt_TQ_temps(INVAR_J1)=TEMPS_tdt;
      tt_TQ_temps(INVAR_J2)=TEMPS_tdt;
      tt_TQ_temps(INVAR_J3)=TEMPS_tdt;
      tt_TQ_temps(DEF_EQUIVALENTE)=TEMPS_tdt;
      tt_TQ_temps(DEF_EPAISSEUR)=TEMPS_tdt;
      tt_TQ_temps(D_EPAISSEUR)=TEMPS_tdt;
      tt_TQ_temps(DEF_LARGEUR)=TEMPS_tdt;
      tt_TQ_temps(D_LARGEUR)=TEMPS_tdt;
      tt_TQ_temps(DEF_MECANIQUE)=TEMPS_tdt;
      tt_TQ_temps(DEF_ASSO_LOI)=TEMPS_tdt;
      tt_TQ_temps(DEF_P_DANS_V_A)=TEMPS_tdt;
      tt_TQ_temps(SIG_EPAISSEUR)=TEMPS_tdt;
      tt_TQ_temps(SIG_LARGEUR)=TEMPS_tdt;
      tt_TQ_temps(FORCE_GENE_EXT)=TEMPS_tdt;
      tt_TQ_temps(FORCE_GENE_INT)=TEMPS_tdt;
      tt_TQ_temps(FORCE_GENE_EXT_t)=TEMPS_t;
      tt_TQ_temps(FORCE_GENE_INT_t)=TEMPS_t;
      tt_TQ_temps(FORCE_GENE_TOT)=TEMPS_tdt;
      tt_TQ_temps(RESIDU_GLOBAL)=TEMPS_tdt;
      tt_TQ_temps(VECT_PRESSION)=TEMPS_tdt;
      tt_TQ_temps(PRESSION_SCALAIRE)=TEMPS_tdt;
      tt_TQ_temps(VECT_FORCE_VOLUM)=TEMPS_tdt;
      tt_TQ_temps(VECT_DIR_FIXE)=TEMPS_tdt;
      tt_TQ_temps(VECT_SURF_SUIV)=TEMPS_tdt;
      tt_TQ_temps(VECT_HYDRODYNA_Fn)=TEMPS_tdt;
      tt_TQ_temps(VECT_HYDRODYNA_Ft)=TEMPS_tdt;
      tt_TQ_temps(VECT_HYDRODYNA_T)=TEMPS_tdt;
      tt_TQ_temps(VECT_LINE)=TEMPS_tdt;
      tt_TQ_temps(VECT_LINE_SUIV)=TEMPS_tdt;
      tt_TQ_temps(VECT_REAC)=TEMPS_tdt;
      tt_TQ_temps(VECT_REAC_N)=TEMPS_tdt;
      tt_TQ_temps(NN_11)=TEMPS_tdt;
      tt_TQ_temps(NN_22)=TEMPS_tdt;
      tt_TQ_temps(NN_33)=TEMPS_tdt;
      tt_TQ_temps(NN_12)=TEMPS_tdt;
      tt_TQ_temps(NN_13)=TEMPS_tdt;
      tt_TQ_temps(NN_23)=TEMPS_tdt;
      tt_TQ_temps(MM_11)=TEMPS_tdt;
      tt_TQ_temps(MM_22)=TEMPS_tdt;
      tt_TQ_temps(MM_33)=TEMPS_tdt;
      tt_TQ_temps(MM_12)=TEMPS_tdt;
      tt_TQ_temps(MM_13)=TEMPS_tdt;
      tt_TQ_temps(MM_23)=TEMPS_tdt;
      tt_TQ_temps(DIRECTION_PLI)=TEMPS_tdt;
      tt_TQ_temps(DIRECTION_PLI_NORMEE)=TEMPS_tdt;
      tt_TQ_temps(INDIC_CAL_PLIS)=TEMPS_tdt;
      tt_TQ_temps(NN_SURF)=TEMPS_tdt;
      tt_TQ_temps(NN_SURF_t)=TEMPS_t;
      tt_TQ_temps(NN_SURF_t0)=TEMPS_0;
      tt_TQ_temps(NOEUD_PROJECTILE_EN_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(NOEUD_FACETTE_EN_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(GLISSEMENT_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(PENETRATION_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(GLISSEMENT_CONTACT_T)=TEMPS_t;
      tt_TQ_temps(PENETRATION_CONTACT_T)=TEMPS_t;
      tt_TQ_temps(FORCE_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(FORCE_CONTACT_T)=TEMPS_t;
      tt_TQ_temps(CONTACT_NB_PENET)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_NB_DECOL)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_CAS_SOLIDE)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_ENERG_PENAL)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_COLLANT)=TEMPS_tdt;
      tt_TQ_temps(NUM_ZONE_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_ENERG_GLISSE_ELAS)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_ENERG_GLISSE_PLAS)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_ENERG_GLISSE_VISQ)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_PENALISATION_N)=TEMPS_tdt;
      tt_TQ_temps(CONTACT_PENALISATION_T)=TEMPS_tdt;
      tt_TQ_temps(NORMALE_CONTACT)=TEMPS_tdt;
      tt_TQ_temps(TEMPS_CPU_USER)=TEMPS_tdt;
      tt_TQ_temps(TEMPS_CPU_LOI_COMP)=TEMPS_tdt;
      tt_TQ_temps(TEMPS_CPU_METRIQUE)=TEMPS_tdt;
      tt_TQ_temps(GENERIQUE_UNE_GRANDEUR_GLOBALE)=TEMPS_tdt;
      tt_TQ_temps(GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR)=TEMPS_tdt;
      tt_TQ_temps(GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR)=TEMPS_tdt;
      tt_TQ_temps(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0)=TEMPS_0;
      tt_TQ_temps(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T)=TEMPS_t;
      tt_TQ_temps(GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT)=TEMPS_tdt;
      tt_TQ_temps(DEPLACEMENT)=TEMPS_tdt;
      tt_TQ_temps(VITESSE)=TEMPS_tdt;
      tt_TQ_temps(DELTA_XI)=TEMPS_tdt;
      tt_TQ_temps(XI_ITER_0)=TEMPS_tdt;
      tt_TQ_temps(MASSE_RELAX_DYN)=TEMPS_tdt;
      tt_TQ_temps(COMP_TORSEUR_REACTION)=TEMPS_tdt;
      tt_TQ_temps(NUM_NOEUD)=TEMPS_tdt;
      tt_TQ_temps(NUM_MAIL_NOEUD)=TEMPS_tdt;
      tt_TQ_temps(NUM_ELEMENT)=TEMPS_tdt;
      tt_TQ_temps(NUM_MAIL_ELEM)=TEMPS_tdt;
      tt_TQ_temps(NUM_PTI)=TEMPS_tdt;
      tt_TQ_temps(NUM_FACE)=TEMPS_tdt;
      tt_TQ_temps(NUM_ARETE)=TEMPS_tdt;
     };

string NomTypeQuelconque(EnumTypeQuelconque id_TypeQuelconque)
// Retourne le nom du type de grandeur correspondant a l'identificateur 
// de type enumere id_TypeQuelconque
{string result;
	switch (id_TypeQuelconque)
	{case RIEN_TYPEQUELCONQUE : result="RIEN_TYPEQUELCONQUE"; break;
		case SIGMA_BARRE_BH_T : result="sigma_barre_BH_t"; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION : result="contrainte_individuelle_a_t_sans_proportion"; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="contrainte_individuelle_a_chaque_loi_a_t"; break;
		case CONTRAINTE_COURANTE : result="CONTRAINTE_COURANTE"; break;
		case DEFORMATION_COURANTE : result="DEFORMATION_COURANTE"; break;
		case VITESSE_DEFORMATION_COURANTE : result="VITESSE_DEFORMATION_COURANTE"; break;
  case ALMANSI : result="ALMANSI"; break; 
  case GREEN_LAGRANGE : result="GREEN_LAGRANGE"; break; 
  case LOGARITHMIQUE : result="LOGARITHMIQUE"; break; 
  case DELTA_DEF : result="DELTA_DEF"; break; 
  case ALMANSI_TOTAL : result="ALMANSI_TOTAL"; break; 
  case GREEN_LAGRANGE_TOTAL : result="GREEN_LAGRANGE_TOTAL"; break; 
  case LOGARITHMIQUE_TOTALE : result="LOGARITHMIQUE_TOTALE"; break; 
  case DEF_PRINCIPALES : result="DEF_PRINCIPALES"; break; 
  case SIGMA_PRINCIPALES : result="SIGMA_PRINCIPALES"; break; 
  case VIT_PRINCIPALES : result="VIT_PRINCIPALES"; break; 
  case DEF_DUALE_MISES : result="DEF_DUALE_MISES"; break; 
  case DEF_DUALE_MISES_MAXI : result="DEF_DUALE_MISES_MAXI"; break; 
  case CONTRAINTE_MISES : result="CONTRAINTE_MISES"; break; 
  case CONTRAINTE_MISES_T : result="CONTRAINTE_MISES_T"; break;
  case CONTRAINTE_TRESCA : result="CONTRAINTE_TRESCA"; break;
  case CONTRAINTE_TRESCA_T : result="CONTRAINTE_TRESCA_T"; break;
  case ERREUR_Q : result="ERREUR_Q"; break;
  case DEF_PLASTIQUE_CUMULEE : result="DEF_PLASTIQUE_CUMULEE"; break; 
  case ERREUR_SIG_RELATIVE : result="ERREUR_SIG_RELATIVE"; break;

		case TEMPERATURE_LOI_THERMO_PHYSIQUE : result="TEMPERATURE_LOI_THERMO_PHYSIQUE"; break;
		case PRESSION_LOI_THERMO_PHYSIQUE : result="PRESSION_LOI_THERMO_PHYSIQUE"; break;
		case TEMPERATURE_TRANSITION : result="TEMPERATURE_TRANSITION"; break;
		case VOLUME_SPECIFIQUE : result="VOLUME_SPECIFIQUE"; break;
		case FLUXD : result="FLUXD"; break;
		case GRADT : result="GRADT"; break;
		case DGRADT : result="DGRADT"; break;
		case DELTAGRADT : result="DELTAGRADT"; break;
		case COEFF_DILATATION_LINEAIRE : result="COEFF_DILATATION_LINEAIRE"; break;
		case CONDUCTIVITE : result="CONDUCTIVITE"; break;
		case CAPACITE_CALORIFIQUE : result="CAPACITE_CALORIFIQUE"; break;
		case MODULE_COMPRESSIBILITE : result="MODULE_COMPRESSIBILITE"; break;
		case COEFF_COMPRESSIBILITE : result="COEFF_COMPRESSIBILITE"; break;
		case MODULE_CISAILLEMENT : result="MODULE_CISAILLEMENT"; break;
		case MODULE_COMPRESSIBILITE_TOTAL : result="MODULE_COMPRESSIBILITE_TOTAL"; break;
		case MODULE_CISAILLEMENT_TOTAL : result="MODULE_CISAILLEMENT_TOTAL"; break;
  case E_YOUNG : result="E_YOUNG"; break;
  case NU_YOUNG : result="NU_YOUNG"; break;
  case MU_VISCO : result="MU_VISCO"; break;
  case MU_VISCO_SPHERIQUE : result="MU_VISCO_SPHERIQUE"; break;
  case MODULE_TANGENT_1D : result="MODULE_TANGENT_1D"; break;
  case COMPRESSIBILITE_TANGENTE : result="COMPRESSIBILITE_TANGENTE"; break;
		case NB_INVERSION : result="NB_INVERSION"; break;
		case HYPER_CENTRE_HYSTERESIS : result="HYPER_CENTRE_HYSTERESIS"; break;
		case SIGMA_REF : result="SIGMA_REF"; break;
		case Q_SIG_HYST_Oi_A_R : result="Q_SIG_HYST_Oi_A_R"; break;
		case Q_SIG_HYST_R_A_T : result="Q_SIG_HYST_R_A_T"; break;
		case Q_DELTA_SIG_HYST : result="Q_DELTA_SIG_HYST"; break;
		case COS_ALPHA_HYSTERESIS : result="COS_ALPHA_HYSTERESIS"; break;
		case COS3PHI_SIG_HYSTERESIS : result="COS3PHI_SIG_HYSTERESIS"; break;
		case COS3PHI_DELTA_SIG_HYSTERESIS : result="COS3PHI_DELTA_SIG_HYSTERESIS"; break;
		case FCT_AIDE : result="FCT_AIDE"; break;
		case NB_ITER_TOTAL_RESIDU : result="NB_ITER_TOTAL_RESIDU"; break;
		case NB_INCRE_TOTAL_RESIDU : result="NB_INCRE_TOTAL_RESIDU"; break;
		case NB_APPEL_FCT : result="NB_APPEL_FCT"; break;
		case NB_STEP : result="NB_STEP"; break;
		case ERREUR_RK : result="ERREUR_RK"; break;
		case PRESSION_HYST_REF : result="PRESSION_HYST_REF"; break;
		case PRESSION_HYST_REF_M1 : result="PRESSION_HYST_REF_M1"; break;
		case PRESSION_HYST_T : result="PRESSION_HYST_T"; break;
		case PRESSION_HYST : result="PRESSION_HYST"; break;
		case UN_DDL_ENUM_ETENDUE : result="UN_DDL_ENUM_ETENDUE"; break;
		case ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"; break;
		case ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"; break;
		case ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T"; break;
		case PROPORTION_LOI_MELANGE : result="PROPORTION_LOI_MELANGE"; break;
		case FONC_PONDERATION : result="FONC_PONDERATION"; break;
		case POSITION_GEOMETRIQUE : result="POSITION_GEOMETRIQUE"; break;
  case POSITION_GEOMETRIQUE_t : result="POSITION_GEOMETRIQUE_t"; break;
  case POSITION_GEOMETRIQUE_t0 : result="POSITION_GEOMETRIQUE_t0"; break;
		case CRISTALINITE : result="CRISTALINITE"; break;
		case VOLUME_ELEMENT : result="VOLUME_ELEMENT"; break;
		case VOLUME_PTI : result="VOLUME_PTI"; break;
		case EPAISSEUR_MOY_INITIALE : result="EPAISSEUR_MOY_INITIALE"; break;
		case EPAISSEUR_MOY_FINALE : result="EPAISSEUR_MOY_FINALE"; break;
		case SECTION_MOY_INITIALE : result="SECTION_MOY_INITIALE"; break;
		case SECTION_MOY_FINALE : result="SECTION_MOY_FINALE"; break;
  case EPAISSEUR_INITIALE : result="EPAISSEUR_INITIALE"; break;
  case EPAISSEUR_FINALE : result="EPAISSEUR_FINALE"; break;
  case SECTION_INITIALE : result="SECTION_INITIALE"; break;
  case SECTION_FINALE : result="SECTION_FINALE"; break;
		case VOL_ELEM_AVEC_PLAN_REF : result="VOL_ELEM_AVEC_PLAN_REF"; break;
		case INTEG_SUR_VOLUME : result="INTEG_SUR_VOLUME"; break;
		case INTEG_SUR_VOLUME_ET_TEMPS : result="INTEG_SUR_VOLUME_ET_TEMPS"; break;
  case STATISTIQUE : result="STATISTIQUE"; break;
  case STATISTIQUE_ET_TEMPS : result="STATISTIQUE_ET_TEMPS"; break;
		case ENERGIE_HOURGLASS : result="ENERGIE_HOURGLASS"; break;
		case PUISSANCE_BULK : result="PUISSANCE_BULK"; break;
		case ENERGIE_BULK : result="ENERGIE_BULK"; break;
		case ENERGIE_STABMEMB_BIEL : result="ENERGIE_STABMEMB_BIEL"; break;
		case FORCE_STABMEMB_BIEL : result="FORCE_STABMEMB_BIEL"; break;
		case TENSEUR_COURBURE : result="TENSEUR_COURBURE"; break;
		case COURBURES_PRINCIPALES : result="COURBURES_PRINCIPALES"; break;
		case DIRECTIONS_PRINC_COURBURE : result="DIRECTIONS_PRINC_COURBURE"; break;
		case DIRECTIONS_PRINC_SIGMA : result="DIRECTIONS_PRINC_SIGMA"; break;
		case DIRECTIONS_PRINC_DEF : result="DIRECTIONS_PRINC_DEF"; break;
		case DIRECTIONS_PRINC_D : result="DIRECTIONS_PRINC_D"; break;
		case REPERE_LOCAL_ORTHO : result="REPERE_LOCAL_ORTHO"; break;
		case REPERE_LOCAL_H : result="REPERE_LOCAL_H"; break;
		case REPERE_LOCAL_B : result="REPERE_LOCAL_B"; break;
  case REPERE_D_ANISOTROPIE : result="REPERE_D_ANISOTROPIE"; break;
  case EPS_TRANSPORTEE_ANISO : result="EPS_TRANSPORTEE_ANISO"; break;
  case SIGMA_DANS_ANISO : result="SIGMA_DANS_ANISO"; break;
  case DELTA_EPS_TRANSPORTEE_ANISO : result="DELTA_EPS_TRANSPORTEE_ANISO"; break;
  case DELTA_SIGMA_DANS_ANISO : result="DELTA_SIGMA_DANS_ANISO"; break;
  case PARA_ORTHO : result="PARA_ORTHO"; break;
		case SPHERIQUE_EPS : result="SPHERIQUE_EPS"; break;
		case Q_EPS : result="Q_EPS"; break;
		case COS3PHI_EPS : result="COS3PHI_EPS"; break;
		case SPHERIQUE_SIG : result="SPHERIQUE_SIG"; break;
		case Q_SIG : result="Q_SIG"; break;
		case COS3PHI_SIG : result="COS3PHI_SIG"; break;
		case SPHERIQUE_DEPS : result="SPHERIQUE_DEPS"; break;
		case V_vol : result="V_vol"; break;
		case Q_DEPS : result="Q_DEPS"; break;
		case COS3PHI_DEPS : result="COS3PHI_DEPS"; break;
		case POTENTIEL : result="POTENTIEL"; break;
  case FCT_POTENTIEL_ND : result="FCT_POTENTIEL_ND"; break;
  case INVAR_B1 : result="INVAR_B1"; break;
  case INVAR_B2 : result="INVAR_B2"; break;
  case INVAR_B3 : result="INVAR_B3"; break;
  case INVAR_J1 : result="INVAR_J1"; break;
  case INVAR_J2 : result="INVAR_J2"; break;
  case INVAR_J3 : result="INVAR_J3"; break;
		case DEF_EQUIVALENTE : result="DEF_EQUIVALENTE"; break;
		case DEF_EPAISSEUR : result="DEF_EPAISSEUR"; break;
		case D_EPAISSEUR : result="D_EPAISSEUR"; break;
		case DEF_LARGEUR : result="DEF_LARGEUR"; break;
		case D_LARGEUR : result="D_LARGEUR"; break;
  case DEF_MECANIQUE : result="DEF_MECANIQUE"; break;
  case DEF_ASSO_LOI : result="DEF_ASSO_LOI"; break;
  case DEF_P_DANS_V_A : result="DEF_P_DANS_V_A"; break;
		case SIG_EPAISSEUR : result="SIG_EPAISSEUR"; break;
		case SIG_LARGEUR : result="SIG_LARGEUR"; break;
		case FORCE_GENE_EXT : result="FORCE_GENE_EXT"; break;
		case FORCE_GENE_INT : result="FORCE_GENE_INT"; break;
  case FORCE_GENE_EXT_t : result="FORCE_GENE_EXT_t"; break;
  case FORCE_GENE_INT_t : result="FORCE_GENE_INT_t"; break;
  case FORCE_GENE_TOT : result="FORCE_GENE_TOT"; break;
  case RESIDU_GLOBAL : result="RESIDU_GLOBAL"; break;
		case VECT_PRESSION : result="VECT_PRESSION"; break;
  case PRESSION_SCALAIRE : result="PRESSION_SCALAIRE"; break;
		case VECT_FORCE_VOLUM : result="VECT_FORCE_VOLUM"; break;
		case VECT_DIR_FIXE : result="VECT_DIR_FIXE"; break;
		case VECT_SURF_SUIV : result="VECT_SURF_SUIV"; break;
		case VECT_HYDRODYNA_Fn : result="VECT_HYDRODYNA_Fn"; break;
		case VECT_HYDRODYNA_Ft : result="VECT_HYDRODYNA_Ft"; break;
		case VECT_HYDRODYNA_T : result="VECT_HYDRODYNA_T"; break;
		case VECT_LINE : result="VECT_LINE"; break;
		case VECT_LINE_SUIV : result="VECT_LINE_SUIV"; break;
		case VECT_REAC : result="VECT_REAC"; break;
		case VECT_REAC_N : result="VECT_REAC_N"; break;
  
  case NN_11 : result="NN_11"; break;
  case NN_22 : result="NN_22"; break;
  case NN_33 : result="NN_33"; break;
  case NN_12 : result="NN_12"; break;
  case NN_13 : result="NN_13"; break;
  case NN_23 : result="NN_23"; break;

  case MM_11 : result="MM_11"; break;
  case MM_22 : result="MM_22"; break;
  case MM_33 : result="MM_33"; break;
  case MM_12 : result="MM_12"; break;
  case MM_13 : result="MM_13"; break;
  case MM_23 : result="MM_23"; break;

  case DIRECTION_PLI : result="DIRECTION_PLI"; break;
  case DIRECTION_PLI_NORMEE : result="DIRECTION_PLI_NORMEE"; break;
  case INDIC_CAL_PLIS : result="INDIC_CAL_PLIS"; break;
  case NN_SURF : result="NN_SURF"; break;
  case NN_SURF_t : result="NN_SURF_t"; break;
  case NN_SURF_t0 : result="NN_SURF_t0"; break;
  case NOEUD_PROJECTILE_EN_CONTACT : result="NOEUD_PROJECTILE_EN_CONTACT"; break;
  case NOEUD_FACETTE_EN_CONTACT : result="NOEUD_FACETTE_EN_CONTACT"; break;
  case GLISSEMENT_CONTACT : result="GLISSEMENT_CONTACT"; break;
  case PENETRATION_CONTACT : result="PENETRATION_CONTACT"; break;
  case GLISSEMENT_CONTACT_T : result="GLISSEMENT_CONTACT_T"; break;
  case PENETRATION_CONTACT_T : result="PENETRATION_CONTACT_T"; break;
  case FORCE_CONTACT : result="FORCE_CONTACT"; break;
  case FORCE_CONTACT_T : result="FORCE_CONTACT_T"; break;
  case CONTACT_NB_PENET : result="CONTACT_NB_PENET"; break;
  case CONTACT_NB_DECOL : result="CONTACT_NB_DECOL"; break;
  case CONTACT_CAS_SOLIDE : result="CONTACT_CAS_SOLIDE"; break;
  case CONTACT_ENERG_PENAL : result="CONTACT_ENERG_PENAL"; break;
  case CONTACT_COLLANT : result="CONTACT_COLLANT"; break;
  case NUM_ZONE_CONTACT : result="NUM_ZONE_CONTACT"; break;
  case CONTACT_ENERG_GLISSE_ELAS : result="CONTACT_ENERG_GLISSE_ELAS"; break;
  case CONTACT_ENERG_GLISSE_PLAS : result="CONTACT_ENERG_GLISSE_PLAS"; break;
  case CONTACT_ENERG_GLISSE_VISQ : result="CONTACT_ENERG_GLISSE_VISQ"; break;
  case CONTACT_PENALISATION_N : result="CONTACT_PENALISATION_N"; break;
  case CONTACT_PENALISATION_T : result="CONTACT_PENALISATION_T"; break;
  case NORMALE_CONTACT : result="NORMALE_CONTACT"; break;
  case TEMPS_CPU_USER : result="TEMPS_CPU_USER"; break;
  case TEMPS_CPU_LOI_COMP : result="TEMPS_CPU_LOI_COMP"; break;
  case TEMPS_CPU_METRIQUE : result="TEMPS_CPU_METRIQUE"; break;
  case GENERIQUE_UNE_GRANDEUR_GLOBALE : result="GENERIQUE_UNE_GRANDEUR_GLOBALE"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR : result="GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR : result="GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0 : result="GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T : result="GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT : result="GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT"; break;
  case DEPLACEMENT : result="DEPLACEMENT"; break;
  case VITESSE : result="VITESSE"; break;
  case DELTA_XI : result="DELTA_XI"; break;
  case XI_ITER_0 : result="XI_ITER_0"; break;
  case MASSE_RELAX_DYN : result="MASSE_RELAX_DYN"; break;
  case COMP_TORSEUR_REACTION : result="COMP_TORSEUR_REACTION"; break;
  case NUM_NOEUD : result="NUM_NOEUD"; break;
  case NUM_MAIL_NOEUD : result="NUM_MAIL_NOEUD"; break;
  case NUM_ELEMENT : result="NUM_ELEMENT"; break;
  case NUM_MAIL_ELEM : result="NUM_MAIL_ELEM"; break;
  case NUM_PTI : result="NUM_PTI"; break;
  case NUM_FACE : result="NUM_FACE"; break;
  case NUM_ARETE : result="NUM_ARETE"; break;

		default :
			cout << "\nErreur : valeur incorrecte du type EnumTypeQuelconque: id_TypeQuelconque="
         << id_TypeQuelconque << " !\n";
			cout << "NomTypeQuelconque(EnumTypeQuelconque ) \n";
			Sortie(1);
	};
	return result;	
};

// idem mais en version courte (sur quelques caractères)
string NomTypeQuelconque_court (EnumTypeQuelconque id_TypeQuelconque)
// Retourne le nom du type de grandeur correspondant a l'identificateur 
// de type enumere id_TypeQuelconque
{   string result;
	switch (id_TypeQuelconque)
	{   case RIEN_TYPEQUELCONQUE : result="RIEN"; break;
		case SIGMA_BARRE_BH_T : result="sig_b"; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION : result="sig_i_SP"; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="sig_i"; break;
		case CONTRAINTE_COURANTE : result="sig_c"; break;
		case DEFORMATION_COURANTE : result="def_c"; break;
		case VITESSE_DEFORMATION_COURANTE : result="D_c"; break;
  case ALMANSI : result="ALMANSI"; break; 
  case GREEN_LAGRANGE : result="GREEN_LAG"; break; 
  case LOGARITHMIQUE : result="LOG"; break; 
  case DELTA_DEF : result="DELTA_DEF"; break; 
  case ALMANSI_TOTAL : result="ALMANSI_TOT"; break; 
  case GREEN_LAGRANGE_TOTAL : result="GREEN_LAG_TOT"; break; 
  case LOGARITHMIQUE_TOTALE : result="LOG_TOT"; break; 
  case DEF_PRINCIPALES : result="DEF_princ"; break; 
  case SIGMA_PRINCIPALES : result="SIG_princ"; break; 
  case VIT_PRINCIPALES : result="VIT_princ"; break; 
  case DEF_DUALE_MISES : result="DefDualMises"; break; 
  case DEF_DUALE_MISES_MAXI : result="DefDualMisesMaxi"; break; 
  case CONTRAINTE_MISES : result="MISES"; break; 
  case CONTRAINTE_MISES_T : result="MISES_T"; break;
  case CONTRAINTE_TRESCA : result="TRESCA"; break;
  case CONTRAINTE_TRESCA_T : result="TRESCA_T"; break;
  case ERREUR_Q : result="ERR_Q"; break;
  case DEF_PLASTIQUE_CUMULEE : result="DEF_PLAST_CUM"; break; 
  case ERREUR_SIG_RELATIVE : result="ERR_SIG_REL"; break;

		case TEMPERATURE_LOI_THERMO_PHYSIQUE : result="TEMP_THERMO_PHYS"; break;
		case PRESSION_LOI_THERMO_PHYSIQUE : result="P_THERMO_PHYS"; break;
		case TEMPERATURE_TRANSITION : result="TEMP_TRANSI"; break;
		case VOLUME_SPECIFIQUE : result="VOL_SPEC"; break;
		case FLUXD : result="FLUXD"; break;
		case GRADT : result="GRADT"; break;
		case DGRADT : result="DGRADT"; break;
		case DELTAGRADT : result="DELTAGRADT"; break;
		case COEFF_DILATATION_LINEAIRE : result="COEFF_DILAT_LIN"; break;
		case CONDUCTIVITE : result="CONDUC"; break;
		case CAPACITE_CALORIFIQUE : result="CAPAC_CALOR"; break;
		case MODULE_COMPRESSIBILITE : result="MOD_COMPRES"; break;
		case COEFF_COMPRESSIBILITE : result="COEFF_COMPRES"; break;
		case MODULE_CISAILLEMENT : result="MOD_CISAIL"; break;
		case MODULE_COMPRESSIBILITE_TOTAL : result="MOD_COMPRES_TOT"; break;
		case MODULE_CISAILLEMENT_TOTAL : result="MOD_CISAIL_TOT"; break;
  case E_YOUNG : result="E_YOUNG"; break;
  case NU_YOUNG : result="NU_YOUNG"; break;
  case MU_VISCO : result="MU_VISCO"; break;
  case MU_VISCO_SPHERIQUE : result="MU_VISCO_SPHERIQUE"; break;
  case MODULE_TANGENT_1D : result="MODULE_TANGENT_1D"; break;
  case COMPRESSIBILITE_TANGENTE : result="COMPRESSIBILITE_TANGENTE"; break;
		case NB_INVERSION : result="NB_INVER"; break;
		case HYPER_CENTRE_HYSTERESIS : result="CENTRE_HYSTER"; break;
		case SIGMA_REF : result="SIG_REF"; break;
		case Q_SIG_HYST_Oi_A_R : result="Q_SIG_HYS_0iR"; break;
		case Q_SIG_HYST_R_A_T : result="Q_SIG_HYS_RaT"; break;
		case Q_DELTA_SIG_HYST : result="Q_D_SIG_HYST"; break;
		case COS_ALPHA_HYSTERESIS : result="COS_ALPHA_HYS"; break;
		case COS3PHI_SIG_HYSTERESIS : result="COS3PHI_SIG_HYS"; break;
		case COS3PHI_DELTA_SIG_HYSTERESIS : result="CO3PHI_D_SIG_HYS"; break;
		case FCT_AIDE : result="FCT_AIDE"; break;
		case NB_ITER_TOTAL_RESIDU : result="NB_ITER_TOT_RES"; break;
		case NB_INCRE_TOTAL_RESIDU : result="NB_INCR_TOT_RES"; break;
		case NB_APPEL_FCT : result="NB_APPEL_FCT"; break;
		case NB_STEP : result="NB_STEP"; break;
		case ERREUR_RK : result="ERREUR_RK"; break;
		case PRESSION_HYST_REF : result="PRESSION_HYST_REF"; break;
		case PRESSION_HYST_REF_M1 : result="PRESSION_HYST_REF_M1"; break;
		case PRESSION_HYST_T : result="PRESSION_HYST_T"; break;
		case PRESSION_HYST : result="PRESSION_HYST"; break;
		case UN_DDL_ENUM_ETENDUE : result="DDL_ENUM_ETENDUE"; break;
		case ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_ELAST_i"; break;
		case ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_PLAST_i"; break;
		case ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_VISQU_i"; break;
		case PROPORTION_LOI_MELANGE : result="PROP_LOI_MEL"; break;
		case FONC_PONDERATION : result="F_POND"; break;
		case POSITION_GEOMETRIQUE : result="POSI_GEOM"; break;
  case POSITION_GEOMETRIQUE_t : result="POSI_GEOM_t"; break;
  case POSITION_GEOMETRIQUE_t0 : result="POSI_GEOM_t0"; break;
		case CRISTALINITE : result="CRISTA"; break;
		case VOLUME_ELEMENT : result="VOLUME_ELEMENT"; break;
		case VOLUME_PTI : result="VOLUME_PTI"; break;
		case EPAISSEUR_MOY_INITIALE : result="EPAISS_MOY_0"; break;
		case EPAISSEUR_MOY_FINALE : result="EPAISS_MOY_tdt"; break;
		case SECTION_MOY_INITIALE : result="SECT_MOY_0"; break;
		case SECTION_MOY_FINALE : result="SECT_MOY_tdt"; break;
  case EPAISSEUR_INITIALE : result="EPAISSEUR_INITIALE"; break;
  case EPAISSEUR_FINALE : result="EPAISSEUR_FINALE"; break;
  case SECTION_INITIALE : result="SECTION_INITIALE"; break;
  case SECTION_FINALE : result="SECTION_FINALE"; break;
		case VOL_ELEM_AVEC_PLAN_REF : result="VOL_ELE_PLREF"; break;
		case INTEG_SUR_VOLUME : result="INTEG_SUR_VOL"; break;
		case INTEG_SUR_VOLUME_ET_TEMPS : result="INTEG_SUR_VOL_ET_TEMPS"; break;
  case STATISTIQUE : result="STATISTIQUE"; break;
  case STATISTIQUE_ET_TEMPS : result="STATISTIQUE_ET_TEMPS"; break;
		case ENERGIE_HOURGLASS : result="ENERG_HOURG"; break;
		case PUISSANCE_BULK : result="PUISS_BULK"; break;
		case ENERGIE_BULK : result="ENERG_BULK"; break;
		case ENERGIE_STABMEMB_BIEL : result="ENERGIE_STABMEMB_BIEL"; break;
		case FORCE_STABMEMB_BIEL : result="FORCE_STABMEMB_BIEL"; break;
		case TENSEUR_COURBURE : result="TENS_COURBURE"; break;
		case COURBURES_PRINCIPALES : result="COURBURES_PRINC"; break;
		case DIRECTIONS_PRINC_COURBURE : result="DIR_PRINC_CURB"; break;
		case DIRECTIONS_PRINC_SIGMA : result="DIR_PRINC_SIG"; break;
		case DIRECTIONS_PRINC_DEF : result="DIR_PRINC_DEF"; break;
		case DIRECTIONS_PRINC_D : result="DIR_PRINC_D"; break;
		case REPERE_LOCAL_ORTHO : result="REP_LOC_ORTHO"; break;
		case REPERE_LOCAL_H : result="REP_LOC_H"; break;
		case REPERE_LOCAL_B : result="REP_LOC_H"; break;
  case REPERE_D_ANISOTROPIE : result="REP_D_ANISO"; break;
  case EPS_TRANSPORTEE_ANISO : result="EPS_TRANS_ANISO"; break;
  case SIGMA_DANS_ANISO : result="SIGMA_DANS_ANISO"; break;
  case DELTA_EPS_TRANSPORTEE_ANISO : result="DELTA_EPS_TRANS_ANISO"; break;
  case DELTA_SIGMA_DANS_ANISO : result="DELTA_SIGMA_DANS_ANISO"; break;
  case PARA_ORTHO : result="PARA_ORTHO"; break;
		case SPHERIQUE_EPS : result="SPHERIQUE_EPS"; break;
		case Q_EPS : result="Q_EPS"; break;
		case COS3PHI_EPS : result="COS3PHI_EPS"; break;
		case SPHERIQUE_SIG : result="SPHERIQUE_SIG"; break;
		case Q_SIG : result="Q_SIG"; break;
		case COS3PHI_SIG : result="COS3PHI_SIG"; break;
		case SPHERIQUE_DEPS : result="SPHERIQUE_DEPS"; break;
		case V_vol : result="V_vol"; break;
		case Q_DEPS : result="Q_DEPS"; break;
		case COS3PHI_DEPS : result="COS3PHI_DEPS"; break;
		case POTENTIEL : result="POTENTIEL"; break;
  case FCT_POTENTIEL_ND : result="FCT_POTENTIEL_ND"; break;
  case INVAR_B1 : result="INVAR_B1"; break;
  case INVAR_B2 : result="INVAR_B2"; break;
  case INVAR_B3 : result="INVAR_B3"; break;
  case INVAR_J1 : result="INVAR_J1"; break;
  case INVAR_J2 : result="INVAR_J2"; break;
  case INVAR_J3 : result="INVAR_J3"; break;
		case DEF_EQUIVALENTE : result="DEF_EQUIVALENTE"; break;
		case DEF_EPAISSEUR : result="DEF_EPAISSEUR"; break;
		case D_EPAISSEUR : result="D_EPAISSEUR"; break;
		case DEF_LARGEUR : result="DEF_LARGEUR"; break;
		case D_LARGEUR : result="D_LARGEUR"; break;
  case DEF_MECANIQUE : result="DEF_MECANIQUE"; break;
  case DEF_ASSO_LOI : result="DEF_ASSO_LOI"; break;
  case DEF_P_DANS_V_A : result="DEF_P_DANS_V_A"; break;
		case SIG_EPAISSEUR : result="SIG_EPAISSEUR"; break;
		case SIG_LARGEUR : result="SIG_LARGEUR"; break;
		case FORCE_GENE_EXT : result="FORCE_GENE_EXT"; break;
		case FORCE_GENE_INT : result="FORCE_GENE_INT"; break;
  case FORCE_GENE_EXT_t : result="FORCE_GENE_EXT_t"; break;
  case FORCE_GENE_INT_t : result="FORCE_GENE_INT_t"; break;
  case FORCE_GENE_TOT : result="FORCE_GENE_TOT"; break;
  case RESIDU_GLOBAL : result="RESIDU_GLOBAL"; break;
		case VECT_PRESSION : result="VECT_PRESSION"; break;
  case PRESSION_SCALAIRE : result="PRESSION_SCALAIRE"; break;
		case VECT_FORCE_VOLUM : result="VECT_FORCE_VOLUM"; break;
		case VECT_DIR_FIXE : result="VECT_DIR_FIXE"; break;
		case VECT_SURF_SUIV : result="VECT_SURF_SUIV"; break;
		case VECT_HYDRODYNA_Fn : result="VECT_HYDRODYNA_Fn"; break;
		case VECT_HYDRODYNA_Ft : result="VECT_HYDRODYNA_Ft"; break;
		case VECT_HYDRODYNA_T : result="VECT_HYDRODYNA_T"; break;
		case VECT_LINE : result="VECT_LINE"; break;
		case VECT_LINE_SUIV : result="VECT_LINE_SUIV"; break;
		case VECT_REAC : result="VECT_REAC"; break;
		case VECT_REAC_N : result="VECT_REAC_N"; break;
  
  case NN_11 : result="NN_11"; break;
  case NN_22 : result="NN_22"; break;
  case NN_33 : result="NN_33"; break;
  case NN_12 : result="NN_12"; break;
  case NN_13 : result="NN_13"; break;
  case NN_23 : result="NN_23"; break;

  case MM_11 : result="MM_11"; break;
  case MM_22 : result="MM_22"; break;
  case MM_33 : result="MM_33"; break;
  case MM_12 : result="MM_12"; break;
  case MM_13 : result="MM_13"; break;
  case MM_23 : result="MM_23"; break;

  case DIRECTION_PLI : result="DIR_PLI"; break;
  case DIRECTION_PLI_NORMEE : result="DIR_PLI_NORM"; break;
  case INDIC_CAL_PLIS : result="INDIC_CAL_PLIS"; break;
  case NN_SURF : result="NN_SURF"; break;
  case NN_SURF_t : result="NN_SURF_t"; break;
  case NN_SURF_t0 : result="NN_SURF_t0"; break;
  case NOEUD_PROJECTILE_EN_CONTACT : result="NOE_PR_CONTACT"; break;
  case NOEUD_FACETTE_EN_CONTACT : result="NOE_F_CONTACT"; break;
  case GLISSEMENT_CONTACT : result="GLISSE_CONTACT"; break;
  case PENETRATION_CONTACT : result="PENE_CONTACT"; break;
  case GLISSEMENT_CONTACT_T : result="GLISSE_CONTACT_T"; break;
  case PENETRATION_CONTACT_T : result="PENE_CONTACT_T"; break;
  case FORCE_CONTACT : result="FORCE_CONTACT"; break;
  case FORCE_CONTACT_T : result="FORCE_CONTACT_T"; break;
  case CONTACT_NB_PENET : result="CONTACT_NB_PENET"; break;
  case CONTACT_NB_DECOL : result="CONTACT_NB_DECOL"; break;
  case CONTACT_CAS_SOLIDE : result="CONTACT_CAS_SOLIDE"; break;
  case CONTACT_ENERG_PENAL : result="CONTACT_ENERG_PENAL"; break;
  case CONTACT_COLLANT : result="CONTACT_COLLANT"; break;
  case NUM_ZONE_CONTACT : result="NUM_ZONE_CONTACT"; break;
  case CONTACT_ENERG_GLISSE_ELAS : result="CONTACT_ENERG_GLISSE_ELAS"; break;
  case CONTACT_ENERG_GLISSE_PLAS : result="CONTACT_ENERG_GLISSE_PLAS"; break;
  case CONTACT_ENERG_GLISSE_VISQ : result="CONTACT_ENERG_GLISSE_VISQ"; break;
  case CONTACT_PENALISATION_N : result="CONTACT_PENALISATION_N"; break;
  case CONTACT_PENALISATION_T : result="CONTACT_PENALISATION_T"; break;
  case NORMALE_CONTACT : result="NORMALE_CONTACT"; break;
  case TEMPS_CPU_USER : result="TEMPS_CPU_USER"; break;
  case TEMPS_CPU_LOI_COMP : result="TEMPS_CPU_LOI_COMP"; break;
  case TEMPS_CPU_METRIQUE : result="TEMPS_CPU_METRIQUE"; break;
  case GENERIQUE_UNE_GRANDEUR_GLOBALE : result="GEN_GRANDEUR_GLOB"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR : result="GEN_CONST_GLOB_INT_USER"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR : result="GEN_CONST_GLOB_DOUBLE_USER"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0 : result="GEN_VAR_GLOB_DOUBLE_USER_0"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T : result="GEN_VAR_GLOB_DOUBLE_USER_T"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT : result="GEN_VAR_GLOB_DOUBLE_USER_TDT"; break;
  case DEPLACEMENT : result="DEPLACEMENT"; break;
  case VITESSE : result="VITESSE"; break;
  case DELTA_XI : result="DELTA_XI"; break;
  case XI_ITER_0 : result="XI_ITER_0"; break;
  case MASSE_RELAX_DYN : result="MASSE_RELAX_DYN"; break;
  case COMP_TORSEUR_REACTION : result="COMP_TORS_REAC"; break;
  case NUM_NOEUD : result="NUM_NOEUD"; break;
  case NUM_MAIL_NOEUD : result="NUM_MAIL_NOEUD"; break;
  case NUM_ELEMENT : result="NUM_ELEMENT"; break;
  case NUM_MAIL_ELEM : result="NUM_MAIL_ELEM"; break;
  case NUM_PTI : result="NUM_PTI"; break;
  case NUM_FACE : result="NUM_FACE"; break;
  case NUM_ARETE : result="NUM_ARETE"; break;
		default :
			cout << "\nErreur : valeur incorrecte du type EnumTypeQuelconque: id_TypeQuelconque="
        << id_TypeQuelconque << " !\n";
			cout << "NomTypeQuelconque_court(EnumTypeQuelconque ) \n";
			Sortie(1);
	};
	return result;	
};

// nom générique, lorsqu'il s'agit de composantes
string NomGeneriqueTypeQuelconque(EnumTypeQuelconque id_TypeQuelconque)
{string result;
 switch (id_TypeQuelconque)
 {case RIEN_TYPEQUELCONQUE : result="RIEN"; break;
  case SIGMA_BARRE_BH_T : result="sig_b"; break;
  case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION : result="sig_i_SP"; break;
  case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="sig_i"; break;
  case CONTRAINTE_COURANTE : result="sig_c"; break;
  case DEFORMATION_COURANTE : result="def_c"; break;
  case VITESSE_DEFORMATION_COURANTE : result="D_c"; break;
  case ALMANSI : result="ALMANSI"; break;
  case GREEN_LAGRANGE : result="GREEN_LAG"; break;
  case LOGARITHMIQUE : result="LOG"; break;
  case DELTA_DEF : result="DELTA_DEF"; break;
  case ALMANSI_TOTAL : result="ALMANSI_TOT"; break;
  case GREEN_LAGRANGE_TOTAL : result="GREEN_LAG_TOT"; break;
  case LOGARITHMIQUE_TOTALE : result="LOG_TOT"; break;
  case DEF_PRINCIPALES : result="DEF_princ"; break;
  case SIGMA_PRINCIPALES : result="SIG_princ"; break;
  case VIT_PRINCIPALES : result="VIT_princ"; break;
  case DEF_DUALE_MISES : result="DefDualMises"; break;
  case DEF_DUALE_MISES_MAXI : result="DefDualMisesMaxi"; break;
  case CONTRAINTE_MISES : result="MISES"; break;
  case CONTRAINTE_MISES_T : result="MISES_T"; break;
  case CONTRAINTE_TRESCA : result="TRESCA"; break;
  case CONTRAINTE_TRESCA_T : result="TRESCA_T"; break;
  case ERREUR_Q : result="ERR_Q"; break;
  case DEF_PLASTIQUE_CUMULEE : result="DEF_PLAST_CUM"; break;
  case ERREUR_SIG_RELATIVE : result="ERR_SIG_REL"; break;

  case TEMPERATURE_LOI_THERMO_PHYSIQUE : result="TEMP_THERMO_PHYS"; break;
  case PRESSION_LOI_THERMO_PHYSIQUE : result="P_THERMO_PHYS"; break;
  case TEMPERATURE_TRANSITION : result="TEMP_TRANSI"; break;
  case VOLUME_SPECIFIQUE : result="VOL_SPEC"; break;
  case FLUXD : result="FLUXD"; break;
  case GRADT : result="GRADT"; break;
  case DGRADT : result="DGRADT"; break;
  case DELTAGRADT : result="DELTAGRADT"; break;
  case COEFF_DILATATION_LINEAIRE : result="COEFF_DILAT_LIN"; break;
  case CONDUCTIVITE : result="CONDUC"; break;
  case CAPACITE_CALORIFIQUE : result="CAPAC_CALOR"; break;
  case MODULE_COMPRESSIBILITE : result="MOD_COMPRES"; break;
  case COEFF_COMPRESSIBILITE : result="COEFF_COMPRES"; break;
  case MODULE_CISAILLEMENT : result="MOD_CISAIL"; break;
  case MODULE_COMPRESSIBILITE_TOTAL : result="MOD_COMPRES_TOT"; break;
  case MODULE_CISAILLEMENT_TOTAL : result="MOD_CISAIL_TOT"; break;
  case E_YOUNG : result="E_YOUNG"; break;
  case NU_YOUNG : result="NU_YOUNG"; break;
  case MU_VISCO : result="MU_VISCO"; break;
  case MU_VISCO_SPHERIQUE : result="MU_VISCO_SPHERIQUE"; break;
  case MODULE_TANGENT_1D : result="MODULE_TANGENT_1D"; break;
  case COMPRESSIBILITE_TANGENTE : result="COMPRESSIBILITE_TANGENTE"; break;
  case NB_INVERSION : result="NB_INVER"; break;
  case HYPER_CENTRE_HYSTERESIS : result="CENTRE_HYSTER"; break;
  case SIGMA_REF : result="SIG_REF"; break;
  case Q_SIG_HYST_Oi_A_R : result="Q_SIG_HYS_0iR"; break;
  case Q_SIG_HYST_R_A_T : result="Q_SIG_HYS_RaT"; break;
  case Q_DELTA_SIG_HYST : result="Q_D_SIG_HYST"; break;
  case COS_ALPHA_HYSTERESIS : result="COS_ALPHA_HYS"; break;
  case COS3PHI_SIG_HYSTERESIS : result="COS3PHI_SIG_HYS"; break;
  case COS3PHI_DELTA_SIG_HYSTERESIS : result="CO3PHI_D_SIG_HYS"; break;
  case FCT_AIDE : result="FCT_AIDE"; break;
  case NB_ITER_TOTAL_RESIDU : result="NB_ITER_TOT_RES"; break;
  case NB_INCRE_TOTAL_RESIDU : result="NB_INCR_TOT_RES"; break;
  case NB_APPEL_FCT : result="NB_APPEL_FCT"; break;
  case NB_STEP : result="NB_STEP"; break;
  case ERREUR_RK : result="ERREUR_RK"; break;
  case PRESSION_HYST_REF : result="PRESSION_HYST_REF"; break;
  case PRESSION_HYST_REF_M1 : result="PRESSION_HYST_REF_M1"; break;
  case PRESSION_HYST_T : result="PRESSION_HYST_T"; break;
  case PRESSION_HYST : result="PRESSION_HYST"; break;
  case UN_DDL_ENUM_ETENDUE : result="DDL_ENUM_ETENDUE"; break;
  case ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_ELAST_i"; break;
  case ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_PLAST_i"; break;
  case ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : result="ENERG_VISQU_i"; break;
  case PROPORTION_LOI_MELANGE : result="PROP_LOI_MEL"; break;
  case FONC_PONDERATION : result="F_POND"; break;
  case POSITION_GEOMETRIQUE : result="POSI_GEOM"; break;
  case POSITION_GEOMETRIQUE_t : result="POSI_GEOM_t"; break;
  case POSITION_GEOMETRIQUE_t0 : result="POSI_GEOM_t0"; break;
  case CRISTALINITE : result="CRISTA"; break;
  case VOLUME_ELEMENT : result="VOLUME_ELEMENT"; break;
  case VOLUME_PTI : result="VOLUME_PTI"; break;
  case EPAISSEUR_MOY_INITIALE : result="EPAISS_MOY_0"; break;
  case EPAISSEUR_MOY_FINALE : result="EPAISS_MOY_tdt"; break;
  case SECTION_MOY_INITIALE : result="SECT_MOY_0"; break;
  case SECTION_MOY_FINALE : result="SECT_MOY_tdt"; break;
  case EPAISSEUR_INITIALE : result="EPAISSEUR_INITIALE"; break;
  case EPAISSEUR_FINALE : result="EPAISSEUR_FINALE"; break;
  case SECTION_INITIALE : result="SECTION_INITIALE"; break;
  case SECTION_FINALE : result="SECTION_FINALE"; break;
  case VOL_ELEM_AVEC_PLAN_REF : result="VOL_ELE_PLREF"; break;
  case INTEG_SUR_VOLUME : result="INTEG_SUR_VOL"; break;
  case INTEG_SUR_VOLUME_ET_TEMPS : result="INTEG_SUR_VOL_ET_TEMPS"; break;
  case STATISTIQUE : result="STATISTIQUE"; break;
  case STATISTIQUE_ET_TEMPS : result="STATISTIQUE_ET_TEMPS"; break;
  case ENERGIE_HOURGLASS : result="ENERG_HOURG"; break;
  case PUISSANCE_BULK : result="PUISS_BULK"; break;
  case ENERGIE_BULK : result="ENERG_BULK"; break;
  case ENERGIE_STABMEMB_BIEL : result="ENERGIE_STABMEMB_BIEL"; break;
  case FORCE_STABMEMB_BIEL : result="FORCE_STABMEMB_BIEL"; break;
  case TENSEUR_COURBURE : result="TENS_COURBURE"; break;
  case COURBURES_PRINCIPALES : result="COURBURES_PRINC"; break;
  case DIRECTIONS_PRINC_COURBURE : result="DIR_PRINC_CURB"; break;
  case DIRECTIONS_PRINC_SIGMA : result="DIR_PRINC_SIG"; break;
  case DIRECTIONS_PRINC_DEF : result="DIR_PRINC_DEF"; break;
  case DIRECTIONS_PRINC_D : result="DIR_PRINC_D"; break;
  case REPERE_LOCAL_ORTHO : result="REP_LOC_ORTHO"; break;
  case REPERE_LOCAL_H : result="REP_LOC_H"; break;
  case REPERE_LOCAL_B : result="REP_LOC_H"; break;
  case REPERE_D_ANISOTROPIE : result="REP_D_ANISO"; break;
  case EPS_TRANSPORTEE_ANISO : result="EPS_TRANS_ANISO"; break;
  case SIGMA_DANS_ANISO : result="SIGMA_DANS_ANISO"; break;
  case DELTA_EPS_TRANSPORTEE_ANISO : result="DELTA_EPS_TRANS_ANISO"; break;
  case DELTA_SIGMA_DANS_ANISO : result="DELTA_SIGMA_DANS_ANISO"; break;
  case PARA_ORTHO : result="PARA_ORTHO"; break;
  case SPHERIQUE_EPS : result="SPHERIQUE_EPS"; break;
  case Q_EPS : result="Q_EPS"; break;
  case COS3PHI_EPS : result="COS3PHI_EPS"; break;
  case SPHERIQUE_SIG : result="SPHERIQUE_SIG"; break;
  case Q_SIG : result="Q_SIG"; break;
  case COS3PHI_SIG : result="COS3PHI_SIG"; break;
  case SPHERIQUE_DEPS : result="SPHERIQUE_DEPS"; break;
  case V_vol : result="V_vol"; break;
  case Q_DEPS : result="Q_DEPS"; break;
  case COS3PHI_DEPS : result="COS3PHI_DEPS"; break;
  case POTENTIEL : result="POTENTIEL"; break;
  case FCT_POTENTIEL_ND : result="FCT_POTENTIEL_ND"; break;
  case INVAR_B1 : result="INVAR_B1"; break;
  case INVAR_B2 : result="INVAR_B2"; break;
  case INVAR_B3 : result="INVAR_B3"; break;
  case INVAR_J1 : result="INVAR_J1"; break;
  case INVAR_J2 : result="INVAR_J2"; break;
  case INVAR_J3 : result="INVAR_J3"; break;
  case DEF_EQUIVALENTE : result="DEF_EQUIVALENTE"; break;
  case DEF_EPAISSEUR : result="DEF_EPAISSEUR"; break;
  case D_EPAISSEUR : result="D_EPAISSEUR"; break;
  case DEF_LARGEUR : result="DEF_LARGEUR"; break;
  case D_LARGEUR : result="D_LARGEUR"; break;
  case DEF_MECANIQUE : result="DEF_MECANIQUE"; break;
  case DEF_ASSO_LOI : result="DEF_ASSO_LOI"; break;
  case DEF_P_DANS_V_A : result="DEF_P_DANS_V_A"; break;
  case SIG_EPAISSEUR : result="SIG_EPAISSEUR"; break;
  case SIG_LARGEUR : result="SIG_LARGEUR"; break;
  case FORCE_GENE_EXT : result="FORCE_GENE_EXT"; break;
  case FORCE_GENE_INT : result="FORCE_GENE_INT"; break;
  case FORCE_GENE_EXT_t : result="FORCE_GENE_EXT_t"; break;
  case FORCE_GENE_INT_t : result="FORCE_GENE_INT_t"; break;
  case FORCE_GENE_TOT : result="FORCE_GENE_TOT"; break;
  case RESIDU_GLOBAL : result="RESIDU_GLOBAL"; break;
  case VECT_PRESSION : result="VECT_PRESSION"; break;
  case PRESSION_SCALAIRE : result="PRESSION_SCALAIRE"; break;
  case VECT_FORCE_VOLUM : result="VECT_FORCE_VOLUM"; break;
  case VECT_DIR_FIXE : result="VECT_DIR_FIXE"; break;
  case VECT_SURF_SUIV : result="VECT_SURF_SUIV"; break;
  case VECT_HYDRODYNA_Fn : result="VECT_HYDRODYNA_Fn"; break;
  case VECT_HYDRODYNA_Ft : result="VECT_HYDRODYNA_Ft"; break;
  case VECT_HYDRODYNA_T : result="VECT_HYDRODYNA_T"; break;
  case VECT_LINE : result="VECT_LINE"; break;
  case VECT_LINE_SUIV : result="VECT_LINE_SUIV"; break;
  case VECT_REAC : result="VECT_REAC"; break;
  case VECT_REAC_N : result="VECT_REAC_N"; break;
  
  case NN_11 : case NN_22 : case NN_33 :
  case NN_12 : case NN_13 : case NN_23 :
    result="NN_"; break;

  case MM_11 : case MM_22 : case MM_33 :
  case MM_12 : case MM_13 : case MM_23 :
    result="MM_"; break;

  case DIRECTION_PLI : result="DIR_PLI"; break;
  case DIRECTION_PLI_NORMEE : result="DIR_PLI_NORM"; break;
  case INDIC_CAL_PLIS : result="INDIC_CAL_PLIS"; break;
  case NN_SURF : result="NN_SURF"; break;
  case NN_SURF_t : result="NN_SURF_t"; break;
  case NN_SURF_t0 : result="NN_SURF_t0"; break;
  case NOEUD_PROJECTILE_EN_CONTACT : result="NOE_PR_CONTACT"; break;
  case NOEUD_FACETTE_EN_CONTACT : result="NOE_F_CONTACT"; break;
  case GLISSEMENT_CONTACT : result="GLISSE_CONTACT"; break;
  case PENETRATION_CONTACT : result="PENE_CONTACT"; break;
  case GLISSEMENT_CONTACT_T : result="GLISSE_CONTACT_T"; break;
  case PENETRATION_CONTACT_T : result="PENE_CONTACT_T"; break;
  case FORCE_CONTACT : result="FORCE_CONTACT"; break;
  case FORCE_CONTACT_T : result="FORCE_CONTACT_T"; break;
  case CONTACT_NB_PENET : result="CONTACT_NB_PENET"; break;
  case CONTACT_NB_DECOL : result="CONTACT_NB_DECOL"; break;
  case CONTACT_CAS_SOLIDE : result="CONTACT_CAS_SOLIDE"; break;
  case CONTACT_ENERG_PENAL : result="CONTACT_ENERG_PENAL"; break;
  case CONTACT_COLLANT : result="CONTACT_COLLANT"; break;
  case NUM_ZONE_CONTACT : result="NUM_ZONE_CONTACT"; break;
  case CONTACT_ENERG_GLISSE_ELAS : result="CONTACT_ENERG_GLISSE_ELAS"; break;
  case CONTACT_ENERG_GLISSE_PLAS : result="CONTACT_ENERG_GLISSE_PLAS"; break;
  case CONTACT_ENERG_GLISSE_VISQ : result="CONTACT_ENERG_GLISSE_VISQ"; break;
  case CONTACT_PENALISATION_N : result="CONTACT_PENALISATION_N"; break;
  case CONTACT_PENALISATION_T : result="CONTACT_PENALISATION_T"; break;
  case NORMALE_CONTACT : result="NORMALE_CONTACT"; break;
  case TEMPS_CPU_USER : result="TEMPS_CPU_USER"; break;
  case TEMPS_CPU_LOI_COMP : result="TEMPS_CPU_LOI_COMP"; break;
  case TEMPS_CPU_METRIQUE : result="TEMPS_CPU_METRIQUE"; break;
  case GENERIQUE_UNE_GRANDEUR_GLOBALE : result="GEN_GRANDEUR_GLOB"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR : result="GEN_CONST_GLOB_INT_USER"; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR : result="GEN_CONST_GLOB_DOUBLE_USER"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0 : result="GEN_VAR_GLOB_DOUBLE_USER_0"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T : result="GEN_VAR_GLOB_DOUBLE_USER_T"; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT : result="GEN_VAR_GLOB_DOUBLE_USER_TDT"; break;
  case DEPLACEMENT : result="DEPLACEMENT"; break;
  case VITESSE : result="VITESSE"; break;
  case DELTA_XI : result="DELTA_XI"; break;
  case XI_ITER_0 : result="XI_ITER_0"; break;
  case MASSE_RELAX_DYN : result="MASSE_RELAX_DYN"; break;
  case COMP_TORSEUR_REACTION : result="COMP_TORSEUR_REACTION"; break;
  case NUM_NOEUD : result="NUM_NOEUD"; break;
  case NUM_MAIL_NOEUD : result="NUM_MAIL_NOEUD"; break;
  case NUM_ELEMENT : result="NUM_ELEMENT"; break;
  case NUM_MAIL_ELEM : result="NUM_MAIL_ELEM"; break;
  case NUM_PTI : result="NUM_PTI"; break;
  case NUM_FACE : result="NUM_FACE"; break;
  case NUM_ARETE : result="NUM_ARETE"; break;

  default :
   cout << "\nErreur : valeur incorrecte du type EnumTypeQuelconque: id_TypeQuelconque="
        << id_TypeQuelconque << " !\n";
   cout << "NomGenerique(EnumTypeQuelconque ) \n";
   Sortie(1);
 };
 return result;
};

						
EnumTypeQuelconque Id_nomTypeQuelconque (const char* nom_TypeQuelconque)
// Retourne la variable de type enumere associe au nom nom_TypeQuelconque
{	// on vérifie si la variable de type enumere existe
	 map < string, EnumTypeQuelconque , std::less < string> >& maa=ClassPourEnumTypeQuelconque::map_EnumTypeQuelconque;
  map < string, EnumTypeQuelconque , std::less < string> >::iterator il,ilfin= maa.end();
  il = maa.find(nom_TypeQuelconque);
  if (il == ilfin)
	   {// on vérifie si la variable de type enumere existe
		   cout << "\nErreur : nom du type de grandeur  '" <<nom_TypeQuelconque <<  " 'inconnu !\n";
		   cout << "Id_nomTypeQuelconque(char* ) \n";
		   Sortie(1);
	   };
	 // retour de la grandeur 
	 return (*il).second;  
};

EnumTypeQuelconque Id_nomTypeQuelconque (const string& nom_TypeQuelconque)
// Retourne la variable de type enumere associe au nom nom_TypeQuelconque
{	// on vérifie si la variable de type enumere existe
	map < string, EnumTypeQuelconque , std::less < string> >& maa=ClassPourEnumTypeQuelconque::map_EnumTypeQuelconque;
 map < string, EnumTypeQuelconque , std::less < string> >::iterator il,ilfin= maa.end();
 il = maa.find(nom_TypeQuelconque);
 if (il == ilfin)
	   {// on vérifie si la variable de type enumere existe
		   cout << "\nErreur : nom du type de grandeur  '" <<nom_TypeQuelconque <<  " 'inconnu !\n";
		   cout << "Id_nomTypeQuelconque(string& ) \n";
// -- pour le débug ---
//cout << " \n le conteneur *** \n ";		
//for (il = maa.begin();il != ilfin; il++)
//  cout << "\n " << (*il).first ;		
// -- pour le débug ---
		   Sortie(1);
	   };
	 // retour de la grandeur 
	 return (*il).second;  
};

// indique si le string correspond à un type quelconque reconnu ou non
bool Existe_typeQuelconque(const string& nom)
{   return ( ClassPourEnumTypeQuelconque::map_EnumTypeQuelconque.find(nom) 
            != ClassPourEnumTypeQuelconque::map_EnumTypeQuelconque.end());};
	
// surcharge de la lecture
istream & operator >> (istream & entree, EnumTypeQuelconque & result)
 { char nom_TypeQuelconque[80];
   entree >> nom_TypeQuelconque;
   result = Id_nomTypeQuelconque ( nom_TypeQuelconque);
   return entree;
 };  

// Retourne le type de grandeur associée au type quelconque
EnumTypeGrandeur Type_de_grandeur_associee(EnumTypeQuelconque typa)
{switch (typa)
	{case RIEN_TYPEQUELCONQUE : return RIEN_TYPEGRANDEUR; break;
		case SIGMA_BARRE_BH_T : return TENSEURBH; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T_SANS_PROPORTION : return TENSEURHH; break;
		case CONTRAINTE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : return TENSEURHH; break;
		case CONTRAINTE_COURANTE : return TENSEURHH; break;
		case DEFORMATION_COURANTE : return TENSEURBB; break;
		case VITESSE_DEFORMATION_COURANTE : return TENSEURBB; break;
  case ALMANSI : return TENSEURBB; break; 
  case GREEN_LAGRANGE : return TENSEURBB; break; 
  case LOGARITHMIQUE : return TENSEURBB; break; 
  case DELTA_DEF : return TENSEURBB; break; 
  case ALMANSI_TOTAL : return TENSEURBB; break; 
  case GREEN_LAGRANGE_TOTAL : return TENSEURBB; break; 
  case LOGARITHMIQUE_TOTALE : return TENSEURBB; break; 
  case DEF_PRINCIPALES : return COORDONNEE; break; 
  case SIGMA_PRINCIPALES : return COORDONNEE; break; 
  case VIT_PRINCIPALES : return COORDONNEE; break;
  case DEF_DUALE_MISES : return SCALAIRE; break;
  case DEF_DUALE_MISES_MAXI : return SCALAIRE; break;
  case CONTRAINTE_MISES : return SCALAIRE; break;
  case CONTRAINTE_MISES_T : return SCALAIRE; break;
  case CONTRAINTE_TRESCA : return SCALAIRE; break;
  case CONTRAINTE_TRESCA_T : return SCALAIRE; break;
  case ERREUR_Q : return SCALAIRE; break;
  case DEF_PLASTIQUE_CUMULEE : return SCALAIRE; break;
  case ERREUR_SIG_RELATIVE : return SCALAIRE; break;
		case TEMPERATURE_LOI_THERMO_PHYSIQUE : return SCALAIRE; break;
		case PRESSION_LOI_THERMO_PHYSIQUE : return SCALAIRE; break;
		case TEMPERATURE_TRANSITION : return SCALAIRE; break;
		case VOLUME_SPECIFIQUE : return SCALAIRE; break;
		case FLUXD : return COORDONNEE; break;
		case GRADT : return COORDONNEE; break;
		case DGRADT : return COORDONNEE; break;
		case DELTAGRADT : return COORDONNEE; break;
		case COEFF_DILATATION_LINEAIRE : return SCALAIRE; break;
		case CONDUCTIVITE : return SCALAIRE; break;
		case CAPACITE_CALORIFIQUE : return SCALAIRE; break;
		case MODULE_COMPRESSIBILITE : return SCALAIRE; break;
		case COEFF_COMPRESSIBILITE : return SCALAIRE; break;
		case MODULE_CISAILLEMENT : return SCALAIRE; break;
		case MODULE_COMPRESSIBILITE_TOTAL : return SCALAIRE; break;
		case MODULE_CISAILLEMENT_TOTAL : return SCALAIRE; break;
  case E_YOUNG : return SCALAIRE; break;
  case NU_YOUNG : return SCALAIRE; break;
  case MU_VISCO : return SCALAIRE; break;
  case MU_VISCO_SPHERIQUE : return SCALAIRE; break;
  case MODULE_TANGENT_1D : return SCALAIRE; break;
  case COMPRESSIBILITE_TANGENTE : return SCALAIRE; break;
		case NB_INVERSION : return SCALAIRE; break;
		case HYPER_CENTRE_HYSTERESIS : return TENSEURBH; break;
		case SIGMA_REF : return TENSEURBH; break;
		case Q_SIG_HYST_Oi_A_R : return SCALAIRE; break;
		case Q_SIG_HYST_R_A_T : return SCALAIRE; break;
		case Q_DELTA_SIG_HYST : return SCALAIRE; break;
		case COS_ALPHA_HYSTERESIS : return SCALAIRE; break;
		case COS3PHI_SIG_HYSTERESIS : return SCALAIRE; break;
		case COS3PHI_DELTA_SIG_HYSTERESIS : return SCALAIRE; break;
		case FCT_AIDE : return SCALAIRE; break;
		case NB_ITER_TOTAL_RESIDU : return SCALAIRE; break;
		case NB_INCRE_TOTAL_RESIDU : return SCALAIRE; break;
		case NB_APPEL_FCT : return SCALAIRE; break;
		case NB_STEP : return SCALAIRE; break;
		case ERREUR_RK : return SCALAIRE; break;
		case PRESSION_HYST_REF : return SCALAIRE; break;
		case PRESSION_HYST_REF_M1 : return SCALAIRE; break;
		case PRESSION_HYST_T : return SCALAIRE; break;
		case PRESSION_HYST : return SCALAIRE; break;
		case UN_DDL_ENUM_ETENDUE : return SCALAIRE; break;
		case ENERGIE_ELASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : return SCALAIRE; break;
		case ENERGIE_PLASTIQUE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : return SCALAIRE; break;
		case ENERGIE_VISQUEUSE_INDIVIDUELLE_A_CHAQUE_LOI_A_T : return SCALAIRE; break;
		case PROPORTION_LOI_MELANGE : return SCALAIRE; break;
		case FONC_PONDERATION : return SCALAIRE; break;
		case POSITION_GEOMETRIQUE : return COORDONNEE; break;
  case POSITION_GEOMETRIQUE_t : return COORDONNEE; break;
  case POSITION_GEOMETRIQUE_t0 : return COORDONNEE; break;
		case CRISTALINITE : return SCALAIRE; break;
		case VOLUME_ELEMENT : return SCALAIRE; break;
		case VOLUME_PTI : return SCALAIRE; break;
		case EPAISSEUR_MOY_INITIALE : return SCALAIRE; break;
		case EPAISSEUR_MOY_FINALE : return SCALAIRE; break;
		case SECTION_MOY_INITIALE : return SCALAIRE; break;
		case SECTION_MOY_FINALE : return SCALAIRE; break;
  case EPAISSEUR_INITIALE : return SCALAIRE; break;
  case EPAISSEUR_FINALE : return SCALAIRE; break;
  case SECTION_INITIALE : return SCALAIRE; break;
  case SECTION_FINALE : return SCALAIRE; break;
		case VOL_ELEM_AVEC_PLAN_REF : return COORDONNEE; break;
		case INTEG_SUR_VOLUME : return VECTEUR; break;
		case INTEG_SUR_VOLUME_ET_TEMPS : return VECTEUR; break;
  case STATISTIQUE : return VECTEUR; break;
  case STATISTIQUE_ET_TEMPS : return VECTEUR; break;
		case ENERGIE_HOURGLASS : return SCALAIRE; break;
		case PUISSANCE_BULK : return SCALAIRE; break;
		case ENERGIE_BULK : return SCALAIRE; break;
		case ENERGIE_STABMEMB_BIEL : return SCALAIRE; break;
		case FORCE_STABMEMB_BIEL : return SCALAIRE; break;
		case TENSEUR_COURBURE : return TENSEURBB; break;
		case COURBURES_PRINCIPALES : return SCALAIRE; break;
		case DIRECTIONS_PRINC_COURBURE : return COORDONNEE; break;
		case DIRECTIONS_PRINC_SIGMA : return COORDONNEE; break;
		case DIRECTIONS_PRINC_DEF : return COORDONNEE; break;
		case DIRECTIONS_PRINC_D : return COORDONNEE; break;
		case REPERE_LOCAL_ORTHO : return COORDONNEE; break;
		case REPERE_LOCAL_H : return COORDONNEE; break;
		case REPERE_LOCAL_B : return COORDONNEE; break;
  case REPERE_D_ANISOTROPIE : return BASE__H; break;
  case EPS_TRANSPORTEE_ANISO : return TENSEURHH; break;
  case SIGMA_DANS_ANISO : return TENSEURHH; break;
  case DELTA_EPS_TRANSPORTEE_ANISO : return TENSEURHH; break;
  case DELTA_SIGMA_DANS_ANISO : return TENSEURHH; break;
  case PARA_ORTHO : return VECTEUR; break;
		case SPHERIQUE_EPS : return SCALAIRE; break;
		case Q_EPS : return SCALAIRE; break;
		case COS3PHI_EPS : return SCALAIRE; break;
		case SPHERIQUE_SIG : return SCALAIRE; break;
		case Q_SIG : return SCALAIRE; break;
		case COS3PHI_SIG : return SCALAIRE; break;
		case SPHERIQUE_DEPS : return SCALAIRE; break;
		case V_vol : return SCALAIRE; break;
		case Q_DEPS : return SCALAIRE; break;
		case COS3PHI_DEPS : return SCALAIRE; break;
		case POTENTIEL : return SCALAIRE; break;
  case FCT_POTENTIEL_ND : return VECTEUR; break;
  case INVAR_B1 : return SCALAIRE; break;
  case INVAR_B2 : return SCALAIRE; break;
  case INVAR_B3 : return SCALAIRE; break;
  case INVAR_J1 : return SCALAIRE; break;
  case INVAR_J2 : return SCALAIRE; break;
  case INVAR_J3 : return SCALAIRE; break;
		case DEF_EQUIVALENTE : return SCALAIRE; break;
		case DEF_EPAISSEUR : return SCALAIRE; break;
		case D_EPAISSEUR : return SCALAIRE; break;
		case DEF_LARGEUR : return SCALAIRE; break;
		case D_LARGEUR : return SCALAIRE; break;
  case DEF_MECANIQUE : return TENSEURBB; break;
  case DEF_ASSO_LOI : return TENSEURBB; break;
  case DEF_P_DANS_V_A : return VECTEUR; break;
		case SIG_EPAISSEUR : return SCALAIRE; break;
		case SIG_LARGEUR : return SCALAIRE; break;
		case FORCE_GENE_EXT : return COORDONNEE; break;
		case FORCE_GENE_INT : return COORDONNEE; break;
  case FORCE_GENE_EXT_t : return COORDONNEE; break;
  case FORCE_GENE_INT_t : return COORDONNEE; break;
  case FORCE_GENE_TOT : return COORDONNEE; break;
  case RESIDU_GLOBAL : return COORDONNEE; break;
		case VECT_PRESSION : return COORDONNEE; break;
  case PRESSION_SCALAIRE : return SCALAIRE; break;
		case VECT_FORCE_VOLUM : return COORDONNEE; break;
		case VECT_DIR_FIXE : return COORDONNEE; break;
		case VECT_SURF_SUIV : return COORDONNEE; break;
		case VECT_HYDRODYNA_Fn : return COORDONNEE; break;
		case VECT_HYDRODYNA_Ft : return COORDONNEE; break;
		case VECT_HYDRODYNA_T : return COORDONNEE; break;
		case VECT_LINE : return COORDONNEE; break;
		case VECT_LINE_SUIV : return COORDONNEE; break;
		case VECT_REAC : return COORDONNEE; break;
		case VECT_REAC_N : return COORDONNEE; break;
  case NN_11 : return SCALAIRE; break;
  case NN_22 : return SCALAIRE; break;
  case NN_33 : return SCALAIRE; break;
  case NN_12 : return SCALAIRE; break;
  case NN_13 : return SCALAIRE; break;
  case NN_23 : return SCALAIRE; break;
  case MM_11 : return SCALAIRE; break;
  case MM_22 : return SCALAIRE; break;
  case MM_33 : return SCALAIRE; break;
  case MM_12 : return SCALAIRE; break;
  case MM_13 : return SCALAIRE; break;
  case MM_23 : return SCALAIRE; break;
  case DIRECTION_PLI : return COORDONNEE; break;
  case DIRECTION_PLI_NORMEE : return COORDONNEE; break;
  case INDIC_CAL_PLIS : return SCALAIRE; break;
  case NN_SURF : return COORDONNEE; break;
  case NN_SURF_t : return COORDONNEE; break;
  case NN_SURF_t0 : return COORDONNEE; break;
  case NOEUD_PROJECTILE_EN_CONTACT : return SCALAIRE; break;
  case NOEUD_FACETTE_EN_CONTACT : return SCALAIRE; break;
  case GLISSEMENT_CONTACT : return COORDONNEE; break;
  case PENETRATION_CONTACT : return COORDONNEE; break;
  case GLISSEMENT_CONTACT_T : return COORDONNEE; break;
  case PENETRATION_CONTACT_T : return COORDONNEE; break;
  case FORCE_CONTACT : return COORDONNEE; break;
  case FORCE_CONTACT_T : return COORDONNEE; break;
  case CONTACT_NB_PENET : return SCALAIRE; break;
  case CONTACT_NB_DECOL : return SCALAIRE; break;
  case CONTACT_CAS_SOLIDE : return SCALAIRE; break;
  case CONTACT_ENERG_PENAL : return SCALAIRE; break;
  case CONTACT_COLLANT : return SCALAIRE; break;
  case NUM_ZONE_CONTACT : return SCALAIRE; break;
  case CONTACT_ENERG_GLISSE_ELAS : return SCALAIRE; break;
  case CONTACT_ENERG_GLISSE_PLAS : return SCALAIRE; break;
  case CONTACT_ENERG_GLISSE_VISQ : return SCALAIRE; break;
  case CONTACT_PENALISATION_N : return SCALAIRE; break;
  case CONTACT_PENALISATION_T : return SCALAIRE; break;
  case NORMALE_CONTACT : return COORDONNEE; break;
  case TEMPS_CPU_USER : return SCALAIRE; break;
  case TEMPS_CPU_LOI_COMP : return SCALAIRE; break;
  case TEMPS_CPU_METRIQUE : return SCALAIRE; break;
  case GENERIQUE_UNE_GRANDEUR_GLOBALE : return SCALAIRE; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_INT_UTILISATEUR : return SCALAIRE; break;
  case GENERIQUE_UNE_CONSTANTE_GLOB_DOUBLE_UTILISATEUR : return SCALAIRE; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_0 : return SCALAIRE; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_T : return SCALAIRE; break;
  case GENERIQUE_UNE_VARIABLE_GLOB_DOUBLE_UTILISATEUR_TDT : return SCALAIRE; break;
  case DEPLACEMENT : return COORDONNEE; break;
  case VITESSE : return COORDONNEE; break;
  case DELTA_XI : return COORDONNEE; break;
  case XI_ITER_0 : return COORDONNEE; break;
  case MASSE_RELAX_DYN : return COORDONNEE; break;
  case COMP_TORSEUR_REACTION : return SCALAIRE; break;
  case NUM_NOEUD : return SCALAIRE; break;
  case NUM_MAIL_NOEUD : return SCALAIRE; break;
  case NUM_ELEMENT : return SCALAIRE; break;
  case NUM_MAIL_ELEM : return SCALAIRE; break;
  case NUM_PTI : return SCALAIRE; break;
  case NUM_FACE : return SCALAIRE; break;
  case NUM_ARETE : return SCALAIRE; break;

		default :
			cout << "\nErreur : valeur incorrecte du type EnumTypeQuelconque: typa= " << typa << " !\n";
			cout << "Type_de_grandeur_associee(EnumTypeQuelconque ) \n";
			Sortie(1);
	};
	return RIEN_TYPEGRANDEUR; // pour taire le warning
};

// retourne le temps auquel la grandeur est définie
Enum_dure EnumTypeQuelconqueTemps(EnumTypeQuelconque typa) 
   {return ClassPourEnumTypeQuelconque::tt_TQ_temps(typa);};

// retour du type d'expression de la grandeur 
// = 1 indique si la grandeur est naturellement exprimé dans le repère globale
// = 0 indique que la grandeur est dans le repère naturelle (ou une combinaison)
int EnumTypeQuelconqueGlobale(EnumTypeQuelconque typa) 
   {return ClassPourEnumTypeQuelconque::tt_GLOB(typa);};
   	   
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnumTypeQuelconque& a)
    { // on ecrit la forme caractère
       sort << NomTypeQuelconque(a) << " ";
       return sort;      
    }; 

