/*! \file Enum_boolddl.h
    \brief def de l'enuméré concernant les booléens ddl.
*/
// FICHIER : Enum_boolddl.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, et une vérification de type plus aisé qu' avec
// les entiers 
// 10 carractere maxi


#ifndef ENUM_BOLLDDL_H
#define ENUM_BOLLDDL_H

//#include "Debug.h"
#include <iostream>
using namespace std;


/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les booléens ddl.

enum Enum_boolddl { LIBRE = 0, FIXE , HSLIBRE,HSFIXE,SOUSLIBRE,SURFIXE
                    ,LISIBLE_LIBRE,LISIBLE_FIXE,LISIBLE_SOUSLIBRE, LISIBLE_SURFIXE 
                    ,HS_LISIBLE_LIBRE,HS_LISIBLE_FIXE
						              ,HS_SOUSLIBRE , HS_SURFIXE , HS_LISIBLE_SOUSLIBRE , HS_LISIBLE_SURFIXE};
/// @}  // end of group

/// Retourne le nom d'un degre de liberte a partir de son identificateur de
/// type enumere id_ddl correspondant
string Nom_boolddl ( Enum_boolddl id_ddl);

/// Retourne l'identificateur de type enumere associe au nom du degre de liberte
/// nom_ddl
 Enum_boolddl Id_nom_boolddl (const string& nom_ddl);
 
/// dit si c'est HS ou pas
/// ramène true si HS, false sinon
bool Est_HS(Enum_boolddl id_ddl);  

/// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_boolddl& a);  
/// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_boolddl& a);

// pour faire de l'inline
#ifndef MISE_AU_POINT
  #include "Enum_boolddl.cc"
  #define  Enum_boolddl_deja_inclus
#endif

#endif
