/*! \file Enum_calcul_masse.h
    \brief def de l'enuméré concernant les types de calcul de masse
* \date      26/12/00
*/
// FICHIER : Enum_calcul_masse.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        26/12/00                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Enumeration des differents type de calcul de la masse.     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef ENUM_CALCUL_MASSE_H
#define ENUM_CALCUL_MASSE_H

#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{
///

/// def de l'enuméré concernant les types de calcul de masse

enum Enum_calcul_masse { MASSE_DIAG_COEF_EGAUX = 1, MASSE_DIAG_COEF_VAR , MASSE_CONSISTANTE };
/// @}  // end of group


// Retourne un nom de type de calcul_masse a partir de son identificateur de
// type enumere id_calcul_masse correspondant
char* Nom_calcul_masse (Enum_calcul_masse id_calcul_masse);

// Retourne l'identificateur de type enumere associe au nom du type
// de calcul_masse nom_calcul_masse
Enum_calcul_masse Id_nom_calcul_masse (char* nom_calcul_masse);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_calcul_masse& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_calcul_masse& a);


#endif


