// FICHIER : Enum_categorie_loi_comp.cp

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.



#include "Enum_categorie_loi_comp.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>



char* Nom_categorie_loi_comp (const Enum_categorie_loi_comp id_comport)
// Retourne le nom de la loi de comportement correspondant a l'identificateur 
// de type enumere id_comport
{
	
   char* result;
	switch (id_comport)
	{
		case CAT_MECANIQUE : result="CAT_MECANIQUE"; break;
		case CAT_THERMO_MECANIQUE : result="CAT_THERMO_MECANIQUE"; break;
		case CAT_THERMO_PHYSIQUE : result="CAT_THERMO_PHYSIQUE"; break;
		case CAT_FROTTEMENT : result="CAT_FROTTEMENT"; break;
		case RIEN_CATEGORIE_LOI_COMP :result="RIEN_CATEGORIE_LOI_COMP"; break;	
		default :
			cout << "\nErreur : valeur incorrecte du type Enum_categorie_loi_comp !: " << id_comport << " \n";
			cout << "Nom_categorie_loi_comp(Enum_categorie_loi_comp ) \n";
			Sortie(1);
	};
	return result;
	
};
						
Enum_categorie_loi_comp Id_nom_categorie_loi_comp (const char* nom_comport)
// Retourne la variable de type enumere associee au nom de la loi 
// de comportement nom_comport
{
	
	Enum_categorie_loi_comp result;
	if ( strcmp(nom_comport,"CAT_MECANIQUE")==0 ) result=CAT_MECANIQUE;
	else if ( strcmp(nom_comport,"CAT_THERMO_MECANIQUE")==0 ) result=CAT_THERMO_MECANIQUE;
	else if ( strcmp(nom_comport,"CAT_THERMO_PHYSIQUE")==0 ) result=CAT_THERMO_PHYSIQUE;
	else if ( strcmp(nom_comport,"CAT_FROTTEMENT")==0 ) result=CAT_FROTTEMENT;
    else if ( strcmp(nom_comport,"RIEN_CATEGORIE_LOI_COMP")==0 ) result=RIEN_CATEGORIE_LOI_COMP;
	else
	{
		cout << "\nErreur : nom de loi de comportement inconnu !: " << nom_comport << " \n";
		cout << "Id_nom_categorie_loi_comp(char* ) \n";
		Sortie(1);
	};
	return result;
	
};


// indique si oui ou non la categorie est mécanique c'est-à-dire satisfait à Loi_comp_abstraite
bool GroupeMecanique(const Enum_categorie_loi_comp id_comport)
  { if (id_comport <= 2) {return true;} else {return false;};
   };

// indique si oui ou non la categorie est thermique c'est-à-dire satisfait à CompThermoPhysiqueAbstraite
bool GroupeThermique(const Enum_categorie_loi_comp id_comport)
  { if (id_comport == 3) {return true;} else {return false;};
   };
   
// indique si oui ou non la categorie est frottement (de contact) c-a-d lié à CompFrotAbstraite
bool GroupeFrottement(const Enum_categorie_loi_comp id_comport)
  { if (id_comport == 4) {return true;} else {return false;};
   };
   	
// ramène un énuméré qui correspond à la catégorie la plus complète des deux passés en arguement
// par exemple la catégorie CAT_THERMO_MECANIQUE englobe la catégorie CAT_MECANIQUE
// au paravant on test si les deux sont du même groupe, sinon erreur
Enum_categorie_loi_comp Categorie_loi_comp_la_plus_complete(const Enum_categorie_loi_comp id_comport1,
                                                            const Enum_categorie_loi_comp id_comport2)
  {// tout d'abord on vérifie qu'ils sont du même groupe
   bool meme_groupe=false;
   if (id_comport1 == id_comport2) return id_comport1; // cas pas de pb
   if (GroupeMecanique(id_comport1) && GroupeMecanique(id_comport2)) // cas du groupe méca
     { if (id_comport1 > id_comport2) {return id_comport1;} else {return id_comport2;};}
   else if (GroupeThermique(id_comport1) && GroupeThermique(id_comport2))
      { if (id_comport1 > id_comport2) {return id_comport1;} else {return id_comport2;};}
   else if (GroupeFrottement(id_comport1) && GroupeFrottement(id_comport2))
      { if (id_comport1 > id_comport2) {return id_comport1;} else {return id_comport2;};}
   else
    { cout << "\n erreur, les deux categories de loi ne sont pas identique"
           << " Enum_categorie_loi_comp Categorie_loi_comp_la_plus_complete(...";
      Sortie(1);
      return RIEN_CATEGORIE_LOI_COMP;
      };     
   };                                                          
// test si deux énumérés correspondent à la même catégorie
bool MemeCategorie(const Enum_categorie_loi_comp id_comport1,const Enum_categorie_loi_comp id_comport2)
 { if (GroupeMecanique(id_comport1) && GroupeMecanique(id_comport2)) return true;
   if (GroupeThermique(id_comport1) && GroupeThermique(id_comport2)) return true;
   if (GroupeFrottement(id_comport1) && GroupeFrottement(id_comport2)) return true;
   return false;
  };
   
   // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_categorie_loi_comp& a)
 { char nom_Enum_comp[50];
   entree >> nom_Enum_comp;
   a = Id_nom_categorie_loi_comp ( nom_Enum_comp);
   return entree;
 };
   // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_categorie_loi_comp& a)
    { // on ecrit la forme caractère
       sort << Nom_categorie_loi_comp(a) << " ";
       return sort;      
     }; 


	
