// FICHIER : Enum_chargement.cc


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "Enum_chargement.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>



string Nom_chargement (const Enum_chargement id_chargement)
// Retourne le nom du type de chargement correspondant a l'identificateur
// de type enumere id_chargement
{
	
	string result;
	switch (id_chargement)
	{
		case RIEN_CHARGEMENT : result="RIEN_CHARGEMENT"; break;
		case UNIFORME : result="UNIFORME"; break;
		case PRESSION : result="PRESSION"; break;
		case PONCTUELLE : result="PONCTUELLE"; break;
		case PRESSDIR : result="PRESSDIR"; break;
		case PHYDRO : result="PHYDRO"; break;
		case LINEIQUE : result="LINEIQUE"; break;
		case VOLUMIQUE : result="VOLUMIQUE"; break;
		case LINEIC_SUIVEUSE : result="LINEIC_SUIVEUSE"; break;
		case P_HYDRODYNA : result="P_HYDRODYNA"; break;
		case TORSEUR_PONCT : result="TORSEUR_PONCT"; break;
		default :
			cout << "\nErreur : valeur incorrecte du type Enum_chargement !\n";
			cout << "NOM_CHARGEMENT(Enum_chargement ) \n";
			Sortie(1);
	};
	return result;
	
};
						
Enum_chargement Id_nom_chargement (const string& nom_chargement)
// Retourne la variable de type enumere associe au nom
// de chargement nom_chargement
{
	
	Enum_chargement result;
	if (  nom_chargement == "RIEN_CHARGEMENT"  )
		result=RIEN_CHARGEMENT;
	else if (  nom_chargement == "UNIFORME"  )
		result=UNIFORME;
	else if (  nom_chargement == "PRESSION"  )
		result=PRESSION;
	else if (  nom_chargement == "PONCTUELLE"  )
		result=PONCTUELLE;
	else if (  nom_chargement == "PRESSDIR"  )
		result=PRESSDIR;
	else if (  nom_chargement == "PHYDRO"  )
		result=PHYDRO;
	else if (  nom_chargement == "LINEIQUE"  )
		result=LINEIQUE;
	else if (  nom_chargement == "VOLUMIQUE"  )
		result=VOLUMIQUE;
	else if (  nom_chargement == "LINEIC_SUIVEUSE"  )
		result=LINEIC_SUIVEUSE;
	else if (  nom_chargement == "P_HYDRODYNA"  )
		result=P_HYDRODYNA;
	else if (  nom_chargement == "TORSEUR_PONCT"  )
		result=TORSEUR_PONCT;
	else
	{
		cout << "\nErreur : nom de chargement inconnu !\n";
		cout << "ID_NOM_CHARGEMENT(string ) \n";
		Sortie(1);
	};
	return result;
	
};

// surcharge de la lecture
istream & operator >> (istream & entree, Enum_chargement & result)
 { char nom_chargement[35];
   entree >> nom_chargement;
   result = Id_nom_chargement ( nom_chargement);
   return entree;
 };  
 
    
   // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_chargement& a)
    { // on ecrit la forme caractère
       sort << Nom_chargement(a) << " ";
       return sort;      
     }; 
	
