/*! \file Enum_contrainte_mathematique.h
    \brief def de l'enuméré concernant les types de contraintes mathématiques
*/
// FICHIER : Enum_contrainte_mathematique.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des methodes permettant de mettre en place une contrainte
// mathematique  sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_contrainte_mathematique et Id_nom_contrainte_mathematique rendent
// possible le lien entre les noms des methodes et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_CONTRAINTE_MATHEMATIQUE_H
#define ENUM_CONTRAINTE_MATHEMATIQUE_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de contraintes mathématiques

enum Enum_contrainte_mathematique { MULTIPLICATEUR_DE_LAGRANGE=1,PENALISATION, NEWTON_LOCAL,PERTURBATION,RIEN_CONTRAINTE_MATHEMATIQUE};
/// @}  // end of group


// ***** !!!! penser à changer  *****
//****** nbmax_caractere_Enum_contrainte_mathematique si nécessaire *****
const int nbmax_caractere_Enum_contrainte_mathematique = 50; 

// Retourne le nom d'une méthode a partir de son identificateur de
// type enumere id_contrainte_mathematique correspondant
string Nom_contrainte_mathematique (const Enum_contrainte_mathematique id_contrainte_mathematique);

// Retourne l'identificateur de type enumere associe au nom de la méthode
//  nom_contrainte_mathematique
Enum_contrainte_mathematique Id_nom_contrainte_mathematique (const string& nom_contrainte_mathematique);


// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_contrainte_mathematique& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_contrainte_mathematique& a);

#endif
