/*! \file Enum_interpol.h
    \brief Définition de l'enuméré concernant les différents types d'interpolation
*/
// FICHIER : Enum_interpol.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms d'interpolation des elements
// sont stockes a l'aide d'un type enumere. Les fonctions Nom_interpol et 
// Id_nom_interpol rendent possible le lien entre les noms des types d'interpolation
// et les identificateurs de type enumere correspondants.


#ifndef ENUM_INTERPOL_H
#define ENUM_INTERPOL_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les différents types d'interpolation

enum Enum_interpol { RIEN_INTERPOL = 1,CONSTANT
           , LINEAIRE, QUADRATIQUE, QUADRACOMPL, CUBIQUE, CUBIQUE_INCOMPL
	          , LINQUAD, HERMITE, SFE1,SFE2,SFE3, SFE3C, QSFE3, QSFE1
           , SFE1_3D,SFE2_3D,SFE3_3D, SFE3C_3D, SEG1, BIE1, BIE2};
/// @}  // end of group


//*** si on ajoute un élément, il faut penser à complèter l'énumération: Enum_PiPoCo					  

// Retourne un nom d'interpolation a partir de son identificateur de
// type enumere id_interpol correspondant
string Nom_interpol (const Enum_interpol id_interpol) ;

// Retourne l'identificateur de type enumere associe au nom du type
// d'interpolation nom_interpol
Enum_interpol Id_nom_interpol (const string& nom_interpol);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_interpol& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_interpol& a);


#endif


