/*! \file Enum_type_geom.h
    \brief def de l'enuméré concernant les types de géométrie
*/
// FICHIER : Enum_type_geom.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des geometrie des elements sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_type_geom et Id_nom_type_geom rendent
// possible le lien entre les noms des geometries et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_TYPE_GEOM_H
#define ENUM_TYPE_GEOM_H
#include <iostream>
using namespace std;


/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de géométrie

enum Enum_type_geom {  POINT_G = 1 , LIGNE , SURFACE, VOLUME, RIEN_TYPE_GEOM };
/// @}  // end of group


// Retourne le nom d'une geometrie a partir de son identificateur de
// type enumere id_type_geom correspondant
string Nom_type_geom (const Enum_type_geom id_type_geom);

// Retourne l'identificateur de type enumere associe au nom de geometrie
// nom_type_geom
Enum_type_geom Id_nom_type_geom (const string& nom_type_geom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_type_geom& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_type_geom& a);

#endif
