/*! \file Enum_type_resolution_matri.h
    \brief Enumeration des differents type de résolution de systèmes  matricielle possible, ainsi que du préconditionnement  éventuel.
* \date      03/01/01
*/

// FICHIER : Enum_type_resolution_matri.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        03/01/01                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Enumeration des differents type de résolution de systèmes  *
 *           matricielle possible, ainsi que du préconditionnement      *
 *           éventuel.                                                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef ENUM_TYPE_RESOLUTION_MATRI_H
#define ENUM_TYPE_RESOLUTION_MATRI_H

#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des differents type de résolution de systèmes  matricielle possible.

enum Enum_type_resolution_matri { RIEN_TYPE_RESOLUTION_MATRI = 1,CHOLESKY, SYMETRISATION_PUIS_CHOLESKY, GAUSS, GAUSS_EXPERT,
   BI_CONJUG, BI_CONJUG_STAB, CONJUG_GRAD, CONJUG_GRAD_SQUARE, CHEBYSHEV, GENE_MINI_RESIDUAL
   ,ITERATION_RICHARSON, QUASI_MINI_RESIDUAL ,DIRECT_DIAGONAL, CRAMER, LU_EQUILIBRE};
   /// @}  // end of group


/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des differents type de préconditionnement  éventuel.

enum Enum_preconditionnement { RIEN_PRECONDITIONNEMENT = 1, DIAGONAL, ICP, ILU };
   /// @}  // end of group


   
//  ===================================================================
//   cas du type de résolution
//  ===================================================================

// Retourne un nom de type de résolution a partir de son identificateur de
// type enumere id_resolution correspondant
string Nom_resolution (Enum_type_resolution_matri id_resolution);

// Retourne l'identificateur de type enumere associe au nom du type
// de resolution nom_resolution
Enum_type_resolution_matri Id_nom_resolution (string& nom_resolution);

// indique si le type existe ou pas, -> 1 ou 0
int Existe_Enum_type_resolution_matri(string& nom_resolution);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_type_resolution_matri& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_type_resolution_matri& a);

//  ===================================================================
//   cas du type de préconditionnement
//  ===================================================================

// Retourne un nom de type de preconditionnement a partir de son identificateur de
// type enumere id_preconditionnement correspondant
string Nom_preconditionnement (Enum_preconditionnement id_preconditionnement);

// Retourne l'identificateur de type enumere associe au nom du type
// de preconditionnement nom_preconditionnement
Enum_preconditionnement Id_nom_preconditionnement (string nom_preconditionnement);

// indique si le type existe ou pas, -> 1 ou 0
int Existe_Enum_preconditionnement(string& nom_preconditionnement);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_preconditionnement& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_preconditionnement& a);

#endif


