/*! \file Enum_type_stocke_deformation.h
    \brief Définir une énumération pour les type de stockage pour  les données de déformations à chaque pt de gauss.
* \date      25/mai/2007
*/

// FICHIER : Enum_type_stocke_deformation.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *     DATE:        25/mai/2007                                         *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Définir une énumération pour les type de stockage pour     *
 *           les données de déformations à chaque pt de gauss.          *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
// Afin de realiser un gain en place memoire, et une vérification de type plus aisé qu' avec
// les entiers 
// 30 carracteres maxi


#ifndef ENUM_TYPE_STOCKE_DEFORMATION_H
#define ENUM_TYPE_STOCKE_DEFORMATION_H

//#include "Debug.h"
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définir une énumération pour les type de stockage pour  les données de déformations à chaque pt de gauss.

enum Enum_type_stocke_deformation { SAVEDEFRESUL_GENERAL = 0, SAVEDEFRESUL_SFE1 };
/// @}  // end of group


// Retourne le nom du type a partir de son identificateur de
// type enumere id_ddl correspondant
char* Nom_type_stockage_def ( Enum_type_stocke_deformation id_ddl);

// Retourne l'identificateur de type enumere associe au nom du type
 Enum_type_stocke_deformation Id_nom_type_stockage_def (char* nom_ddl);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_type_stocke_deformation& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_type_stocke_deformation& a);

// pour faire de l'inline
#ifndef MISE_AU_POINT
  #include "Enum_type_stocke_deformation.cc"
  #define  Enum_type_stocke_deformation_deja_inclus
#endif

#endif
