// FICHIER : ReferencePtiAF.h
// CLASSE : ReferencePtiAF

 /************************************************************************
 *            UNIVERSITE DE BRETAGNE SUD (UBS) --- LORIENT              *
 ************************************************************************
 *           IRDL - Equipe DE GENIE MECANIQUE ET MATERIAUX (EG2M)       *
 * Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
 * tel. 02.97.32.82.47                 http://IRDL.fr                   *
 ************************************************************************
 *     DATE:        22/04/2020                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  Tel 0297874571   fax : 02.97.87.45.72               *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Def, stockage et manipulation des références pour les      *
 *           pti de faces et d'arêtes   .                               *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef REFERENCEPTIAF_H
#define REFERENCEPTIAF_H

#include "Reference.h"


class ReferencePtiAF : public Reference
{
	public :

		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		ReferencePtiAF ( string nom = "rien_actuellement");
		
		// indic = 1 : il s'agit de reference de noeud
		// indic = 2 : il s'agit de reference d'element
		// =3 -> surface relatives à des éléments, 
		// =4 -> arête relatives à des éléments,
		// =5 -> des noeuds relatifs à des éléments
		// =6 -> de points d'intégrations relatifs à des éléments
  // =7 -> de points d'intégrations relatifs à des surfaces d'éléments
  // =8 -> de points d'intégrations relatifs à des arrete d'éléments

		// Constructeur fonction  du nb de maillage et du type de ref		
		ReferencePtiAF (int nbmaille , int indic);

		// Constructeur fonction  de trois  tableaux de numeros, le premier pour les numéros
		// d'éléments le deuxième pour les numéros de faces, ou d'aretes,
		// le troisième pour le nb du pt d'integ,
  // puis le  nb de maillage, et enfin le type de ref
		ReferencePtiAF (const Tableau<int>& tabelem,const Tableau<int>& tabAF,const Tableau<int>& tabPti
		             ,int nbmaille , int indic, string nom = "rien_actuellement");
		// idem mais avec des listes d'entiers plutôt que des tableaux
		ReferencePtiAF (const list <int>& list_elem,const list <int>& list_numAF
               ,const list <int>& list_num_pti
		             ,int nbmaille , int indic, string nom = "rien_actuellement");
		
		// Constructeur fonction d'un nom de  reference
		// du nb de maillage, du type de ref		
		ReferencePtiAF(string nom,int nbmaille , int indic);
				
		// Constructeur de copie
		ReferencePtiAF(const ReferencePtiAF& ref);
		
		
		// DESTRUCTEUR :
		
		~ReferencePtiAF();
		
		
		// METHODES :
		
		// création d'une référence du même type contenant les mêmes info
		// que la référence donnée en argument: utilisation du constructeur de copie
		Reference* Nevez_Ref_copie() const
		  {ReferencePtiAF* ref_ret=new ReferencePtiAF(*this); return ref_ret;} ;

		Reference& operator= (const Reference& ref);
		
		// Retourne le tableau des numeros d'élément
		const Tableau<int>& Tab_Elem() const
		  {	return tab_Elem;	};
		
		// Retourne le tableau des numeros de faces, ou d'arêtes, ou de noeud d'element, ou de points d'integ
		const Tableau<int>& Tab_FA() const
		  {	return tab_FA;	};

  // Retourne le tableau des numeros des points d'integ
  const Tableau<int>& Tab_Pti() const
    { return tab_pti; };

		// Retourne le ieme element du tableau d'éléments
		// de la reference (acces lecture uniquement)
		int NumeroElem(int i) const 
		    {	return tab_Elem(i);	};
		
		// Retourne le ieme element du tableau de faces, ou d'arêtes, ou de noeud d'element, ou de points d'integ
		// de la reference (acces lecture uniquement)
		int NumeroFA(int i) const 
		    {	return tab_FA(i);	};
		
  // Retourne le ieme element du tableau de points d'integ
  // de la reference (acces lecture uniquement)
  int NumeroPti(int i) const
      { return tab_pti(i); };
  

		// Retourne le nombre de numeros de la reference
		int Taille ()  const 
		      {	return tab_Elem.Taille();	};
		      
		// change un numéro i de référence
		// nbe: le numéro d'élément, nbAF: le numéro de de faces, ou d'arêtes, nbPti
		//       le num de points d'integ
		void Change_num_PtiAF_dans_ref(int i,int nbe, int nbAF, int nbPti);
			
  // change les 3 tableaux de la ref
		void Change_tab_num(const Tableau<int>& tabele,const Tableau<int>& tabAF
                      ,const Tableau<int>& tabPti);

		// indique si le numéro d'élément, le numéro de faces ou d'arêtes, et le num de points
		// d'integ, passé  en argument, fait partie de la référence
		// nbe: le numéro d'élément, nbAF: le numéro de de faces, ou d'arêtes,
		//       nbPti le num  de points d'integ
		bool Contient_PtiAF(int nbe, int nbAF, int nbPti) const;
 
  // supprime les doublons internes éventuels dans la référence
  virtual void Supprime_doublons_internes();

		// Affiche les donnees liees a la reference
		void Affiche () const; 
			
//-------- méthodes découlant de virtuelles ----------------
		
	 // Affiche les donnees des références dans le flux passé en paramètre
	 void Affiche_dans_lis(ofstream&  sort) const  ;
  
  // lecture d'une liste de reference 
  bool LectureReference(UtilLecture & entreePrinc);
  
  // affichage et definition interactive des commandes 
  // nbMaxi: nombre maxi d'éléments pour les exemples
  // cas : =1 premier passage, il s'agit de références de noeuds uniquement, 
  //       donc ici cela génére une erreur
  // cas : =2 second passage, il s'agit de l'ensemble des possibilités de références
  void Info_commande_Ref(int nbMaxi,UtilLecture * entreePrinc,int cas);
           
	 //----- lecture écriture dans base info -----
  // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info(ifstream& ent,const int cas);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info(ofstream& sort,const int cas);
        
	protected :
		
		Tableau<int> tab_Elem; // tableau des numeros des éléments de la reference	
		Tableau<int> tab_FA; // tableau des numeros des faces ou arêtes de la reference
  Tableau<int> tab_pti; // tableau des points d'intégrations
		
};

#ifndef MISE_AU_POINT
  #include "ReferencePtiAF.cc"
  #define  REFERENCEPTIAF_H_deja_inclus
#endif

#endif
		
				
