
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "Visualisation_geomview.h"
#include <stdlib.h>
#include <iomanip>
// les ordres possibles
#include "Mail_initiale_geomview.h"
#include "Deformees_geomview.h"
#include "Fin_geomview.h"
#include "Visuali_geomview.h"
#include "Increment_vrml.h"
#include "ChoixDesMaillages_vrml.h"
#include "Animation_geomview.h"
#include "Isovaleurs_geomview.h"
#include "Frontiere_initiale_geomview.h"
#include "Banniere.h"

    // CONSTRUCTEURS :
Visualisation_geomview::Visualisation_geomview ()   // par defaut
  {  cout << "\n  ce constructeur ne doit pas etre utilise !! \n" ;
     cout << " Visualisation_geomview::Visualisation_geomview () // par defaut " << endl;
  }; 
    
// le bon constructeur
Visualisation_geomview::Visualisation_geomview (UtilLecture* ent) :
 lesValVecPropres(),ordre_possible(),entreePrinc(ent)
 ,fin_o(NULL),visuali(NULL),choix_inc(NULL)
 ,list_incre(),list_balaie(NULL),boite()
 ,animation(false)
 ,activ_sort_geomview(false)
  { // définition des ordres possibles
    list <OrdreVisu>::iterator it;
    OrdreVisu* ptordre;
    // maillage initial
    ptordre = new Mail_initiale_geomview(); ordre_possible.push_back(ptordre);
    // frontière du maillage initial
    ptordre = new Frontiere_initiale_geomview(); ordre_possible.push_back(ptordre);
    // Choix de la visualisation des isovaleurs
    //NB: il faut que l'isovaleur soit avant la déformée !! pour que la déformée puisse
    // utiliser les couleurs définis dans isovaleurs 
    ptordre = new Isovaleurs_geomview(); ordre_possible.push_back(ptordre);
    choix_isovaleur = ptordre; // récup de l'ordre
    // déformée
    ptordre = new Deformees_geomview(); ordre_possible.push_back(ptordre);
    ptdeformee = ptordre; // récup de l'ordre
    // Choix du ou des incréments à visualiser. 
    ptordre = new Increment_vrml(); ordre_possible.push_back(ptordre);
    choix_inc = ptordre; // récup de l'ordre 
    // Choix du ou des maillages à visualiser. 
    ptordre = new ChoixDesMaillages_vrml(); ordre_possible.push_back(ptordre);
    choix_mail = ptordre; // récup de l'ordre 
    // gestion de l'animation si nécessaire (doit être à la fin) dans la liste.
    // juste avant les 2 arrêts possibles 
    ptordre = new Animation_geomview(); ordre_possible.push_back(ptordre);
    anim = ptordre; // récup de l'ordre
    // transmission de la position à la déformée et à l'animation qui en ont besoin pour les couleurs
    ((Deformees_geomview*) ptdeformee)->Jonction_isovaleur((Isovaleurs_geomview*)choix_isovaleur);
    ((Animation_geomview*) anim)->Jonction_isovaleur((Isovaleurs_geomview*)choix_isovaleur);
    // transmission du choix des maillages à l'animation et à la déformée qui en ont besoin
    ((Animation_geomview*) anim)->Jonction_ChoixDesMaillages((ChoixDesMaillages_vrml*)choix_mail);
    ((Deformees_geomview*) ptdeformee)->Jonction_ChoixDesMaillages((ChoixDesMaillages_vrml*)choix_mail);
    // transmission de la deformée à l'animation qui en a besoin
    ((Animation_geomview*) anim)->Jonction_Deformees_geomview((Deformees_geomview*)ptdeformee);
    // ordre de visualisation
    ptordre = new Visuali_geomview(); ordre_possible.push_back(ptordre);
    visuali = ptordre; // récup de l'ordre 
    // fin de la visualisation
    ptordre = new Fin_geomview(); ordre_possible.push_back(ptordre);
    fin_o = ptordre; // récup de l'ordre fin
    
   /* string ordre,comment;
    comment = "évolution force/déplacement : "; ordre = "evolFD";
    commentaires_ordres.push_back(comment); ordre_possible.push_back(ordre);*/
   };
    
    // DESTRUCTEUR :
Visualisation_geomview::~Visualisation_geomview ()
  { // destruction de la liste d'ordre
    list <OrdreVisu*>::iterator il,ifin;
    ifin = ordre_possible.end();
    for (il=ordre_possible.begin();il!=ifin;il++)
      delete (*il);
    };

    // METHODES PUBLIQUES :
    
// affichage des différentes possibilités (ordres possibles)
// ramène un numéro qui renseigne le programme appelant
// =-1  : signifie que l'on veut stopper la visualisation
// = 0  : signifie que l'on demande la visualisation effective
int Visualisation_geomview::OrdresPossible()
{    // si l'on passe ici cela signifie que la visualisation geomview est active
     activ_sort_geomview= true;
     string reponse; // réponse utilisateur
     // on commence par inactiver les ordres de visualisation et de fin
     fin_o->Inactive_ordre(); visuali->Inactive_ordre();
     // on boucle
     while ( !(fin_o->Actif()) && !(visuali->Actif()))
      { 
        while ((reponse.length () == 0) || (Existe(reponse)==NULL))
        { // affichage des options possibles
          Affiche_options();
          // lecture de la commande voulu
          reponse = lect_chaine();
          OrdreVisu* ord = Existe(reponse);
          if (ord == NULL)
           { // dans le cas d'un ordre non connu
             cout << "\n ordre non connu !!, verifiez votre demande \n";
            }
          else
           // demande d'activation de l'ordre 
            ord->ChoixOrdre();
          }
       // on vide la réponse pour une autre boucle éventuelle 
       reponse ="";    
       }
      // récupération de la liste des incréments à visualiser
      list_balaie = &((Increment_vrml*)choix_inc)->List_choisit();
      // def du retour
      if (fin_o->Actif())
        return -1;
      else
        return 0;   
   };
   
// information de l'instance de la liste d'incrément disponible pour la visualisation
void Visualisation_geomview::List_increment_disponible(list <int> & list_incr) 
  { list_incre = list_incr; // stockage interne
    // puis passage à l'instance d'ordre qui gère les incréments
    ((Increment_vrml*)choix_inc)->Init_list_inc(list_incre);
    } ;

// indique le choix de la liste d'incrément à utiliser pour l'initialisation
// des fonctions d'initialisation
const list<int> &  Visualisation_geomview::List_balaie_init()
 {  // choix entre tous les incréments : option par défaut 
    // ou une liste plus restreinte d'incrément, utile lorsqu'il y a vraiment 
    // beaucoup d'incrément    
    bool choix_valide = false;
    string rep;
    while (!choix_valide)
      { 
        cout << "\n === choix des increments utilises pour l'initialisation de la visualisation ===="
             << "\n option par defaut : tous les increments             (rep 1)"
             << "\n choix d'un nombre plus petit d'increment             (rep 2)";
        cout << "\n reponse ? "; rep = lect_return_defaut(true,"1");cout << "\n";
        if ((rep == "1") || (rep == "2"))
          // cas d'un choix valide
          choix_valide = true;
        else
          cout << "\n la reponse : " << rep << " n'est pas utilisable, recommencez !";         
       }
    // exécution   
    if (rep == "1")
      // cas de tous les incréments
      return list_incre;
    else if (rep == "2")
      // cas d'un choix d'incrément, appel du choix d'incrément
      { ((Increment_vrml*)choix_inc)->ChoixOrdre();
        return (((Increment_vrml*)choix_inc)->List_choisit());                    
       }
    // pour éviter un warning, car normalement on ne passe jamais ici
    return list_incre;
  }; 

// impose une liste d'incrément à utiliser 
void  Visualisation_geomview::List_balaie_init(const list<int> & )
//void  Visualisation_geomview::List_balaie_init(const list<int> & list_init)
  {  ((Increment_vrml*)choix_inc)->Impos_list(list_incre);
     // récupération de la liste des incréments à visualiser
     list_balaie = &((Increment_vrml*)choix_inc)->List_choisit();     
   };  

// information de l'instance du nombre de maillages disponibles pour la visualisation
// par défaut tous les maillages seront visualisés, ceci est descidé aussi ici
void Visualisation_geomview::List_maillage_disponible(int nombre_maillage_dispo) 
   { nb_maillage_dispo = nombre_maillage_dispo;
     // on défini la liste de tous les maillages par défaut
     ((ChoixDesMaillages_vrml*)choix_mail)->Init_nb_maill(nb_maillage_dispo);
    };
    
// initialisation des ordres disponibles 
// par exemple pour les isovaleurs on définit la liste des isovaleurs disponibles et les extrémas
void Visualisation_geomview::Initialisation(ParaGlob * paraGlob,LesMaillages * lesMaillages
                       ,LesReferences* lesReferences
                       ,LesLoisDeComp* lesLoisDeComp,DiversStockage* diversStockage
                       ,Charge* charge,LesCondLim* lesCondLim,LesContacts* lesContacts
                       ,Resultats* resultats,OrdreVisu::EnumTypeIncre type_incre,int incre
                       ,const map < string, const double * , std::less <string> >& listeVarGlob
                       ,const List_io < TypeQuelconque >& listeVecGlob
                       ,bool fil_calcul)
 {
   // on passe en revue l'ensemble des ordres possibles
   list <OrdreVisu*>::iterator s_or,s_or_fin;
   s_or_fin = ordre_possible.end();
   for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
        (*s_or)->Initialisation(paraGlob,lesMaillages,lesReferences
                      ,lesLoisDeComp ,diversStockage
                      ,charge,lesCondLim,lesContacts,resultats,type_incre,incre
                      ,listeVarGlob,listeVecGlob
                      ,fil_calcul);
 };

// méthode principale pour activer la visualisation
void Visualisation_geomview::Visu(ParaGlob * paraGlob,LesMaillages * lesMaillages,LesReferences* lesReferences
                       ,LesLoisDeComp* lesLoisDeComp,DiversStockage* diversStockage
                       ,Charge* charge,LesCondLim* lesCondLim,LesContacts* lesContacts
                       ,Resultats* resultats,OrdreVisu::EnumTypeIncre type_incre,int incre
                       ,const map < string, const double * , std::less <string> >& listeVarGlob
                       ,const List_io < TypeQuelconque >& listeVecGlob)
   
{  // récup de la liste de maillage
   const list <int>& li_mail =  ((ChoixDesMaillages_vrml*)choix_mail)->List_choisit(); 
   list <int>::const_iterator ik,ikfin;
   ikfin = li_mail.end();
   // on définit un tableau des maillages à sortir, plus pratique pour les itérations
   // dans les routines internes (et plus rapide)
   Tableau <int> tab_mail(li_mail.size()); int iidd = 1;
   for (ik= li_mail.begin();ik!=ikfin;ik++,iidd++) tab_mail(iidd)=*ik;
   // initialisation de l'animation :
   // s'il y a plus d'un seul incrément différent de zéro on est en animation
   if (type_incre==OrdreVisu::PREMIER_INCRE) 
   { list <int>::const_iterator  il,ilfin=list_balaie->end();
     if (*(list_balaie->begin())==0)
       // cas du premier incrément = maillage initial
        if(list_balaie->size() > 2) animation = true; else animation = false;
     else
       // il n'y a que des incrément différent de l'initial
        if(list_balaie->size() > 1) animation = true; else animation = false;
     }
             
   // ici on passe en revue l'ensemble des ordres possibles
   list <OrdreVisu*>::iterator s_or,s_or_fin;
   s_or_fin = ordre_possible.end();
   // s'il n'y a qu'un seul incrément à visualisé on le signale
   bool unseul_incre = true; if (list_balaie->size() > 1) unseul_incre = false;
   // s'il s'agit du premier incrément visualisé et que l'on est en animation
   // on le signale
   for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
       { if ((*s_or) != anim) // l'animation n'est appelé qu'à la fin
            (*s_or)->ExeOrdre(paraGlob,tab_mail,lesMaillages,unseul_incre,lesReferences
                      ,lesLoisDeComp ,diversStockage
                      ,charge,lesCondLim,lesContacts,resultats,*entreePrinc
                      ,type_incre,incre,animation,listeVarGlob,listeVecGlob);
         };
   // cas de l'ordre d'animation qui est appelé à la fin 
   // au dernier incrément
   // le numéro de maillage n'a pas d'importance
//   if ((*(--(list_balaie->end())) == incre) && animation)
   if ((type_incre==OrdreVisu::DERNIER_INCRE) && animation)
     { // dans le cas où l'on doit faire de l'animation et que  celle-ci
       // n'est pas activée on l'active et la renseigne
       if (!(anim->Actif())) anim->ChoixOrdre();
       anim->ExeOrdre(paraGlob,1,lesMaillages,unseul_incre,lesReferences
                      ,lesLoisDeComp ,diversStockage
                      ,charge,lesCondLim,lesContacts,resultats,*entreePrinc
                      ,type_incre,incre,animation,listeVarGlob,listeVecGlob);
      };                
      
   // calcul de la boite maxi d'encombrement 
   // utilisé à la fin pour définir les # points de vue
   // on balaie les différents maillages
   for (ik= li_mail.begin();ik!=ikfin;ik++)
     { int numMail = *ik;
       // pour chaque maillage on définit la boite maxi
       Tableau <Vecteur>  boite_inter  = lesMaillages->Taille_boite(numMail);
       // calcul de la plus grande boite
       Calcul_maxi_boite ( boite_inter);      
      };
   // on vide le buffer au cas ou
   entreePrinc->Sort_princ_geomview() << endl;
   entreePrinc->Sort_princ_geomview() << flush;
   entreePrinc->Sort_legende_geomview() << endl << flush;
};

                       
// == définition des paramètres de visualisation 
// titre, navigation, éclairage
// et initialisation des paramètres de la classe
void Visualisation_geomview::Contexte_debut_visualisation()
  {ostream &sort = entreePrinc->Sort_princ_geomview();
   // écriture de l'entête
   sort << "\n # ----------------------------------"
        << "\n #     format  Geomview utf8         "
        << "\n # ----------------------------------"
        << "\n # ";
   // le titre
   sort << "\n # "
        << "\n #  Visualisation elements finis : Herezh++ V" << ParaGlob::NbVersion()
        << "\n #"
//        << "info [ \"Copyright (c) 1997-2003, Gérard Rio (gerard.rio@univ-ubs.fr) http://www-lg2m.univ-ubs.fr\" ]\n"
        << "\n # ";
   sort << Banniere::CopiPirate() << " \n";
   animation = false; 
   ((Animation_geomview*)anim)->Init_liste_coordonnee();
   };
    
// (points de vue) et enchainement si nécessaire
void Visualisation_geomview::Contexte_fin_visualisation()
  {// définition des points de vue avec l'angle adoc
 //  pour l'instant on ne définit pas de point de vue  DefinitionViewPoint();
   };                  
    
// lecture des paramètres de l'ordre dans un flux
void Visualisation_geomview::Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc)
 { // si dans le flot il existe l'identificateur adoc on lit sinon on passe
   if (strstr(entreePrinc.tablcarCVisu,"debut_visualisation_geomview")!=NULL)
     {// essaie de lecture
      try 
        { string nom; 
          (*entreePrinc.entCVisu)  >> nom ;
          if (nom != "debut_visualisation_geomview")
            { cout << "\n Erreur en lecture de la visualisation geomview a partir d'un fichier .CVisu,"
                   << " le premier enregistrement doit etre le mot clef: debut_visualisation_geomview "
                   << " la visualisation geomview n'est pas validee !! ";
              }
          else
           { // on valide la visualisation
             activ_sort_geomview=true;
             entreePrinc.NouvelleDonneeCVisu();
             // on passe en revue tous les ordres disponibles
             while (strstr(entreePrinc.tablcarCVisu,"fin_visualisation_geomview")==NULL)
               { list <OrdreVisu*>::iterator s_or,s_or_fin;
                 s_or_fin = ordre_possible.end();
                 for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
                   (*s_or)->Lecture_parametres_OrdreVisu(entreePrinc);
                 }  
            }       
         }    
      catch (ErrSortieFinale)
           // cas d'une direction voulue vers la sortie
           // on relance l'interuption pour le niveau supérieur
         { ErrSortieFinale toto;
           throw (toto);
         }
      catch (...)// erreur de lecture
       {  cout << "\n Erreur en lecture de la visualisation geomview a partir d'un fichier .CVisu,"
               << " la visualisation geomview n'est pas validee !! ";
          activ_sort_geomview=false;
          if (ParaGlob::NiveauImpression() >= 4)     
             cout  << "\n Visualisation::Lecture_parametres_OrdreVisu(..";
        }
      // on passe à un nouvel enregistrement
      entreePrinc.NouvelleDonneeCVisu();
      }  
  };

// écriture des paramètres de l'ordre dans un flux
void Visualisation_geomview::Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc)
 { // récup du flot  
   ostream & sort = (*(entreePrinc.Sort_CommandeVisu()));
   // on s'occupe des ordres spécifiques à la visualisation geomview
   // il n'y a écriture que si la visualisation geomview est active
   if(activ_sort_geomview) 
    {// tout d'abord l'activation du type geomview    
     // on commente le fonctionnement
     sort << "\n #  =================================================================================";
     sort << "\n #  ||       *****         demande d'une visualisation geomview:       *****       || "
          << "\n #  ================================================================================="
          << "\n #  un mot cle de debut (debut_visualisation_geomview)"
          << "\n #  un mot cle de fin ( fin_visualisation_geomview) apres tous les ordres particuliers"
          << "\n #  la seule presence du premier mots cle suffit a activer la visualisation geomview"
          << "\n #  la presence du second permet une meilleur lisibilite du fichier, mais n'est pas indispensable" ;
     // on sort les mots clés d'activation du type geomview
     sort << "\n debut_visualisation_geomview \n" ;
     // on passe en revue tous les ordres disponibles
     list <OrdreVisu*>::iterator s_or,s_or_fin;
     s_or_fin = ordre_possible.end();
     for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
           (*s_or)->Ecriture_parametres_OrdreVisu(entreePrinc);
     // fin de l'activation geomview      
     sort << "\n fin_visualisation_geomview"
          << "\n #  ================================================================================="
          << "\n #  ||                          fin de la  visualisation geomview                  || "
          << "\n #  ================================================================================= \n \n";
     }
  };
    
//============================== méthodes privées =====================

     
    // test si la réponse fait partie des ordres possibles
    // si l'on trouve un ordre ok on ramène un pointeur sur l'ordre
    // sinon on ramène un pointeur null
    OrdreVisu* Visualisation_geomview::Existe(string& reponse)
     { // on passe en revue la liste d'ordre
       list <OrdreVisu*>::iterator s_or,s_or_fin;
       s_or_fin = ordre_possible.end();
       for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
          if ((*s_or)->OrdreVrai(reponse))
               return (*s_or); // cas d'un ordre correcte
       return NULL; // pas d'ordre correcte trouvé
       };
     
      
    // affichage des options possibles
    void  Visualisation_geomview::Affiche_options()
     { // écriture d'une ligne de transition
       cout << setw(60) << setfill('-') << "\n " << setfill(' ');
       list <OrdreVisu*>::iterator s_or,s_or_fin;
       s_or_fin = ordre_possible.end();
       for (s_or = ordre_possible.begin();s_or != s_or_fin; s_or++)
           (*s_or)->Affiche_ordre();
       cout << "\n reponse ? ";      
     };

    // calcul de la plus grande boite
    void Visualisation_geomview::Calcul_maxi_boite 
            (Tableau <Vecteur>& boite_inter)
      {  // tout d'abord le cas particulier de la boite non initialisé
         if (boite.Taille() == 0)
          { boite = boite_inter;
            return;
           } 
         // cas d'une boite déjà existante
         // différent tests
         int dima = boite(1).Taille();
         for (int it=1;it<=dima;it++)
            { if (boite(1)(it) > boite_inter(1)(it))
                boite(1)(it) = boite_inter(1)(it);
              if (boite(2)(it) < boite_inter(2)(it))
                boite(2)(it) = boite_inter(2)(it);
             }
       }; 
    // définition des points de vue avec l'angle adoc
    void Visualisation_geomview::DefinitionViewPoint()
      { // l'objectif est de positionner les viewpoints relativement à la boite        
        Vecteur diag = boite(2)-boite(1);
        double lmax = diag.Norme();
        Vecteur centre = (boite(1) + boite(2))/2.;
        // si la dimension est différente de 3 on force à 3 
        // d'où les autres coordonnées sont nulles
        centre.Change_taille(3);
        // def de la distance d'observation grande pour minimiser la déformation
        double dist = 20.*lmax;
        // calcul de l'angle du cone de vue
        double angle = atan(lmax/dist); 
        ostream &sort = entreePrinc->Sort_princ_geomview();
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)<<" "<<centre(2)<<" "<<centre(3)+ dist
             << "\n fieldOfView " << angle
             << "\n description \"suivant Z \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)<<" "<<centre(2)+ dist<<" "<<centre(3)
             << "\n orientation 1.0 0.0 0.0 -1.57"
             << "\n fieldOfView  " << angle
             << "\n description \"suivant Y \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)+ dist<<" "<<centre(2)<<" "<<centre(3)
             << "\n orientation 0.0 1.0 0.0 1.57"
             << "\n fieldOfView " << angle
             << "\n description \"suivant X \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)+ dist<<" "<<centre(2)<<" "<<centre(3)+ dist
             << "\n orientation 0.0 1.0 0.0 0.785"
             << "\n fieldOfView " << angle
             << "\n description \"suivant ZX \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)<<" "<<centre(2)+ dist<<" "<<centre(3)+ dist
             << "\n orientation 1.0 0.0 0.0 -0.785"
             << "\n fieldOfView " << angle
             << "\n description \"suivant Z \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)+ dist<<" "<<centre(2)+ dist<<" "<<centre(3)
             << "\n orientation -1.0 1.0 0.0 1.57"
             << "\n fieldOfView " << angle
             << "\n description \"suivant XY \""
             << "\n }";
        sort << "\n Viewpoint {"
             << "\n position " <<centre(1)+ dist<<" "<<centre(2)+ dist<<" "<<centre(3)+ dist
             << "\n orientation -1.0 1.0 0.0 0.785"
             << "\n fieldOfView " << angle
             << "\n description \"suivant XYZ \""
             << "\n }";
       };
