
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        24/09/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Visualisation du maillage initiale en Gid.                 *
 *           Uniquement l'ordre d'exécution est différent de la classe  *
 *                      vrml.                                           *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef MAIL_INITIALE_GID_T 
#define MAIL_INITIALE_GID_T

#include "OrdreVisu.h"  

/// @addtogroup Les_sorties_Gid
///  @{
///


class Mail_initiale_Gid : public OrdreVisu 
{
  public :
    // CONSTRUCTEURS :
     // par defaut
     Mail_initiale_Gid () ; 
     
     // constructeur de copie 
     Mail_initiale_Gid (const Mail_initiale_Gid& algo);
               
    // DESTRUCTEUR :
     ~Mail_initiale_Gid () ;  
    
    // METHODES PUBLIQUES :
    // initialisation : permet d'initialiser les différents paramètres de l'ordre
    // lors d'un premier passage des différents incréments
    // en virtuelle, a priori est défini si nécessaire dans les classes dérivées
    // incre : numéro d'incrément qui en cours
    void Initialisation(ParaGlob * ,LesMaillages *,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*
                      ,Resultats*,EnumTypeIncre type_incre,int incre
                      ,const map < string, const double * , std::less <string> >& listeVarGlob
                      ,const List_io < TypeQuelconque >& listeVecGlob
                      ,bool fil_calcul) ;
    // execution de l'ordre
    // tab_mail : donne les numéros de maillage concerné
    // incre : numéro d'incrément qui en cours
    // type_incre : indique si c'est le premier le dernier ou l'incrément courant a visualiser ou pas
    // animation : indique si l'on est en animation ou pas
    // unseul_incre : indique si oui ou non il y a un seul increment à visualiser
    void ExeOrdre(ParaGlob * ,const Tableau <int>& tab_mail ,LesMaillages *,bool unseul_incre,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*
                      ,Resultats*,UtilLecture & entreePrinc,OrdreVisu::EnumTypeIncre type_incre,int incre
                      ,bool animation,const map < string, const double * , std::less <string> >& listeVarGlob
                      ,const List_io < TypeQuelconque >& listeVecGlob);
    
    // choix de l'ordre, cet méthode peut entraîner la demande d'informations
    // supplémentaires si nécessaire. qui sont ensuite gérer par la classe elle même
    void ChoixOrdre();                   
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    
    // ramène le tableau de listes de sous maillage 
    const Tableau < Tableau < List_io <  Element* > > > & 
                   Tableau_de_sous_maillage() const {return tabtab_sous_mesh;};
    // ramène le tableau de le listes de types d'éléments différents
    const Tableau < List_io <Element::Signature> >& 
                   Tab_liste_type_element() const { return tabli_sig_elem;};                 
    // ramène le tableau de listes de noms des sous maillages
    const Tableau < List_io <string> >& 
                   Tab_listes_nom_sousMaillage() const { return tabli_sous_mesh;};
    // ramène le  décalages  pour les numéros de noeuds du maillage "nmail"
    int DecalNumNoeud(int nmail) const {return decal_noeud(nmail);};
    // ramène le décalages pour les numéros d'éléments du maillage "nmail"
    int DecalNumElement(int nmail) const {return decal_element(nmail);};
                                          
  private :  
    // VARIABLES PROTEGEES :
    // tableau de liste de sous maillages
    // chaque sous maillage représente un type d'élément fini particulier
    // l'indice du tableau= le numéro de maillage
    Tableau < Tableau < List_io <  Element* > > > tabtab_sous_mesh;
    // tableau de listes de type d'éléments différents 
    Tableau < List_io <Element::Signature> > tabli_sig_elem;
    // tableau de listes des nom des sous_maillages créés
    Tableau < List_io <string> > tabli_sous_mesh;
    // tableau des décalages pour les numéros de noeuds et d'éléments
    Tableau <int > decal_noeud; 
    Tableau <int > decal_element;
    

    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
