
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        24/09/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Gérer et définir les fichiers de visualisation Gid.         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef VISUALISATION_GID_H
#define VISUALISATION_GID_H

#include "UtilLecture.h"
// #include "bool.h"
#include "MotCle.h"
#include "string.h"
#include "Tableau_T.h"
#include "LesMaillages.h"
#include "LesReferences.h" 
#include "LesCondLim.h"  
#include <list>
#include "LesContacts.h"
#include "LesValVecPropres.h"
#include "LesLoisDeComp.h"
#include "DiversStockage.h"
#include "Charge.h"
#include "LesContacts.h"
#include "Resultats.h"
#include "OrdreVisu.h"

/// @addtogroup Les_sorties_Gid
///  @{
///


class Visualisation_Gid
{
  public :
    // CONSTRUCTEURS :
    // le constructeur par défaut ne doit pas être utilisé
    // il y a dans ce cas un message d'erreur et arrêt
    Visualisation_Gid ();   
    // le bon constructeur
    Visualisation_Gid (UtilLecture* ent);
    
    // DESTRUCTEUR :
    ~Visualisation_Gid ();

    // METHODES PUBLIQUES :
    // affichage des différentes possibilités (ordres possibles)
    // ramène un numéro qui renseigne le programme appelant
    // =-1  : signifie que l'on veut stopper la visualisation
    // = 0  : signifie que l'on demande la visualisation effective
    int OrdresPossible();
    
    // information de l'instance de la liste d'incrément disponible pour la visualisation
    void List_increment_disponible(list <int> & list_incr) ;
    // indique le choix de la liste d'incrément à utiliser pour l'initialisation
    // des fonctions d'initialisation
    const list<int> &  List_balaie_init(); 
    // impose une liste d'incrément à utiliser 
    void  List_balaie_init(const list<int> & list_init);    
    // indique le choix de la liste d'incrément à visualiser
    const list<int> &  List_balaie() {return *list_balaie;};
    // information de l'instance du nombre de maillages disponibles pour la visualisation
    // par défaut tous les maillages seront visualisés, ceci est descidé aussi ici
    void List_maillage_disponible(int nombre_maillage_dispo) ;
    
    // initialisation des ordres disponibles 
    // par exemple pour les isovaleurs on définit la liste des isovaleurs disponibles et les extrémas
    void Initialisation(ParaGlob * paraGlob,LesMaillages * lesMaillages,LesReferences* lesReferences
                       ,LesLoisDeComp* lesLoisDeComp,DiversStockage* diversStockage
                       ,Charge* charge,LesCondLim* lesCondLim,LesContacts* lesContacts
                       ,Resultats* resultats,OrdreVisu::EnumTypeIncre type_incre,int incre
                       ,const map < string, const double * , std::less <string> >& listeVarGlob
                       ,const List_io < TypeQuelconque >& listeVecGlob
                       ,bool fil_calcul);

    // méthode principale pour activer la visualisation
    // sort : le flux de visualisation
    void Visu(ParaGlob * paraGlob,LesMaillages * lesMaillages,LesReferences* lesReferences
                       ,LesLoisDeComp* lesLoisDeComp,DiversStockage* diversStockage
                       ,Charge* charge,LesCondLim* lesCondLim,LesContacts* lesContacts
                       ,Resultats* resultats,OrdreVisu::EnumTypeIncre type_incre,int incre
                       ,const map < string, const double * , std::less <string> >& listeVarGlob
                       ,const List_io < TypeQuelconque >& listeVecGlob);
                       
    // == définition des paramètres de visualisation 
    // titre, navigation, éclairage
    // et initialisation des paramètres de la classe
    void Contexte_debut_visualisation(); 
    // (points de vue) et enchainement si nécessaire
    void Contexte_fin_visualisation();                  
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // demande si la visualisation Gid est validé ou pas
    bool Visu_Gid_valide() {return activ_sort_Gid;};
    // inactive la visualisation Gid
    void Inactiv_Visu_Gid() {activ_sort_Gid=false;};
            
  private :  
    // VARIABLES PROTEGEES :
    UtilLecture* entreePrinc; // gestion des entrees/sorties
    list <OrdreVisu*> ordre_possible; // l'ensemble des ordres possibles
    // la sauvegarde de valeurs propres et vecteurs propres éventuelles
    LesValVecPropres lesValVecPropres; 
    OrdreVisu* fin_o; // ordre de fin de la visualisation
    OrdreVisu* mailInit; // Maillage initial
    OrdreVisu* visuali; // ordre de visualiser
    OrdreVisu* choix_inc; // ordre de choix des incréments
    OrdreVisu* ptdeformee; // déformée   
    list <int>  list_incre; // liste des incréments possibles
    const list <int>*  list_balaie; // liste des incréments à visualiser
    int nb_maillage_dispo; // le nombre de maillage disponible
    OrdreVisu* choix_mail; // ordre de choix des maillages
    OrdreVisu* choix_isovaleur; // ordre de visualisation d'isovaleurs
    // indication si l'on est en animation ou pas
    bool animation;
    // indication si la visualisation de type Gid est active ou pas, par défaut = faux
    bool activ_sort_Gid;

    // METHODES PROTEGEES :
    // test si la réponse fait partie des ordres possibles
    // si l'on trouve un ordre ok on ramène un pointeur sur l'ordre
    // sinon on ramène un pointeur null
    OrdreVisu* Existe(string& reponse);
    // affichage des options possibles
    void  Affiche_options();
         
                       
 };
 /// @}  // end of group

#endif  
