
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        07/01/2008                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Visualisation de la déformée avec Gmsh.                    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef DEFORMEES_GMSH_T
#define DEFORMEES_GMSH_T

#include "OrdreVisu.h"
#include "Isovaleurs_Gmsh.h"
#include "OrdreVisu.h"
#include "ChoixDesMaillages_vrml.h"
#include "Basiques.h"

/// @addtogroup Les_sorties_gmsh
///  @{
///


class Deformees_Gmsh : public OrdreVisu
{
  public :
    // CONSTRUCTEURS :
     // par defaut
     Deformees_Gmsh () ;
     // constructeur à utiliser
     Deformees_Gmsh(const string& comment_som, const string& explication, const string& ordre) :
         OrdreVisu(comment_som,explication,ordre)
            {};
      
     // constructeur de copie 
     Deformees_Gmsh (const Deformees_Gmsh& algo);
               
    // DESTRUCTEUR :
     ~Deformees_Gmsh () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'ordre
    // tab_mail : donne les numéros de maillage concerné
    // incre : numéro d'incrément qui en cours
    // type_incre : indique si c'est le premier le dernier ou l'incrément courant a visualiser ou pas
    // animation : indique si l'on est en animation ou pas
    // unseul_incre : indique si oui ou non il y a un seul increment à visualiser
    void ExeOrdre(ParaGlob * ,const Tableau <int>& tab_mail ,LesMaillages *,bool unseul_incre,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*
                      ,Resultats*,UtilLecture & entreePrinc,OrdreVisu::EnumTypeIncre type_incre,int incre
                      ,bool animation,const map < string, const double * , std::less <string> >& listeVarGlob
                      ,const List_io < TypeQuelconque >& listeVecGlob);
    // choix de l'ordre, cet méthode peut entraîner la demande d'informations
    // supplémentaires si nécessaire. qui sont ensuite gérer par la classe elle même
    void ChoixOrdre(); 
    
    // --- méthodes particulière -----
    // initialisation d'une liaison avec une instance de classe d'isovaleur
    void Jonction_isovaleur(const Isovaleurs_Gmsh *  iso) {isovaleurs_Gmsh = iso;}; 
    // initialisation d'une liaison avec une instance de classe de choix des maillages
    void Jonction_ChoixDesMaillages(const ChoixDesMaillages_vrml* choix_m) {choix_mail = choix_m;};                      
    // initialisation d'une liaison avec une instance de classe de maillage initiale
    void Jonction_MaillageInitiale(Mail_initiale_Gmsh *  mailIni) {mailInitial = mailIni;}; 
    
    // récup de la liste des noms de matières utilisés dans les shapes
    const list <string> & Noms_matiere() const { return noms_matiere;};                     
    // récup de la liste des noms de couleurs utilisés dans les shapes
    const list <string> & Noms_couleurs() const { return noms_couleurs;};                     
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // récupération du nom pour la sortie gmsh en particulier
    const string & Nom_deplace() const {return nom_deplace;};
    // récupération du nom pour la sortie gmsh en particulier= NULL si ce n'est pas nécessaire
    const string * Nom_Vitesse() const
      {if (avec_vitesse) return &nom_vitesse;else return NULL;};
    // récupération du nom pour la sortie gmsh en particulier= NULL si ce n'est pas nécessaire
    const string * Nom_Acceleration() const
      {if (avec_acceleration) return &nom_acceleration;else return NULL;};
    // sortie de la déformée dans le cas du nouveau format
    void SortieDeformee(const Tableau <int>& tab_mail,LesMaillages * lesMail
                   ,ostream & sort,int incre);
    // idem pour le champ de vitesse dans le cas du nouveau format
    void SortieVitesse(const Tableau <int>& tab_mail,LesMaillages * lesMail
                   ,ostream & sort,int incre);
    // idem pour le champ d'accélération dans le cas du nouveau format
    void SortieAcceleration(const Tableau <int>& tab_mail,LesMaillages * lesMail
                   ,ostream & sort,int incre);
 
    
  protected :  
    // VARIABLES PROTEGEES :
    // définition des alertes sur les min max
    list <DeuxDoubles> li_bornes; // min max des alertes
    list <string> li_nom_bornes;  // nom des alertes
    Mail_initiale_Gmsh * mailInitial; // pour une jonction avec le maillage initial
    int avec_vitesse; // indique si l'on veut également les vitesses si elles sont disponibles
    int avec_acceleration; // indique si l'on veut également les accélérations si elles sont disponibles
    
    string nom_deplace; // le nom de sortie qui sert pour le fichier de sortie gmsh
    string nom_vitesse; // idem pour la vitesse
    string nom_acceleration; // idem pour l'acceleration
    
    const Isovaleurs_Gmsh *  isovaleurs_Gmsh; // contient lorqu'il est actif les couleurs des noeuds
                                              // et d'autres utilitaires
    list < Deuxentiers >  num_mail_incr; // numéro de maillage et d'incrément correspondant
                                        // à  couleurs noeud 
    list <string> noms_matiere; // liste des noms de matériaux utilisée                                    
    list <string> noms_couleurs; // liste des noms de couleurs utilisée                                    

    const ChoixDesMaillages_vrml* choix_mail; // contient lorqu'il est actif le choix des maillages
                               
    
    // METHODES PROTEGEES :
	 // sortie de la déformée dans le cas de l'ancien format
	 void SortieDeformee_ancien_format(const Tableau <int>& tab_mail,LesMaillages * lesMail
												,ostream & sort,int incre);

 };
 /// @}  // end of group

#endif  
