
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Choix du ou des maillages à visualiser.                    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef CHOIXDESMAILLAGES_VRML_T
#define CHOIXDESMAILLAGES_VRML_T

#include "OrdreVisu.h"

/// @addtogroup Les_sorties_vrml
///  @{
///


class ChoixDesMaillages_vrml : public OrdreVisu
{
  public :
    // CONSTRUCTEURS :
     // par defaut
     ChoixDesMaillages_vrml () ; 
     
     // constructeur de copie 
     ChoixDesMaillages_vrml (const ChoixDesMaillages_vrml& ord);
               
    // DESTRUCTEUR :
     ~ChoixDesMaillages_vrml () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'ordre
    // unseul_incre : indique si oui ou non il y a un seul increment à visualiser
    void ExeOrdre(ParaGlob * ,const Tableau <int>& ,LesMaillages *,bool ,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*
                      ,LesContacts*,Resultats*,UtilLecture & ,OrdreVisu::EnumTypeIncre ,int 
                      ,bool,const map < string, const double * , std::less <string> >&
                      ,const List_io < TypeQuelconque >& listeVecGlob)
             {};
    // choix de l'ordre, cet méthode peut entraîner la demande d'informations
    // supplémentaires si nécessaire. qui sont ensuite gérer par la classe elle même
    void ChoixOrdre(); 
    // initialisation du nombre de maillages possibles
    // et définition de tous les maillages par défaut dans la liste des maillages
    // a visualiser
    void Init_nb_maill(int& nb_maillage_poss); 
    // ramène la liste des maillages choisit
    // dans le cas où les numéros sont négatifs, indique que c'est uniquement
    // les frontières où les données relatives aux frontières qui sont choisit
    const list <int>& List_choisit() const 
        { return lis_sor;};                    
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
            
  private :  
    // VARIABLES PROTEGEES :
  int* nb_maillage; // nombre de maillage  possible    
  list <int> lis_sor; // liste de maillage choisit   
    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
