
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Demande d'un numéro ou d'une plage d'incrément.            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef INCREMENT_VRML_T
#define INCREMENT_VRML_T

#include "OrdreVisu.h"

/// @addtogroup Les_sorties_vrml
///  @{
///


class Increment_vrml : public OrdreVisu
{
  public :
    // CONSTRUCTEURS :
     // par defaut
     Increment_vrml () ; 
     
     // constructeur de copie 
     Increment_vrml (const Increment_vrml& algo);
               
    // DESTRUCTEUR :
     ~Increment_vrml () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'ordre
    // tab_mail : donne les numéros de maillage concerné
    // unseul_incre : indique si oui ou non il y a un seul increment à visualiser
    void ExeOrdre(ParaGlob * ,const Tableau <int>& ,LesMaillages *,bool ,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*
                      ,LesContacts*,Resultats*,UtilLecture & ,OrdreVisu::EnumTypeIncre  ,int 
                      ,bool,const map < string, const double * , std::less <string> >&
                      ,const List_io < TypeQuelconque >& listeVecGlob)
             {};
    // choix de l'ordre, cet méthode peut entraîner la demande d'informations
    // supplémentaires si nécessaire. qui sont ensuite gérer par la classe elle même
    void ChoixOrdre(); 
    // initialisation de la liste d'incrément possible
    void Init_list_inc(list<int>& li_inc)
        {lis_inc = & li_inc;}; 
    // initialisation de la liste à l'incrément 0 et au dernier increment
    // s'il est différent
    void Init_list_inc_DebuEtFin();   
    // ramène la liste des incréments choisit
    const list <int>& List_choisit()
        { return lis_sor;}; 
    // imposition d'une liste d'incrément donné
    void Impos_list(const list<int>& li_inc) {lis_sor = li_inc;}; 
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
            
  private :  
    // VARIABLES PROTEGEES :
  list <int>* lis_inc; // liste d'incrément possible
       // c'est uniquement le pointeur qui est sauvegardé, il n'y a pas de new
       // associé dans les méthodes
  list <int> lis_sor; // liste d'incrément lue    
    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
