
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Visualisation des isovaleurs en vrml.                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef ISOVALEURS_VRML_T
#define ISOVALEURS_VRML_T

#include "OrdreVisu.h"
#include "spectre.h"

/// @addtogroup Les_sorties_vrml
///  @{
///


class Isovaleurs_vrml : public OrdreVisu
{
  public :
    // CONSTRUCTEURS :
     // par defaut
     Isovaleurs_vrml () ; 
     
     // constructeur de copie 
     Isovaleurs_vrml (const Isovaleurs_vrml& algo);
               
    // DESTRUCTEUR :
     ~Isovaleurs_vrml () ;  
    
    // METHODES PUBLIQUES :
    // initialisation : permet d'initialiser les différents paramètres de l'ordre
    // lors d'un premier passage des différents incréments
    // en virtuelle, a priori est défini si nécessaire dans les classes dérivées
    // incre : numéro d'incrément qui en cours
    void Initialisation(ParaGlob * ,LesMaillages *,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*
                      ,Resultats*,EnumTypeIncre type_incre,int incre
                      ,const map < string, const double * , std::less <string> >& listeVarGlob
                      ,const List_io < TypeQuelconque >& listeVecGlob
                      ,bool fil_calcul) ;
    // execution de l'ordre
    // tab_mail : donne les numéros de maillage concerné
    // incre : numéro d'incrément qui en cours
    // type_incre : indique si c'est le premier le dernier ou l'incrément courant a visualiser ou pas
    // animation : indique si l'on est en animation ou pas
    // unseul_incre : indique si oui ou non il y a un seul increment à visualiser
    void ExeOrdre(ParaGlob * ,const Tableau <int>& tab_mail ,LesMaillages *,bool unseul_incre,LesReferences*
                      ,LesLoisDeComp* ,DiversStockage*,Charge*,LesCondLim*,LesContacts*
                      ,Resultats*,UtilLecture & entreePrinc,OrdreVisu::EnumTypeIncre type_incre,int incre
                      ,bool animation,const map < string, const double * , std::less <string> >& listeVarGlob
                      ,const List_io < TypeQuelconque >& listeVecGlob);
    // choix de l'ordre, cet méthode peut entraîner la demande d'informations
    // supplémentaires si nécessaire. qui sont ensuite gérer par la classe elle même
    void ChoixOrdre();                   
            
    // initialisation de la liste des différentes isovaleurs possibles
    void Init_liste_isovaleur(LesMaillages * lesMail,LesContacts* lesContacts);
    
    // ramène une référence constante des couleurs des noeuds des incréments
    const list < Tableau < Rgb > > & List_couleur() const {return couleurs_noeud;};
    // ramène une référence constante des numéros de maillage et d'incrément correspondants aux couleurs
    const list < Deuxentiers > &  List_num_mail_incr() const {return num_mail_incr;};
    
    // lecture des paramètres de l'ordre dans un flux
    void Lecture_parametres_OrdreVisu(UtilLecture & entreePrinc);
    // écriture des paramètres de l'ordre dans un flux
    void Ecriture_parametres_OrdreVisu(UtilLecture & entreePrinc);
            
protected :
    // VARIABLES PROTEGEES :
    bool choix_peau; // si oui visualisation uniquement de la peau
    bool choix_legende; // si oui affichage de la légende
    int choix_min_max; // si 1 min max calculé automatiquement et globalement
                       // si 2 min max calculé en % à chaque incr / à un pourcentage
                       // si 3 min max calculé auto en % globalement
                       // si 0 min max fixé
    double valMini,valMaxi; // les miniMax totaux ou pour chaque incrément
    double pour_legende_min,pour_legende_max; // min et max en % : set éventuellement, dépend de choix_min_max
    int nb_iso_val; // nombre d'isovaleurs dans la légende
    bool absolue;       // par défaut on sort en absolue les tenseurs
                        // pour pouvoir correctement les visualiser, même pour les elem 2D
    
    Tableau <List_io < Ddl_enum_etendu > > tabnoeud_type_ddl; // ddl aux noeuds possibles à visualiser
    Tableau < Ddl_enum_etendu>  type_ddl_retenu; // ddl à visualiser, un par maillage
    Tableau <bool> choix_var_ddl; // indique si c'est une variation ou le ddl
    Tableau <List_io < Ddl_enum_etendu > > tabelement_type_ddl; // ddl aux elements possibles à visualiser
    
    // on définit les valeurs des couleurs aux noeuds
    // chaque élément de la liste correspond à un incrément
    // (i) : correspond au noeud i
    list < Tableau < Rgb > >  couleurs_noeud;
    list < Tableau < double > >  val_noeud; // tableau de travail
    
    list < Deuxentiers >  num_mail_incr; // numéro de maillage et d'incrément correspondant
                                        // à  couleurs noeud 
    
    Spectre spectre_coul; // le spectre de couleur utilisé pour les isovaleurs
    
    // METHODES PROTEGEES :
      
    // sortie du dessin de la légende
    virtual void Sortie_dessin_legende(UtilLecture & entreePrinc);
    
    // constructeur utilisé par les classes dérivées
    // en fait il s'agit de transmettre directement les infos à la classe mère Visu
    Isovaleurs_vrml(const string& comment_som, const string& explication
                              , const string& ordre) :
                    OrdreVisu(comment_som,explication,ordre) 
                              {};
      
    // définition interactives des paramètres généraux des isovaleurs
    void ParametresGeneraux();
    // choix de l'isovaleur à visualiser
    void ChoixIsovaleur();
    // choix de grandeur existantes aux points d'intégrations et à ramener aux noeuds
    // pour préparer une visualisation d'isovaleurs
    void TransfertGrandeursPtIntegAuNoeud();

 };
 /// @}  // end of group

#endif  
