// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// FICHIER : Epai.h
// CLASSE : Epai
/************************************************************************
 *     DATE:        08/07/2008                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Conteneur très basique pour les épaisseurs               *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                $     *
 ************************************************************************/

#ifndef EPAI_SIMPLE_H
#define EPAI_SIMPLE_H

#include <iostream>
#include <fstream>

//
//------------------------------------------------------------------
//!     Conteneur très basique pour les épaisseurs
//------------------------------------------------------------------

      /// \author    Gérard Rio
      /// \version   1.0
      /// \date       08/07/2008


 class Epai 
   {public: 
    // VARIABLES PUBLIQUES :
      double epaisseur0,epaisseur_t,epaisseur_tdt;
    // CONSTRUCTEURS :
    Epai(): epaisseur0(0.),epaisseur_t(0.),epaisseur_tdt(0.) {};
    Epai(const double ep0,const double ept, const double eptdt) :
      epaisseur0(ep0),epaisseur_t(ept),epaisseur_tdt(eptdt)
       {};
    Epai(const Epai & a) :
     epaisseur0(a.epaisseur0),epaisseur_t(a.epaisseur_t)
     ,epaisseur_tdt(a.epaisseur_tdt)
       {};
    // DESTRUCTEUR :
    ~Epai() {};
    
    // METHODES PUBLIQUES :
    // changement de toutes les valeurs à une valeur déterminée
    void Change_tout(const double& val) {epaisseur0=epaisseur_t=epaisseur_tdt=val;};
    // surcharge de l'affectation 
    Epai& operator= (const Epai& a)
     { epaisseur0 = a.epaisseur0; epaisseur_t = a.epaisseur_t; 
       epaisseur_tdt=a.epaisseur_tdt;
       return (*this);
      };
    // surcharge de l'operator de lecture
    friend  istream & operator >> (istream & ent, Epai & de)
     { ent >> de.epaisseur0 >> de.epaisseur_t
           >> de.epaisseur_tdt; 
       return ent;
     };
    // surcharge de l'operator d'ecriture
    friend  ostream & operator << (ostream & sort , const Epai & de)
     { sort << " " << de.epaisseur0   <<" " << de.epaisseur_t 
            <<" " << de.epaisseur_tdt << " "; 
       return sort;
      };
 
 };
 
#endif  
