// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// FICHIER : Sect.h
// CLASSE : Sect
/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Conteneur très basique pour les sections                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef SECT_SIMPLE_H
#define SECT_SIMPLE_H

#include <iostream>
#include <fstream>

 //
 //------------------------------------------------------------------
 //!     Conteneur très basique pour les sections
 //------------------------------------------------------------------

       /// \author    Gérard Rio
       /// \version   1.0
       /// \date       06/03/2023

 class Sect
   {public: 
    // VARIABLES PUBLIQUES :
      double section0,section_t,section_tdt;
    // CONSTRUCTEURS :
    Sect(): section0(0.),section_t(0.),section_tdt(0.) {};
    Sect(const double ep0,const double ept, const double eptdt) :
      section0(ep0),section_t(ept),section_tdt(eptdt)
       {};
    Sect(const Sect & a) :
     section0(a.section0),section_t(a.section_t)
     ,section_tdt(a.section_tdt)
       {};
    // DESTRUCTEUR :
    ~Sect() {};
    
    // METHODES PUBLIQUES :
    // surcharge de l'affectation 
    Sect& operator= (const Sect& a)
     { section0 = a.section0; section_t = a.section_t; 
       section_tdt=a.section_tdt;
       return (*this);
      };
    // surcharge de l'operator de lecture
    friend  istream & operator >> (istream & ent, Sect & de)
     { ent >> de.section0 >> de.section_t
           >> de.section_tdt; 
       return ent;
     };
    // surcharge de l'operator d'ecriture
    friend  ostream & operator << (ostream & sort , const Sect & de)
     { sort << " " << de.section0   <<" " << de.section_t 
            <<" " << de.section_tdt << " "; 
       return sort;
      };
 
 };
 
#endif  
