// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "Courbe_un_moins_cos.h"
#include "Sortie.h"
#include "ConstMath.h"
#include "MathUtil.h"
#include "ParaGlob.h"


    // CONSTRUCTEURS :
Courbe_un_moins_cos::Courbe_un_moins_cos(string nom) :
 Courbe1D(nom,COURBE_UN_MOINS_COS)
 ,ax(ConstMath::tresgrand),bx(ConstMath::tresgrand),cx(ConstMath::tresgrand)
 // on met des valeurs bidons pour pouvoir tester la présence de coefficients correcte (dans complet)
  {};
    
    // de copie
Courbe_un_moins_cos::Courbe_un_moins_cos(const Courbe_un_moins_cos& Co) :
 Courbe1D(Co)
 ,ax(Co.ax),bx(Co.bx),cx(Co.cx)
  {};
    // de copie à partir d'une instance générale
Courbe_un_moins_cos::Courbe_un_moins_cos(const Courbe1D& Coo) :
 Courbe1D(Coo)
  { if (Coo.Type_courbe() != COURBE_UN_MOINS_COS)
      { cout << "\n erreur dans le constructeur de copie pour une courbe COURBE_UN_MOINS_COS "
             << " à partir d'une instance générale ";
        cout << "\n Courbe_un_moins_cos::Courbe_un_moins_cos(const Courbe1D& Co) ";
           Sortie(1);          
          };
    // définition des données
    Courbe_un_moins_cos & Co = (Courbe_un_moins_cos&) Coo;
    ax = Co.ax; bx = Co.bx;cx = Co.cx;
  };

    // DESTRUCTEUR :
Courbe_un_moins_cos::~Courbe_un_moins_cos() 
  {};
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
void Courbe_un_moins_cos::Affiche() const 
  { cout << "\n Courbe_un_moins_cos : nom_ref= " << nom_ref;
    cout << "\n a=" << ax << " b= " << bx << " c= " << cx;
   };

    // vérification que tout est ok, pres à l'emploi
    // ramène true si ok, false sinon
bool Courbe_un_moins_cos::Complet_courbe()const
 { bool ret = Complet_var(); // on regarde du coté de la classe mère tout d'abord
   // puis les variables propres
   if ((ax==ConstMath::tresgrand) && (bx==ConstMath::tresgrand) && (cx==ConstMath::tresgrand))
           ret = false;
   if (!ret && (ParaGlob::NiveauImpression() >0))
      { cout << "\n ***** la courbe n'est pas complete ";
        this->Affiche();
        };
   return ret;
  } ;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
void Courbe_un_moins_cos::LectDonnParticulieres_courbes(const string& nom,UtilLecture * entreePrinc)
  {  if (nom == "")  { *(entreePrinc->entree) >> nom_ref;}
     else {nom_ref=nom;};
     entreePrinc->NouvelleDonnee();  // lecture d'une nouvelle ligne
     string titi;
     // on lit l'entête
     if(strstr(entreePrinc->tablcar,"a=")==0)
         { cout << "\n erreur en lecture du coefficient a ";
           string toto = "\n Courbe_un_moins_cos::LectureDonneesParticulieres ";
           toto += "(UtilLecture * entreePrinc) " ;
 //          cout << "\n Courbe_un_moins_cos::LectureDonneesParticulieres "
 //               << "(UtilLecture * entreePrinc) " << endl ;
           entreePrinc->MessageBuffer("**Courbe_un_moins_cos::LectureDonneesParticulieres**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          }
     *(entreePrinc->entree) >> titi >> ax;
     if(strstr(entreePrinc->tablcar,"b=")==0)
         { cout << "\n erreur en lecture du coefficient b ";
           cout << "\n Courbe_un_moins_cos::LectureDonneesParticulieres "
                << "(UtilLecture * entreePrinc) " << endl ;
           entreePrinc->MessageBuffer("**erreur**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          }
     *(entreePrinc->entree) >> titi >> bx;
     if(strstr(entreePrinc->tablcar,"c=")==0)
         { cout << "\n erreur en lecture du coefficient c ";
           cout << "\n Courbe_un_moins_cos::LectureDonneesParticulieres "
                << "(UtilLecture * entreePrinc) " << endl ;
           entreePrinc->MessageBuffer("**erreur**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          }
     *(entreePrinc->entree) >> titi >> cx;
     // lecture des coeffs
//     string toto;
//     *(entreePrinc->entree) >> toto >> ax;
//     *(entreePrinc->entree) >> toto >> bx;
//     *(entreePrinc->entree) >> toto >> cx ; 
//     Affiche();    
   };
    
// def info fichier de commande
void Courbe_un_moins_cos::Info_commande_Courbes1D(UtilLecture & entreePrinc) 
  {  
     ofstream & sort = *(entreePrinc.Commande_pointInfo()); // pour simplifier
     sort << "\n#............................................"
          << "\n# exemple de definition d'une courbe demi sinus" 
          << "\n#"
          << "\n courbe_exemple    COURBE_UN_MOINS_COS  # nom de la courbe puis le type de la courbe"
          << "\n      # def des coeff de la courbe demi sinus f(t)= c/2*(1-cos((x-a)*Pi/(b-a)))"
          << "\n      # pour x < a => f=0, pour x>b => f=c"
          << "\n      a= 0. b= 1. c= 1."
          << endl;
   };
        
    // ramène la valeur 
double Courbe_un_moins_cos::Valeur(double x)  
  { if (x < ax) return 0.;
    if (x > bx) return cx;
    return 0.5*cx*(1.-cos((x-ax)*ConstMath::Pi/(bx-ax)));
  };

    // ramène la valeur et la dérivée en paramètre
Courbe1D::ValDer Courbe_un_moins_cos::Valeur_Et_derivee(double x)  
  {  ValDer ret; // def de la valeur de retour
     if (x < ax) {ret.valeur= 0.; ret.derivee = 0.;}
     else if (x > bx) {ret.valeur= cx; ret.derivee = 0.;}
     else
       {ret.valeur= 0.5*cx*(1.-cos((x-ax)*ConstMath::Pi/(bx-ax)));
        ret.derivee =  0.5*cx*ConstMath::Pi/(bx-ax)*sin((x-ax)*ConstMath::Pi/(bx-ax));
        }
    return ret;
   };
    
    // ramène la dérivée 
double Courbe_un_moins_cos::Derivee(double x)  
  { if (x < ax) return 0.;
    if (x > bx) return 0.;
    return 0.5*cx*ConstMath::Pi/(bx-ax)*sin((x-ax)*ConstMath::Pi/(bx-ax));
   };
        
    // ramène la valeur et les dérivées première et seconde en paramètre
Courbe1D::ValDer2 Courbe_un_moins_cos::Valeur_Et_der12(double x)
  {  ValDer2 ret; // def de la valeur de retour
     if (x < ax) {ret.valeur= 0.; ret.derivee = 0.;ret.der_sec = 0.;}
     else if (x > bx) {ret.valeur= cx; ret.derivee = 0.;ret.der_sec = 0.;}
     else
       {ret.valeur= 0.5*cx*(1.-cos((x-ax)*ConstMath::Pi/(bx-ax)));
        ret.derivee =  0.5*cx*ConstMath::Pi/(bx-ax)*sin((x-ax)*ConstMath::Pi/(bx-ax));
        ret.der_sec =  0.5*cx*Sqr(ConstMath::Pi/(bx-ax))*cos((x-ax)*ConstMath::Pi/(bx-ax));
        };
    return ret;
   };
        
    // ramène la dérivée seconde
double Courbe_un_moins_cos::Der_sec(double x)
  {  return 0.5*cx*Sqr(ConstMath::Pi/(bx-ax))*cos((x-ax)*ConstMath::Pi/(bx-ax));
   };
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
Courbe1D::Valbool  Courbe_un_moins_cos::Valeur_stricte(double x)  
  { Valbool ret; // def de la valeur de retour
    if (x < ax) {ret.valeur = 0.;ret.dedans = false;}
    else if (x > bx) {ret.valeur = cx;ret.dedans = false;} 
    else 
      {ret.valeur =  0.5*cx*(1.-cos((x-ax)*ConstMath::Pi/(bx-ax)));
       ret.dedans = true;
       }
    return ret;
   }; 
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
Courbe1D::ValDerbool  Courbe_un_moins_cos::Valeur_Et_derivee_stricte(double x)  
  {  ValDerbool ret; // def de la valeur de retour
     if (x < ax) {ret.valeur= 0.; ret.derivee = 0.;ret.dedans = false;}
     else if (x > bx) {ret.valeur= cx; ret.derivee = 0.;ret.dedans = false;}
     else
       {ret.valeur= 0.5*cx*(1.-cos((x-ax)*ConstMath::Pi/(bx-ax)));
        ret.derivee =  0.5*cx*ConstMath::Pi/(bx-ax)*sin((x-ax)*ConstMath::Pi/(bx-ax));
        ret.dedans = true;
        }
    return ret;
   };
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
void Courbe_un_moins_cos::Lecture_base_info(ifstream& ent,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { string nom;
        // lecture et vérification de l'entête
        ent >> nom;
        if (nom != "Courbe_un_moins_cos")
         { cout << "\n erreur dans la vérification du type de courbe lue ";    
           cout << "\n Courbe_un_moins_cos::Lecture_base_info(... ";
           Sortie(1); 
           } 
        // lecture des infos
        ent  >> nom >> ax >> nom >> bx >> nom >> cx ;
       }
   };

    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
void Courbe_un_moins_cos::Ecriture_base_info(ofstream& sort,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { sort << " Courbe_un_moins_cos  ";
        sort << "  a= " << ax  << " b= " << bx <<" c= " << cx ;
       }
   };
         
// sortie du schemaXML: en fonction de enu 
void Courbe_un_moins_cos::SchemaXML_Courbes1D(ofstream& ,const Enum_IO_XML enu)
  {
	switch (enu)
	{ case XML_TYPE_GLOBAUX :
		{
		 break;
		}
		case XML_IO_POINT_INFO :
		{
		 break;
		}
		case XML_IO_POINT_BI :
		{
		 break;
		}
		case XML_IO_ELEMENT_FINI :
		{
		 break;
		}
	};		
  };
     
