

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        1/10/98                                             *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Concernant le potentiel hyperélastique nb 10 .              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef HYPER10_H
#define HYPER10_H

#include "Hyper3D.h"


/// @addtogroup Les_lois_hyperelastiques
///  @{
///


class Hyper10 : 
  public  Hyper3D
{
  public :
    // VARIABLES PUBLIQUES :
    
 // CONSTRUCTEURS :
 // Constructeur par defaut
 Hyper10 ();
 // Constructeur de copie
 Hyper10 (const Hyper10& loi) ;
 // DESTRUCTEUR :
 ~Hyper10 () 
   {};
		
// Lecture des donnees de la classe sur fichier
void LectureDonneesParticulieres (UtilLecture * );

 // affichage de la loi
 void Affiche();
 // test si la loi est complete
 // = 1 tout est ok, =0 loi incomplete
 int TestComplet();

		  	
      	
//  METHODES internes spécifiques à l'hyperélasticité isotrope découlant de 
//  méthodes virtuelles de HyperD

             
 // calcul du potentiel et de ses dérivées non compris la phase
 void Potentiel(double & Ieps,double & V,double& Qeps,
             double& E,double& EV,double& EQeps) ; 
 // calcul du potentiel et de ses dérivées avec la phase
 void PotentielPhase(double & Ieps,double & V,double& Qeps,
           double& cos3phi,double & sin3phi,
           double& E,double& EV,double& EQeps,double& EPhi) ; 
 // calcul  du potentiel sans phase et dérivées avec  ses variations par rapport aux ddl
 void Potentiel_et_var(double & Ieps,Tableau<double> & dIeps,
          double & V,Tableau<double> & dV,double& Qeps,
          Tableau<double> & dQeps,
          double& E,Tableau <double>& dE,double& EV,Tableau <double>& dEV,
          double& EQeps,Tableau <double>& dEQeps,double& EVV,Tableau <double>& dEVV,
          double& EQQ,Tableau <double>& dEQQ,double& EVQ,Tableau <double>& dEVQ ) ;
 // calcul  du potentiel avec phase et dérivées avec  ses variations par rapport aux ddl
 void PotentielPhase_et_var(double & Ieps,Tableau<double> & dIeps,
          double & V,Tableau<double> & dV,double& Qeps,
          Tableau<double> & dQeps,double& cos3phi,Tableau<double> & dcos3phi,
          double & sin3phi,Tableau<double> & dsin3phi,
          double& E,Tableau <double>& dE,double& EV,Tableau <double>& dEV,
          double& EQeps,Tableau <double>& dEQeps,
          double& EPhi,Tableau <double>& dEPhi,double& EVV,Tableau <double>& dEVV,
          double& EQQ,Tableau <double>& dEQQ,double& EVQ,Tableau <double>& dEVQ ,
          double& Ephiphi,Tableau <double>& dEphiphi,double& EQphi,Tableau <double>& dEQphi,
          double& EVphi,Tableau <double>& dEVphi );
 
  // VARIABLES PROTEGEES :
  double  K;      // module de dilatation
  double  Qor;    // seuil
  double  mur;    // pente à l'origine
  double  mu_inf; // pente à l'infini 
    
    // CONSTRUCTEURS :
    
    // DESTRUCTEUR :
    
    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
