

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        22/03/99                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Loi Hyper élastique fondée sur le potentiel développé par   *
 *     Tréloar, mais en incorporant une déformation volumique.          *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef TRELOARN_H
#define TRELOARN_H

#include "Hyper3DN.h"


/// @addtogroup Les_lois_hyperelastiques
///  @{
///


class TreloarN : public  Hyper3DN
{
  public :
    // VARIABLES PUBLIQUES :
    
 // CONSTRUCTEURS :
 // Constructeur par defaut
 TreloarN ();
 // Constructeur de copie
 TreloarN (const TreloarN& loi) ;
 // DESTRUCTEUR :
 ~TreloarN () 
   {};
		
// Lecture des donnees de la classe sur fichier
void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

 // affichage de la loi
 void Affiche() const ;
 // test si la loi est complete
 // = 1 tout est ok, =0 loi incomplete
 int TestComplet();
	 
 //----- lecture écriture de restart -----
 // cas donne le niveau de la récupération
 // = 1 : on récupère tout
 // = 2 : on récupère uniquement les données variables (supposées comme telles)
 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
 // cas donne le niveau de sauvegarde
 // = 1 : on sauvegarde tout
 // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
 void Ecriture_base_info_loi(ofstream& sort,const int cas);
            
// affichage et definition interactive des commandes particulières à chaques lois
void Info_commande_LoisDeComp(UtilLecture& lec);  		  	  
        
// création d'une loi à l'identique et ramène un pointeur sur la loi créée
Loi_comp_abstraite* Nouvelle_loi_identique() const {return (new TreloarN(*this));};
     
//  METHODES  découlant de  méthodes virtuelles définies dans  HyperDN

 // calcul du potentiel et de ses dérivées premières
void Potentiel (double& jacobien_0,double& Ieps,double& V,double& bIIb,
             double& E,double& EV,double& EbIIb,double& EIeps); 
 // calcul  du potentiel et de ses dérivées premières et secondes
void Potentiel_et_var( double& jacobien_0,double&  Ieps,double&  V,double& bIIb,
          double& E,double& EV,double& EbIIb,double& EIeps ,
          double& EVV,double& EbIIb2,double& EIeps2,
          double& EVbIIb,double& EVIeps,double& EbIIbIeps );
            
 
  // VARIABLES PROTEGEES :
   // paramètres de la loi de comportement
    double K;  // coefficient de compressibilité volumique
    double C;  // coefficient de Treloar 

    // METHODES PROTEGEES :

 };
 /// @}  // end of group

#endif  
