

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Gestion des differentes lois de comportement.              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef LESLOISDECOMP_H
#define LESLOISDECOMP_H


#include "LoiAbstraiteGeneral.h"
#include "UtilLecture.h"
#include "string"
#include "LesReferences.h"
#include "MotCle.h"
#include <list>
#include "LesCourbes1D.h"

class LesLoisDeComp
{
  public :
    // classe de stockage
    class RefLoi
      {// surcharge de l'operator de lecture typée
       friend istream & operator >> (istream &, LesLoisDeComp::RefLoi &);
       // surcharge de l'operator d'ecriture typée
       friend ostream & operator << (ostream &, const LesLoisDeComp::RefLoi &);
       public :  string* nom_maillage; // nom du maillage associé
                 string st1; // nom d'une reference
                 string st2; // nom d'une loi
         RefLoi() : st1(),st2(),nom_maillage(NULL) {};        
  //       RefLoi(string & s1,string & s2) : nom_maillage(NULL),st1(s1), st2(s2) {};
         RefLoi(string* nom, string & s1,string & s2) 
           : nom_maillage(NULL),st1(s1), st2(s2) { if (nom != NULL)nom_maillage=new string(*nom); };
         RefLoi (const RefLoi & a) 
           : nom_maillage(NULL),st1(a.st1), st2(a.st2) 
              { if (a.nom_maillage != NULL) nom_maillage=new string(*(a.nom_maillage)); };
         ~RefLoi() {if (nom_maillage != NULL) delete nom_maillage;}; 
         RefLoi& operator = (const RefLoi & a);  
         // changement de nom de maillage
         void Change_nom_maillage(const string& nouveau);
	    //----- lecture écriture de restart -----
	    // cas donne le niveau de la récupération
        // = 1 : on récupère tout
        // = 2 : on récupère uniquement les données variables (supposées comme telles)
	    void Lecture_base_info(ifstream& ent,const int cas);
        // cas donne le niveau de sauvegarde
        // = 1 : on sauvegarde tout
        // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	    void Ecriture_base_info(ofstream& sort,const int cas) const;
       }; 
    class Loi
      {// surcharge de l'operator de lecture typée
       friend istream & operator >> (istream &, LesLoisDeComp::Loi &);
       // surcharge de l'operator d'ecriture typée
       friend ostream & operator << (ostream &, const LesLoisDeComp::Loi &);
       public :  string st1; // nom d'une loi
         LoiAbstraiteGeneral * pt ; // pointeur de loi
         Loi() {};        
         Loi(string & s1,LoiAbstraiteGeneral * p ) :
            st1(s1)
           { pt = p;}; 
         Loi (const Loi & a) : 
            st1(a.st1)
              { pt = a.pt;};
         Loi& operator = (const Loi & a)  
           { st1 = a.st1;pt = a.pt; return *this; };        
	    //----- lecture écriture de restart -----
	    // cas donne le niveau de la récupération
        // = 1 : on récupère tout
        // = 2 : on récupère uniquement les données variables (supposées comme telles)
	    void Lecture_base_info(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
        // cas donne le niveau de sauvegarde
        // = 1 : on sauvegarde tout
        // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	    void Ecriture_base_info(ofstream& sort,const int cas);
       }; 
        
    friend class Loi;
    // CONSTRUCTEURS :
    LesLoisDeComp (); // par defaut
    // DESTRUCTEUR :
    
    // METHODES PUBLIQUES :

    // lecture des lois de comportement
    void LecturelesLoisDeComp(UtilLecture& lec,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD); 
    
    // affichage des lois de comportement
    void Affiche() const ;  
    
    // affichage et definition interactive des commandes 
    void Info_commande_lesLoisDeComp(UtilLecture& lec,LesReferences& lesRef);
          
    // ramene le tableau de reference de loi
    // tableau des références de loi qui servent directement dans le calcul avec les éléments
    inline const Tableau <RefLoi> & TabRefLoi() const 
      { return tabRefLoi; };
      
    // ramene le tableau des  loi
    // tableau de toutes les lois lues: même celles qui ne servent pas directement, mais qui
    // soit ne servent pas du tout, ou soit servent indirectement via par exemple des umat
    inline const Tableau <Loi> & TabLoi() const 
      { return tabLoi; };
      
    // ramene le pointeur de LoiAbstraiteGeneral en fonction 
    // 1) du nom de la loi, 2) du nom d'une reference
    //  ou null si la recherche n'abouti pas
    LoiAbstraiteGeneral * PtLoi_abstraite(const string& st) const ;
    
    // indique aux différentes loi que l'on doit utiliser une loi tangente simplifiée
    // ou non suivant la valeur du paramètre de passage
    void Loi_simplifie(bool ordre);
    // récup du type de loi: simplifiée ou non
    bool Test_loi_simplife(); 
         
    // lecture de donnée en fonction d'un indicateur : int type
    // pour l'instant ne fait rien
    void   LectureDonneesExternes(UtilLecture& ,LesReferences& ,const int ,const string&) {};
    
    //indique aux lois Umat éventuelles le numéro d'itération 
    void MiseAJour_umat_nbiter(const int& num_iter);
    // ou le numéro d'incrément 
    void MiseAJour_umat_nbincr(const int& num_incr);
	 
    //----- lecture écriture de restart -----
    // cas donne le niveau de la récupération
       // = 1 : on récupère tout
       // = 2 : on récupère uniquement les données variables (supposées comme telles)
    void Lecture_base_info(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                               ,LesFonctions_nD& lesFonctionsnD);
       // cas donne le niveau de sauvegarde
       // = 1 : on sauvegarde tout
       // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
    void Ecriture_base_info(ofstream& sort,const int cas);
	 
    // définition d'un pointeur de loi en fonction d'une chaine de caractères
    static LoiAbstraiteGeneral * Def_loi(string & nom);
     
    // sortie sur fichier des temps cpu spécifiques à toutes les lois
    void Sortie_temps_cpu(UtilLecture& lec);

  private :  
    // VARIABLES PROTEGEES :
    // tableau des références de loi qui servent directement dans le calcul avec les éléments
    Tableau <RefLoi> tabRefLoi;
    // tableau de toutes les lois lues: même celles qui ne servent pas directement, mais qui
    // soit ne servent pas du tout, ou soit servent indirectement via par exemple des umat
    Tableau <Loi> tabLoi;
    // la liste des lois umat éventuelles pour optimiser les routines MiseAJour_umat_nbiter
    // et MiseAJour_umat_nbincr
    list <LoiAbstraiteGeneral *> list_de_umat;
    
    // la liste de tous les types de lois actuellement possibles (mais pas forcément lue)
    static list <LoiAbstraiteGeneral *> list_de_loi;
         
    // METHODES PROTEGEES :
    // lecture des references de materiaux
    void LecRef(UtilLecture& lec,LesReferences& lesRef);
    // lecture des  materiaux
    void LecMateriaux(UtilLecture& lec,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

 };
 
#endif  
