

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "LoiAbstraiteGeneral.h"


		// CONSTRUCTEURS :
LoiAbstraiteGeneral::LoiAbstraiteGeneral () :  // Constructeur par defaut
   id_comp(RIEN_COMP),dim(0),categorie_loi_comp(RIEN_CATEGORIE_LOI_COMP)
   ,enu_ddl_type_pt(NU_DDL)
		 { // la dimension est testée lorsque l'on regarde si la loi est complète
		 };  
// Constructeur utile si l'identificateur du nom de la loi
// de comportement et la dimension sont connus ainsi que la catégorie de la loi	
LoiAbstraiteGeneral::LoiAbstraiteGeneral (Enum_comp id_compor, int dimension
                                          ,Enum_categorie_loi_comp id_categorie,Enum_ddl ddl) :
   id_comp(id_compor),dim(dimension),categorie_loi_comp(id_categorie),enu_ddl_type_pt(ddl)
   { // la dimension est testée lorsque l'on regarde si la loi est complète
       };
// Constructeur utile si l'identificateur du nom de la loi
// de comportement et la dimension sont connus ainsi que la catégorie de la loi	
LoiAbstraiteGeneral::LoiAbstraiteGeneral (string nom,int dimension
                                          ,Enum_categorie_loi_comp id_categorie,Enum_ddl ddl) :
   id_comp(Id_nom_comp(nom)),dim(dimension),categorie_loi_comp(id_categorie),enu_ddl_type_pt(ddl)
   { // la dimension est testée lorsque l'on regarde si la loi est complète
    };
// Constructeur de copie	
LoiAbstraiteGeneral::LoiAbstraiteGeneral (const LoiAbstraiteGeneral& a) :
   id_comp(a.id_comp),dim(a.dim),categorie_loi_comp(a.categorie_loi_comp) 
   ,enu_ddl_type_pt(a.enu_ddl_type_pt)   
      {// la dimension est testée lorsque l'on regarde si la loi est complète
      };
// DESTRUCTEUR  :
LoiAbstraiteGeneral::~LoiAbstraiteGeneral ()
		{ };
// METHODES NON VIRTUELLES :

// test si la loi est complete
int LoiAbstraiteGeneral::TestComplet()
    { int ret =1;
      if (id_comp == RIEN_COMP) 
       { cout << " \n identificateur de la loi de comportement non défini "
                   << '\n';
         ret = 0;
       }
      if (dim == 0)
       { cout << " \n  la dimension de la loi " <<  Nom_comp(id_comp) 
              << " n'est pas défini " << "\n ";
         ret = 0;
       };
                
       return ret; 
    }; 


// --------------- fonctions protégées appelées par les classes dérivées ------------

	    // cas donne le niveau de la récupération
        // = 1 : on récupère tout
        // = 2 : on récupère uniquement les données variables (supposées comme telles)
void LoiAbstraiteGeneral::Lect_base_info_loi (ifstream& ent,const int cas
           ,LesReferences& ,LesCourbes1D& ,LesFonctions_nD& lesFonctionsnD)
  { string toto; 
	if (cas == 1) 
	 { // le nom de la loi de comportement
       ent >> toto ;
       if (toto != "type_de_loi_de_comportement:")
         { cout << "\n erreur en lecture d'une loi de comportement, nom lu: " << toto
                << "\n LoiAbstraiteGeneral::Lect_base_info_loi (...";
           Sortie(1);     
          }
       // lecture de la suite des données, on considère qu'il n'y a plus d'erreur car l'écriture
       // normalement est automatique
       string nom,categorie;
       ent >> nom >> toto >> dim >> toto >> categorie; 
       id_comp = Id_nom_comp(nom.c_str());
       categorie_loi_comp =  Id_nom_categorie_loi_comp(categorie.c_str());  
	  }
   };

        // cas donne le niveau de sauvegarde
        // = 1 : on sauvegarde tout
        // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
void LoiAbstraiteGeneral::Ecrit_base_info_loi(ofstream& sort,const int cas) const
  { if (cas == 1) 
     { // le nom de la loi de comportement
       sort << "\n type_de_loi_de_comportement: " << Nom_comp(id_comp) << " "
       // la dimension
            << " dimension " << dim << " "
       // la catégorie
            << " categorie: " << Nom_categorie_loi_comp(categorie_loi_comp) << " ";     
      }
   }; 

// fonction utilitaire pour lire une grandeur qui dépend d'une fonction
// si la courbe existe déjà dans LesCourbes1D, ramène un pointeur dessus
// si la courbe n'exite pas déjà, création indépendamment de LesCourbes (donc via un new)
// et ramène un pointeur dessus
Courbe1D * LoiAbstraiteGeneral::Lecture_courbe(UtilLecture * entreePrinc,LesCourbes1D& lesCourbes1D)
{ Courbe1D * retour_courbe = NULL; string nom;
  // lecture de la courbe
  *(entreePrinc->entree) >>  nom;    
  // on regarde si la courbe existe, si oui on récupère la référence 
  if (lesCourbes1D.Existe(nom)) 
    { retour_courbe = lesCourbes1D.Trouve(nom);
    }
  else
   { // sinon il faut la lire maintenant
     string non_courbe("_");   
     retour_courbe = Courbe1D::New_Courbe1D(non_courbe,Id_Nom_Courbe1D (nom.c_str()));
     // lecture de la courbe
     retour_courbe->LectDonnParticulieres_courbes (non_courbe,entreePrinc);
   };
  // retour de la courbe
  return retour_courbe;
}





