// FICHIER : Loi_iso_elas3D.h
// CLASSE : Loi_iso_elas3D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
*     DATE:        15/09/2020                                          *
*                                                                $     *
*     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
*                                                                $     *
*     PROJET:      Herezh++                                            *
*                                                                $     *
************************************************************************
* La classe Loi_iso_elas3D permet de calculer la contrainte et ses     *
* derivees pour une loi isotrope elastique en 3D sans condition        *
* particulieres.                                                       *
* Il s'agit d'une classe derivee de la classe Loi_comp_abstraite.      * 
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*                                                                      *
*     VERIFICATION:                                                    *
*                                                                      *
*     !  date  !   auteur   !       but                          !     *
*     ------------------------------------------------------------     *
*     !        !            !                                    !     *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*     MODIFICATIONS:                                                   *
*     !  date  !   auteur   !       but                          !     *
*     ------------------------------------------------------------     *
*                                                                $     *
************************************************************************/

// La classe Loi_iso_elas3D permet de calculer la contrainte et ses derivees pour une loi
// isotrope elastique en 3D sans condition particulieres.
// Il s'agit d'une classe derivee de la classe Loi_comp_abstraite.


#ifndef LOI_ISO_ELAS3D_H
#define LOI_ISO_ELAS3D_H


#include "Loi_comp_abstraite.h"

/// @addtogroup Les_lois_hooke
///  @{
///



class Loi_iso_elas3D : public Loi_comp_abstraite
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Loi_iso_elas3D ();

		// Contructeur fonction du E et du nu
		Loi_iso_elas3D(const double& EE,const double& nunu);
		
		// Constructeur de copie
		Loi_iso_elas3D (const Loi_iso_elas3D& loi) ;
		
		// DESTRUCTEUR :
		~Loi_iso_elas3D ();
 
  // initialise les donnees particulieres a l'elements
  // de matiere traite ( c-a-dire au pt calcule)
  // Il y a creation d'une instance de SaveResul particuliere
  // a la loi concernee
  // la SaveResul classe est remplie par les instances heritantes
  // le pointeur de SaveResul est sauvegarde au niveau de l'element
  // c'a-d que les info particulieres au point considere sont stocke
  // au niveau de l'element et non de la loi.
  class SaveResulLoi_iso_elas3D: public SaveResul
   { public :
         SaveResulLoi_iso_elas3D(): // constructeur par défaut :
           E(-ConstMath::trespetit),nu(-2.*ConstMath::trespetit)
           ,E_t(-ConstMath::trespetit),nu_t(-2.*ConstMath::trespetit),map_type_quelconque() {};
         SaveResulLoi_iso_elas3D(const SaveResulLoi_iso_elas3D& sav): // de copie
           E(sav.E),nu(sav.nu),E_t(sav.E_t),nu_t(sav.nu_t)
           ,map_type_quelconque(sav.map_type_quelconque) {};
    
         virtual ~SaveResulLoi_iso_elas3D() {}; // destructeur
         // définition d'une nouvelle instance identique
         // appelle du constructeur via new
         SaveResul * Nevez_SaveResul() const{return (new SaveResulLoi_iso_elas3D(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a)
           { SaveResulLoi_iso_elas3D& sav = *((SaveResulLoi_iso_elas3D*) &a);
             E=sav.E; nu=sav.nu;E_t=sav.E_t; nu_t=sav.nu_t;
             map_type_quelconque = sav.map_type_quelconque;
             return *this;
           };
         //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
         void Lecture_base_info (ifstream& ent,const int cas)
           {string toto; ent >> toto >> E >> toto >> nu; E_t=E; nu_t = nu; };
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
         void Ecriture_base_info(ofstream& sort,const int cas)
           {sort << "\n E= "<< E <<" nu= "<< nu << " ";};
    
         // mise à jour des informations transitoires
         void TdtversT()
          {E_t = E; nu_t = nu;
           // mise à jour de la liste des grandeurs quelconques internes
           Mise_a_jour_map_type_quelconque();
          };
         void TversTdt()
          {E = E_t; nu = nu_t;
           // mise à jour de la liste des grandeurs quelconques internes
           Mise_a_jour_map_type_quelconque();
          };
    
         // affichage à l'écran des infos
         void Affiche() const
          { cout <<"\n E= "<< E <<" nu= "<< nu << " ";};
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma) {};
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         virtual SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {return NULL;};

    
         //-------------------------------------------------------------------
         // données
         //-------------------------------------------------------------------
         double E,E_t,nu,nu_t; // les paramètres matériaux réellement utilisés
         
         // --- gestion d'une map de grandeurs quelconques éventuelles ---

         // une map de grandeurs quelconques particulière qui peut servir aux classes appelantes
         // il s'agit ici d'une map interne qui a priori ne doit servir qu'aux class loi de comportement
         // un exemple d'utilisation est une loi combinée qui a besoin de grandeurs spéciales définies
         // -> n'est pas sauvegardé, car a priori il s'agit de grandeurs redondantes
         map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> > map_type_quelconque;

         // récupération des type quelconque sous forme d'un arbre pour faciliter la recherche
         const map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> >* Map_type_quelconque()
               const {return &map_type_quelconque;};
        private:
          void Mise_a_jour_map_type_quelconque();
     
          // ---- fin gestion d'une liste de grandeurs quelconques éventuelles ---
   };

  SaveResul * New_et_Initialise()
    { SaveResulLoi_iso_elas3D * pt = new SaveResulLoi_iso_elas3D();
      // insertion éventuelle de conteneurs de grandeurs quelconque
      Insertion_conteneur_dans_save_result(pt);
      return pt;
    };
 
  friend class SaveResulLoi_iso_elas3D;

	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
          
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Loi_iso_elas3D(*this)); };
		  	  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
            
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);

  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
       (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul * ,list<int>& decal) const ;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
 
 
  // insertion des conteneurs ad hoc concernant le stockage de grandeurs quelconques
  // passée en paramètre, dans le save result: ces conteneurs doivent être valides
  // c-a-d faire partie de listdeTouslesQuelc_dispo_localement
  virtual void Insertion_conteneur_dans_save_result(SaveResul * saveResul);
 
  // activation du stockage de grandeurs quelconques qui pourront ensuite être récupéré
  // via le conteneur SaveResul, si la grandeur n'existe pas ici, aucune action
  virtual void Activation_stockage_grandeurs_quelconques(list <EnumTypeQuelconque >& listEnuQuelc);

	protected :

  // donnees protegees
	 double E,nu; // paramètres de la loi
	 Courbe1D* E_temperature; // courbe éventuelle d'évolution de E en fonction de la température
  Fonction_nD * E_nD; // fonction nD éventuelle pour E
  Fonction_nD * nu_nD; // fonction nD éventuelle pour nu
	 short int cas_calcul; // indique le choix entre différents types de calcul possible
	                       // = 0 : calcul normal
	                       // = 1 : calcul seulement déviatorique (la partie sphérique est mise à zéro)
	                       // = 2 : calcul seulement sphérique (la partie déviatorique est mise à zéro)
  // on introduit un certain nombre de tenseur du quatrième ordre, qui vont nous servir pour
  // Calcul_dsigma_deps, dans le cas où on n'est pas en orthonormee
		Tenseur3HHHH  I_x_I_HHHH,I_xbarre_I_HHHH,I_x_eps_HHHH,Ixbarre_eps_HHHH;
	   		
  // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

		  	
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
                (const PtIntegMecaInterne& ,const Deformation &
                 ,Enum_dure,const ThermoDonnee&
                 ,const Met_abstraite::Impli* ex_impli
                 ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                 ,const Met_abstraite::Umat_cont* ex_umat
                 ,const List_io<Ddl_etendu>* exclure_dd_etend
                 ,const List_io<const TypeQuelconque *>* exclure_Q
                 ) {};

};
/// @}  // end of group


#endif		

		

