// FICHIER : Loi_Umat.h
// CLASSE : Loi_Umat


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        11/03/2005                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Loi Umat permet d'utiliser des loi_umat découlant d'un    *
 *            d'un autre programme (processus). Le passage d'information*
 *            s'effectue via deux pipes nommés.                         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef LOI_UMAT_H
#define LOI_UMAT_H


#include "Loi_comp_abstraite.h"
#include "UmatAbaqus.h" 
#include "Tenseur3.h"
#include "Tenseur2.h"
#include "Tenseur1.h"
#include "TenseurQ.h"
#include "TenseurQ-2.h"
#include "TenseurQ-1.h"


class Loi_Umat : public Loi_comp_abstraite
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
  // par défaut c'est une loi 3D mais cela pourrait-être des CP
		Loi_Umat (Enum_comp enu = LOI_VIA_UMAT);
		
		
		// Constructeur de copie
		Loi_Umat (const Loi_Umat& loi) ;
		
		// DESTRUCTEUR :
		
		~Loi_Umat ();
		
				
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_Loi_Umat: public SaveResul 
		 { public : 
	        SaveResul_Loi_Umat(); // constructeur par défaut (a ne pas utiliser)
	        // le constructeur courant
	        SaveResul_Loi_Umat(list <SaveResul*>& l_des_SaveResul,list <TenseurHH* >& l_siHH
	                                     ,list <TenseurHH* >& l_siHH_t);
	        // constructeur de copie 
	        SaveResul_Loi_Umat(const SaveResul_Loi_Umat& sav );
	        // destructeur
	        ~SaveResul_Loi_Umat(){};
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_Loi_Umat(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() ;
	        void TversTdt() ;
			  
         // affichage à l'écran des infos
         void Affiche() const
          { cout << "\n SaveResul_Loi_Umat: ";
          save_pour_loi_ext->Affiche(); 
         };
   
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma)
            {return save_pour_loi_ext->ChBase_des_grandeurs(beta,gamma);};
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // données protégées
	        SaveResul* save_pour_loi_ext;
         double hsurh0,h_tsurh0;
   };

  // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise();
				
	 // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
          
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const
    {Loi_Umat::SaveResul_Loi_Umat& sav = *((Loi_Umat::SaveResul_Loi_Umat*) &saveResul);
     return sav.hsurh0;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Loi_Umat(*this)); };
		
		// ----- gestion des incréments et itérations --------
		//  mise à jour des infos nécessaires pour l'umat
		
		void Mise_a_jour_num_increment(int num) {umatAbaqus.nb_increment=num;};
		void Mise_a_jour_num_iteration(int num) {umatAbaqus.nb_step=num;};
		
		// idem pour le numéro d'élément et le numéro de point d'intégration
		void Mise_a_jour_nbe_nbptinteg(int nbe, int nbptinteg) 
		        {umatAbaqus.nb_elem = nbe;umatAbaqus.nb_pt_integ = nbptinteg;};

	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
     // affichage et definition interactive des commandes particulières à chaques lois
     void Info_commande_LoisDeComp(UtilLecture& lec);
			       
	 // retour du nom de la loi associée
	 const string& NomDeLaLoi() const {return nom_de_la_loi;}; 
	 
	 // définition de la loi ext dans le cas d'une umat directe sans pipe
	 void DefLoiExt(LoiAbstraiteGeneral* lext) {loi_ext=lext;	};
	 // indique si oui ou non la loi utilise une umat interne
	 bool Utilise_une_umat_interne() const {return utilisation_umat_interne;};
		  	       
	protected :

        // donnees protegees
        string nom_de_la_loi; // nom de la loi
        bool utilisation_umat_interne; // indique si oui ou non on utilise une umat interne ou alors via pipe
        // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//        int permet_affichage; // pour permettre un affichage spécifique dans les méthodes, pour les erreurs et warning
        UmatAbaqus umatAbaqus; // le conteneur d'information
        // par défaut on définit 3 métriques ce qui nous permet de naviguer entre les 3 dimensions
        // certaines ne servent pas, mais comme il s'agit de la définition de la loi, on immagine
        // qu'il y aura peut d'exemplaires d'où un impact de stockage négligeable
        Met_abstraite umat_met3D; //  métrique 3D particulière à l'Umat
        Met_abstraite umat_met2D; //  métrique 2D particulière à l'Umat
        Met_abstraite umat_met1D; //  métrique 1D particulière à l'Umat
        Tenseur2HHHH d_sigma_deps_2D; // un tenseur de travail pour  la méthode Calcul_DsigmaHH_tdt en CP
        Tenseur1HHHH d_sigma_deps_1D; // un tenseur de travail pour  la méthode Calcul_DsigmaHH_tdt en CP2

        // élément par défaut pour umat_met
        // def des  bases naturelles et duales par rapport aux coordonnées initiales
        // l'indice x est pour rappeler que les coordonnées matérielles sont les x_0
        BaseB gixB_0,gixB_t,gixB_tdt;  BaseH gixH_0,gixH_t,gixH_tdt; 
        // idem pour les métriques, sachant que la métrique initiale est l'identité
        TenseurBB* gabBB_tdt; TenseurHH* gabHH_tdt; // tenseurs dans Ia
        TenseurBB* gabBB_t; TenseurHH* gabHH_t; // tenseurs dans Ia
        // les grandeurs qui pour l'instant ne servent pas, on y met un pointeur nul
        BaseB * ggaB_0null, * ggaB_tnull;BaseH * ggaH_0null;
        TenseurBB * ggradVmoyBB_tnull, * ggradVmoyBB_tdtnull;
        TenseurBB * ggradVBB_tdtnull;
 
        // --//\\-- grandeurs particulières pour le cas contraintes planes
        Met_abstraite::Umat_cont*  umat_cont_3D; // version 3D
        Met_abstraite::Umat_cont*  umat_cont_2D; // version 2D
        BaseB Ip2_B; // base orthonormée locale

        // -- variables nécessaires pour la création de umat_cont_3D
        // certaines grandeurs sont associées à un pointeur qui peut soit être nulle soit pointer sur le conteneur
        // l'intérêt est que le fait d'avoir un pointeur nul est parfois utilisé pour éviter un calcul
        BaseB giB_0_3D;
        BaseH giH_0_3D;
        BaseB giB_t_3D;
        BaseH giH_t_3D;
        BaseB  giB_tdt_3D;
        BaseH  giH_tdt_3D;
        Tenseur3BB  gijBB_0_3D;
        Tenseur3HH  gijHH_0_3D;
        Tenseur3BB  gijBB_t_3D;
        Tenseur3HH  gijHH_t_3D;
        Tenseur3BB  gijBB_tdt_3D;
        Tenseur3HH  gijHH_tdt_3D;
 
        TenseurBB * gradVmoyBB_t_3D_P;  Tenseur_ns3BB  gradVmoyBB_t_3D;
        TenseurBB * gradVmoyBB_tdt_3D_P; Tenseur_ns3BB  gradVmoyBB_tdt_3D;
        TenseurBB * gradVBB_tdt_3D_P;  Tenseur_ns3BB  gradVBB_tdt_3D;
        double jacobien_tdt_3D;double jacobien_t_3D;double jacobien_0_3D;  // pour les jacobiens on considère qu'ils existent toujours
        // -- variables nécessaires pour la création de umat_cont_2D
        BaseB Ip3B_0_3D; // coordonnées de la base locale ortho dans le repère globale
        BaseH Ip3H_0_3D; //   3 vecteurs de dimensions 3, les deux premiers sont dans le plan des CP

        // puis les grandeurs pointées dans umat_cont_2D
        // certaines grandeurs sont associées à un pointeur qui peut soit être nulle soit pointer sur le conteneur
        // l'intérêt est que le fait d'avoir un pointeur nul est parfois utilisé pour éviter un calcul
 
        BaseB giB_0_2D;  // coordonnée de giB_0 localement dans la base Ip2B_0 = IP2H_0 car orthonormée
        BaseH giH_0_2D;  // coordonnée de giH_0 localement dans la base Ip2B_0 = IP2H_0 car orthonormée
        BaseB giB_t_2D;
        BaseH giH_t_2D;
        BaseB  giB_tdt_2D;
        BaseH  giH_tdt_2D;
        Tenseur2BB  gijBB_0_2D;
        Tenseur2HH  gijHH_0_2D;
        Tenseur2BB  gijBB_t_2D;
        Tenseur2HH  gijHH_t_2D;
        Tenseur2BB  gijBB_tdt_2D;
        Tenseur2HH  gijHH_tdt_2D;

        TenseurBB * gradVmoyBB_t_2D_P;  Tenseur_ns3BB  gradVmoyBB_t_2D;
        TenseurBB * gradVmoyBB_tdt_2D_P; Tenseur_ns3BB  gradVmoyBB_tdt_2D;
        TenseurBB * gradVBB_tdt_2D_P;  Tenseur_ns3BB  gradVBB_tdt_2D;
        double jacobien_tdt_2D;double jacobien_t_2D;double jacobien_0_2D;  // pour les jacobiens on considère qu'ils existent toujours
        // --//\\-- fin grandeurs particulières pour le cas contraintes planes

        // un compteur pour alouer un numéro d'ordre qui tiens lieu de numéro d'élément
        static int compteur_simili_num_elemEtPtInteg,max_simili_num_ptinteg;
        // un pointeur de loi qui est utilisé lorsque on n'utilise pas de pipe
        // en développement ou dans le cas non unix
        LoiAbstraiteGeneral* loi_ext;
		
        // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
                (const PtIntegMecaInterne& ,const Deformation &
                 ,Enum_dure,const ThermoDonnee&
                 ,const Met_abstraite::Impli* ex_impli
                 ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                 ,const Met_abstraite::Umat_cont* ex_umat
                 ,const List_io<Ddl_etendu>* exclure_dd_etend
                 ,const List_io<const TypeQuelconque *>* exclure_Q
                 ) {};

  // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
  // pour répercuter les modifications de la température
  // ici utiliser pour modifier la température des lois élémentaires
  // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
  void RepercuteChangeTemperature(Enum_dure temps);

  //---  méthodes internes
  // passage des grandeurs métriques de l'ordre 2 à 3: cas implicite
  void Passage_metrique_ordre2_vers_3(const Met_abstraite::Umat_cont& ex);
 
  // fonction de travail
  static int Choix_dim(Enum_comp enu);
 
  // calcul des modules de compressibilité et de cisaillement
  // en fonction des résultats de l'umat
  void Calcul_compressibilite_cisaillement(const Met_abstraite::Umat_cont& ex
                                          ,double & module_compressibilite,double & module_cisaillement);

};


#endif		

		

