// FICHIER : Loi_maxwell2D_D.h
// CLASSE : Loi_maxwell2D_D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        08/06/2003                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Loi_maxwell2D_D definit une loi 2D de type en   *
 *            déformations planes de maxwell, c'est-à-dire une loi de   *
 *            hook en série avec à amortisseur. La partie visqueuse est *
 *            soit purement déviatorique ou complète.                   *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef LOI_MAXWELL2D_D_H
#define LOI_MAXWELL2D_D_H


#include "Loi_comp_abstraite.h"


/// @addtogroup Les_lois_viscoelastique
///  @{

class Loi_maxwell2D_D : public Loi_comp_abstraite
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Loi_maxwell2D_D ();
		
		
		// Constructeur de copie
		Loi_maxwell2D_D (const Loi_maxwell2D_D& loi) ;
		
		// DESTRUCTEUR :
		
		~Loi_maxwell2D_D ();
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_Loi_maxwell2D_D: public SaveResul 
		 { public : 
	        SaveResul_Loi_maxwell2D_D(): sig33(0.),sig33_t(0.) {}; // constructeur par défaut 
	        // le constructeur courant
	        SaveResul_Loi_maxwell2D_D(const double& sigo33,const double& sigo33_t):
	          sig33(sigo33),sig33_t(sigo33_t) {};
	        // constructeur de copie 
	        SaveResul_Loi_maxwell2D_D(const SaveResul_Loi_maxwell2D_D& sav ):
	          sig33(sav.sig33),sig33_t(sav.sig33_t) {};
	        // destructeur
	        ~SaveResul_Loi_maxwell2D_D() {};
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_Loi_maxwell2D_D(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a)
           {SaveResul_Loi_maxwell2D_D& sav = *((SaveResul_Loi_maxwell2D_D*) &a);
             sig33=sav.sig33;sig33_t=sav.sig33_t;
             return *this;};
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() {sig33_t=sig33;};
	        void TversTdt() {sig33=sig33_t;};
			  
			      // affichage à l'écran des infos
			      void Affiche() const
			       { cout << "\n SaveResul_Loi_maxwell2D_D: sig33= " << sig33
				             << " sig33_t= " << sig33_t << " ";
				      };
   
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // ici il n'y a pas de données tensorielles donc rien n'a faire
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma){};
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() {return 0.;};
	        
	        // données protégées
	        // ici on considère que g^33=g_33=1 
	        double sig33; // contrainte normale courante
	        double sig33_t; // la dernière enregistrée
   };

  // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise() 
				    { SaveResul_Loi_maxwell2D_D * pt = new SaveResul_Loi_maxwell2D_D(); return pt;};
				
	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
	 
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
        
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return 0.;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Loi_maxwell2D_D(*this)); };
		
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
				  	
  // activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
  //  ici concerne de la vérification d'existence, mais c'est appelé une fois, c'est ça l'intérêt
  virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
 
	protected :
	   // donnée de la loi
	   double E,nu,mu; // module d'young, coeff de poisson et coef de viscosité sur D
	   double mu_p; // coef de viscosité éventuel sur la partie sphérique de la contrainte
		//---- dépendance éventuelle à la température ------
	   Courbe1D* mu_p_temperature; // courbe éventuelle d'évolution de mu_p en fonction de la température
	   bool existe_mu_p; // indique si oui ou non il y a de la viscosité sur la partie sphérique
	   Courbe1D* E_temperature; // courbe éventuelle d'évolution de E en fonction de la température
	   Courbe1D* mu_temperature; // courbe éventuelle d'évolution de mu en fonction de la température
	   int type_derive; // type de dérivée objective utilisée pour sigma
		//---- dépendance éventuelle à D ------
	   int depend_de_D; // indique si oui ou non on a une viscosité et/ou le module d'Young dépendant de D
		  // =0 : viscosité linéaire (pas de dépendance à D) , =1 : la viscosité (déviatorique et/ou sphérique) seule dépend de D, 
		  // =2 : le module d'Young seul dépend de D, = 3: la viscosité et le module d'Young dépendent de D,
	   // si oui, implique que fac_mu_cissionD et/ou fac_E_cissionD existent
	   Courbe1D* fac_mu_cissionD; // courbe multiplicative éventuelle de mu, fonction de sqrt(D_barre:D_barre)
	   Courbe1D* fac_E_cissionD; // courbe multiplicative éventuelle de E, fonction de sqrt(D_barre:D_barre)
		//---- dépendance éventuelle à eps ------
	   int depend_de_eps; // indique si oui ou non on a une viscosité et/ou le module d'Young dépendant de eps
		  // =0 : viscosité linéaire (pas de dépendance à eps) , =1 : la viscosité (déviatorique et/ou sphérique) seule dépend de eps, 
		  // =2 : le module d'Young seul dépend de eps, = 3: la viscosité et le module d'Young dépendent de eps,
	   // si oui, implique que fac_mu_Mises_Eps et/ou fac_E_Mises_Eps existent
	   Courbe1D* fac_mu_Mises_Eps; // courbe multiplicative éventuelle de mu, fonction de sqrt(2/3*Eps_barre:Eps_barre)
	   Courbe1D* fac_E_Mises_Eps; // courbe multiplicative éventuelle de E, fonction de sqrt(2/3*Eps_barre:Eps_barre)
  //--------- fin dépendance diverse -------------
	   bool seule_deviatorique; // drapeau indiquant éventuellement un calcul uniquement déviatorique
	   //------ modèle particulier au polymère -----------
	   // prise en compte de la cristalinité, de la pression, température, a priori uniquement sur la partie
	   // déviatoire (donc n'intervient pas sur la partie sphérique)
	   bool depend_cristalinite; // indique si oui ou non, on utilise ce modèle particulier
	   double nc,tauStar,D1,D2,D3,A1,At2,C1; // l'ensemble des paramètres pour la dépendance à la cristalinité
	   double taux_crista; // taux de cristalinité
	   bool volumique_visqueux; // indique si oui ou non dans le cas de la cristalinité, la partie volumique est
	                            // visqueuse
	   // crista_aux_noeuds indique si la cristalinité est calculée à partir des noeuds (valeur par défaut)
	   // ou directement à partir d'une valeur obtenue au point d'intégration
	   bool crista_aux_noeuds; 
 
    // ---codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
 
    // fonction surchargée dans les classes dérivée si besoin est
    virtual void CalculGrandeurTravail
            (const PtIntegMecaInterne& ptintmeca
             ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
             ,const Met_abstraite::Impli* ex_impli
             ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
             ,const Met_abstraite::Umat_cont* ex_umat
             ,const List_io<Ddl_etendu>* exclure_dd_etend
             ,const List_io<const TypeQuelconque *>* exclure_Q
            );
    // calcul de la viscosité dépendante de la cristalinité
    double ViscositeCristaline(double & P, double & gamma_point);

};
/// @}  // end of group


#endif		

		

