// FICHIER : Coordonnee1B.cc
// CLASSE : Coordonnee1B

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

//#include "Debug.h"

# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "ConstMath.h"
#include "MathUtil.h"

#include "Coordonnee1.h"


#ifndef  COORDONNEE1_H_deja_inclus


// Constructeur par defaut
// il y a initialisation des coordonnées à zéro 
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B () :
 CoordonneeB(1,coord1)
{   coord1[0] = 0.; 
};

// Constructeur suivant un booleen
// quelque soit la valeur du booleen il n'y a pas initialisation des coordonnées 
// ceci  pour aller plus vite par rapport au constructeur par défaut
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B (bool  ):
 CoordonneeB(1,coord1)
{  };

#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B (double x):
 CoordonneeB(1,coord1)
// Constructeur pour un point a deux dimensions
{	coord1[0]=x;
};

// constructeur fonction d'une adresse memoire ou sont stockee les coordonnees
// et d'une dimension ( l'existance de la place mémoire est a la charge
//  de l'utilisateur.
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B (double* t):
 CoordonneeB(1,coord1)
  { coord1[0]=t[0];
   };  						

// Constructeur de copie
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B (const Coordonnee1B& c):
 CoordonneeB(1,coord1)
{  coord1[0]=c.coord1[0];
};

// Constructeur de copie pour une instance indiférenciée
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::Coordonnee1B (const CoordonneeB& c):
 CoordonneeB(1,coord1)
{	
    #ifdef MISE_AU_POINT
	 if (c.Dimension () != 1)
		{ cout << "\nErreur la dimension de c est différente de 1 !! !\n";
		  cout << "Coordonnee1B::Coordonnee1B (const Coordonnee& c) \n";
		  Sortie(1);
		 } 
    #endif
    coord1[0]=c(1);
};

// DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B::~Coordonnee1B () 
{ dim = 0; // pour l'appel de ~Coordonnee()
  #ifdef MISE_AU_POINT
    coord=NULL;
  #endif
 };
// Desallocation de la place memoire allouee
// fonction définie dans la classe mère générique mais qui n'a pas de 
// sens ici
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::Libere ()
 {       
		 #ifdef MISE_AU_POINT
		   cout << "\nErreur cette fonction n'a pas cours !! !\n";
		   cout << "Coordonnee1B::Libere () \n";
		   Sortie(1);
         #endif
  };		


// Renvoie le nombre de coordonnees
#ifndef MISE_AU_POINT
  inline 
#endif
int Coordonnee1B::Dimension ()  const
 { return 1; };
 
// changement de la dimension
// fonction définie dans la classe mère générique mais qui n'a pas de 
// sens ici
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::Change_dim(int )
  { 
		 #ifdef MISE_AU_POINT
		   cout << "\nErreur cette fonction n'a pas cours !! !\n";
		   cout << "Coordonnee1B::Change_dim(int dimen) \n";
		   Sortie(1);
         #endif
  };
  
// Surcharge de l'operateur = : realise l'affectation entre deux points 
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B& Coordonnee1B::operator= (const Coordonnee1B& c)
  {	 coord1[0]=c.coord1[0];
     return (*this);			
   };
  		
		
// Surcharge de l'operateur - : renvoie l'oppose d'un point
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B Coordonnee1B::operator- () const 
		{	return Coordonnee1B(-coord1[0]);			
		};
		
// Surcharge de l'operateur - : realise la soustraction des
// coordonnees de deux points
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B Coordonnee1B::operator- (const Coordonnee1B& c) const 
		{ return Coordonnee1B(coord1[0]-c.coord1[0]);			
		};
		
// Surcharge de l'operateur + : realise l'addition des 
// coordonnees de deux points
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B Coordonnee1B::operator+ (const Coordonnee1B& c) const 
		{	return Coordonnee1B(coord1[0]+c.coord1[0]);		
		};
		
// Surcharge de l'operateur += 
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::operator+= (const Coordonnee1B& c)
		{  coord1[0]+=c.coord1[0];
		};

// Surcharge de l'operateur -= 
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::operator-= (const Coordonnee1B& c)
		{ coord1[0]-=c.coord1[0];
		};		

// Surcharge de l'operateur *= 
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::operator*= (double val)
		{	coord1[0]*=val;
		};	
		
// Surcharge de l'operateur * : multiplication de coordonnees par un scalaire
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B Coordonnee1B::operator* (double val) const 
		{ 
		  return Coordonnee1B(val*coord1[0]);
		};
		
// Surcharge de l'operateur * : produit scalaire entre coordonnees 
#ifndef MISE_AU_POINT
  inline 
#endif
double Coordonnee1B::operator* (const Coordonnee1H& c) const 
 { return ( coord1[0] * c.coord1[0]  );
  };

		
//  produit scalaire entre coordonnees covariantes et covariantes
#ifndef MISE_AU_POINT
  inline 
#endif
double Coordonnee1B::ScalBB(const Coordonnee1B& c) const 
 { return ( coord1[0] * c.coord1[0]  );
  };


// Surcharge de l'operateur / : division de coordonnees par un scalaire
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B Coordonnee1B::operator/ (double val) const 
 {
   #ifdef MISE_AU_POINT
    if(Dabs(val) <= ConstMath::trespetit)
    { cout << "\n erreur, division par zero ";
      cout << "\nCoordonnee1B::operator/ (double val) " << endl;
      Sortie (1);
     }
   #endif
   return Coordonnee1B(coord1[0] / val);  
  };
		
// Surcharge de l'operateur /= : division de coordonnees par un scalaire
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::operator/= (double val)
 {
  #ifdef MISE_AU_POINT
   if(Dabs(val) <= ConstMath::trespetit)
    { cout << "\n erreur, division par zero ";
      cout << "\nCoordonnee1B::operator/= (double val) " << endl;
      Sortie (1);
     }
  #endif
   coord1[0]/=val;
  };

// Surcharge de l'operateur == : test d'egalite 
// Renvoie 1 si les deux positions sont identiques
// Renvoie 0 sinon
#ifndef MISE_AU_POINT
  inline 
#endif
int Coordonnee1B::operator== (const Coordonnee1B& c) const 
		{  if (c.coord1[0]!=coord1[0]) 
				 return 0;
		   else 
		    	 return 1;
		};

// mise a zero des coordonnées
#ifndef MISE_AU_POINT
  inline 
#endif
void Coordonnee1B::Zero() 
		 { coord1[0] = 0.;
		  };  
		   
// Calcul de la norme euclidienne des composantes du point
#ifndef MISE_AU_POINT
  inline 
#endif
double Coordonnee1B::Norme ()  const 
 { return sqrt( coord1[0] * coord1[0] );
  };
// norme le vecteur coordonnée
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee1B& Coordonnee1B::Normer ()
 { double norme = this->Norme();
   #ifdef MISE_AU_POINT
   if(Dabs(norme) <= ConstMath::trespetit)
    { cout << "\n erreur, division par zero ";
      cout << "\nCoordonnee1B::Normer () " << endl;
      Sortie (1);
     }
  #endif
  *this /=  norme;
  return *this ;
 };
		
// somme de tous les composantes 
#ifndef MISE_AU_POINT
  inline 
#endif
double Coordonnee1B::Somme() const 
  { 
    return coord1[0] ;
   };

// Surcharge de l'operateur * : multiplication entre un scalaire et des coordonnees
#ifndef MISE_AU_POINT
  inline 
#endif
 Coordonnee1B operator* (double val,const Coordonnee1B& c)
 { return Coordonnee1B(val*c.coord1[0]);};


#ifndef MISE_AU_POINT
  inline 
#endif
    // sortie du schemaXML: en fonction de enu 
void Coordonnee1B::SchemaXML_Coordonnee(ofstream& sort,const Enum_IO_XML enu)
  {
	switch (enu)
	{ case XML_TYPE_GLOBAUX :
  	   {sort << "\n <!--  *************************** CoordonneeB dim 1 ***************************  -->"
  	         << "\n<xs:complexType name=\"COORDONNEE_1B\" >"
  	         << "\n    <xs:annotation>"
  	         << "\n      <xs:documentation> coordonnee de dimension  1: elements: "
  	         << "\n          1 reel "
  	         << "\n      </xs:documentation>"
  	         << "\n    </xs:annotation>"
  	         << "\n    <xs:sequence>"
  	         << "\n       <xs:element name=\"coordonnees\" type=\"xs:double\" />"
  	         << "\n    </xs:sequence>"
  	         << "\n</xs:complexType>";
		 break;
		}
		case XML_IO_POINT_INFO :
		{
		 break;
		}
		case XML_IO_POINT_BI :
		{
		 break;
		}
		case XML_IO_ELEMENT_FINI :
		{
		 break;
		}
	};		
  };

#endif
