

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "Tenseur2.h" 
#include "MathUtil.h"
#include "ConstMath.h"
#include <iomanip>
#include "Util.h"

#ifndef  Tenseur2_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal
//------------------------------------------------------------------
//          cas des composantes deux fois contravariantes 
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::ChangementIndex::ChangementIndex() :
  idx_i(3),idx_j(3),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=1;             idx_j(3)=2;
    odVect(1,1)=1;odVect(1,2)=3;
    odVect(2,1)=3;odVect(2,2)=2;
  }; 

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::Tenseur2HH() :  
 ipointe() // par défaut
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] = 0.; t[1] = 0.; t[2] = 0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::Tenseur2HH(const double val) :  
 ipointe() 
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val; t[1] =val; t[2] =val;
  };
// initialisation avec 3 valeurs différentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::Tenseur2HH
(const double val1,const double val2,const  double val3) :  
 ipointe() 
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val1; t[1] =val2; t[2] =val3;
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::~Tenseur2HH() 
{//if(listdouble3.end() != listdouble3.begin())        // si la liste n'est pas vide
     listdouble3.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::Tenseur2HH (const  TenseurHH & B) :  
 ipointe()
  { dimension = 2;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HH::Tenseur2HH( etc..");
    #endif
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HH::Tenseur2HH (const  Tenseur2HH & B):  
 ipointe()
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2];
  };        
    // METHODES PUBLIQUES :
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2HH::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;};
// operations 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator + (const TenseurHH & B)const 
  { if (B.Dimension() == -2) return (B + (*this)); // cas B non symetrique
    TenseurHH * res;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HH::operator + ( etc..");
    #endif
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2HH::operator += (const TenseurHH & B)
   {
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HH::operator += ( etc..");
    #endif
    for (int i = 0; i<=2; i++)
       this->t[i] += B.t[i];}; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator - () const 
  {  TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator - ( const TenseurHH & B) const 
  {  TenseurHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator - ( etc..");
     #endif
     if (B.Dimension() == 2) // cas B non symetrique
       {res =  new Tenseur2HH;
        LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        for (int i = 0; i<=2; i++)
          res->t[i] = this->t[i] - B.t[i]; //soustraction des données
        }
     else
       {res =  new Tenseur_ns2HH;    
        LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        for (int i = 0; i<=2; i++)
          res->t[i] = this->t[i] - B.t[i]; //soustraction des données
        res->t[3] = this->t[2] - B.t[3];  
        }
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HH::operator -= ( const TenseurHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HH::operator -= ( etc..");
     #endif
     for (int i = 0; i<=2; i++)
       this->t[i] -= B.t[i];}; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator = ( const TenseurHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator = ( etc..");
     #endif
     if (B.Dimension() == 2) // cas symetrique
       { for (int i = 0; i<=2; i++)
          this->t[i] = B.t[i];
        }
     else // cas non symetrique de stockage
     // pour les 2D cela ne signifie pas : non symetrique car pendant les calculs il peut y avoir
     // génération de terme non symétrique,  la verif est donc a faire a l'affectation c-a-d ici
      {
      #ifdef MISE_AU_POINT
       double Z = B.MaxiComposante(); 
       if (!diffpourcent(B(2,1),B(1,2),Z,ConstMath::unpeupetit)
           || (Abs(Z) < ConstMath::petit) )
      #endif
         // on a retrouve un tenseur symetrique
         {  this->t[0] = B.t[0];
            this->t[1] = B.t[1];
            this->t[2] = 0.5*(B.t[2] + B.t[3]) ;
          }
      #ifdef MISE_AU_POINT
       else // erreur d'affectation     
         { if (ParaGlob::NiveauImpression() > 5)
                cout << "\n tenseur = " << ((Tenseur_ns2HH&) B);
           if (ParaGlob::NiveauImpression() > 5) 
              cout << " erreur d\'affectation, tenseur non symetrique, on symetrise, "
                   << "\n Tenseur2HH::operator = ( etc.."; 
           this->t[0] = B.t[0];
           this->t[1] = B.t[1];
           this->t[2] = 0.5*(B.t[2] + B.t[3]) ;
          }           
      #endif
      }
          
    LesMaillonsHH::Libere(); // destruction des tenseurs intermediaires
    return *this; }; //affectation des données;
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator * ( const double & b) const 
  {  TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HH::operator *= ( const double & b)
  { for (int i = 0; i<=2; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator / ( const double & b) const 
  {  TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HH::operator /= ( const double & b)
  { for (int i = 0; i<=2; i++)
       this->t[i] /= b ;}; //division des données
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this, les données en trop sont ignorées
void Tenseur2HH::Affectation_3D_a_2D(const Tenseur3HH & B)  
{ this->t[0] = B.t[0];this->t[1] = B.t[1];this->t[2] = B.t[3];
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur2HH::Affectation_trans_dimension(const TenseurHH & B,bool plusZero)
{ switch (B.Dimension())
   {case 2: case -2: *this = B; break; // affectation normale
    case 3:
      { const Tenseur3HH & bn  = *((Tenseur3HH *) &B);
        this->Affectation_3D_a_2D(bn);
        break;
      }
    case -3:
      { const Tenseur3HH  bn  = B; // on crée un nouveau tenseur transitoire
        this->Affectation_3D_a_2D(bn); // qui est tout de suite supprimé
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur2HH::Affectation_trans_dimension( const TenseurHH & B, ..");
   }
};

// produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeH Tenseur2HH::operator * ( const CoordonneeB & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur2HH::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeH v(dimension);
    v(1) = this->t[0] * B(1) + this->t[2] * B(2);
    v(2) = this->t[2] * B(1) + this->t[1] * B(2);
    return v;
  };            
 // produit contracte contracté une fois puis deux fois
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::operator * ( const TenseurBH & B) const 
   { TenseurHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HH::operator * ( etc..");
     #endif
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
     res->t[1] = this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;
     res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
     double a = this->t[0] * B.t[3] +  this->t[2] * B.t[1] ;
     if (difftrespetit(a,res->t[2]))
       // le resultat n'est pas symetrique
       { TenseurHH * ress;
         ress =  new Tenseur_ns2HH;
         LesMaillonsHH::NouveauMaillon( ress); // ajout d'un tenseur intermediaire
         for (int i=0;i<=2;i++)
           ress->t[i] = res->t[i];
         ress->t[3] = a;
         return *ress;
       }    
     else
       return *res; };
     
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB & Tenseur2HH::operator * ( const TenseurBB & B) const 
   { TenseurHB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator * ( etc..");
     #endif
     res =  new Tenseur2HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (B.Dimension() == 2)
      { res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
        res->t[1] = this->t[2] * B.t[2] +  this->t[1] * B.t[1] ;
        res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
        res->t[3] = this->t[0] * B.t[2] +  this->t[2] * B.t[1] ;
      }
     else // cas où B est non symétrique
      { res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
        res->t[1] = this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;
        res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
        res->t[3] = this->t[0] * B.t[3] +  this->t[2] * B.t[1] ;
       }    
     return *res; };
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur2HH::operator && ( const TenseurBB & B) const 
   { double b;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator && ( etc..");
     #endif
     if (B.Dimension() == 2)
        b = this->t[0] * B.t[0] +  this->t[2] * B.t[2] +
            this->t[2] * B.t[2] +  this->t[1] * B.t[1] ;
     else
        b = this->t[0] * B.t[0] +  this->t[2] * B.t[2] +
            this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;  
     return b; }; 
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2HH::operator == ( const TenseurHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator == ( etc..");
     #endif
    for (int i = 0; i<=2; i++)
       if (this->t[i] != B.t[i]) res = 0 ;
    if ((B.Dimension() == -2) && difftrespetit(B.t[2],B.t[3]))
         res = 0;         
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2HH::operator != ( const TenseurHH & B) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2HH::operator != ( etc..");
     #endif
    if ((*this) == B) 
      return 0;
    else  
      return 1; };
// determinant de la matrice des coordonnees     
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur2HH::Det()  const 
  {  return (this->t[0] * this->t[1] - this->t[2] * this->t[2]) ; };     
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2HH::Inverse() const 
   {TenseurBB * res;
    res =  new Tenseur2BB;
    LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  res->t[0] = t[0];res->t[1] = t[1];res->t[2] = t[2];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat2x2(((Tenseur2HH *) res)->ipointe);
			 }
			 break;
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
		 case CRAMER : // méthode historique
			{ // calcul du determinant
			  double det = this->t[0] * this->t[1] - this->t[2] * this->t[2] ;
			 #ifdef MISE_AU_POINT	 	 
				if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur2HH::Inverse() \n";
					  Sortie(1);
					};
			#endif 
			  res->t[0] = this->t[1] / det;
			  res->t[2] = - this->t[2] / det;
			  res->t[1] = this->t[0] / det;
			  break;
			} 
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur2HH::Inverse() \n";
					  Sortie(1);
			  };
	 };
//	 res->Ecriture(cout); // pour le debug
    return *res;
   };                    
                         
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHH & Tenseur2HH::Transpose() const 
   { TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
    return *res;};           
    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur2HH::Baisse2Indices() const
   { TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     return *res;};           
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& Tenseur2HH::BaissePremierIndice() const
   { TenseurBH * res;
     res =  new Tenseur2BH;
     LesMaillonsBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     res->t[3] = this->t[2];
     return *res;};           

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& Tenseur2HH::BaisseDernierIndice() const
   { TenseurHB * res;
     res =  new Tenseur2HB;
     LesMaillonsHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     res->t[3] = this->t[2];
     return *res;};           

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2HH::MaxiComposante() const
  { return DabsMaxiTab(t,  3) ;
   };
             
    // retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
double& Tenseur2HH::Coor( const int i, const int j)
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << " j = " << j << '\n';
			  cout << "TenseurHH::Coor(int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    
 
    // retourne la composante i,j en lecture uniquement
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2HH::operator () ( const int i, const int j) const 
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurHH::OPERATOR() ( const int, const int ) const \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    
 
//fonctions static définissant le produit tensoriel de deux vecteurs
// si les vecteurs sont égaux le tenseur est symétrique sinon il est non symétrique
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HH::Prod_tensoriel(const CoordonneeH & aH, const CoordonneeH & bH)
  {  TenseurHH * res;
     #ifdef MISE_AU_POINT
       if ((aH.Dimension() != 2) || (bH.Dimension() != 2))
        { cout << "\n erreur de dimension dans les coordonnees d'entree, dim1 et dim2 ="
               << aH.Dimension() << " " << bH.Dimension() 
               << "\n Tenseur2HH::Prod_tensoriel( etc.." << endl;
          Sortie(1);
         }  
     #endif
     if (aH == bH) // cas d'un résultat symetrique
       {res =  new Tenseur2HH;
        LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        res->t[0] = aH(1) * aH(1); res->t[1] = aH(2) * aH(2); res->t[2] = aH(1) * aH(2);
        }
     else // cas d'un résultat non symétrique
       {res =  new Tenseur_ns2HH;    
        LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        res->t[0] = aH(1) * bH(1); res->t[3] = aH(1) * bH(2); 
        res->t[2] = aH(2) * bH(1); res->t[1] = aH(2) * bH(2); 
        }
    return *res ;};  
    
// lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2HH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 2; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2HH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2HH ";
    // puis les datas
     for (int i = 0; i<= 2; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2HH & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != 2) A.Message(2,"operator >> (istream & entree, Tenseur2HH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 2; i++)
        entree >> A.t[i];
    return entree;      
  };
 #ifndef MISE_AU_POINT
  inline 
#endif
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2HH & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2HH ";
    // puis les datas
     for (int i = 0; i<= 2; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };

//------------------------------------------------------------------
//          cas des composantes deux fois covariantes
//------------------------------------------------------------------

	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::ChangementIndex::ChangementIndex() :
  idx_i(3),idx_j(3),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=1;             idx_j(3)=2;
    odVect(1,1)=1;odVect(1,2)=3;
    odVect(2,1)=3;odVect(2,2)=2;
  }; 

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::Tenseur2BB() :  
 ipointe() // par défaut
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] = 0.; t[1] = 0.; t[2] = 0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::Tenseur2BB( const double val):  
 ipointe()  
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val; t[1] =val; t[2] =val;
  };
// initialisation avec 3 valeurs differentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::Tenseur2BB
( const double val1,  const double val2, const  double val3):  
 ipointe()  
  { dimension = 2;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val1; t[1] =val2; t[2] =val3;
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::~Tenseur2BB() 
{//if(listdouble3.end() != listdouble3.begin())        // si la liste n'est pas vide
     listdouble3.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::Tenseur2BB (const  TenseurBB & B):  
 ipointe() 
  { dimension = 2;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BB::Tenseur2BB ( etc..");
     #endif
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BB::Tenseur2BB ( const  Tenseur2BB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble3.push_front(Reels3());  // allocation
    ipointe = listdouble3.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2];
  };        
    // METHODES PUBLIQUES :
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2BB::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;};
// operations 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator + ( const TenseurBB & B) const 
  {  if (B.Dimension() == -2) return (B + (*this));
     TenseurBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BB::operator + ( etc..");
     #endif
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2BB::operator += ( const TenseurBB & B)
   {
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BB::operator += ( etc..");
     #endif
     for (int i = 0; i<=2; i++)
       this->t[i] += B.t[i];}; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator - () const 
  {  TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = - this->t[i]; //oppose des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator - ( const TenseurBB & B) const 
  {  TenseurBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator - ( etc..");
     #endif
     if (B.Dimension() == 2) // cas B non symetrique
       { res =  new Tenseur2BB;
         LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
         for (int i = 0; i<=2; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
        }    
     else
       {res =  new Tenseur_ns2BB;    
        LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        for (int i = 0; i<=2; i++)
          res->t[i] = this->t[i] - B.t[i]; //soustraction des données
        res->t[3] = this->t[2] - B.t[3];  
        }
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BB::operator -= ( const TenseurBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BB::operator -= ( etc..");
     #endif
     for (int i = 0; i<=2; i++)
       this->t[i] -= B.t[i];}; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator = ( const TenseurBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator = ( etc..");
     #endif
     if (B.Dimension() == 2) // cas symetrique
       { for (int i = 0; i<=2; i++)
          this->t[i] = B.t[i];
        }
     else // cas non symetrique de stockage
     // pour les 2D cela ne signifie pas : non symetrique car pendant les calculs il peut y avoir
     // génération de terme non symétrique,  la verif est donc a faire a l'affectation c-a-d ici
      {
      #ifdef MISE_AU_POINT
       double Z = B.MaxiComposante(); 
       if (!diffpourcent(B(2,1),B(1,2),Z,ConstMath::unpeupetit)
           || (Abs(Z) < ConstMath::petit) )
      #endif
         // on a retrouve un tenseur symetrique
         {  this->t[0] = B.t[0];
            this->t[1] = B.t[1];
            this->t[2] = 0.5*(B.t[2] + B.t[3]) ;
          }
      #ifdef MISE_AU_POINT
       else // erreur d'affectation     
         { if (ParaGlob::NiveauImpression() > 5)
                cout << "\n tenseur = " << ((Tenseur_ns2BB&) B);
           if (ParaGlob::NiveauImpression() > 5) 
              cout << " erreur d\'affectation, tenseur non symetrique, on symetrise, "
                   << "\n Tenseur2BB::operator = ( etc.."; 
           this->t[0] = B.t[0];
           this->t[1] = B.t[1];
           this->t[2] = 0.5*(B.t[2] + B.t[3]) ;
          }           
      #endif
      }
    LesMaillonsBB::Libere(); // destruction des tenseurs intermediaires
    return *this; }; //affectation des données;
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator * ( const double & b) const 
  {  TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BB::operator *= ( const double & b)
  { for (int i = 0; i<=2; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator / ( const double & b) const 
  {  TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=2; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BB::operator /= ( const double & b)
  { for (int i = 0; i<=2; i++)
       this->t[i] /= b ;}; //division des données
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this, les données en trop sont ignorées
void Tenseur2BB::Affectation_3D_a_2D(const Tenseur3BB & B)  
{ this->t[0] = B.t[0];this->t[1] = B.t[1];this->t[2] = B.t[3];
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur2BB::Affectation_trans_dimension(const TenseurBB & B,bool plusZero)
{ switch (B.Dimension())
   {case 2: case -2: *this = B; break; // affectation normale
    case 3:
      { const Tenseur3BB & bn  = *((Tenseur3BB *) &B);
        this->Affectation_3D_a_2D(bn);
        break;
      }
    case -3:
      { const Tenseur3BB  bn  = B; // on crée un nouveau tenseur transitoire
        this->Affectation_3D_a_2D(bn); // qui est tout de suite supprimé
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur2BB::Affectation_trans_dimension( const TenseurBB & B, ..");
   }
};

// produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeB Tenseur2BB::operator * ( const CoordonneeH & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur2BB::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeB v(dimension);
    v(1) = this->t[0] * B(1) + this->t[2] * B(2);
    v(2) = this->t[2] * B(1) + this->t[1] * B(2);
    return v;
  };                       
 // produit contracte contracté une fois puis deux fois
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::operator * ( const TenseurHB & B) const 
   { TenseurBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BB::operator * ( etc..");
     #endif
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
     res->t[1] = this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;
     res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
     double a = this->t[0] * B.t[3] +  this->t[2] * B.t[1] ;	 	 
     if (difftrespetit(a,res->t[2]))
       // le resultat n'est pas symetrique
       { TenseurBB * ress;
         ress = new Tenseur_ns2BB;
         LesMaillonsBB::NouveauMaillon( ress); // ajout d'un tenseur intermediaire
         for (int i=0;i<=2;i++)
           ress->t[i] = res->t[i];
         ress->t[3] = a;
         return *ress;
       }    
     else
       return *res;
   };
 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH & Tenseur2BB::operator * ( const TenseurHH & B) const 
   { TenseurBH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator * ( etc..");
     #endif
     res =  new Tenseur2BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (B.Dimension() == 2)
      {  res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
         res->t[1] = this->t[2] * B.t[2] +  this->t[1] * B.t[1] ;
         res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
         res->t[3] = this->t[0] * B.t[2] +  this->t[2] * B.t[1] ;
       }
     else
      { res->t[0] = this->t[0] * B.t[0] +  this->t[2] * B.t[2] ;
        res->t[1] = this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;
        res->t[2] = this->t[2] * B.t[0] +  this->t[1] * B.t[2] ;
        res->t[3] = this->t[0] * B.t[3] +  this->t[2] * B.t[1] ;
       }             
     return *res; };
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur2BB::operator && ( const TenseurHH & B) const 
   { double b;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator && ( etc..");
     #endif
     if (B.Dimension() == 2)
        b = this->t[0] * B.t[0] +  this->t[2] * B.t[2] +
            this->t[2] * B.t[2] +  this->t[1] * B.t[1] ;
     else
        b = this->t[0] * B.t[0] +  this->t[2] * B.t[2] +
            this->t[2] * B.t[3] +  this->t[1] * B.t[1] ;  
     return b; };    
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2BB::operator == ( const TenseurBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator == ( etc..");
     #endif
    for (int i = 0; i<=2; i++)
       if (this->t[i] != B.t[i]) res = 0 ;
    if ((B.Dimension() == -2) && difftrespetit(B.t[2],B.t[3]))
         res = 0;         
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2BB::operator != ( const TenseurBB & B) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BB::operator != ( etc..");
     #endif
    if ((*this) == B) 
      return 0;
    else  
      return 1; };
// calcul du determinant de la matrice des coordonnees      
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur2BB::Det() const  
  { return (this->t[0] * this->t[1] - this->t[2] * this->t[2]) ; };            
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2BB::Inverse() const 
   {TenseurHH * res;
    res =  new Tenseur2HH;
    LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  res->t[0] = t[0];res->t[1] = t[1];res->t[2] = t[2];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat2x2(((Tenseur2BB*) res)->ipointe);
			 }
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
//	Sortie(1);
			 break;
		 case CRAMER : // méthode historique
			{ // calcul du determinant
			  double det = this->t[0] * this->t[1] - this->t[2] * this->t[2] ;
			#ifdef MISE_AU_POINT	 	 
				if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur2BB::Inverse() \n";
					  this->Ecriture(cout);
					  Sortie(1);
					};
			#endif 
		     res->t[0] = this->t[1] / det;
			  res->t[2] = - this->t[2] / det;
			  res->t[1] = this->t[0] / det;
			 break;
			} 
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur2BB::Inverse() \n";
					  Sortie(1);
			  };
	 };
//	 res->Ecriture(cout); // pour le debug
//	 cout << endl;
    return *res;
   };                    
                         
 #ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::Transpose() const 
   { TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
    return *res;};           

    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur2BB::Monte2Indices() const
   { TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     return *res;};           
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& Tenseur2BB::MonteDernierIndice() const
   { TenseurBH * res;
     res =  new Tenseur2BH;
     LesMaillonsBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     res->t[3] = this->t[2];
     return *res;};           

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& Tenseur2BB::MontePremierIndice() const
   { TenseurHB * res;
     res =  new Tenseur2HB;
     LesMaillonsHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[2];
     res->t[3] = this->t[2];
     return *res;};           
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2BB::MaxiComposante() const
  { return DabsMaxiTab(t,  3) ;
   };
                                                               
    // retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
double& Tenseur2BB::Coor( const int i, const int j)
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurBB::Coor(int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    }; 
       
    // retourne la composante i,j en lecture uniquement
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2BB::operator () ( const int i, const int j) const
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurBB::OPERATOR() ( const int, const int )  const\n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    

 
//fonctions static définissant le produit tensoriel de deux vecteurs
// si les vecteurs sont égaux le tenseur est symétrique sinon il est non symétrique
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BB::Prod_tensoriel(const CoordonneeB & aB, const CoordonneeB & bB)
  {  TenseurBB * res;
     #ifdef MISE_AU_POINT
       if ((aB.Dimension() != 2) || (bB.Dimension() != 2))
        { cout << "\n erreur de dimension dans les coordonnees d'entree, dim1 et dim2 ="
               << aB.Dimension() << " " << bB.Dimension() 
               << "\n Tenseur2BB::Prod_tensoriel( etc.." << endl;
          Sortie(1);
         }  
     #endif
     if (aB == bB) // cas d'un résultat symetrique
       {res =  new Tenseur2BB;
        LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        res->t[0] = aB(1) * aB(1); res->t[1] = aB(2) * aB(2); res->t[2] = aB(1) * aB(2);
        }
     else // cas d'un résultat non symétrique
       {res =  new Tenseur_ns2BB;    
        LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
        res->t[0] = aB(1) * bB(1); res->t[3] = aB(1) * bB(2); 
        res->t[2] = aB(2) * bB(1); res->t[1] = aB(2) * bB(2); 
        }
    return *res ;};
    
// lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2BB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 2; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2BB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2BB ";
    // puis les datas
     for (int i = 0; i<= 2; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
      
 #ifndef MISE_AU_POINT
  inline 
#endif
// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2BB & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != 2) A.Message(2,"operator >> (istream & entree, Tenseur2BB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 2; i++)
        entree >> A.t[i];
    return entree;      
  };
 #ifndef MISE_AU_POINT
  inline 
#endif
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2BB & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2BB ";
    // puis les datas
     for (int i = 0; i<= 2; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };
 
 
#endif    
