

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "Tenseur2.h"
#include "ConstMath.h"  
#include "MathUtil.h"
#include "Util.h"


 
#ifndef  Tenseur2_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal
//------------------------------------------------------------------
//          cas des composantes mixtes BH
//------------------------------------------------------------------
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::ChangementIndex::ChangementIndex() :
  idx_i(4),idx_j(4),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=2;idx_i(4)=1;  idx_j(3)=1;idx_j(4)=2;
    odVect(1,1)=1;odVect(1,2)=4;
    odVect(2,1)=3;odVect(2,2)=2;
  }; 

#ifndef MISE_AU_POINT
  inline 
#endif
 Tenseur2BH::Tenseur2BH() :  
 ipointe()  // par defaut
   { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
     t[0] = 0.; t[1] = 0.; t[2] = 0.;t[3] = 0.;
   };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::Tenseur2BH( const double val):  
 ipointe()  
  { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val; t[1] =val; t[2] =val; t[3] =val;
  };
// initialisation avec 4 valeurs différentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::Tenseur2BH
       ( const double val1, const  double val2, const  double val3, const  double val4):  
 ipointe()  
  { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val1; t[1] =val2; t[2] =val3; t[3] = val4;
  };      
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::Tenseur2BH ( const  TenseurBH & B):  
 ipointe() 
  { this->dimension = B.dimension;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::Tenseur2BH( etc..");
    #endif
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2]; this->t[3] = B.t[3];
  };         
 // constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::Tenseur2BH ( const  Tenseur2BH & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2]; this->t[3] = B.t[3];
  };         
 #ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BH::~Tenseur2BH() 
{//if(listdouble4.end() != listdouble4.begin())        // si la liste n'est pas vide
     listdouble4.erase(ipointe);} ; // suppression de l'élément de la liste
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2BH::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;t[3] =val;};
// operations 
 #ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH & Tenseur2BH::operator + ( const TenseurBH & B) const 
   {  TenseurBH * ptr;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator + ( etc..");
      #endif
      ptr =  new Tenseur2BH;
      LesMaillonsBH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = this->t[i] + B.t[i]; //somme des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2BH::operator += ( const TenseurBH & B)
    { 
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator += ( etc..");
      #endif
      for (int i = 0; i<=3; i++)
       this->t[i] += B.t[i];}; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator - () const 
    {  TenseurBH * ptr;
      ptr =  new Tenseur2BH;
      LesMaillonsBH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = - this->t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator - ( const TenseurBH & B) const 
    { TenseurBH * ptr;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator + ( etc..");
      #endif
      ptr =  new Tenseur2BH;
      LesMaillonsBH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = this->t[i] - B.t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2BH::operator -= ( const TenseurBH & B)
     { 
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator -= ( etc..");
      #endif
      for (int i = 0; i<=3; i++)
         this->t[i] -= B.t[i];}; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator = ( const TenseurBH & B)
  { 
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator = ( etc..");
     #endif
     for (int i = 0; i<=3; i++)
       this->t[i] = B.t[i];
     LesMaillonsBH::Libere(); // destruction des tenseurs intermediaires
     return *this; }; //affectation des données;
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator * ( const double & b) const 
    { TenseurBH * res;
      res =  new Tenseur2BH;
      LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=3; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2BH::operator *= ( const double & b)
      { for (int i = 0; i<=3; i++)
       this->t[i] *= b;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator / ( const double & b) const 
    { TenseurBH * res;
      res =  new Tenseur2BH;
      LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=3; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2BH::operator /= ( const double & b)
      { for (int i = 0; i<=3; i++)
       this->t[i] /= b;}; //division des données 
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this, les données en trop sont ignorées
void Tenseur2BH::Affectation_3D_a_2D(const Tenseur3BH & B)  
{ this->t[0] = B.t[0];this->t[1] = B.t[4];this->t[2] = B.t[3];this->t[3] = B.t[1];
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur2BH::Affectation_trans_dimension(const TenseurBH & B,bool plusZero)
{ switch (B.Dimension())
   {case 2:  *this = B; break; // affectation normale
    case 3:
      { const Tenseur3BH & bn  = *((Tenseur3BH *) &B);
        this->Affectation_3D_a_2D(bn);
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur2BH::Affectation_trans_dimension( const TenseurBH & B, ..");
   }
};

// produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeB Tenseur2BH::operator * ( const CoordonneeB & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur2BH::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeB v(dimension);
    v(1) = this->t[0] * B(1) + this->t[3] * B(2);
    v(2) = this->t[2] * B(1) + this->t[1] * B(2);
    return v;
  };            
        
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur2BH::operator * ( const TenseurBB & B) const  // produit une fois contracte
    { TenseurBB * res;
      #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 2) Message(2,"Tenseur2BH::operator * ( etc..");
      #endif
      // on cree systematiquement un tenseur non symetrique
      res =  new Tenseur_ns2BB;
      LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
      if (B.Dimension() == 2) // cas symetrique
       { res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
         res->t[1] = this->t[2] * B.t[2] + this->t[1] * B.t[1];
         res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
         res->t[3] = this->t[0] * B.t[2] + this->t[3] * B.t[1] ;
        }
       else  // cas ou B n'est pas symetrique
        {
         res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
         res->t[1] = this->t[2] * B.t[3] + this->t[1] * B.t[1];
         res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
         res->t[3] = this->t[0] * B.t[3] + this->t[3] * B.t[1];
         };      
      return  (*res); };           
              
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2BH::operator * ( const TenseurBH & B) const  // produit une fois contracte
    { TenseurBH * res;
      res =  new Tenseur2BH;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator * ( etc..");
      #endif
      LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
       res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
       res->t[1] = this->t[2] * B.t[3] + this->t[1] * B.t[1];
       res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
       res->t[3] = this->t[0] * B.t[3] + this->t[3] * B.t[1];
       return  (*res); };
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2BH::operator && ( const TenseurBH & B) const  // produit deux fois contracte
    { double b;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator && ( etc..");
      #endif
      b = (this->t[0] * B.t[0] + this->t[3] * B.t[2]) +
          (this->t[2] * B.t[3] + this->t[1] * B.t[1]);
      return b;};     
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2BH::Trace() const    // trace du tenseur
    { double b;
      b = this->t[0] + this->t[1] ;
      return b;};     
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2BH::II() const    // second invariant
//    { double b;
//      // on met Dabs car dans les petits nombres on peut avoir des nombres négatifs !!
//      b =  Dabs( this->t[0] * this->t[0] + 2. * this->t[3] * this->t[2]
//          + this->t[1] * this->t[1]);
//      return b;};     
    { //return Dabs((*this && *this)); // on met Dabs car dans les petits nombres on peut avoir des nombres
      double ret = (*this && *this);
//      if (Dabs(ret) < 0.1*ConstMath::pasmalpetit)
        // on part du principe qu'il s'agit du produit doublement contracté d'un tenseur symétrique
        // donc normalement il devrait donner un nombre positif, donc s'il est très petit, on le met arbitrairement à 0
      if (ret < 0.)
        {ret = 0.;}
      return ret;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2BH::III()  const   // troisieme invariant
    { double b,b1,b2,b3,b4;
      b1 =  this->t[0] * this->t[0] +  this->t[3] * this->t[2];
      b2 =  this->t[0] * this->t[3] +  this->t[3] * this->t[1];
      b3 =  this->t[2] * this->t[0] +  this->t[1] * this->t[2];
      b4 =  this->t[2] * this->t[3] +  this->t[1] * this->t[1];
      b = b1 * this->t[0] + b2 * this->t[2] +
          b3 * this->t[3] + b4 * this->t[1];
      return b;};              
                
    // valeurs propre  dans le vecteur  de retour, classée par ordres "décroissants"
    // cas indique le cas de valeur propre: 
    // quelque soit la dimension: cas = -1, si l'extraction des valeurs propres n'a pas pu se faire
    //                            dans ce cas les valeurs propres de retour sont nulles par défaut
    // dim = 2 , cas = 1 si deux valeurs propres distinctes, cas = 0 si deux val propres identiques
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee Tenseur2BH::ValPropre(int& cas) const 
  { Coordonnee ret(2);
    double b=(this->t[0]+this->t[1]); // la trace
	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	   // donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )  
       { cas=0;ret(1)=ret(2)=b/2.;return ret;
       };
    // sinon ..
	   b = -b; // on utilise l'inverse pour la suite
    double c=this->t[0] * this->t[1] - this->t[2] * this->t[3]; // déterminant
    int caas;
    alg_zero.SecondDegre(1.,b,c,ret(1),ret(2),caas);
    if (caas <= 0)
    	{ // cas d'un pb pourtant on sait qu'il doit y avoir une solution. On essaie une solution avec
    	  // des conditions de précision relachées: ici la procédure est plus poussive !!
    	  if ((Dabs(b) < ConstMath::petit)  && (Dabs(c)< ConstMath::petit))
    	    // cela signifie que les valeurs propres sont pratiquements nulles
    	    	{ ret(1) = ret(2) = 0.; 
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	}
    	  else if ((Dabs(b*b - 4. * c) < ConstMath::petit) &&  ((b*b - 4. * c) < 0.))
    	    // cas d'un discriminant négatif mais avec des valeurs faibles
    	    	{ ret(1) = ret(2) = -b * 0.5 ; // -b/(2a)
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	};	
    	};
    
    #ifdef MISE_AU_POINT
      if (caas <= 0)
       { cout << "\n warning **** recherche de valeurs propre du tenseur: pas de racine correcte ! "
              << "\n tenseur= " << *this
              << "\n on laisse le calcul se poursuivre cependant"
              << "\n Coordonnee Tenseur2BH::ValPropre()";
        };      
    #endif
    // classement des valeurs propres
    if (ret(2) > ret(1))
      { double x = ret(1); ret(1) = ret(2); ret(2)=x;};
    // def du cas
    if ( difftrespetit(ret(1),ret(2))) { cas = 1;} else {cas = 0;};
    // s'il y a eu erreur dans la recherche de valeurs propre on modifie le cas
    if (caas <= 0) cas=-1;
    // retour
    return ret;
  };
   
    // idem met en retour la matrice mat contiend par colonne les vecteurs propre
    // elle doit avoir la dimension du tenseur
    // le premier vecteur propre est exprime dans le repere dual 
    // le second vecteur propre est exprimé dans le repère naturel 
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee Tenseur2BH::ValPropre(int& cas, Mat_pleine& mat) const 
  { Coordonnee ret(2);
    double b=(this->t[0]+this->t[1]); // la trace
	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	   // donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    // on ramène le tenseur identité en absolue comme base de vecteurs propres car toute base est ok
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )  
       { mat.Initialise(0.); mat(1,1)=1.;mat(2,2)=1.;
         cas=0;ret(1)=ret(2)=b/2.;return ret;
       };
    // sinon       
	   b = -b; // on utilise l'inverse pour la suite
    double c=this->t[0] * this->t[1] - this->t[2] * this->t[3];
    int caas;
    alg_zero.SecondDegre(1.,b,c,ret(1),ret(2),caas);
    if (caas <= 0)
    	{ // cas d'un pb pourtant on sait qu'il doit y avoir une solution. On essaie une solution avec
    	  // des conditions de précision relachées: ici la procédure est plus poussive !!
    	  if ((Dabs(b) < ConstMath::petit)  && (Dabs(c)< ConstMath::petit))
    	    // cela signifie que les valeurs propres sont pratiquements nulles
    	    	{ ret(1) = ret(2) = 0.; 
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	}
    	  else if ((Dabs(b*b - 4. * c) < ConstMath::petit) &&  ((b*b - 4. * c) < 0.))
    	    // cas d'un discriminant négatif mais avec des valeurs faibles
    	    	{ ret(1) = ret(2) = -b * 0.5 ; // -b/(2a)
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	};	
    	};
    #ifdef MISE_AU_POINT
      if (caas <= 0)
       { cout << "\n warning **** recherche de valeurs propre du tenseur: pas de racine correcte ! "
              << "\n tenseur= " << *this
              << "\n on laisse le calcul se poursuivre cependant"
              << "\n Coordonnee Tenseur2BH::ValPropre( Mat_pleine& mat)";
       };
    #endif

    #ifdef MISE_AU_POINT	 	 
    if  ((mat.Nb_ligne() != 2) || (mat.Nb_colonne() != 2))
     { cout << "\nErreur : la matrice en parametre doit etre de dimension "
            << 2 << " !\n";
       cout << "Tenseur2BH::ValPropre( Mat_pleine& mat)  \n";
       Sortie(1);
     };
    #endif
    // classement des valeurs propres
    if (ret(2) > ret(1))
      { double x = ret(1); ret(1) = ret(2); ret(2)=x;};
    // calcul du premier vecteur normé
    double norme1= sqrt(this->t[3] * this->t[3] 
                  + (this->t[0] - ret(1))*(this->t[0] - ret(1)) );
    double norme2= sqrt(this->t[2] * this->t[2] 
                  + (this->t[1] - ret(1))*(this->t[1] - ret(1)) );
    // on prend le maxi des normes
    bool premier= true; double norme=norme1;
    if (norme1 < norme2) {premier = false; norme = norme2;};             
    if (norme < ConstMath::pasmalpetit)
      { if (ParaGlob::NiveauImpression() > 4)
          cout << "\n attention, construction des vecteurs propres impossible  car valeurs propres nulles"
               << "\n on annulle les vecteurs propres ";
        #ifdef MISE_AU_POINT
         if ((ParaGlob::NiveauImpression() > 3)&&(ParaGlob::NiveauImpression() < 5))
           cout << "\n attention, construction des vecteurs propres impossible  car valeurs propres nulles"
               << "\n on met des vecteurs par defaut ";
         if (ParaGlob::NiveauImpression() > 3)
           cout << "\n tenseur= " << *this << "\n valeur propres= " << ret << ", norme = " << norme
                << "\n Coordonnee Tenseur2BH::ValPropre( Mat_pleine& mat)";
        #endif
        mat.Zero();mat(1,1)=mat(2,2)=1.;
        return ret;
       };
    if (premier)                 
     {mat(1,1) = - this->t[3] / norme;
      mat(2,1) = (this->t[0] - ret(1)) / norme;
     }
    else
     {mat(1,1) = (this->t[1] - ret(1)) / norme;     
      mat(2,1) = - this->t[2]  / norme;
     };
    // calcul du second vecteur normé  normal au premier
    mat(1,2) = - mat(2,1); mat(2,2) = mat(1,1); // le deuxième vecteur 
      
    // def du cas
    if ( difftrespetit(ret(1),ret(2))) { cas = 1;} else {cas = 0;};
    // s'il y a eu erreur dans la recherche de valeurs propre on modifie le cas
    if (caas <= 0) cas=-1;

    #ifdef MISE_AU_POINT
    // en débug on vérifie que les valeurs propres fonctionnent correctement
    //               , cas = 1 si 2 val propres différentes
    //               , cas = 0 si les 2 sont identiques
    if (ParaGlob::NiveauImpression() > 2)
     { CoordonneeB Vp(2);
       switch (cas)
        { case 0: Vp(1)=Vp(2)=ret(1); break;
          case 1: Vp(1) = ret(1); Vp(2)=ret(2); break;
        };
       CoordonneeB Vpropre(mat(1,1),mat(2,1));
       CoordonneeB W6 = (*this)* Vpropre;
       // normalement W6 = ret(1)*Vpropre
       double diff = (W6 - (Vpropre * ret(1))).Norme();
       double w6_norme = W6.Norme();
       double V1_norme = (Vpropre * ret(1)).Norme();
       if (diffpourcent(w6_norme,V1_norme,MaX(w6_norme,V1_norme),ConstMath::unpeupetit))
//       if (Dabs(diff) > ConstMath::unpeupetit)
          {cout << "\n erreur le vecteur propre 1 ";
           Vpropre.Affiche();
           cout << "\n n'est pas un vecteur propre pour la premiere valeur propre: "
                << Vp(1) << " concernant la matrice: ";
           this->Ecriture(cout);
           cout << "\n A*V = "<<W6 << "(\nVpropre * val1)= " << (Vpropre * ret(1))
                << "\n diff= "<<diff << " W6.Norme()= " << w6_norme
                << " (Vpropre * ret(1)).Norme()= " << V1_norme;
              
           cout << "\n cas= "<< cas ;
           cout << "\n les val propres: " << ret(1) << " et "<<ret(2) ;
              
           cout  << "\n Tenseur2HB::ValPropre(..."  << endl;
          };
       // on vérifie que le produit scalaire des deux vecteurs propres est nul
       diff = (mat(1,1)*mat(1,2)+mat(2,1)*mat(2,2));
       int i,j;
       double max_mat = mat.MaxiValAbs(i, j);
       if (Dabs(diff) > (ConstMath::unpeupetit * max_mat))
          {cout << "\n erreur le produit scalaire des deux vecteurs propres n'est pas nul !! ";
           cout << "\n V1= " << mat(1,1) <<" "<<mat(2,1) << " V2= " << mat(2,1) <<" "<<mat(2,2)  ;
           cout  << "\n Tenseur2HB::ValPropre(..."  << endl;
          };
     };
    
    #endif
    // retour
    return ret;
   };

    // ici il s'agit uniquement de calculer les vecteurs propres, les valeurs propres
    // étant déjà connues
    // en retour VP les vecteurs propre : doivent avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel (pour les tenseurs dim 3
    // pour dim=2:le premier vecteur propre est exprime dans le repere naturel 
    //            le second vecteur propre est exprimé dans le repère dual
    // pour dim=1 le vecteur est dans la base naturelle (mais ça importe pas)
    //   en sortie cas = -1 s'il y a eu un problème, dans ce cas, V_P est quelconque
    //             sinon si tout est ok, cas est identique en sortie avec l'entrée
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2BH::VecteursPropres(const Coordonnee& valPropre,int& cas, Tableau <Coordonnee>& V_P) const
  { // on dimensionne en conséquence
    Coordonnee toto(2); // un vecteur nul
    V_P.Change_taille(2,toto); // tableau initialisé à 0
  
    double b=(this->t[0]+this->t[1]); // la trace

    double intens2 = (this->II() - b*b*0.5);
    double sqrt_II_b = sqrt(Dabs(intens2)); // l'intensité du déviateur
    if ((intens2 < 0.) && (sqrt_II_b > Dabs(b)))
     {
      // veut dire que l'on est très fortement négatif, on considère que l'erreur est trop grande
      #ifdef MISE_AU_POINT
      if  (cas != 0)
       { cout << "\nErreur : l'intensite du deviateur "<< intens2 << " est negative et superieure "
              << " a la trace par exemple, on considere que l'erreur est trop grande, le calcul "
              << " des valeurs propres n'est pas possible ";
         cout << "\n Tenseur2HB::VecteursPropres(...  \n";
       };
      #endif
      cas=-1;
      return;
     };

//	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	   // donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )  
    // on ramène le tenseur identité en absolue comme base de vecteurs propres car toute base est ok
       {
        #ifdef MISE_AU_POINT
        if  (cas != 0)
         { cout << "\nErreur : les valeurs propres devraient etre identiques alors qu'en entree: cas= " << cas ;
           cout << "\n Tenseur2BH::VecteursPropres(...  \n";
           cas=-1;
           return;
         };
        #endif
         V_P(1).Zero(); V_P(1)(1)=1.;
         V_P(2).Zero(); V_P(2)(2)=1.;
         cas=0;return;
       };
    // sinon ce n'est pas sphérique
    // calcul des vecteurs normés
    double norme1= sqrt(this->t[3] * this->t[3] 
                  + (this->t[0] - valPropre(1))*(this->t[0] - valPropre(1)) );
    double norme2= sqrt(this->t[2] * this->t[2] 
                  + (this->t[1] - valPropre(1))*(this->t[1] - valPropre(1)) );
    // on prend le maxi des normes
    bool premier= true; double norme=norme1;
    if (norme1 < norme2) {premier = false; norme = norme2;};             
    if (norme < ConstMath::pasmalpetit)
      { // la construction des vecteurs propres est impossible
        if (ParaGlob::NiveauImpression() > 4)
          cout << "\n attention, construction des vecteurs propres impossible car valeurs propres nulles"
               << "\n on met des vecteurs par defaut ";
        #ifdef MISE_AU_POINT
         if ((ParaGlob::NiveauImpression() > 3)&&(ParaGlob::NiveauImpression() < 4))
           cout << "\n attention, construction des vecteurs propres impossible  car valeurs propres nulles"
                << "\n on annulle les vecteurs propres ";
         if (ParaGlob::NiveauImpression() > 3)
           cout << "\n tenseur= " << *this << "\n valeur propres= " << valPropre
                << ", norme = " << norme
                << "\n Tenseur2BH::VecteursPropres(...";
        #endif
        V_P(1).Zero(); V_P(1)(1)=1.;
        V_P(2).Zero(); V_P(2)(2)=1.;
        cas = -1;
        return ;
       };
    if (premier)                 
     {V_P(1)(1)= - this->t[3] / norme;
      V_P(1)(2)= (this->t[0] - valPropre(1)) / norme;
      }
    else
     {V_P(1)(1)= (this->t[1] - valPropre(1)) / norme;
      V_P(1)(2)= - this->t[2]  / norme;
      };
    // puis le second vecteur
    V_P(2)(1) = - V_P(1)(2); V_P(2)(2) = V_P(1)(1); // le deuxième vecteur
    // retour
    return ;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2BH::Det()  const     // determinant de la matrice des coordonnees
    { double b;
      b = this->t[0] * this->t[1] - this->t[2] * this->t[3];
      return b;};               
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2BH::operator == ( const TenseurBH & B) const 
  { int res = 1;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator == ( etc..");
    #endif
    for (int i = 0; i<=3; i++)
       if (this->t[i] != B.t[i]) res = 0 ;
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2BH::operator != ( const TenseurBH & B) const 
  { 
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2BH::operator != ( etc..");
     #endif
    if ((*this) == B) 
      return 0;
    else  
      return 1; };
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH & Tenseur2BH::Inverse() const 
   {TenseurBH * res;
    res =  new Tenseur2BH;
    LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  res->t[0] = t[0];res->t[1] = t[1];res->t[2] = t[2];res->t[3] = t[3];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;res->t[3]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat2x2(((Tenseur2BH*) res)->ipointe);
			 }
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
//	Sortie(1);
			 break;
		 case CRAMER : // méthode historique
			{ // calcul du determinant
				double det = this->Det() ;
			#ifdef MISE_AU_POINT	 	 
				if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur2BH::Inverse() \n";
					  Sortie(1);
					};
			#endif
			  // transposee de la matrice des cofacteurs 
			  res->t[0] = this->t[1] / det;
			  res->t[2] = - this->t[2] / det;
			  res->t[1] = this->t[0] / det;
			  res->t[3] = - this->t[3] / det;    
			 break;
			} 
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur2BH::Inverse() \n";
					  Sortie(1);
			  };
	 };
//	 res->Ecriture(cout); // pour le debug
    return *res;
   };                    
 #ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB & Tenseur2BH::Transpose() const 
   { TenseurHB * res;
     res =  new Tenseur2HB;
     LesMaillonsHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[3];
     res->t[3] = this->t[2];
    return *res;};


#ifndef MISE_AU_POINT
  inline 
#endif
// permute Bas Haut, mais reste dans le même tenseur
void Tenseur2BH::PermuteHautBas()
   { double xi=this->t[3];
     t[3] = t[2];
     t[2] = xi;
   };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2BH::MaxiComposante() const
  { return DabsMaxiTab(t,  4) ;
   };
             
// retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
 double& Tenseur2BH::Coor( const int i, const int j)
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurBH::Coor(int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[3]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break; }
         default : return t[0];                       			
		} 
    }; 
       
// retourne la composante i,j en lecture uniquement
#ifndef MISE_AU_POINT
  inline 
#endif
 double Tenseur2BH::operator () ( const int i, const int j) const
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurBH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[3]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break; }
         default : return t[0];                       			
		} 
    };    
 
//fonctions static définissant le produit tensoriel de deux vecteurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH & Tenseur2BH::Prod_tensoriel(const CoordonneeB & aB, const CoordonneeH & bH)
  {  TenseurBH * res;
     #ifdef MISE_AU_POINT
       if ((aB.Dimension() != 2) || (bH.Dimension() != 2))
        { cout << "\n erreur de dimension dans les coordonnées d'entrée, dim1 et dim2 ="
               << aB.Dimension() << " " << bH.Dimension() 
               << "\n Tenseur2BH::Prod_tensoriel( etc.." << endl;
          Sortie(1);
         }  
     #endif
     res =  new Tenseur2BH;    
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = aB(1) * bH(1); res->t[3] = aB(1) * bH(2); 
     res->t[2] = aB(2) * bH(1); res->t[1] = aB(2) * bH(2); 
    return *res ;};  
 
// lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2BH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 3; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2BH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2BH ";
    // puis les datas
     for (int i = 0; i<= 3; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2BH & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != 2) A.Message(2,"operator >> (istream & entree, Tenseur2BH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<=3; i++)
        entree >> A.t[i];
    return entree;      
  };
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2BH & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2BH ";
    // puis les datas
     for (int i = 0; i<=3; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };

//------------------------------------------------------------------
//          cas des composantes mixtes HB
//------------------------------------------------------------------
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HB::ChangementIndex::ChangementIndex() :
  idx_i(4),idx_j(4),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=2;idx_i(4)=1;  idx_j(3)=1;idx_j(4)=2;
    odVect(1,1)=1;odVect(1,2)=4;
    odVect(2,1)=3;odVect(2,2)=2;
  }; 

#ifndef MISE_AU_POINT
  inline 
#endif
 Tenseur2HB::Tenseur2HB():  
 ipointe()   // par defaut
   { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
     t[0] = 0.; t[1] = 0.; t[2] = 0.;t[3] = 0.;
   };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HB::Tenseur2HB( const double val):  
 ipointe()  
  { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val; t[1] =val; t[2] =val; t[3] =val;
  };
// initialisation avec 4 valeurs différentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HB::Tenseur2HB
       ( const double val1, const  double val2, const  double val3, const double val4):  
 ipointe()  
  { dimension = 2;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] =val1; t[1] =val2; t[2] =val3; t[3] = val4;
  }; 
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HB::Tenseur2HB ( const  TenseurHB & B):  
 ipointe() 
  { this->dimension = B.dimension;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::Tenseur2HB( etc..");
    #endif
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2]; this->t[3] = B.t[3];
  };         
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HB::Tenseur2HB (  const Tenseur2HB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble4.push_front(Reels4());  // allocation
    ipointe = listdouble4.begin(); // recup de la position de la maille dans la liste
     t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    this->t[0] = B.t[0]; this->t[1] = B.t[1];
    this->t[2] = B.t[2]; this->t[3] = B.t[3];
  };         

#ifndef MISE_AU_POINT
  inline 
#endif
 Tenseur2HB::~Tenseur2HB() 
{//if(listdouble4.end() != listdouble4.begin())        // si la liste n'est pas vide
     listdouble4.erase(ipointe);} ; // suppression de l'élément de la liste
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2HB::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;t[3] =val;};
// operations 
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator + ( const TenseurHB & B) const 
   {  TenseurHB * ptr;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator + ( etc..");
      #endif
      ptr =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = this->t[i] + B.t[i]; //somme des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2HB::operator += ( const TenseurHB & B)
    { 
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator += ( etc..");
      #endif
      for (int i = 0; i<=3; i++)
       this->t[i] += B.t[i];}; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator - () const 
    { TenseurHB * ptr;
      ptr =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = - this->t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator - ( const TenseurHB & B) const 
    {  TenseurHB * ptr;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator - ( etc..");
      #endif
      ptr =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=3; i++)
        (*ptr).t[i] = this->t[i] - B.t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2HB::operator -= ( const TenseurHB & B)
     {
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator -= ( etc..");
      #endif
      for (int i = 0; i<=3; i++)
         this->t[i] -= B.t[i];}; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator = ( const TenseurHB & B)
  { 
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator = ( etc..");
    #endif
    for (int i = 0; i<=3; i++)
       this->t[i] = B.t[i];
    LesMaillonsHB::Libere(); // destruction des tenseurs intermediaires
    return *this; }; //affectation des données;
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator * ( const double & b) const 
    { TenseurHB * res;
      res =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=3; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2HB::operator *= ( const double & b)
      { for (int i = 0; i<=3; i++)
       this->t[i] *= b;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator / ( const double & b) const 
    { TenseurHB * res;
      res =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=3; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
 void Tenseur2HB::operator /= ( const double & b)
      { for (int i = 0; i<=3; i++)
       this->t[i] /= b;}; //division des données
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this, les données en trop sont ignorées
void Tenseur2HB::Affectation_3D_a_2D(const Tenseur3HB & B)  
{ this->t[0] = B.t[0];this->t[1] = B.t[4];this->t[2] = B.t[3];this->t[3] = B.t[1];
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur2HB::Affectation_trans_dimension(const TenseurHB & B,bool plusZero)
{ switch (B.Dimension())
   {case 2:  *this = B; break; // affectation normale
    case 3:
      { const Tenseur3HB & bn  = *((Tenseur3HB *) &B);
        this->Affectation_3D_a_2D(bn);
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur2HB::Affectation_trans_dimension( const TenseurHB & B, ..");
   }
};

 // produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeH Tenseur2HB::operator * ( const CoordonneeH & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur2HB::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeH v(dimension);
    v(1) = this->t[0] * B(1) + this->t[3] * B(2);
    v(2) = this->t[2] * B(1) + this->t[1] * B(2);
    return v;
  };            

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur2HB::operator * ( const TenseurHH & B) const   // produit une fois contracte      
     { TenseurHH * res;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator * ( etc..");
      #endif
      // on cree systematiquement un tenseur non symetrique
      res =  new Tenseur_ns2HH;
      LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
      if (B.Dimension() == 2) // cas symetrique
       { res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
         res->t[1] = this->t[2] * B.t[2] + this->t[1] * B.t[1];
         res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
         res->t[3] = this->t[0] * B.t[2] + this->t[3] * B.t[1] ;
        }
       else  // cas ou B n'est pas symetrique
        {
         res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
         res->t[1] = this->t[2] * B.t[3] + this->t[1] * B.t[1];
         res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
         res->t[3] = this->t[0] * B.t[3] + this->t[3] * B.t[1];
         };      
      
      return  (*res); };          
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurHB & Tenseur2HB::operator * ( const TenseurHB & B) const  // produit une fois contracte
    { TenseurHB * res;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator * ( etc..");
      #endif
      res =  new Tenseur2HB;
      LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
       res->t[0] = this->t[0] * B.t[0] + this->t[3] * B.t[2];
       res->t[1] = this->t[2] * B.t[3] + this->t[1] * B.t[1];
       res->t[2] = this->t[2] * B.t[0] + this->t[1] * B.t[2];
       res->t[3] = this->t[0] * B.t[3] + this->t[3] * B.t[1];
       return  (*res); };
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2HB::operator && ( const TenseurHB & B) const  // produit deux fois contracte
    { double b;
      #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator && ( etc..");
      #endif
      b = (this->t[0] * B.t[0] + this->t[3] * B.t[2]) +
          (this->t[2] * B.t[3] + this->t[1] * B.t[1]);
      return b;};     
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2HB::Trace() const    // trace du tenseur
    { double b;
      b = this->t[0] + this->t[1] ;
      return b;};     
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2HB::II() const    // second invariant
//    { double b;
//      // on met Dabs car dans les petits nombres on peut avoir des nombres négatifs !!
//      b =  Dabs( this->t[0] * this->t[0] + 2. * this->t[3] * this->t[2]
//          + this->t[1] * this->t[1]);
//      return b;};
    { //return Dabs((*this && *this)); // on met Dabs car dans les petits nombres on peut avoir des nombres
      double ret = (*this && *this);
//      if (Dabs(ret) < 0.1*ConstMath::pasmalpetit)
        // on part du principe qu'il s'agit du produit doublement contracté d'un tenseur symétrique
        // donc normalement il devrait donner un nombre positif, donc s'il est très petit, on le met arbitrairement à 0
      if (ret < 0.)
        {ret = 0.;}
      return ret;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2HB::III()  const   // troisieme invariant
    { double b,b1,b2,b3,b4;
      b1 =  this->t[0] * this->t[0] +  this->t[3] * this->t[2];
      b2 =  this->t[0] * this->t[3] +  this->t[3] * this->t[1];
      b3 =  this->t[2] * this->t[0] +  this->t[1] * this->t[2];
      b4 =  this->t[2] * this->t[3] +  this->t[1] * this->t[1];
      b = b1 * this->t[0] + b2 * this->t[2] +
          b3 * this->t[3] + b4 * this->t[1];
      return b;};              
                
    // valeurs propre  dans le vecteur  de retour, classée par ordres "décroissants"
    // cas indique le cas de valeur propre: 
    // quelque soit la dimension: cas = -1, si l'extraction des valeurs propres n'a pas pu se faire
    //                            dans ce cas les valeurs propres de retour sont nulles par défaut
    // dim = 2 , cas = 1 si deux valeurs propres distinctes, cas = 0 si deux val propres identiques
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee Tenseur2HB::ValPropre(int& cas) const 
  { Coordonnee ret(2);
    double b=(this->t[0]+this->t[1]); // la trace
	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	// donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )  
       { cas=0;ret(1)=ret(2)=b/2.;return ret;
         };
    // sinon ..
	   b = -b; // on utilise l'inverse pour la suite
    double c=this->t[0] * this->t[1] - this->t[2] * this->t[3];
    int caas;
    alg_zero.SecondDegre(1.,b,c,ret(1),ret(2),caas);
    if (caas <= 0)
    	{ // cas d'un pb pourtant on sait qu'il doit y avoir une solution. On essaie une solution avec
    	  // des conditions de précision relachées: ici la procédure est plus poussive !!
    	  if ((Dabs(b) < ConstMath::petit)  && (Dabs(c)< ConstMath::petit))
    	    // cela signifie que les valeurs propres sont pratiquements nulles
    	    	{ ret(1) = ret(2) = 0.; 
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	}
    	  else if ((Dabs(b*b - 4. * c) < ConstMath::petit) &&  ((b*b - 4. * c) < 0.))
    	    // cas d'un discriminant négatif mais avec des valeurs faibles
    	    	{ ret(1) = ret(2) = -b * 0.5 ; // -b/(2a)
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	};	
    	};
    #ifdef MISE_AU_POINT
      if (caas <= 0)
       { cout << "\n warning **** recherche de valeurs propre du tenseur: pas de racine correcte ! "
              << "\n tenseur= " << *this
              << "\n on laisse le calcul se poursuivre cependant"
              << "\n Coordonnee Tenseur2HB::ValPropre()";
        };      
    #endif
    // classement des valeurs propres
    if (ret(2) > ret(1))
      { double x = ret(1); ret(1) = ret(2); ret(2)=x;};
    // def du cas
    if ( difftrespetit(ret(1),ret(2))) { cas = 1;} else {cas = 0;};
    // s'il y a eu erreur dans la recherche de valeurs propre on modifie le cas
    if (caas <= 0) cas=-1;
    // retour
    return ret;
   };
   
    // idem met en retour la matrice mat contiend par colonne les vecteurs propre
    // elle doit avoir la dimension du tenseur
    // le premier vecteur propre est exprimé dans le repere naturel 
    // le second vecteur propre est exprimé dans le repère dual 
#ifndef MISE_AU_POINT
  inline 
#endif
Coordonnee Tenseur2HB::ValPropre(int& cas, Mat_pleine& mat) const 
  { Coordonnee ret(2);
    double b=(this->t[0]+this->t[1]); // la trace
	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	   // donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )  
    // on ramène le tenseur identité en absolue comme base de vecteurs propres car toute base est ok
       { mat.Initialise(0.); mat(1,1)=1.;mat(2,2)=1.;
         cas=0;ret(1)=ret(2)=b/2.;return ret;
         };
    // sinon       
	   b = -b; // on utilise l'inverse pour la suite
    double c=this->t[0] * this->t[1] - this->t[2] * this->t[3];
    int caas;
    alg_zero.SecondDegre(1.,b,c,ret(1),ret(2),caas);
    if (caas <= 0)
    	{ // cas d'un pb pourtant on sait qu'il doit y avoir une solution. On essaie une solution avec
    	  // des conditions de précision relachées: ici la procédure est plus poussive !!
    	  if ((Dabs(b) < ConstMath::petit)  && (Dabs(c)< ConstMath::petit))
    	    // cela signifie que les valeurs propres sont pratiquements nulles
    	    	{ ret(1) = ret(2) = 0.; 
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	}
    	  else if ((Dabs(b*b - 4. * c) < ConstMath::petit) &&  ((b*b - 4. * c) < 0.))
    	    // cas d'un discriminant négatif mais avec des valeurs faibles
    	    	{ ret(1) = ret(2) = -b * 0.5 ; // -b/(2a)
    	    	  caas = 0; // pour dire que l'on a réglé le pb
    	    	};	
    	};
    #ifdef MISE_AU_POINT
     if (caas <= 0)
       { cout << "\n warning **** recherche de valeurs propre du tenseur: pas de racine correcte ! "
              << "\n tenseur= " << *this
              << "\n on laisse le calcul se poursuivre cependant"
              << "\n Coordonnee Tenseur2HB::ValPropre( Mat_pleine& mat)";
        };      
    #endif

    #ifdef MISE_AU_POINT	 	 
     if  ((mat.Nb_ligne() != 2) || (mat.Nb_colonne() != 2))
     { cout << "\nErreur : la matrice en parametre doit etre de dimension " 
            << 2 << " !\n";
       cout << "Tenseur2HB::ValPropre( Mat_pleine& mat)  \n";
       Sortie(1);
     };
    #endif
    // classement des valeurs propres
    if (ret(2) > ret(1))
      { double x = ret(1); ret(1) = ret(2); ret(2)=x;};
    // calcul des vecteurs normés
    double norme1= sqrt(this->t[3] * this->t[3] 
                  + (this->t[0] - ret(1))*(this->t[0] - ret(1)) );
    double norme2= sqrt(this->t[2] * this->t[2] 
                  + (this->t[1] - ret(1))*(this->t[1] - ret(1)) );
    // on prend le maxi des normes
    bool premier= true; double norme=norme1;
    if (norme1 < norme2) {premier = false; norme = norme2;};             
    if (norme < ConstMath::pasmalpetit)
      { if (ParaGlob::NiveauImpression() > 4)
          cout << "\n attention, construction des vecteurs propres impossible car valeurs propres nulles"
               << "\n on met des vecteurs par defaut ";
        #ifdef MISE_AU_POINT
         if ((ParaGlob::NiveauImpression() > 3)&&(ParaGlob::NiveauImpression() < 4))
           cout << "\n attention, construction des vecteurs propres impossible  car valeurs propres nulles"
                << "\n on annulle les vecteurs propres ";
         if (ParaGlob::NiveauImpression() > 3)
           cout << "\n tenseur= " << *this << "\n valeur propres= " << ret << ", norme = " << norme
                << "\n Coordonnee Tenseur2HB::ValPropre( Mat_pleine& mat)";
        #endif
        mat.Zero();mat(1,1)=mat(2,2)=1.;
        return ret;
       };
    if (premier)                 
     {mat(1,1) = - this->t[3] / norme;
      mat(2,1) = (this->t[0] - ret(1)) / norme;
      }
    else
     {mat(1,1) = (this->t[1] - ret(1)) / norme;     
      mat(2,1) = - this->t[2]  / norme;
     };
    mat(1,2) = - mat(2,1); mat(2,2) = mat(1,1); // le deuxième vecteur           

    // def du cas
    if ( difftrespetit(ret(1),ret(2))) { cas = 1;} else {cas = 0;};
    // s'il y a eu erreur dans la recherche de valeurs propre on modifie le cas
    if (caas <= 0) cas=-1;
   
    #ifdef MISE_AU_POINT
    // en débug on vérifie que les valeurs propres fonctionnent correctement
    //               , cas = 1 si 2 val propres différentes
    //               , cas = 0 si les 2 sont identiques
    if (ParaGlob::NiveauImpression() > 2)
     { CoordonneeH Vp(2);
       switch (cas)
        { case 0: Vp(1)=Vp(2)=ret(1); break;
          case 1: Vp(1) = ret(1); Vp(2)=ret(2); break;
        };
       CoordonneeH Vpropre(mat(1,1),mat(2,1));
       CoordonneeH W6 = (*this)* Vpropre;
       // normalement W6 = ret(1)*Vpropre
       double diff = (W6 - (Vpropre * ret(1))).Norme();
       double w6_norme = W6.Norme();
       double V1_norme = (Vpropre * ret(1)).Norme();
       if (diffpourcent(w6_norme,V1_norme,MaX(w6_norme,V1_norme),ConstMath::unpeupetit))
//       if (Dabs(diff) > ConstMath::unpeupetit)
          {cout << "\n erreur le vecteur propre 1 ";
           Vpropre.Affiche();
           cout << "\n n'est pas un vecteur propre pour la premiere valeur propre: "
                << Vp(1) << " concernant la matrice: ";
           this->Ecriture(cout);
           cout << "\n A*V = "<<W6 << "\n(Vpropre * val1)= " << (Vpropre * ret(1))
                << "\n diff= "<<diff << " W6.Norme()= " << w6_norme
                << " (Vpropre * ret(1)).Norme()= " << V1_norme;

           cout << "\n cas= "<< cas ;
           cout << "\n les val propres: " << ret(1) << " et "<<ret(2) ;
              
           cout  << "\n Tenseur2HB::ValPropre(..."  << endl;
          };
       // on vérifie que le produit scalaire des deux vecteurs propres est nul
       diff = (mat(1,1)*mat(1,2)+mat(2,1)*mat(2,2));
       int i,j;
       double max_mat = mat.MaxiValAbs(i, j);
       if (Dabs(diff) > (ConstMath::unpeupetit * max_mat))
          {cout << "\n erreur le produit scalaire des deux vecteurs propres n'est pas nul !! ";
           cout << "\n V1= " << mat(1,1) <<" "<<mat(2,1) << " V2= " << mat(2,1) <<" "<<mat(2,2)  ;
           cout  << "\n Tenseur2HB::ValPropre(..."  << endl;
          };
     };
    #endif
   
    // retour
    return ret;
   };

    // ici il s'agit uniquement de calculer les vecteurs propres, les valeurs propres
    // étant déjà connues
    // en retour VP les vecteurs propre : doivent avoir la dimension du tenseur
    // les vecteurs propre sont exprime dans le repere naturel (pour les tenseurs dim 3
    // pour dim=2:le premier vecteur propre est exprime dans le repere naturel 
    //            le second vecteur propre est exprimé dans le repère dual
    // pour dim=1 le vecteur est dans la base naturelle (mais ça importe pas)
    //   en sortie cas = -1 s'il y a eu un problème, dans ce cas, V_P est quelconque
    //             sinon si tout est ok, cas est identique en sortie avec l'entrée
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2HB::VecteursPropres(const Coordonnee& valPropre,int& cas, Tableau <Coordonnee>& V_P) const
  { // on dimensionne en conséquence
    Coordonnee toto(2); // un vecteur nul
    V_P.Change_taille(2,toto); // tableau initialisé à 0
  
    double b=(this->t[0]+this->t[1]); // la trace
    // la formule qui suit à été vérifiée !! elle est bonne en 2D
    // intens2 doit représenter l'intensité au carré du déviateur
    // si par hazard c'est négatif on peut se poser des questions
    double intens2 = (this->II() - b*b*0.5);
    double sqrt_II_b = sqrt(Dabs(intens2)); // l'intensité du déviateur
    if ((intens2 < 0.) && (sqrt_II_b > Dabs(b)))
     {
      // veut dire que l'on est très fortement négatif, on considère que l'erreur est trop grande
      #ifdef MISE_AU_POINT
      if  (cas != 0)
       { cout << "\nErreur : l'intensite du deviateur "<< intens2 << " est negative et superieure "
              << " a la trace par exemple, on considere que l'erreur est trop grande, le calcul "
              << " des valeurs propres n'est pas possible ";
         cout << "\n Tenseur2HB::VecteursPropres(...  \n";
       };
      #endif
      cas=-1;
      return;
     }
    else
     // sinon c'est ok, et on continue
   
//	   double sqrt_II_b = sqrt(Dabs(this->II() - b*b*0.5)) ; // l'intensité du déviateur
    // si le tenseur est sphérique, les deux valeurs propres sont identique au 1/2 de la trace
	   // donc si l'intensité/trace est plus petite qu'environ 10-7 (unpeupetit)
    if (sqrt_II_b <= ConstMath::unpeupetit * Dabs(b) )
    // on ramène le tenseur identité en absolue comme base de vecteurs propres car toute base est ok
       {
        #ifdef MISE_AU_POINT
        if  (cas != 0)
         { cout << "\nErreur : les valeurs propres devraient etre identiques alors qu'en entree: cas= " << cas ;
           cout << "\n Tenseur2HB::VecteursPropres(...  \n";
           cas=-1;
           return;
         };
        #endif
         V_P(1).Zero(); V_P(1)(1)=1.;
         V_P(2).Zero(); V_P(2)(2)=1.;
         cas=0;return;
       };
    // sinon ce n'est pas sphérique
    // calcul des vecteurs normés
    double norme1= sqrt(this->t[3] * this->t[3] 
                  + (this->t[0] - valPropre(1))*(this->t[0] - valPropre(1)) );
    double norme2= sqrt(this->t[2] * this->t[2] 
                  + (this->t[1] - valPropre(1))*(this->t[1] - valPropre(1)) );
    // on prend le maxi des normes
    bool premier= true; double norme=norme1;
    if (norme1 < norme2) {premier = false; norme = norme2;};             
    if (norme < ConstMath::pasmalpetit)
      { // la construction des vecteurs propres est impossible
        if (ParaGlob::NiveauImpression() > 4)
          cout << "\n attention, construction des vecteurs propres impossible car valeurs propres nulles"
               << "\n on met des vecteurs par defaut ";
        #ifdef MISE_AU_POINT
         if ((ParaGlob::NiveauImpression() > 3)&&(ParaGlob::NiveauImpression() < 4))
           cout << "\n attention, construction des vecteurs propres impossible  car valeurs propres nulles"
                << "\n on annulle les vecteurs propres ";
         if (ParaGlob::NiveauImpression() > 3)
           cout << "\n tenseur= " << *this << "\n valeur propres= " << valPropre
                << ", norme = " << norme
                << "\n Tenseur2HB::VecteursPropres(...";
        #endif
        V_P(1).Zero(); V_P(1)(1)=1.;
        V_P(2).Zero(); V_P(2)(2)=1.;
        cas = -1;
        return ;
       };
    if (premier)                 
     {V_P(1)(1)= - this->t[3] / norme;
      V_P(1)(2)= (this->t[0] - valPropre(1)) / norme;
      }
    else
     {V_P(1)(1)= (this->t[1] - valPropre(1)) / norme;
      V_P(1)(2)= - this->t[2]  / norme;
      };
    // puis le second vecteur
    V_P(2)(1) = - V_P(1)(2); V_P(2)(2) = V_P(1)(1); // le deuxième vecteur
    // retour
    return ;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
 double  Tenseur2HB::Det()  const     // determinant de la matrice des coordonnees
    { double b;
      b = this->t[0] * this->t[1] - this->t[2] * this->t[3];
      return b;};
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2HB::operator == ( const TenseurHB & B) const 
  { int res = 1;
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator == ( etc..");
    #endif
    for (int i = 0; i<=3; i++)
       if (this->t[i] != B.t[i]) res = 0 ;
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2HB::operator != ( const TenseurHB & B) const 
  { 
    #ifdef MISE_AU_POINT
       if (B.Dimension() != 2) Message(2,"Tenseur2HB::operator != ( etc..");
    #endif  
    if ((*this) == B) 
      return 0;
    else  
      return 1; }; 
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB & Tenseur2HB::Inverse() const 
   {TenseurHB * res;
    res =  new Tenseur2HB;
    LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  res->t[0] = t[0];res->t[1] = t[1];res->t[2] = t[2];res->t[3] = t[3];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;res->t[3]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat2x2(((Tenseur2HB*) res)->ipointe);
			 }
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
//	Sortie(1);
			 break;
		 case CRAMER : // méthode historique
			{ // calcul du determinant
			  double det = this->Det() ;
			#ifdef MISE_AU_POINT	 	 
				if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur2HB::Inverse() \n";
					  Sortie(1);
					};
			#endif
			  // transposee de la matrice des cofacteurs 
			  res->t[0] = this->t[1] / det;
			  res->t[2] = - this->t[2] / det;
			  res->t[1] = this->t[0] / det;
			  res->t[3] = - this->t[3] / det;    
			 break;
			} 
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur2HB::Inverse() \n";
					  Sortie(1);
			  };
	 };
//	 res->Ecriture(cout); // pour le debug
    return *res;
   };                    
             
#ifndef MISE_AU_POINT
  inline 
#endif
 TenseurBH & Tenseur2HB::Transpose() const 
   { TenseurBH * res;
     res =  new Tenseur2BH;
     LesMaillonsBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[1];
     res->t[2] = this->t[3];
     res->t[3] = this->t[2];
    return *res;};           
               
#ifndef MISE_AU_POINT
  inline 
#endif
// permute Bas Haut, mais reste dans le même tenseur
void Tenseur2HB::PermuteHautBas()
   { double xi=this->t[3];
     t[3] = t[2];
     t[2] = xi;
   };
          
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2HB::MaxiComposante() const
  { return DabsMaxiTab(t,  4) ;
   };
             
// retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
 double& Tenseur2HB::Coor( const int i, const int j)
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurHH::Coor(int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[3]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break; }
         default : return t[0];                       			
		} 
    }; 
       
// retourne la composante i,j en lecture uniquement 
#ifndef MISE_AU_POINT
  inline 
#endif
 double Tenseur2HB::operator () ( const int i, const int j) const 
   { 
#ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)) || ((j!=1)&&(j!=2)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "TenseurHH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
#endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[3]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[2]; break;
                       case 2 : return t[1]; break;
                       default : return t[0]; }
                   break; }
         default : return t[0];                       			
		} 
    };    
 
//fonctions static définissant le produit tensoriel de deux vecteurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB & Tenseur2HB::Prod_tensoriel(const CoordonneeH & aH, const CoordonneeB & bB)
  {  TenseurHB * res;
     #ifdef MISE_AU_POINT
       if ((aH.Dimension() != 2) || (bB.Dimension() != 2))
        { cout << "\n erreur de dimension dans les coordonnées d'entrée, dim1 et dim2 ="
               << aH.Dimension() << " " << bB.Dimension() 
               << "\n Tenseur2HB::Prod_tensoriel( etc.." << endl;
          Sortie(1);
         }  
     #endif
     res =  new Tenseur2HB;    
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = aH(1) * bB(1); res->t[3] = aH(1) * bB(2); 
     res->t[2] = aH(2) * bB(1); res->t[1] = aH(2) * bB(2); 
    return *res ;};  
 
// lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2HB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<= 3; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2HB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2HB ";
    // puis les datas
     for (int i = 0; i<= 3; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2HB & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != 2) A.Message(2,"operator >> (istream & entree, Tenseur2HB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i<=3; i++)
        entree >> A.t[i];
    return entree;      
  };
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2HB & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2HB ";
    // puis les datas
     for (int i = 0; i<=3; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };
 
#endif    
