

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "ConstMath.h" 
#include "MathUtil.h"
//#include "Debug.h"
#include "Tenseur2_TroisSym.h"
#include "Tenseur2.h"
#include "CharUtil.h"


#ifndef  TenseurQ2_TroisSym_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes HHHH
//------------------------------------------------------------------ 
  

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::TenseurQ2_troisSym_HHHH():  
 ipointe()  // par défaut
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<6;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::TenseurQ2_troisSym_HHHH( const double& val) :  
 ipointe() 
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<6;i++) t[i]=val;
  };
  
// initialisation à partir des 6 coefficients indépendants
// (1111) (2222) (1122) (1212) (1211) (1222)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::TenseurQ2_troisSym_HHHH(const double& x1111,const double& x2222,const double& x1122
                            ,const double& x1212,const double& x1211,const double& x1222):  
 ipointe() 
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]=x1111;  t[1]= x2222;
    t[2]= x1122; t[3]= x1212;
    t[4]= x1211; t[5]= x1222;
  }; 
 
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::~TenseurQ2_troisSym_HHHH() 
{ listdouble6.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
// il n'y a pas de vérification des symétries  seules les grandeurs
// (1111) (2222) (1122) (1212) (1211) (1222) sont utilisées 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::TenseurQ2_troisSym_HHHH ( const TenseurHHHH & B):  
 ipointe() 
  { listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  t[1]= B(2,2,2,2);
    t[2]= B(1,1,2,2); t[3]= B(1,2,1,2);
    t[4]= B(1,2,1,1); t[5]= B(1,2,2,2);
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_HHHH::TenseurQ2_troisSym_HHHH (  const TenseurQ2_troisSym_HHHH & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 6;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2_troisSym_HHHH::Inita(double val)   
	{ for (int i=0;i< 6;i++)
         t[i] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator + ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator + ( etc..");
     #endif
     res =  new TenseurQ2_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2_troisSym_HHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator - () const 
  {  TenseurHHHH * res;
     res =  new TenseurQ2_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator - ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator - ( etc..");
     #endif
     res =  new TenseurQ2_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_HHHH::operator -= ( const TenseurHHHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator * ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ2_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_HHHH::operator *= ( const double & b)
  {for (int i = 0; i< 6; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::operator / ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ2_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2_troisSym_HHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_HHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2_troisSym_HHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 6; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ2_troisSym_HHHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2) 
           Message(2,"TenseurQ2_troisSym_HHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     // deux cas suivant que le tenseur aBB est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (aBB.Dimension()==2)
      {// cas symétrique
       const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
       res->Coor(1,1)=t[0] * a2BB(1,1) + t[2] * a2BB(2,2) + 2. * t[4] *  a2BB(1,2) ;
       res->Coor(2,2)=t[2] * a2BB(1,1) + t[1] * a2BB(2,2) + 2. * t[5] *  a2BB(1,2) ;
       res->Coor(1,2)=t[4] * a2BB(1,1) + t[5] * a2BB(2,2) + 2. * t[3] *  a2BB(1,2) ;
       }
     else
      {// cas non symétrique
       const Tenseur_ns2BB & a2BB = *((Tenseur_ns2BB*) &aBB); // passage en dim 2
       res->Coor(1,1)=t[0] * a2BB(1,1) + t[2] * a2BB(2,2) + t[4] * ( a2BB(1,2) + a2BB(2,1)) ;
       res->Coor(2,2)=t[2] * a2BB(1,1) + t[1] * a2BB(2,2) + t[5] * ( a2BB(1,2) + a2BB(2,1)) ;
       res->Coor(1,2)=t[4] * a2BB(1,1) + t[5] * a2BB(2,2) + t[3] * ( a2BB(1,2) + a2BB(2,1)) ;
       }
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2_troisSym_HHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new TenseurQ2_troisSym_HHHH(*this);
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2_troisSym_HHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 30 : case 306 :
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                this->Change(i,j,k,l,aHHHH(i,j,k,l));
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ2_troisSym_HHHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2_troisSym_HHHH::operator == ( const TenseurHHHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_HHHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2_troisSym_HHHH::Change (int i, int j, int k, int l, const double& val) 
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[0] = val; break;                              // 1111
                                       case 2: t[4] = val; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: t[4] = val; break;                              // 1121
                                       case 2: t[2] = val; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] = val; break;                              // 1211
                                       case 2: t[3] = val; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: t[3] = val; break;                              // 1221
                                       case 2: t[5] = val; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] = val; break;                              // 2111
                                       case 2: t[3] = val; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: t[3] = val; break;                              // 2121
                                       case 2: t[5] = val; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[2] = val; break;                              // 2211
                                       case 2: t[5] = val; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: t[5] = val; break;                              // 2221
                                       case 2: t[1] = val; break; }  break; } break; }  break; // 2222
     default:
        cout << "\n erreur en modification d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n TenseurQ2_troisSym_BBBB::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);     
    } 
}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2_troisSym_HHHH::ChangePlus (int i, int j, int k, int l, const double& val)
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[0] += val; break;                              // 1111
                                       case 2: t[4] += val; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: t[4] += val; break;                              // 1121
                                       case 2: t[2] += val; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] += val; break;                              // 1211
                                       case 2: t[3] += val; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: t[3] += val; break;                              // 1221
                                       case 2: t[5] += val; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] += val; break;                              // 2111
                                       case 2: t[3] += val; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: t[3] += val; break;                              // 2121
                                       case 2: t[5] += val; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[2] += val; break;                              // 2211
                                       case 2: t[5] += val; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: t[5] += val; break;                              // 2221
                                       case 2: t[1] += val; break; }  break; } break; }  break; // 2222
     default:
        cout << "\n erreur en modification d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n TenseurQ2_troisSym_BBBB::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
    }
};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2_troisSym_HHHH::operator () (int i, int j, int k, int l) const 
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[0]; break;                              // 1111
                                       case 2: return t[4]; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: return t[4]; break;                              // 1121
                                       case 2: return t[2]; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[4]; break;                              // 1211
                                       case 2: return t[3]; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: return t[3]; break;                              // 1221
                                       case 2: return t[5]; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[4]; break;                              // 2111
                                       case 2: return t[3]; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: return t[3]; break;                              // 2121
                                       case 2: return t[5]; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[2]; break;                              // 2211
                                       case 2: return t[5]; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: return t[5]; break;                              // 2221
                                       case 2: return t[1]; break; }  break; } break; }  break; // 2222
    }
 cout << "\n erreur en acces lecture d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
      << "\n double  TenseurQ2_troisSym_HHHH::operator () (int i, int j, int k, int l) const";   
 Sortie(1); return 1; // pour faire taire le compilo    
}; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2_troisSym_HHHH::MaxiComposante() const
  { return DabsMaxiTab(t,6) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2_troisSym_HHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 6; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2_troisSym_HHHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ2_troisSym_HHHH ";
    // puis les datas
     for (int i = 0; i< 6; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2_troisSym_HHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 206) A.Message(2,"operator >> (istream & entree, TenseurQ2_troisSym_HHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 6; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2_troisSym_HHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2_troisSym_HHHH ";
    // puis les datas
    for (int i = 0; i< 6; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::TenseurQ2_troisSym_BBBB() :  
 ipointe() // par défaut
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<6;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::TenseurQ2_troisSym_BBBB( const double& val) :  
 ipointe() 
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<6;i++) t[i]=val;
  };
  
// initialisation à partir des 6 coefficients indépendants
// (1111) (2222) (1122) (1212) (1211) (1222)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::TenseurQ2_troisSym_BBBB(const double& x1111,const double& x2222,const double& x1122
                            ,const double& x1212,const double& x1211,const double& x1222):  
 ipointe() 
  { dimension = 206;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]=x1111;  t[1]= x2222;
    t[2]= x1122; t[3]= x1212;
    t[4]= x1211; t[5]= x1222;
  }; 
 
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::~TenseurQ2_troisSym_BBBB() 
{ listdouble6.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
// il n'y a pas de vérification des symétries  seules les grandeurs
// (1111) (2222) (1122) (1212) (1211) (1222) sont utilisées 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::TenseurQ2_troisSym_BBBB ( const TenseurBBBB & B) :  
 ipointe()
  { listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  t[1]= B(2,2,2,2);
    t[2]= B(1,1,2,2); t[3]= B(1,2,1,2);
    t[4]= B(1,2,1,1); t[5]= B(1,2,2,2);
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2_troisSym_BBBB::TenseurQ2_troisSym_BBBB (  const TenseurQ2_troisSym_BBBB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble6.push_front(Reels6());  // allocation
    ipointe = listdouble6.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 6;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2_troisSym_BBBB::Inita(double val)   
	{ for (int i=0;i< 6;i++)
         t[i] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator + ( etc..");
     #endif
     res =  new TenseurQ2_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2_troisSym_BBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new TenseurQ2_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator - ( etc..");
     #endif
     res =  new TenseurQ2_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_BBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ2_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_BBBB::operator *= ( const double & b)
  {for (int i = 0; i< 6; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ2_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2_troisSym_BBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 6; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2_troisSym_BBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2_troisSym_BBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 6; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ2_troisSym_BBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2) 
           Message(2,"TenseurQ2_troisSym_BBBB::operator && ( const TenseurBB & aHH)");
     #endif
     TenseurBB * res;
     // deux cas suivant que le tenseur aHH est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (aHH.Dimension()==2)
      {// cas symétrique
       const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
       res->Coor(1,1)=t[0] * a2HH (1,1) + t[2] * a2HH (2,2) + 2. * t[4] *  a2HH (1,2) ;
       res->Coor(2,2)=t[2] * a2HH (1,1) + t[1] * a2HH (2,2) + 2. * t[5] *  a2HH (1,2) ;
       res->Coor(1,2)=t[4] * a2HH (1,1) + t[5] * a2HH (2,2) + 2. * t[3] *  a2HH (1,2) ;
       }
     else
      {// cas non symétrique
       const Tenseur_ns2HH & a2HH  = *((Tenseur_ns2HH*) &aHH); // passage en dim 2
       res->Coor(1,1)=t[0] * a2HH (1,1) + t[2] * a2HH (2,2) + t[4] * ( a2HH (1,2) + a2HH (2,1)) ;
       res->Coor(2,2)=t[2] * a2HH (1,1) + t[1] * a2HH (2,2) + t[5] * ( a2HH (1,2) + a2HH (2,1)) ;
       res->Coor(1,2)=t[4] * a2HH (1,1) + t[5] * a2HH (2,2) + t[3] * ( a2HH (1,2) + a2HH (2,1)) ;       
       }
     return *res ;
  };
  

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2_troisSym_BBBB::Transpose1et2avec3et4() const   
   { TenseurBBBB * res;
     res =  new TenseurQ2_troisSym_BBBB(*this);
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2_troisSym_BBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 30 : case 306 :
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                this->Change(i,j,k,l,aBBBB(i,j,k,l));
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ2_troisSym_BBBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2_troisSym_BBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 206) Message(2,"TenseurQ2_troisSym_BBBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 6; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2_troisSym_BBBB::Change (int i, int j, int k, int l, const double& val) 
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[0] = val; break;                              // 1111
                                       case 2: t[4] = val; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: t[4] = val; break;                              // 1121
                                       case 2: t[2] = val; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] = val; break;                              // 1211
                                       case 2: t[3] = val; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: t[3] = val; break;                              // 1221
                                       case 2: t[5] = val; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] = val; break;                              // 2111
                                       case 2: t[3] = val; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: t[3] = val; break;                              // 2121
                                       case 2: t[5] = val; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[2] = val; break;                              // 2211
                                       case 2: t[5] = val; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: t[5] = val; break;                              // 2221
                                       case 2: t[1] = val; break; }  break; } break; }  break; // 2222
     default:
        cout << "\n erreur en modification d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n TenseurQ2_troisSym_HHHH::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);     
    } 
}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2_troisSym_BBBB::ChangePlus (int i, int j, int k, int l, const double& val)
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[0] += val; break;                              // 1111
                                       case 2: t[4] += val; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: t[4] += val; break;                              // 1121
                                       case 2: t[2] += val; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] += val; break;                              // 1211
                                       case 2: t[3] += val; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: t[3] += val; break;                              // 1221
                                       case 2: t[5] += val; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[4] += val; break;                              // 2111
                                       case 2: t[3] += val; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: t[3] += val; break;                              // 2121
                                       case 2: t[5] += val; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: t[2] += val; break;                              // 2211
                                       case 2: t[5] += val; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: t[5] += val; break;                              // 2221
                                       case 2: t[1] += val; break; }  break; } break; }  break; // 2222
     default:
        cout << "\n erreur en modification d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n TenseurQ2_troisSym_HHHH::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
    }
};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2_troisSym_BBBB::operator () (int i, int j, int k, int l) const 
 { switch (i)
   { case 1: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[0]; break;                              // 1111
                                       case 2: return t[4]; break; } break;                     // 1112
                           case 2: switch (l)
                                    {  case 1: return t[4]; break;                              // 1121
                                       case 2: return t[2]; break; }  break; }  break;          // 1122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[4]; break;                              // 1211
                                       case 2: return t[3]; break; }  break;                    // 1212
                           case 2: switch (l)
                                    {  case 1: return t[3]; break;                              // 1221
                                       case 2: return t[5]; break; }  break; } break; }  break; // 1222
     case 2: switch (j)
              { case 1: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[4]; break;                              // 2111
                                       case 2: return t[3]; break; } break;                     // 2112
                           case 2: switch (l)
                                    {  case 1: return t[3]; break;                              // 2121
                                       case 2: return t[5]; break; }  break; }  break;          // 2122
                case 2: switch (k)
                         { case 1: switch (l)
                                     { case 1: return t[2]; break;                              // 2211
                                       case 2: return t[5]; break; }  break;                    // 2212
                           case 2: switch (l)
                                    {  case 1: return t[5]; break;                              // 2221
                                       case 2: return t[1]; break; }  break; } break; }  break; // 2222
    } 
 cout << "\n erreur en acces lecture d'une composante de tenseur: ("<<i<<","<<j<<","<<k<<","<<l<<")"
      << "\n double  TenseurQ2_troisSym_BBBB::operator () (int i, int j, int k, int l) const";   
 Sortie(1); return 1; // pour faire taire le compilo    
}; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2_troisSym_BBBB::MaxiComposante() const
  { return DabsMaxiTab(t,6) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2_troisSym_BBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 6; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2_troisSym_BBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ2_troisSym_BBBB ";
    // puis les datas
     for (int i = 0; i< 6; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2_troisSym_BBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 206) A.Message(2,"operator >> (istream & entree, TenseurQ2_troisSym_BBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 6; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2_troisSym_BBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2_troisSym_BBBB ";
    // puis les datas
    {for (int i = 0; i< 6; i++)
        {sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";}}
    return sort;      
  };

 
  
#endif
