

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "ConstMath.h" 
#include "MathUtil.h"
//#include "Debug.h"
#include "Tenseur3_TroisSym.h"
#include "Tenseur3.h"
#include "CharUtil.h"


#ifndef  TenseurQ3_TroisSym_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes HHHH
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
//    t[0]= x1111; t[1]= x2222; t[2]= x3333; t[3]= x1122; t[4]= x1133; t[5]= x2233;
//    t[6]= x1211; t[7]= x1222; t[8]= x1233; t[9]= x1311; t[10]= x1322; t[11]= x1333;
//    t[12]= x2311; t[13]= x2322; t[14]= x2333;
//    t[15]= x1212; t[16]= x1313; t[17]= x2323; t[18]= x1213; t[19]= x1223; t[20]= x1323; 
TenseurQ3_troisSym_HHHH::ChangementIndex::ChangementIndex() :
 odVect(3) 
  { odVect(1,1,1,1)=0;  odVect(1,1,1,2)=6;  odVect(1,1,1,3)=9; 
    odVect(1,1,2,1)=6;  odVect(1,1,2,2)=3;  odVect(1,1,2,3)=12;
    odVect(1,1,3,1)=9;  odVect(1,1,3,2)=12; odVect(1,1,3,3)=4;
    odVect(1,2,1,1)=6;  odVect(1,2,1,2)=15; odVect(1,2,1,3)=18; //
    odVect(1,2,2,1)=15; odVect(1,2,2,2)=7;  odVect(1,2,2,3)=19;
    odVect(1,2,3,1)=18; odVect(1,2,3,2)=19; odVect(1,2,3,3)=8;
    odVect(1,3,1,1)=9;  odVect(1,3,1,2)=18; odVect(1,3,1,3)=16; //
    odVect(1,3,2,1)=18; odVect(1,3,2,2)=10; odVect(1,3,2,3)=20;
    odVect(1,3,3,1)=16; odVect(1,3,3,2)=20; odVect(1,3,3,3)=11;

    odVect(2,1,1,1)=6;  odVect(2,1,1,2)=15; odVect(2,1,1,3)=18; 
    odVect(2,1,2,1)=15; odVect(2,1,2,2)=7;  odVect(2,1,2,3)=19;
    odVect(2,1,3,1)=18; odVect(2,1,3,2)=19; odVect(2,1,3,3)=8;
    odVect(2,2,1,1)=3;  odVect(2,2,1,2)=7;  odVect(2,2,1,3)=10; //
    odVect(2,2,2,1)=7;  odVect(2,2,2,2)=1;  odVect(2,2,2,3)=13;
    odVect(2,2,3,1)=10; odVect(2,2,3,2)=13; odVect(2,2,3,3)=5;
    odVect(2,3,1,1)=11; odVect(2,3,1,2)=19; odVect(2,3,1,3)=20; //
    odVect(2,3,2,1)=19; odVect(2,3,2,2)=13; odVect(2,3,2,3)=17;
    odVect(2,3,3,1)=20;  odVect(2,3,3,2)=17; odVect(2,3,3,3)=14;

    odVect(3,1,1,1)=9;  odVect(3,1,1,2)=18; odVect(3,1,1,3)=16; 
    odVect(3,1,2,1)=18; odVect(3,1,2,2)=10; odVect(3,1,2,3)=20;
    odVect(3,1,3,1)=16; odVect(3,1,3,2)=20; odVect(3,1,3,3)=11;
    odVect(3,2,1,1)=12; odVect(3,2,1,2)=19; odVect(3,2,1,3)=20; //
    odVect(3,2,2,1)=19; odVect(3,2,2,2)=13; odVect(3,2,2,3)=17;
    odVect(3,2,3,1)=20;  odVect(3,2,3,2)=17; odVect(3,2,3,3)=14;
    odVect(3,3,1,1)=4;  odVect(3,3,1,2)=8;  odVect(3,3,1,3)=11; //
    odVect(3,3,2,1)=8;  odVect(3,3,2,2)=3;  odVect(3,3,2,3)=14;
    odVect(3,3,3,1)=11; odVect(3,3,3,2)=14; odVect(3,3,3,3)=2;
  }; 

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::TenseurQ3_troisSym_HHHH() :  
 ipointe() // par défaut
  { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<21;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::TenseurQ3_troisSym_HHHH( const double& val) :  
 ipointe() 
  { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<21;i++) t[i]=val;
  };
  
// initialisation à partir des 21 coefficients indépendants
// (1111) (2222) (3333) (1122) (1133) (2233)
// (1211) (1222) (1233) (1311) (1322) (1333) (2311) (2322) (2333) 
// (1212) (1313) (2323) (1213) (1223) (1323)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::TenseurQ3_troisSym_HHHH
        ( const double& x1111,const double& x2222,const double& x3333,const double& x1122,const double& x1133,const double& x2233
         ,const double& x1211,const double& x1222,const double& x1233,const double& x1311,const double& x1322,const double& x1333
         ,const double& x2311,const double& x2322,const double& x2333
         ,const double& x1212,const double& x1313,const double& x2323,const double& x1213,const double& x1223,const double& x1323)
 :  
 ipointe()
  { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]= x1111; t[1]= x2222; t[2]= x3333; t[3]= x1122; t[4]= x1133; t[5]= x2233;
    t[6]= x1211; t[7]= x1222; t[8]= x1233; t[9]= x1311; t[10]= x1322; t[11]= x1333;
    t[12]= x2311; t[13]= x2322; t[14]= x2333;
    t[15]= x1212; t[16]= x1313; t[17]= x2323; t[18]= x1213; t[19]= x1223; t[20]= x1323; 
  }; 
 
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::~TenseurQ3_troisSym_HHHH() 
{ listdouble21.erase(ipointe);} ; // suppression de l'élément de la liste

// constructeur a partir d'une instance non differenciee  
// il n'y a pas de vérification des symétries  seules les grandeurs suivantes sont utilisés:
// (1111) (2222) (3333) (1122) (1133) (2233)
// (1211) (1222) (1233) (1311) (1322) (1333) (2311) (2322) (2333) 
// (1212) (1313) (2323) (1213) (1223) (1323)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::TenseurQ3_troisSym_HHHH ( const TenseurHHHH & B) :  
 ipointe()
  { listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]= B(1,1,1,1); t[1]= B(2,2,2,2); t[2]= B(3,3,3,3); t[3]= B(1,1,2,2); t[4]= B(1,1,3,3); t[5]= B(2,2,3,3);
    t[6]= B(1,2,1,1); t[7]= B(1,2,2,2); t[8]= B(1,2,3,3); t[9]= B(1,3,1,1); t[10]= B(1,3,2,2); t[11]= B(1,3,3,3);
    t[12]= B(2,3,1,1); t[13]= B(2,3,2,2); t[14]= B(2,3,3,3);
    t[15]= B(1,2,1,2); t[16]= B(1,3,1,3); t[17]= B(2,3,2,3); t[18]= B(1,2,1,3); t[19]= B(1,2,2,3); t[20]= B(1,3,2,3); 
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_HHHH::TenseurQ3_troisSym_HHHH (  const TenseurQ3_troisSym_HHHH & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 21;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ3_troisSym_HHHH::Inita(double val)   
	{ for (int i=0;i< 21;i++)
         t[i] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator + ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator + ( etc..");
     #endif
     res =  new TenseurQ3_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ3_troisSym_HHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator - () const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator - ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator - ( etc..");
     #endif
     res =  new TenseurQ3_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_HHHH::operator -= ( const TenseurHHHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator * ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_HHHH::operator *= ( const double & b)
  {for (int i = 0; i< 21; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::operator / ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3_troisSym_HHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_HHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3_troisSym_HHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 21; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ3_troisSym_HHHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(2,"TenseurQ3_troisSym_HHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     // deux cas suivant que le tenseur aBB est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (aBB.Dimension()==3)
      {// cas symétrique
       const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
       for (int i=1;i<=3;i++) for (int j=1;j<=i;j++)
        {double x=0.; 
         for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
           x += t[cdex.odVect(i,j,k,l)] * a3BB(k,l);
         res->Coor(i,j) = x;
        };
       }
     else
      {// cas non symétrique, même calcul que si dessus, simplement du fait de la def explicite du
       // tenseur a3BB, on doit aller plus vite en adressage de l'opérateur ()
       const Tenseur_ns3BB & a3BB = *((Tenseur_ns3BB*) &aBB); // passage en dim 3
       for (int i=1;i<=3;i++) for (int j=1;j<=i;j++)
        {double x=0.; 
         for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
           x += t[cdex.odVect(i,j,k,l)] * a3BB(k,l);
         res->Coor(i,j) = x;
        };
       }
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3_troisSym_HHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new TenseurQ3_troisSym_HHHH(*this);
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ3_troisSym_HHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int i=1;i<4;i++)
            for (int j=1;j<4;j++)
             for (int k=1;k<4;k++)
              for (int l=1;l<4;k++)
                t[cdex.odVect(i,j,k,l)] = aHHHH(i,j,k,l);
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[cdex.odVect(i,j,k,l)] = aHHHH(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ3_troisSym_HHHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ3_troisSym_HHHH::operator == ( const TenseurHHHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_HHHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ3_troisSym_HHHH::Change (int i, int j, int k, int l, const double& val) 
 { t[cdex.odVect(i,j,k,l)]=val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ3_troisSym_HHHH::ChangePlus (int i, int j, int k, int l, const double& val)
 { t[cdex.odVect(i,j,k,l)] += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ3_troisSym_HHHH::operator () (int i, int j, int k, int l) const 
 { return t[cdex.odVect(i,j,k,l)];};
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ3_troisSym_HHHH::MaxiComposante() const
  { return DabsMaxiTab(t,21) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ3_troisSym_HHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 21; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ3_troisSym_HHHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ3_troisSym_HHHH ";
    // puis les datas
     for (int i = 0; i< 21; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ3_troisSym_HHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 306) A.Message(2,"operator >> (istream & entree, TenseurQ3_troisSym_HHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 21; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ3_troisSym_HHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ3_troisSym_HHHH ";
    // puis les datas
    for (int i = 0; i< 21; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
//    t[0]= x1111; t[1]= x2222; t[2]= x3333; t[3]= x1122; t[4]= x1133; t[5]= x2233;
//    t[6]= x1211; t[7]= x1222; t[8]= x1233; t[9]= x1311; t[10]= x1322; t[11]= x1333;
//    t[12]= x2311; t[13]= x2322; t[14]= x2333;
//    t[15]= x1212; t[16]= x1313; t[17]= x2323; t[18]= x1213; t[19]= x1223; t[20]= x1323; 
TenseurQ3_troisSym_BBBB::ChangementIndex::ChangementIndex() :
 odVect(3) 
  { odVect(1,1,1,1)=0;  odVect(1,1,1,2)=6;  odVect(1,1,1,3)=9; 
    odVect(1,1,2,1)=6;  odVect(1,1,2,2)=3;  odVect(1,1,2,3)=12;
    odVect(1,1,3,1)=9;  odVect(1,1,3,2)=12; odVect(1,1,3,3)=4;
    odVect(1,2,1,1)=6;  odVect(1,2,1,2)=15; odVect(1,2,1,3)=18; //
    odVect(1,2,2,1)=15; odVect(1,2,2,2)=7;  odVect(1,2,2,3)=19;
    odVect(1,2,3,1)=18; odVect(1,2,3,2)=19; odVect(1,2,3,3)=8;
    odVect(1,3,1,1)=9;  odVect(1,3,1,2)=18; odVect(1,3,1,3)=16; //
    odVect(1,3,2,1)=18; odVect(1,3,2,2)=10; odVect(1,3,2,3)=20;
    odVect(1,3,3,1)=16; odVect(1,3,3,2)=20; odVect(1,3,3,3)=11;

    odVect(2,1,1,1)=6;  odVect(2,1,1,2)=15; odVect(2,1,1,3)=18; 
    odVect(2,1,2,1)=15; odVect(2,1,2,2)=7;  odVect(2,1,2,3)=19;
    odVect(2,1,3,1)=18; odVect(2,1,3,2)=19; odVect(2,1,3,3)=8;
    odVect(2,2,1,1)=3;  odVect(2,2,1,2)=7;  odVect(2,2,1,3)=10; //
    odVect(2,2,2,1)=7;  odVect(2,2,2,2)=1;  odVect(2,2,2,3)=13;
    odVect(2,2,3,1)=10; odVect(2,2,3,2)=13; odVect(2,2,3,3)=5;
    odVect(2,3,1,1)=11; odVect(2,3,1,2)=19; odVect(2,3,1,3)=20; //
    odVect(2,3,2,1)=19; odVect(2,3,2,2)=13; odVect(2,3,2,3)=17;
    odVect(2,3,3,1)=20;  odVect(2,3,3,2)=17; odVect(2,3,3,3)=14;

    odVect(3,1,1,1)=9;  odVect(3,1,1,2)=18; odVect(3,1,1,3)=16; 
    odVect(3,1,2,1)=18; odVect(3,1,2,2)=10; odVect(3,1,2,3)=20;
    odVect(3,1,3,1)=16; odVect(3,1,3,2)=20; odVect(3,1,3,3)=11;
    odVect(3,2,1,1)=12; odVect(3,2,1,2)=19; odVect(3,2,1,3)=20; //
    odVect(3,2,2,1)=19; odVect(3,2,2,2)=13; odVect(3,2,2,3)=17;
    odVect(3,2,3,1)=20;  odVect(3,2,3,2)=17; odVect(3,2,3,3)=14;
    odVect(3,3,1,1)=4;  odVect(3,3,1,2)=8;  odVect(3,3,1,3)=11; //
    odVect(3,3,2,1)=8;  odVect(3,3,2,2)=3;  odVect(3,3,2,3)=14;
    odVect(3,3,3,1)=11; odVect(3,3,3,2)=14; odVect(3,3,3,3)=2;
  }; 

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::TenseurQ3_troisSym_BBBB() :  
 ipointe() // par défaut
  { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<21;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::TenseurQ3_troisSym_BBBB( const double& val) :  
 ipointe() 
  { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<21;i++) t[i]=val;
  };
  
// initialisation à partir des 21 coefficients indépendants
// (1111) (2222) (3333) (1122) (1133) (2233)
// (1211) (1222) (1233) (1311) (1322) (1333) (2311) (2322) (2333) 
// (1212) (1313) (2323) (1213) (1223) (1323)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::TenseurQ3_troisSym_BBBB
        ( const double& x1111,const double& x2222,const double& x3333,const double& x1122,const double& x1133,const double& x2233
         ,const double& x1211,const double& x1222,const double& x1233,const double& x1311,const double& x1322,const double& x1333
         ,const double& x2311,const double& x2322,const double& x2333
         ,const double& x1212,const double& x1313,const double& x2323,const double& x1213,const double& x1223,const double& x1323)
 :  
 ipointe() { dimension = 306;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]= x1111; t[1]= x2222; t[2]= x3333; t[3]= x1122; t[4]= x1133; t[5]= x2233;
    t[6]= x1211; t[7]= x1222; t[8]= x1233; t[9]= x1311; t[10]= x1322; t[11]= x1333;
    t[12]= x2311; t[13]= x2322; t[14]= x2333;
    t[15]= x1212; t[16]= x1313; t[17]= x2323; t[18]= x1213; t[19]= x1223; t[20]= x1323; 
  }; 
 
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::~TenseurQ3_troisSym_BBBB() 
{ listdouble21.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
// constructeur a partir d'une instance non differenciee  
// il n'y a pas de vérification des symétries  seules les grandeurs suivantes sont utilisés:
// (1111) (2222) (3333) (1122) (1133) (2233)
// (1211) (1222) (1233) (1311) (1322) (1333) (2311) (2322) (2333) 
// (1212) (1313) (2323) (1213) (1223) (1323)
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::TenseurQ3_troisSym_BBBB ( const TenseurBBBB & B) :  
 ipointe()
  { listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0]= B(1,1,1,1); t[1]= B(2,2,2,2); t[2]= B(3,3,3,3); t[3]= B(1,1,2,2); t[4]= B(1,1,3,3); t[5]= B(2,2,3,3);
    t[6]= B(1,2,1,1); t[7]= B(1,2,2,2); t[8]= B(1,2,3,3); t[9]= B(1,3,1,1); t[10]= B(1,3,2,2); t[11]= B(1,3,3,3);
    t[12]= B(2,3,1,1); t[13]= B(2,3,2,2); t[14]= B(2,3,3,3);
    t[15]= B(1,2,1,2); t[16]= B(1,3,1,3); t[17]= B(2,3,2,3); t[18]= B(1,2,1,3); t[19]= B(1,2,2,3); t[20]= B(1,3,2,3); 
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3_troisSym_BBBB::TenseurQ3_troisSym_BBBB (  const TenseurQ3_troisSym_BBBB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble21.push_front(Reels21());  // allocation
    ipointe = listdouble21.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 21;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ3_troisSym_BBBB::Inita(double val)   
	{ for (int i=0;i< 21;i++)
         t[i] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator + ( etc..");
     #endif
     res =  new TenseurQ3_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ3_troisSym_BBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator - ( etc..");
     #endif
     res =  new TenseurQ3_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_BBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_BBBB::operator *= ( const double & b)
  {for (int i = 0; i< 21; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3_troisSym_BBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 21; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3_troisSym_BBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3_troisSym_BBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 21; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ3_troisSym_BBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"TenseurQ3_troisSym_BBBB::operator && ( const TenseurBB & aHH)");
     #endif
     TenseurBB * res;
     // deux cas suivant que le tenseur aHH est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (aHH.Dimension()==3)
      {// cas symétrique
       const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
       for (int i=1;i<=3;i++) for (int j=1;j<=i;j++)
        {double x=0.; 
         for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
           x += t[cdex.odVect(i,j,k,l)] * a3HH(k,l);
         res->Coor(i,j) = x;
        };
       }
     else
      {// cas non symétrique, même calcul que si dessus, simplement du fait de la def explicite du
       // tenseur a3BB, on doit aller plus vite en adressage de l'opérateur ()
       const Tenseur_ns3HH & a3HH = *((Tenseur_ns3HH*) &aHH); // passage en dim 3
       for (int i=1;i<=3;i++) for (int j=1;j<=i;j++)
        {double x=0.; 
         for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
           x += t[cdex.odVect(i,j,k,l)] * a3HH(k,l);
         res->Coor(i,j) = x;
        };  
       }
     return *res ;
  };
  

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3_troisSym_BBBB::Transpose1et2avec3et4() const   
   { TenseurBBBB * res;
     res =  new TenseurQ3_troisSym_BBBB(*this);
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ3_troisSym_BBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int i=1;i<4;i++)
            for (int j=1;j<4;j++)
             for (int k=1;k<4;k++)
              for (int l=1;l<4;k++)
                t[cdex.odVect(i,j,k,l)] = aBBBB(i,j,k,l);
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[cdex.odVect(i,j,k,l)] = aBBBB(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ3_troisSym_BBBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ3_troisSym_BBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 306) Message(2,"TenseurQ3_troisSym_BBBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 21; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ3_troisSym_BBBB::Change (int i, int j, int k, int l, const double& val) 
 { t[cdex.odVect(i,j,k,l)]=val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ3_troisSym_BBBB::ChangePlus (int i, int j, int k, int l, const double& val)
 { t[cdex.odVect(i,j,k,l)] += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ3_troisSym_BBBB::operator () (int i, int j, int k, int l) const 
 { return t[cdex.odVect(i,j,k,l)];};
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ3_troisSym_BBBB::MaxiComposante() const
  { return DabsMaxiTab(t,21) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ3_troisSym_BBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 21; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ3_troisSym_BBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ3_troisSym_BBBB ";
    // puis les datas
     for (int i = 0; i< 21; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ3_troisSym_BBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 306) A.Message(2,"operator >> (istream & entree, TenseurQ3_troisSym_BBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 21; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ3_troisSym_BBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ3_troisSym_BBBB ";
    // puis les datas
    {for (int i = 0; i< 21; i++)
        {sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";}}
    return sort;      
  };

 
  
#endif
