

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "Tenseur3.h"
#include "ConstMath.h"  
#include "MathUtil.h"
#include "Util.h"


#ifndef  Tenseur3_H_deja_inclus
 
// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal
//-----------------------------------------------------------------------
//          cas des composantes deux fois contravariantes non symetriques
//          pour les differencier la dimension = -3
//-----------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::ChangementIndex::ChangementIndex() :
  idx_i(9),idx_j(9),odVect(3)
  { idx_i(1)=1;idx_i(2)=1;idx_i(3)=1;  idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;
    idx_i(4)=2;idx_i(5)=2;idx_i(6)=2;  idx_j(4)=1;idx_j(5)=2;idx_j(6)=3;
    idx_i(7)=3;idx_i(8)=3;idx_i(9)=3;  idx_j(7)=1;idx_j(8)=2;idx_j(9)=3;
    odVect(1,1)=1;odVect(1,2)=2;odVect(1,3)=3;
    odVect(2,1)=4;odVect(2,2)=5;odVect(2,3)=6;
    odVect(3,1)=7;odVect(3,2)=8;odVect(3,3)=9;
  }; 

#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::Tenseur_ns3HH() :  
 ipointe()  // par defaut
   { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
     for ( int i=0; i< 9; i++)
        t[i] = 0.;        
   };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::Tenseur_ns3HH( const double val) :  
 ipointe() 
  { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
     for ( int i=0; i< 9; i++)
        t[i] = val;        
  };
// initialisation avec 9 valeurs différentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::Tenseur_ns3HH
      ( const double val1,  const double val2,  const double val3,   // 1ere  ligne
        const double val4,  const double val5,  const double val6,   // 2ieme ligne
        const double val7,  const double val8,  const double val9)   // 3ieme ligne
 :  
 ipointe()
  { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] = val1;t[1] = val2;t[2] = val3; 
    t[3] = val4;t[4] = val5;t[5] = val6; 
    t[6] = val7;t[7] = val8;t[8] = val9; 
  };      
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::Tenseur_ns3HH (  const TenseurHH & B) :  
 ipointe()
  { this->dimension = -3 ;
    #ifdef MISE_AU_POINT
    if (Abs(B.dimension) != 3)
      { cout << "\n erreur de dimension, elle devrait etre = 3 ou -3 ";
        cout << "\n Tenseur_ns3HH::Tenseur_ns3HH ( TenseurHH &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (B.dimension == -3)
      {for ( int i=0; i< 9; i++) t[i] = B.t[i]; } // cas d'un tenseur non symétrique
    else  // cas d'un tenseur symétrique
      { Tenseur3HH& C = *((Tenseur3HH*) & B);
        t[0] = C.t[0];t[1] = C.t[3];t[2] = C.t[5]; 
        t[3] = C.t[3];t[4] = C.t[1];t[5] = C.t[4]; 
        t[6] = C.t[5];t[7] = C.t[4];t[8] = C.t[2]; 
      };        
  };         
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::Tenseur_ns3HH ( const  Tenseur_ns3HH & B) :  
 ipointe()
  { this->dimension = B.dimension ;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for ( int i=0; i< 9; i++)
        t[i] = B.t[i];        
  };         
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3HH::~Tenseur_ns3HH() 
{//if(listdouble9.end() != listdouble9.begin())        // si la liste n'est pas vide
     listdouble9.erase(ipointe);} ; // suppression de l'élément de la liste
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3HH::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;
    t[3] =val; t[4] =val; t[5] =val;    
    t[6] =val; t[7] =val; t[8] =val; 
   };
// operations 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator + (const TenseurHH & B) const 
   {  TenseurHH * ptr;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator + ( etc..");
     #endif
      ptr =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          (*ptr).t[i] = this->t[i] + B.t[i]; //somme des données
      else 
       {  (*ptr).t[0] = this->t[0] + B.t[0]; //somme des données
          (*ptr).t[1] = this->t[1] + B.t[3]; //somme des données
          (*ptr).t[2] = this->t[2] + B.t[5]; //somme des données
          (*ptr).t[3] = this->t[3] + B.t[3]; //somme des données
          (*ptr).t[4] = this->t[4] + B.t[1]; //somme des données
          (*ptr).t[5] = this->t[5] + B.t[4]; //somme des données
          (*ptr).t[6] = this->t[6] + B.t[5]; //somme des données
          (*ptr).t[7] = this->t[7] + B.t[4]; //somme des données
          (*ptr).t[8] = this->t[8] + B.t[2]; //somme des données
        }            
        
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3HH::operator += ( const TenseurHH & B)
    { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator + ( etc..");
     #endif
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          this->t[i] += B.t[i];
      else 
       {  this->t[0] +=  B.t[0]; //somme des données
          this->t[1] +=  B.t[3]; //somme des données
          this->t[2] +=  B.t[5]; //somme des données
          this->t[3] +=  B.t[3]; //somme des données
          this->t[4] +=  B.t[1]; //somme des données
          this->t[5] +=  B.t[4]; //somme des données
          this->t[6] +=  B.t[5]; //somme des données
          this->t[7] +=  B.t[4]; //somme des données
          this->t[8] +=  B.t[2]; //somme des données
        }            
    }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator - () const 
    {  TenseurHH * ptr;
      ptr =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=8; i++)
        (*ptr).t[i] = - this->t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator - ( const TenseurHH & B) const 
    {  TenseurHH * ptr;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator - ( etc..");
     #endif
      ptr =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          (*ptr).t[i] = this->t[i] - B.t[i]; // soustraction des données
      else 
       {  (*ptr).t[0] = this->t[0] - B.t[0]; //soustraction des données
          (*ptr).t[1] = this->t[1] - B.t[3]; //soustraction des données
          (*ptr).t[2] = this->t[2] - B.t[5]; //soustraction des données
          (*ptr).t[3] = this->t[3] - B.t[3]; //soustraction des données
          (*ptr).t[4] = this->t[4] - B.t[1]; //soustraction des données
          (*ptr).t[5] = this->t[5] - B.t[4]; //soustraction des données
          (*ptr).t[6] = this->t[6] - B.t[5]; //soustraction des données
          (*ptr).t[7] = this->t[7] - B.t[4]; //soustraction des données
          (*ptr).t[8] = this->t[8] - B.t[2]; //soustraction des données
        }            
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3HH::operator -= ( const TenseurHH & B)
     { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator -= ( etc..");
     #endif
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          this->t[i] -= B.t[i];
      else 
       {  this->t[0] -=  B.t[0]; //soustraction des données
          this->t[1] -=  B.t[3]; //soustraction des données
          this->t[2] -=  B.t[5]; //soustraction des données
          this->t[3] -=  B.t[3]; //soustraction des données
          this->t[4] -=  B.t[1]; //soustraction des données
          this->t[5] -=  B.t[4]; //soustraction des données
          this->t[6] -=  B.t[5]; //soustraction des données
          this->t[7] -=  B.t[4]; //soustraction des données
          this->t[8] -=  B.t[2]; //soustraction des données
        }            
          }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator = ( const TenseurHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator = ( etc..");
     #endif
     if (B.Dimension() == -3)
      for (int i = 0; i<=8; i++)
       this->t[i] = B.t[i];
      else 
       {  this->t[0] =  B.t[0]; //
          this->t[1] =  B.t[3]; //
          this->t[2] =  B.t[5]; //
          this->t[3] =  B.t[3]; //
          this->t[4] =  B.t[1]; //
          this->t[5] =  B.t[4]; //
          this->t[6] =  B.t[5]; //
          this->t[7] =  B.t[4]; //
          this->t[8] =  B.t[2]; //
        }                    
    LesMaillonsHH::Libere(); // destruction des tenseurs intermediaires
    return *this; }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator * ( const double & b) const 
    { TenseurHH * res;
      res =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3HH::operator *= ( const double & b)
      { for (int i = 0; i<=8; i++)
       this->t[i] *= b;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator / ( const double & b) const 
    { TenseurHH * res;
      res =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3HH::operator /= ( const double & b)
      { for (int i = 0; i<=8; i++)
       this->t[i] /= b;}; //division des données 
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées, 
// plusZero = true: les données manquantes sont mises à 0 
void Tenseur_ns3HH::Affectation_2D_a_3D(const Tenseur_ns2HH & B,bool plusZero)  
{ this->t[0] = B.t[0];this->t[4] = B.t[1];this->t[3] = B.t[2];this->t[1] = B.t[3];
  if (plusZero)
  { this->t[2] = this->t[5] = this->t[8] = this->t[7] = this->t[6] = 0.;};
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur_ns3HH::Affectation_trans_dimension(const TenseurHH & B,bool plusZero)
{ switch (B.Dimension())
   {case 3: case -3: *this = B; break; // affectation normale
    case -2:
      { const Tenseur_ns2HH & bn  = *((Tenseur2HH *) &B);
        this->Affectation_2D_a_3D(bn,plusZero);
        break;
      }
    case 2:
      { const Tenseur_ns2HH  bn  = B; // on crée un nouveau tenseur transitoire
        this->Affectation_2D_a_3D(bn,plusZero); // qui est tout de suite supprimé
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur_ns3BB::Affectation_trans_dimension( const TenseurHH & B, ..");
   };
};

// produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeH Tenseur_ns3HH::operator * ( const CoordonneeB & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != -dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur_ns3HH::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeH v(-dimension);
    v(1) = this->t[0] * B(1) + this->t[1] * B(2) + this->t[2] * B(3);
    v(2) = this->t[3] * B(1) + this->t[4] * B(2) + this->t[5] * B(3);
    v(3) = this->t[6] * B(1) + this->t[7] * B(2) + this->t[8] * B(3);
    return v;
  };            
        
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::operator * ( const TenseurBH & B) const  // produit une fois contracte
    { Tenseur_ns3HH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 3) Message(3,"Tenseur_ns3HH::operator * ( etc..");
     #endif
      res =  new Tenseur_ns3HH;
      LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
      // on calcul a priori un tenseur antisymetrique, car le test de symetrie
      // est plus long que le calcul, ici c'est au moment de l'affection que la symetrie
      // est testee     
      res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
      res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[6];
      res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[6];
      res->t[1] = this->t[0] * B.t[1] +  this->t[1] * B.t[4]  + this->t[2] * B.t[7];
      res->t[4] = this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
      res->t[7] = this->t[6] * B.t[1] +  this->t[7] * B.t[4]  + this->t[8] * B.t[7];
      res->t[2] = this->t[0] * B.t[2] +  this->t[1] * B.t[5]  + this->t[2] * B.t[8];
      res->t[5] = this->t[3] * B.t[2] +  this->t[4] * B.t[5]  + this->t[5] * B.t[8];
      res->t[8] = this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
      return  (*res);
   };           
              
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB & Tenseur_ns3HH::operator * ( const TenseurBB & B)  const // produit une fois contracte
    { TenseurHB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator * ( etc..");
     #endif
     res =  new Tenseur3HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (B.Dimension() == -3) 
     { res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
       res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[6];
       res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[6];
       res->t[1] = this->t[0] * B.t[1] +  this->t[1] * B.t[4]  + this->t[2] * B.t[7];
       res->t[4] = this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
       res->t[7] = this->t[6] * B.t[1] +  this->t[7] * B.t[4]  + this->t[8] * B.t[7];
       res->t[2] = this->t[0] * B.t[2] +  this->t[1] * B.t[5]  + this->t[2] * B.t[8];
       res->t[5] = this->t[3] * B.t[2] +  this->t[4] * B.t[5]  + this->t[5] * B.t[8];
       res->t[8] = this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
      }
     else
     { res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[5];
       res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[5];
       res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[5];
       res->t[1] = this->t[0] * B.t[3] +  this->t[1] * B.t[1]  + this->t[2] * B.t[4];
       res->t[4] = this->t[3] * B.t[3] +  this->t[4] * B.t[1]  + this->t[5] * B.t[4];
       res->t[7] = this->t[6] * B.t[3] +  this->t[7] * B.t[1]  + this->t[8] * B.t[4];
       res->t[2] = this->t[0] * B.t[5] +  this->t[1] * B.t[4]  + this->t[2] * B.t[2];
       res->t[5] = this->t[3] * B.t[5] +  this->t[4] * B.t[4]  + this->t[5] * B.t[2];
       res->t[8] = this->t[6] * B.t[5] +  this->t[7] * B.t[4]  + this->t[8] * B.t[2];
      }
       return  (*res); };
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur_ns3HH::operator && ( const TenseurBB & B) const  // produit deux fois contracte
    { double b = 0;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator && ( etc..");
     #endif
     if (B.Dimension() == -3) 
      { b += this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
        b += this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
        b += this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
       }
     else
      { b += this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[5];
        b += this->t[3] * B.t[3] +  this->t[4] * B.t[1]  + this->t[5] * B.t[4];
        b += this->t[6] * B.t[5] +  this->t[7] * B.t[4]  + this->t[8] * B.t[2];
       }          
      return b;
    };     
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur_ns3HH::Det()  const     // determinant de la matrice des coordonnees
    { double b = 0;
      b += this->t[0] *(this->t[4] * this->t[8] - this->t[5] * this->t[7]);
      b -= this->t[3] *(this->t[1] * this->t[8] - this->t[2] * this->t[7]);
      b += this->t[6] *(this->t[1] * this->t[5] - this->t[2] * this->t[4]);
      return b;};               
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur_ns3HH::operator == ( const TenseurHH & B) const 
  { int res = 1;
    #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator == ( etc..");
    #endif
    int i1 = 0;
    for (int i=1; i<= 3;i++)
      for (int j=1; j<=3; j++)
       { if (this->t[i1] != B(i,j)) res = 0 ;
         i1++;
        } 
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur_ns3HH::operator != ( const TenseurHH & B) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3HH::operator != ( etc..");
     #endif
    if ((*this) == B) 
      return 0;
    else  
      return 1; };
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3HH::Inverse() const 
   {TenseurBB * res;
    res =  new Tenseur_ns3BB;
    LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  for (int i = 0; i< 9; i++)
             res->t[i] = t[i];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat3x3(((Tenseur_ns3BB *) res)->ipointe);
			 }
			 break;
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
		 case CRAMER : // méthode historique
			{ // calcul du determinant
			  double det = Det();
			 #ifdef MISE_AU_POINT	 	 
			  if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur_ns3HH::Inverse() \n";
					  Sortie(1);
					};
			 #endif
			  det =1./det; 
			  res->t[0] = (this->t[4]*this->t[8] - this->t[5]*this->t[7])*det;
			  res->t[3] = (this->t[5]*this->t[6] - this->t[3]*this->t[8])*det;
			  res->t[6] = (this->t[3]*this->t[7] - this->t[4]*this->t[6])*det;
			  res->t[1] = (this->t[2]*this->t[7] - this->t[1]*this->t[8])*det;
			  res->t[4] = (this->t[0]*this->t[8] - this->t[2]*this->t[6])*det;
			  res->t[7] = (this->t[1]*this->t[6] - this->t[0]*this->t[7])*det;
			  res->t[2] = (this->t[1]*this->t[5] - this->t[2]*this->t[4])*det;
			  res->t[5] = (this->t[2]*this->t[3] - this->t[0]*this->t[5])*det;
			  res->t[8] = (this->t[0]*this->t[4] - this->t[1]*this->t[3])*det;
			 }
			 break;
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur_ns3HH::Inverse() \n";
					  Sortie(1);
					};
			 break; 
	 };
//	 res->Ecriture(cout); // pour le debug
    return *res;
   };    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3HH::Transpose() const 
   { TenseurHH * res;
     res =  new Tenseur_ns3HH;
     LesMaillonsHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[3];
     res->t[2] = this->t[6];
     res->t[3] = this->t[1];
     res->t[4] = this->t[4];
     res->t[5] = this->t[7];
     res->t[6] = this->t[2];
     res->t[7] = this->t[5];
     res->t[8] = this->t[8];
    return *res;};           

    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur_ns3HH::Baisse2Indices() const
   { TenseurBB * res;
     res =  new Tenseur_ns3BB;
     LesMaillonsBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& Tenseur_ns3HH::BaissePremierIndice() const
   { TenseurBH * res;
     res =  new Tenseur3BH;
     LesMaillonsBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // même stockage HH non symétique et BH
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& Tenseur_ns3HH::BaisseDernierIndice() const
   { TenseurHB * res;
     res =  new Tenseur3HB;
     LesMaillonsHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // même stockage HH non symétique et BH
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur_ns3HH::MaxiComposante() const
  { return DabsMaxiTab(t,  9) ;
   };
             
// retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
double& Tenseur_ns3HH::Coor( const int i, const int j)
   { 
    #ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)&&(i!=3)) || ((j!=1)&&(j!=2)&&(j!=3)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "Tenseur_ns3HH::Coor(int,int ) \n";
			  Sortie(1);
			};
    #endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[1]; break;
                       case 3 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[3]; break;
                       case 2 : return t[4]; break;
                       case 3 : return t[5]; break;
                       default : return t[0]; }
                   break;}
         case 3 : { switch (j) 
                     { case 1 : return t[6]; break;
                       case 2 : return t[7]; break;
                       case 3 : return t[8]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };
        
// retourne la composante i,j en lecture seulement
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur_ns3HH::operator () ( const int i, const int j) const 
   { 
    #ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)&&(i!=3)) || ((j!=1)&&(j!=2)&&(j!=3)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "Tenseur_ns3HH::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
    #endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[1]; break;
                       case 3 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[3]; break;
                       case 2 : return t[4]; break;
                       case 3 : return t[5]; break;
                       default : return t[0]; }
                   break;}
         case 3 : { switch (j) 
                     { case 1 : return t[6]; break;
                       case 2 : return t[7]; break;
                       case 3 : return t[8]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    
 
    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur_ns3HH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur_ns3HH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur_ns3HH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur_ns3HH ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur_ns3HH & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != -3) A.Message(-3,"operator >> (istream & entree, Tenseur_ns3HH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur_ns3HH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> A.t[i];
    return entree;      
  };
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur_ns3HH & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur_ns3HH ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };

//--------------------------------------------------------------------
//          cas des composantes deux fois covariantes non symetriques
//          pour les differencier la dimension = -3
//--------------------------------------------------------------------

	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::ChangementIndex::ChangementIndex() :
  idx_i(9),idx_j(9),odVect(3)
  { idx_i(1)=1;idx_i(2)=1;idx_i(3)=1;  idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;
    idx_i(4)=2;idx_i(5)=2;idx_i(6)=2;  idx_j(4)=1;idx_j(5)=2;idx_j(6)=3;
    idx_i(7)=3;idx_i(8)=3;idx_i(9)=3;  idx_j(7)=1;idx_j(8)=2;idx_j(9)=3;
    odVect(1,1)=1;odVect(1,2)=2;odVect(1,3)=3;
    odVect(2,1)=4;odVect(2,2)=5;odVect(2,3)=6;
    odVect(3,1)=7;odVect(3,2)=8;odVect(3,3)=9;
  }; 

#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::Tenseur_ns3BB()  :  
 ipointe() // par defaut
   { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for ( int i=0; i< 9; i++)
        t[i] = 0.;        
   };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::Tenseur_ns3BB( const double val) :  
 ipointe() 
  { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for ( int i=0; i< 9; i++)
        t[i] = val;        
  };
// initialisation avec 9 valeurs différentes  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::Tenseur_ns3BB
      ( const double val1,  const double val2,  const double val3,   // 1ere  ligne
        const double val4,  const double val5,  const double val6,   // 2ieme ligne
        const double val7,  const double val8,  const double val9)   // 3ieme ligne
 :  
 ipointe()
  { dimension = -3;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    t[0] = val1;t[1] = val2;t[2] = val3; 
    t[3] = val4;t[4] = val5;t[5] = val6; 
    t[6] = val7;t[7] = val8;t[8] = val9; 
  };      
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::Tenseur_ns3BB ( const TenseurBB & B) :  
 ipointe()
  { this->dimension = -3 ;
    #ifdef MISE_AU_POINT
    if (Abs(B.dimension) != 3)
      { cout << "\n erreur de dimension, elle devrait etre = 3 ou -3  ";
        Message(3,"Tenseur_ns3BB::Tenseur_ns3BB( etc..");
      }  
    #endif
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (B.dimension == -3)
      {for ( int i=0; i< 9; i++) t[i] = B.t[i]; } // cas d'un tenseur non symétrique
    else  // cas d'un tenseur symétrique
      { Tenseur3BB& C = *((Tenseur3BB*) & B);
        t[0] = C.t[0];t[1] = C.t[3];t[2] = C.t[5]; 
        t[3] = C.t[3];t[4] = C.t[1];t[5] = C.t[4]; 
        t[6] = C.t[5];t[7] = C.t[4];t[8] = C.t[2]; 
      };        
  };         
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::Tenseur_ns3BB ( const  Tenseur_ns3BB & B) :  
 ipointe()
  { this->dimension = B.dimension ;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for ( int i=0; i< 9; i++)
        t[i] = B.t[i];        
  };         
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur_ns3BB::~Tenseur_ns3BB() 
{//if(listdouble9.end() != listdouble9.begin())        // si la liste n'est pas vide
     listdouble9.erase(ipointe);} ; // suppression de l'élément de la liste
// initialise toutes les composantes à val
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3BB::Inita(double val)
  { t[0] =val; t[1] =val; t[2] =val;
    t[3] =val; t[4] =val; t[5] =val;    
    t[6] =val; t[7] =val; t[8] =val; 
   };
// operations 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator + (const TenseurBB & B) const 
   {  TenseurBB * ptr;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator + ( etc..");
     #endif
      ptr =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          (*ptr).t[i] = this->t[i] + B.t[i]; //somme des données
      else 
       {  (*ptr).t[0] = this->t[0] + B.t[0]; //somme des données
          (*ptr).t[1] = this->t[1] + B.t[3]; //somme des données
          (*ptr).t[2] = this->t[2] + B.t[5]; //somme des données
          (*ptr).t[3] = this->t[3] + B.t[3]; //somme des données
          (*ptr).t[4] = this->t[4] + B.t[1]; //somme des données
          (*ptr).t[5] = this->t[5] + B.t[4]; //somme des données
          (*ptr).t[6] = this->t[6] + B.t[5]; //somme des données
          (*ptr).t[7] = this->t[7] + B.t[4]; //somme des données
          (*ptr).t[8] = this->t[8] + B.t[2]; //somme des données
        }            
        
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3BB::operator += ( const TenseurBB & B)
    { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator + ( etc..");
     #endif
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          this->t[i] += B.t[i];
      else 
       {  this->t[0] +=  B.t[0]; //somme des données
          this->t[1] +=  B.t[3]; //somme des données
          this->t[2] +=  B.t[5]; //somme des données
          this->t[3] +=  B.t[3]; //somme des données
          this->t[4] +=  B.t[1]; //somme des données
          this->t[5] +=  B.t[4]; //somme des données
          this->t[6] +=  B.t[5]; //somme des données
          this->t[7] +=  B.t[4]; //somme des données
          this->t[8] +=  B.t[2]; //somme des données
        }            
    }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator - () const 
    {  TenseurBB * ptr;
      ptr =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      for (int i = 0; i<=8; i++)
        (*ptr).t[i] = - this->t[i]; // soustraction des données
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator - ( const TenseurBB & B) const 
    {  TenseurBB * ptr;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator - ( etc..");
     #endif
      ptr =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( ptr); // ajout d'un tenseur intermediaire
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          (*ptr).t[i] = this->t[i] - B.t[i]; // soustraction des données
      else 
       {  (*ptr).t[0] = this->t[0] - B.t[0]; //soustraction des données
          (*ptr).t[1] = this->t[1] - B.t[3]; //soustraction des données
          (*ptr).t[2] = this->t[2] - B.t[5]; //soustraction des données
          (*ptr).t[3] = this->t[3] - B.t[3]; //soustraction des données
          (*ptr).t[4] = this->t[4] - B.t[1]; //soustraction des données
          (*ptr).t[5] = this->t[5] - B.t[4]; //soustraction des données
          (*ptr).t[6] = this->t[6] - B.t[5]; //soustraction des données
          (*ptr).t[7] = this->t[7] - B.t[4]; //soustraction des données
          (*ptr).t[8] = this->t[8] - B.t[2]; //soustraction des données
        }            
    return (*ptr) ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3BB::operator -= ( const TenseurBB & B)
     { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator -= ( etc..");
     #endif
      if (B.Dimension() == -3)
        for (int i = 0; i<=8; i++)
          this->t[i] -= B.t[i];
      else 
       {  this->t[0] -=  B.t[0]; //soustraction des données
          this->t[1] -=  B.t[3]; //soustraction des données
          this->t[2] -=  B.t[5]; //soustraction des données
          this->t[3] -=  B.t[3]; //soustraction des données
          this->t[4] -=  B.t[1]; //soustraction des données
          this->t[5] -=  B.t[4]; //soustraction des données
          this->t[6] -=  B.t[5]; //soustraction des données
          this->t[7] -=  B.t[4]; //soustraction des données
          this->t[8] -=  B.t[2]; //soustraction des données
        }            
          }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator = ( const TenseurBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator = ( etc..");
     #endif
     if (B.Dimension() == -3)
      for (int i = 0; i<=8; i++)
       this->t[i] = B.t[i];
      else 
       {  this->t[0] =  B.t[0]; //
          this->t[1] =  B.t[3]; //
          this->t[2] =  B.t[5]; //
          this->t[3] =  B.t[3]; //
          this->t[4] =  B.t[1]; //
          this->t[5] =  B.t[4]; //
          this->t[6] =  B.t[5]; //
          this->t[7] =  B.t[4]; //
          this->t[8] =  B.t[2]; //
        }                    
    LesMaillonsBB::Libere(); // destruction des tenseurs intermediaires
    return *this; }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator * ( const double & b) const 
    { TenseurBB * res;
      res =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3BB::operator *= ( const double & b)
      { for (int i = 0; i<=8; i++)
       this->t[i] *= b;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator / ( const double & b) const 
    { TenseurBB * res;
      res =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur_ns3BB::operator /= ( const double & b)
      { for (int i = 0; i<=8; i++)
       this->t[i] /= b;}; //division des données 
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées, 
// plusZero = true: les données manquantes sont mises à 0 
void Tenseur_ns3BB::Affectation_2D_a_3D(const Tenseur_ns2BB & B,bool plusZero)  
{ this->t[0] = B.t[0];this->t[4] = B.t[1];this->t[3] = B.t[2];this->t[1] = B.t[3];
  if (plusZero)
  { this->t[2] = this->t[5] = this->t[8] = this->t[7] = this->t[6] = 0.;};
};		 
             
#ifndef MISE_AU_POINT
  inline 
#endif
// affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
// plusZero = true: les données manquantes sont mises à 0
// si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
// des données possibles
void Tenseur_ns3BB::Affectation_trans_dimension(const TenseurBB& B,bool plusZero)
{ switch (B.Dimension())
   {case 3: case -3: *this = B; break; // affectation normale
    case -2:
      { const Tenseur_ns2BB & bn  = *((Tenseur2BB *) &B);
        this->Affectation_2D_a_3D(bn,plusZero);
        break;
      }
    case 2:
      { const Tenseur_ns2BB  bn  = B; // on crée un nouveau tenseur transitoire
        this->Affectation_2D_a_3D(bn,plusZero); // qui est tout de suite supprimé
        break;
      }
    case 1:
      { if (plusZero)
          this->Inita(0.);
        this->t[0] = B.t[0]; //on affecte le seul terme
        break;
      }
    default:
      cout << "\n this= " << *this << " B= "; B.Ecriture(cout);
      Message(3,
         "erreur d\'affectation, Tenseur_ns3BB::Affectation_trans_dimension( const TenseurBB & B, ..");
   };
};

// produit contracte avec un vecteur
#ifndef MISE_AU_POINT
  inline 
#endif
CoordonneeB Tenseur_ns3BB::operator * ( const CoordonneeH & B) const 
  { 
    #ifdef MISE_AU_POINT	 	 
	 if (B.Dimension() != -dimension)
	    { cout << "\nErreur : dimensions vecteur tenseur non egales !\n";
		  cout << " Tenseur_ns3BB::operator *\n";
		  Sortie(1);
		};
    #endif
    CoordonneeB v(-dimension);
    v(1) = this->t[0] * B(1) + this->t[1] * B(2) + this->t[2] * B(3);
    v(2) = this->t[3] * B(1) + this->t[4] * B(2) + this->t[5] * B(3);
    v(3) = this->t[6] * B(1) + this->t[7] * B(2) + this->t[8] * B(3);
    return v;
  };            
        
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::operator * ( const TenseurHB & B) const  // produit une fois contracte
    { Tenseur_ns3BB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 3) Message(3,"Tenseur_ns3BB::operator * ( etc..");
     #endif
      res =  new Tenseur_ns3BB;
      LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
      // on calcul a priori un tenseur antisymetrique, car le test de symetrie
      // est plus long que le calcul, ici c'est au moment de l'affection que la symetrie
      // est testee     
      res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
      res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[6];
      res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[6];
      res->t[1] = this->t[0] * B.t[1] +  this->t[1] * B.t[4]  + this->t[2] * B.t[7];
      res->t[4] = this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
      res->t[7] = this->t[6] * B.t[1] +  this->t[7] * B.t[4]  + this->t[8] * B.t[7];
      res->t[2] = this->t[0] * B.t[2] +  this->t[1] * B.t[5]  + this->t[2] * B.t[8];
      res->t[5] = this->t[3] * B.t[2] +  this->t[4] * B.t[5]  + this->t[5] * B.t[8];
      res->t[8] = this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
      return  (*res);
   };           
              
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH & Tenseur_ns3BB::operator * ( const TenseurHH & B) const  // produit une fois contracte
    { TenseurBH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator * ( etc..");
     #endif
     res =  new Tenseur3BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     if (B.Dimension() == -3) 
     { res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
       res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[6];
       res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[6];
       res->t[1] = this->t[0] * B.t[1] +  this->t[1] * B.t[4]  + this->t[2] * B.t[7];
       res->t[4] = this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
       res->t[7] = this->t[6] * B.t[1] +  this->t[7] * B.t[4]  + this->t[8] * B.t[7];
       res->t[2] = this->t[0] * B.t[2] +  this->t[1] * B.t[5]  + this->t[2] * B.t[8];
       res->t[5] = this->t[3] * B.t[2] +  this->t[4] * B.t[5]  + this->t[5] * B.t[8];
       res->t[8] = this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
      }
     else
     { res->t[0] = this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[5];
       res->t[3] = this->t[3] * B.t[0] +  this->t[4] * B.t[3]  + this->t[5] * B.t[5];
       res->t[6] = this->t[6] * B.t[0] +  this->t[7] * B.t[3]  + this->t[8] * B.t[5];
       res->t[1] = this->t[0] * B.t[3] +  this->t[1] * B.t[1]  + this->t[2] * B.t[4];
       res->t[4] = this->t[3] * B.t[3] +  this->t[4] * B.t[1]  + this->t[5] * B.t[4];
       res->t[7] = this->t[6] * B.t[3] +  this->t[7] * B.t[1]  + this->t[8] * B.t[4];
       res->t[2] = this->t[0] * B.t[5] +  this->t[1] * B.t[4]  + this->t[2] * B.t[2];
       res->t[5] = this->t[3] * B.t[5] +  this->t[4] * B.t[4]  + this->t[5] * B.t[2];
       res->t[8] = this->t[6] * B.t[5] +  this->t[7] * B.t[4]  + this->t[8] * B.t[2];
      }
       return  (*res); };
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur_ns3BB::operator && ( const TenseurHH & B)  const // produit deux fois contracte
    { double b = 0;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator && ( etc..");
     #endif
     if (B.Dimension() == -3) 
      { b += this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[6];
        b += this->t[3] * B.t[1] +  this->t[4] * B.t[4]  + this->t[5] * B.t[7];
        b += this->t[6] * B.t[2] +  this->t[7] * B.t[5]  + this->t[8] * B.t[8];
       }
     else
      { b += this->t[0] * B.t[0] +  this->t[1] * B.t[3]  + this->t[2] * B.t[5];
        b += this->t[3] * B.t[3] +  this->t[4] * B.t[1]  + this->t[5] * B.t[4];
        b += this->t[6] * B.t[5] +  this->t[7] * B.t[4]  + this->t[8] * B.t[2];
       }          
      return b;
    };     
#ifndef MISE_AU_POINT
  inline 
#endif
double  Tenseur_ns3BB::Det() const      // determinant de la matrice des coordonnees
    { double b = 0;
      b += this->t[0] *(this->t[4] * this->t[8] - this->t[5] * this->t[7]);
      b -= this->t[3] *(this->t[1] * this->t[8] - this->t[2] * this->t[7]);
      b += this->t[6] *(this->t[1] * this->t[5] - this->t[2] * this->t[4]);
      return b;};               
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur_ns3BB::operator == ( const TenseurBB & B) const 
  { int res = 1;
    #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator == ( etc..");
    #endif
    int i1 = 0;
    for (int i=1; i<= 3;i++)
      for (int j=1; j<=3; j++)
       { if (this->t[i1] != B(i,j)) res = 0 ;
         i1++;
        } 
    return res; };  
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur_ns3BB::operator != ( const TenseurBB & B) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 3) Message(3,"Tenseur_ns3BB::operator != ( etc..");
     #endif
    if ((*this) == B) 
      return 0;
    else  
      return 1; };
// calcul du tenseur inverse par rapport au produit contracte
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH & Tenseur_ns3BB::Inverse() const 
   {TenseurHH * res;
    res =  new Tenseur_ns3HH;
    LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
	 // choix sur la méthode d'inversion
	 switch (ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique()) 
	  { case  LU_EQUILIBRE:
	      { // on recopie this dans le nouveau tenseur
			  for (int i = 0; i< 9; i++)
             res->t[i] = t[i];
				 
// pour le débug
//res->t[0]=3.; res->t[1]=2.;res->t[2]=1.;				 
				 
			  // appel de l'inversion
			  Util::Inverse_mat3x3(((Tenseur_ns3HH *) res)->ipointe);
			 }
			 break;
//cout << "\n comp \n ";
//   res->Ecriture(cout); cout << "\n";
		 case CRAMER : // méthode historique
			{ // calcul du determinant
			  double det = Det();
			 #ifdef MISE_AU_POINT	 	 
			  if  (Dabs(det) <= ConstMath::trespetit)
					{ cout << "\nErreur : le determinant du tenseur est nul !\n";
					  cout << "Tenseur_ns3BB::Inverse() \n";
					  Sortie(1);
					};
			 #endif
			  det =1./det; 
			  res->t[0] = (this->t[4]*this->t[8] - this->t[5]*this->t[7])*det;
			  res->t[3] = (this->t[5]*this->t[6] - this->t[3]*this->t[8])*det;
			  res->t[6] = (this->t[3]*this->t[7] - this->t[4]*this->t[6])*det;
			  res->t[1] = (this->t[2]*this->t[7] - this->t[1]*this->t[8])*det;
			  res->t[4] = (this->t[0]*this->t[8] - this->t[2]*this->t[6])*det;
			  res->t[7] = (this->t[1]*this->t[6] - this->t[0]*this->t[7])*det;
			  res->t[2] = (this->t[1]*this->t[5] - this->t[2]*this->t[4])*det;
			  res->t[5] = (this->t[2]*this->t[3] - this->t[0]*this->t[5])*det;
			  res->t[8] = (this->t[0]*this->t[4] - this->t[1]*this->t[3])*det;
			 }
			 break;
		 default:
			 { cout << "\nErreur **** : la methode de resolution de l'inversion de tenseur "
			        << ParaGlob::param->ParaAlgoControleActifs().Type_calnum_inversion_metrique() << " n'est pas implante \n";
					  cout << "Tenseur_ns3BB::Inverse() \n";
					  Sortie(1);
					};
			 break; 
	 };
//	 res->Ecriture(cout); // pour le debug
    return *res;
   };    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB & Tenseur_ns3BB::Transpose() const 
   { TenseurBB * res;
     res =  new Tenseur_ns3BB;
     LesMaillonsBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0];
     res->t[1] = this->t[3];
     res->t[2] = this->t[6];
     res->t[3] = this->t[1];
     res->t[4] = this->t[4];
     res->t[5] = this->t[7];
     res->t[6] = this->t[2];
     res->t[7] = this->t[5];
     res->t[8] = this->t[8];
    return *res;};           

    // ---- manipulation d'indice ---- -> création de nouveaux tenseurs
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur_ns3BB::Monte2Indices() const
   { TenseurHH * res;
     res =  new Tenseur_ns3HH;
     LesMaillonsHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& Tenseur_ns3BB::MonteDernierIndice() const
   { TenseurBH * res;
     res =  new Tenseur3BH;
     LesMaillonsBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // même stockage HH non symétique et BH
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& Tenseur_ns3BB::MontePremierIndice() const
   { TenseurHB * res;
     res =  new Tenseur3HB;
     LesMaillonsHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // même stockage HH non symétique et BH
     for (int i = 0; i<=8; i++)
        res->t[i] = this->t[i]; 
     return *res;};           
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur_ns3BB::MaxiComposante() const
  { return DabsMaxiTab(t,  9) ;
   };
             
// retourne la composante i,j en lecture et écriture
#ifndef MISE_AU_POINT
  inline 
#endif
double& Tenseur_ns3BB::Coor( const int i, const int j)
   { 
    #ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)&&(i!=3)) || ((j!=1)&&(j!=2)&&(j!=3)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "Tenseur_ns3BB::Coor(int,int ) \n";
			  Sortie(1);
			};
    #endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[1]; break;
                       case 3 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[3]; break;
                       case 2 : return t[4]; break;
                       case 3 : return t[5]; break;
                       default : return t[0]; }
                   break;}
         case 3 : { switch (j) 
                     { case 1 : return t[6]; break;
                       case 2 : return t[7]; break;
                       case 3 : return t[8]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    
 
// retourne la composante i,j en lecture seulement
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur_ns3BB::operator () ( const int i, const int j) const 
   { 
    #ifdef MISE_AU_POINT	 	 
	 if ( ((i!=1)&&(i!=2)&&(i!=3)) || ((j!=1)&&(j!=2)&&(j!=3)) )
			{ cout << "\nErreur : composante inexistante !\n";
			  cout << " i = " << i << "j = " << j << '\n';
			  cout << "Tenseur_ns3BB::OPERATOR() (int,int ) \n";
			  Sortie(1);
			};
    #endif  
      switch (i)
       { case 1 : { switch (j) 
                     { case 1 : return t[0]; break;
                       case 2 : return t[1]; break;
                       case 3 : return t[2]; break;
                       default : return t[0]; }
                   break;}
         case 2 : { switch (j) 
                     { case 1 : return t[3]; break;
                       case 2 : return t[4]; break;
                       case 3 : return t[5]; break;
                       default : return t[0]; }
                   break;}
         case 3 : { switch (j) 
                     { case 1 : return t[6]; break;
                       case 2 : return t[7]; break;
                       case 3 : return t[8]; break;
                       default : return t[0]; }
                   break;}
         default : return t[0];                       			
		} 
    };    
 
    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur_ns3BB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur_ns3BB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur_ns3BB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur_ns3BB ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur_ns3BB & A)
  { int dim = A.Dimension();
     #ifdef MISE_AU_POINT
       if (dim != -3) A.Message(-3,"operator >> (istream & entree, Tenseur_ns3BB & A)");
    #endif
     // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur_ns3BB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
   for (int i = 0; i< 9; i++)
        entree >> A.t[i];
    return entree;      
  };
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur_ns3BB & A)
  { //int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur_ns3BB ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
     return sort;      
  };


#endif         
