

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *  UNIVERSITE DE BRETAGNE SUD (UBS) --- ENSIBS DE LORIENT              *
 ************************************************************************
 *           LIMATB- Equipe DE GENIE MECANIQUE ET MATERIAUX (EG2M)      *
 * Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
 * tel. 02.97.87.45.70 fax. 02.97.87.45.72 http://www-lg2m.univ-ubs.fr  *
 ************************************************************************
 *     DATE:        14/06/2015                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  phone  0297874573, fax : 0297874588                 *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Definition des classes derivees tenseur du 4ieme ordre    *
 *            de dimension1. Ici de nombreuses fonctions ne sont pas    *
 *            pas disponible du à la forme particulière de stockage     *
 *            par contre c'est économique en stockage.                  *
 *            Stockage: (ijkl) = avec 1 valeurs                         *
 *            en fait tenseur symétrique sur ij et sur kl tel que:      *
 *            A(i,j,k,l) = A(j,i,k,l) = A(i,j,l,k) = A(j,i,l,k)         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef TENSEURQ1_H
#define TENSEURQ1_H

#include <iostream> 
#include "TenseurQ.h"  
#include "PtTabRel.h"
# include "Tableau2_T.h"

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 1HHHH
//------------------------------------------------------------------ 
class  TenseurBBHH; 
class  TenseurHHBB;
    
  
class Tenseur1HHHH : public TenseurHHHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1HHHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1HHHH &);
 
  public :
    // Constructeur
    Tenseur1HHHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur1HHHH(const double val);
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // booleen = false : produit tensoriel barre, qui conserve les symétries
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    Tenseur1HHHH(bool normal, const TenseurHH & aHH, const TenseurHH & bHH);

    // DESTRUCTEUR :
    ~Tenseur1HHHH() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur1HHHH (const  TenseurHHHH &);
    // constructeur de copie  
    Tenseur1HHHH (const  Tenseur1HHHH &);
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHHH & operator + ( const TenseurHHHH &) const ;
    void operator += ( const TenseurHHHH &);
    TenseurHHHH & operator - () const ;  // oppose du tenseur
    TenseurHHHH & operator - ( const TenseurHHHH &) const ;
    void  operator -= ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const Tenseur1HHHH & B)
       { return this->operator=((TenseurHHHH &) B); };
    TenseurHHHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    TenseurHH& operator && ( const TenseurBB & )  const ;
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) ;
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurHHHH &  Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHHH & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHHHH & B,bool plusZero);
    
    // transférer un tenseur général accessible en indice, dans un tenseur 9 Tenseur1HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
    void TransfertDunTenseurGeneral(const TenseurHHHH & aHHHH);
    
    // ---- manipulation d'indice ----
    // on baisse les deux premiers indices -> création d'un tenseur TenseurBBHH 
    TenseurBBHH& Baisse2premiersIndices();
    // on baisse les deux derniers indices -> création d'un tenseur TenseurHHBB 
    TenseurHHBB& Baisse2derniersIndices();
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
    TenseurHHHH & Baselocale(TenseurHHHH & A,const BaseH & gi) const ;
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat
    // retour d'une reference sur A
    // A = A^{ijkl) g_i rond g_j rond g_k rond g_l =  A'^{efgh) gp_i rond gpp_j rond g_k rond gp_l
    // g_i = beta_i^j gp_j --> A'^{efgh) = A^{ijkl) beta_i^e beta_j^f beta_k^g beta_l^h
    TenseurHHHH & ChangeBase(TenseurHHHH & A,const BaseB & gi) const;
    
    // test
    int operator == ( const TenseurHHHH &) const ;
    
    // change la composante i,j,k,l du tenseur
    // acces en  ecriture, 
   	void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	  listdouble1Iter ipointe;
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex1HHHH;
	
	   // fonction pour le poduit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurBB & F) const;
        
  }; 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
class Tenseur1BBBB : public TenseurBBBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1BBBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1BBBB &);
 
  public :
    // Constructeur
    Tenseur1BBBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur1BBBB(const double val);
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    Tenseur1BBBB(bool normal, const TenseurBB & aBB, const TenseurBB & bBB);

    // DESTRUCTEUR :
    ~Tenseur1BBBB() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur1BBBB (const  TenseurBBBB &);
    // constructeur de copie  
    Tenseur1BBBB (const  Tenseur1BBBB &);
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBBB & operator + ( const TenseurBBBB &) const ;
    void operator += ( const TenseurBBBB &);
    TenseurBBBB & operator - () const ;  // oppose du tenseur
    TenseurBBBB & operator - ( const TenseurBBBB &) const ;
    void  operator -= ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const Tenseur1HHHH & B)
       { return this->operator=((TenseurBBBB &) B); };
    TenseurBBBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    TenseurBB& operator && ( const TenseurHH & )  const ;
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    static TenseurBBBB &  Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) ;
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurBBBB &  Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBBB & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBBBB & B,bool plusZero);
 
    // transférer un tenseur général accessible en indice, dans un tenseur 1 Tenseur1HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
    void TransfertDunTenseurGeneral(const TenseurBBBB & aBBBB);
    
    // ---- manipulation d'indice ----
    // on monte les deux premiers indices -> création d'un tenseur TenseurHHBB 
    TenseurHHBB& Monte2premiersIndices();
    // on monte les deux derniers indices -> création d'un tenseur TenseurBBHH 
    TenseurBBHH& Monte2derniersIndices();    
    // on monte les 4 indices -> création d'un tenseur TenseurHHHH 
    TenseurHHHH& Monte4Indices();    
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
    TenseurBBBB & Baselocale(TenseurBBBB & A,const BaseB & gi) const ;
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
    // A = A_{ijkl) g^i rond g^j rond g^k rond g_l =  A'_{efgh) gp^i rond gpp^j rond g^k rond gp^l
    // g^i = gamma^i_j gp^j --> A'_{efgh) = A_{ijkl) gamma^i_e gamma^j_f gamma^k_g gamma^l_h
    TenseurBBBB & ChangeBase(TenseurBBBB & A,const BaseH & gi) const;
    
    
    // test
    int operator == ( const TenseurBBBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	  listdouble1Iter ipointe;
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex1BBBB;
	
	   // fonction pour le poduit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurHH & F) const;

  }; 

//  
//------------------------------------------------------------------
//          cas des composantes mixte 1BBHH
//------------------------------------------------------------------
      
class Tenseur1BBHH : public TenseurBBHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1BBHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1BBHH &);
 
  public :
    // Constructeur
    Tenseur1BBHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur1BBHH(const double val);
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
    Tenseur1BBHH(const TenseurBB & aBB, const TenseurHH & bHH);

    // DESTRUCTEUR :
    ~Tenseur1BBHH() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur1BBHH (const  TenseurBBHH &);
    // constructeur de copie  
    Tenseur1BBHH (const  Tenseur1BBHH &);
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBHH & operator + ( const TenseurBBHH &) const ;
    void operator += ( const TenseurBBHH &);
    TenseurBBHH & operator - () const ;  // oppose du tenseur
    TenseurBBHH & operator - ( const TenseurBBHH &) const ;
    void  operator -= ( const TenseurBBHH &);
    TenseurBBHH & operator = ( const TenseurBBHH &);
    TenseurBBHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    TenseurBB& operator && ( const TenseurBB & )  const ;
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
    static TenseurBBHH &  Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHBB & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBBHH & B,bool plusZero);
 
    // test
    int operator == ( const TenseurBBHH &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	  listdouble1Iter ipointe;
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex1BBHH;
	    
	   // fonction pour le poduit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurHH & F) const;   
};	   
//  
//------------------------------------------------------------------
//          cas des composantes mixte 2HHBB
//------------------------------------------------------------------
      
class Tenseur1HHBB : public TenseurHHBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur1HHBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur1HHBB &);
 
  public :
    // Constructeur
    Tenseur1HHBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur1HHBB(const double val);
    // initialisation à partir d'un produit tensoriel avec 1 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
    Tenseur1HHBB(const TenseurHH & aHH, const TenseurBB & bBB);

    // DESTRUCTEUR :
    ~Tenseur1HHBB() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur1HHBB (const  TenseurHHBB &);
    // constructeur de copie  
    Tenseur1HHBB (const  Tenseur1HHBB &);
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHBB & operator + ( const TenseurHHBB &) const ;
    void operator += ( const TenseurHHBB &);
    TenseurHHBB & operator - () const ;  // oppose du tenseur
    TenseurHHBB & operator - ( const TenseurHHBB &) const ;
    void  operator -= ( const TenseurHHBB &);
    TenseurHHBB & operator = ( const TenseurHHBB &);
    TenseurHHBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    TenseurHH& operator && ( const TenseurHH & )  const ;
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
    static TenseurHHBB &  Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBHH & Transpose1et2avec3et4() const  ;
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHHBB & B,bool plusZero);
 
    // test
    int operator == ( const TenseurHHBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	  listdouble1Iter ipointe;
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex1HHBB;
	
	   // fonction pour le produit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurBB & F) const;
};	   	   
     
#ifndef MISE_AU_POINT
  #include "TenseurQ1-1.cc"
  #include "TenseurQ1-2.cc"
  #define  TenseurQ1_H_deja_inclus
#endif
    
 
#endif  
