

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ-1.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur1.h"
#include "CharUtil.h"

#ifndef  TenseurQ1_H_deja_inclus
   
// 1) pour le passage repère absolu en repère local
//  T1 et T2 sont des tenseurs, Base représente une base
// pour des tenseurs T1 et T2 symétriques HHHH ou BBBB      
template <class T1,class T2,class Base>  
 T1&  produit44_1(T1& A,T2& Nous, const Base & gi)
    {   int dim = 1;
        if ((A.Dimension()!=11)|(Nous.Dimension()!=11))
        	{cout << "\n erreur la methode n'est valide que pour la dimension 1"
        	      << "\n ici pour des tenseurs Tenseur1HHHH ou Tenseur1BBBB"
        	      << "\n T1&  produit44_1(T1& A,T2& Nous, const Base & gi)";
        	 Sortie(1);     
        	};
        int dimint = gi.Dimension(); //peut être de dim 1, 2 ou 3
        *(A.t)=0.;
        for (int al=1; al<= dimint; al++)
          for (int be=1; be<= dimint; be++)
           for (int cl=1; cl<= dimint; cl++)
            for (int de=1; de<= dimint; de++)
            	{ int abcd=(Tenseur1HHHH::cdex1HHHH.odVect(al,be)-1)*6+Tenseur1HHHH::cdex1HHHH.odVect(cl,de)-1;
            	  *(A.t) += Nous.t[abcd] * gi(1)(al)*gi(1)(be)
                             *gi(1)(cl)*gi(1)(de);
            	};
        return A;
      };

// 2) pour le changement de base
//  T1 et T2 sont des tenseurs, Base représente une base
// pour des tenseurs T1 et T2 symétriques HHHH ou BBBB      
    // A = A?{ijkl) g?i rond g?j rond g?k rond g?l =  A'?{efgh) gp?i rond gp?j rond gp?k rond gp?l
    // g?i = beta?i?j gp?j --> A'?{efgh) = A?{ijkl) beta?i?e beta?j?f beta?k?g beta?l?h
    // ici seul g?1 est utilisé et ces coordonnées: g?1?i sont celles qu'il dans gp?j
    // c'est à dire: coordonnées de l'ancien vecteur dans la nouvelle base
template <class T1,class T2,class Base>
 T1&  produit55_1(T1& A,T2& Nous, const Base & gi)
    {   int dim_gi = gi(1).Dimension();
        #ifndef MISE_AU_POINT
         // on vérifie que la dimention de retour convient
         // on doit avoir : g1 = beta(1,j) g'j
         bool erreur_dim = false;
         switch (A.Dimension())
          {case 11: if (dim_gi != 1) erreur_dim = true; break;
           case 22: if (dim_gi != 2) erreur_dim = true; break;
           case 33: if (dim_gi != 3) erreur_dim = true; break;
           default:
            cout << "\n erreur*** cas non pris en compte: A.Dimension()= "
                 << A.Dimension()
                 << "\n T1&  produit55_1(T1& A,T2& Nous, const Base & gi) ";
            Sortie(1);
          }
         if (erreur_dim)
            cout << "\n erreur*** de dimension:  A.Dimension()= "
                 << A.Dimension() << ", dim_gi= "<< dim_gi
                 << "\n T1&  produit55_1(T1& A,T2& Nous, const Base & gi) ";
            Sortie(1);
        #endif

//        if ((A.Dimension()!=11)|(Nous.Dimension()!=11))
//         {cout << "\n erreur la methode n'est valide que pour la dimension 1"
//               << "\n ici pour des tenseurs Tenseur1HHHH ou Tenseur1BBBB"
//               << "\n T1&  produit55_1(T1& A,T2& Nous, const Base & gi)";
//          Sortie(1);
//         };

         switch (A.Dimension())
          {case 11:
            { *(A.t)= *(Nous.t) * PUISSN(gi(1)(1), 4);
              break;
            }
           case 22: case 33:
            {int dim_giP1 = dim_gi+1;
             for (int i=1;i< dim_giP1; i++)
               for (int j=1; j<= i; j++)
                 for (int k=1;k< dim_giP1; k++)
                   for (int l=1; l<= k; l++)
                    { double val = (*(Nous.t)) * gi.Coordo(1)(i) * gi.Coordo(1)(j)
                                           * gi.Coordo(1)(k) * gi.Coordo(1)(l);
                      A.Change(i,j,k,l,val) ;
                    };
              break;
            }
           default:
            cout << "\n erreur*** cas non pris en compte: A.Dimension()= "
                 << A.Dimension()
                 << "\n T1&  produit55_1(T1& A,T2& Nous, const Base & gi) ";
            Sortie(1);
          }
     
        return A;      
      };

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 2HHHH
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::ChangementIndex::ChangementIndex() :
  idx_i(1),idx_j(1),odVect(2)
  { idx_i(1)=1;idx_j(1)=1;
    odVect(1,1)=1;
  };

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::Tenseur1HHHH() :
 ipointe() // par défaut
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::Tenseur1HHHH( const double val) :
 ipointe() 
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=val;
  };
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::Tenseur1HHHH(bool , const TenseurHH & aHH, const TenseurHH & bHH) :
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
    const Tenseur1HH & b1HH = *((Tenseur1HH*) &bHH); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1HH.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                           +"Tenseur1HHHH::Tenseur1HHHH(bool normal, const"
                           +" TenseurHH & aHH, const TenseurHH & bHH);");
       if (Dabs(b1HH.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                           +"Tenseur1HHHH::Tenseur1HHHH(bool normal, const"
                           +" TenseurHH & aHH, const TenseurHH & bHH);");
    #endif
    *t =   a1HH(1,1) * b1HH(1,1);
  };

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::~Tenseur1HHHH()
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste

// constructeur a partir d'une instance non differenciee  
// dans le cas d'un tenseur quelconque, celui-ci
// est converti à condition que les symétries existent sinon erreur en debug
// opération longue dans ce cas !
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::Tenseur1HHHH ( const TenseurHHHH & B) :
 ipointe()
  { dimension = 11;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 11)
//      { cout << "\n erreur de dimension, elle devrait etre = 11 ";
//        cout << "\n Tenseur1HHHH::Tenseur1HHHH ( TenseurHHHH &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 11 ) // cas d'un tenseur du même type
      { *t = *B.t;
      }
    else
      {// cas d'un tenseur quelconque, on récupère uniquement le premier terme
       // on va mettre un message car j'ai peur que l'on fasse des conversions non voulues
       Message(1,string("\n conversion d'un tenseur de dimension ")
                 + ChangeEntierSTring(B.dimension)
                 + "Tenseur1HHHH::Tenseur1HHHH ( const TenseurHHHH & B)");
       Sortie(1);
       ///*t = B(1,1,1,1);
      };
  };

// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHHH::Tenseur1HHHH (  const Tenseur1HHHH & B) :
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t = *B.t;
  };
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur1HHHH::Inita(double val)
	{ *t = val; };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator + ( const TenseurHHHH & B) const
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 11) Message(1,"Tenseur1HHHH::operator + ( etc..");
     #endif
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) + *(B.t); //somme des données
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur1HHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHHH::operator += ( etc..");
     #endif
     *(this->t) += *(B.t);
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
   
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator - () const
  {  TenseurHHHH * res;
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = - *(this->t); //oppose
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator - ( const TenseurHHHH & B) const
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHHH::operator - ( etc..");
     #endif
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) - *(B.t);
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHHH::operator -= ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHHH::operator -= ( etc..");
     #endif
     *(this->t) -= *(B.t);
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //soustraction des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHHH::operator = ( etc..");
     #endif
     *(this->t) = *(B.t);
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
  }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator * ( const double & b) const
  {  TenseurHHHH * res;
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) * b;
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHHH::operator *= ( const double & b)
  {*(this->t) *= b;
  }; //multiplication des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::operator / ( const double & b) const
  {  TenseurHHHH * res;
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1HHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(res->t) = *(this->t) / b;
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1HHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(this->t) /= b;
  }; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur1HHHH::operator && ( const TenseurBB & aBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 1)
           Message(1,"Tenseur1HHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur1HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
     res->Coor(1,1) = *t * a1BB(1,1);
     return *res ;
  };

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  Tenseur1HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new Tenseur1HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
    const Tenseur1HH & b1HH = *((Tenseur1HH*) &bHH); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    *(res->t) = a1HH(1,1) * b1HH(1,1);
    return *res;
  };

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  Tenseur1HHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new Tenseur1HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
    const Tenseur1HH & b1HH = *((Tenseur1HH*) &bHH); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(1);
           }      
       if (Dabs(b1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(1);
           }      
    #endif
    *(res->t) = a1HH(1,1) * b1HH(1,1);
    return *res;
  };      

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   }; 

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur1HHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n Tenseur1HHHH::Affectation_trans_dimension(");
       };
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // transférer un tenseur général accessible en indice, dans un tenseur 9 Tenseur1HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
void Tenseur1HHHH::TransfertDunTenseurGeneral(const TenseurHHHH & aHHHH)
   {  *t = aHHHH(1,1,1,1);
   };
       
    // ---- manipulation d'indice ----
    // on baisse les deux premiers indices -> création d'un tenseur TenseurBBHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH& Tenseur1HHHH::Baisse2premiersIndices()
   { TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   }; 
    // on baisse les deux derniers indices -> création d'un tenseur TenseurHHBB 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB& Tenseur1HHHH::Baisse2derniersIndices()
   { TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   }; 
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::Baselocale(TenseurHHHH & A,const BaseH & gi) const
   {  return produit44_1(A,*this,gi);
   };

    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat
    // retour d'une reference sur A
    // A = A^{ijkl) g_i rond g_j rond g_k rond g_l =  A'^{efgh) gp_i rond gpp_j rond g_k rond gp_l
    // g_i = beta_i^j gp_j --> A'^{efgh) = A^{ijkl) beta_i^e beta_j^f beta_k^g beta_l^h
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur1HHHH::ChangeBase(TenseurHHHH & A,const BaseB & gi) const
   {  return produit55_1(A,*this,gi);
   };
             
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur1HHHH::operator == ( const TenseurHHHH & B) const 
  {  int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHHH::operator == ( etc..");
     #endif
     if (*t != *(B.t)) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur1HHHH::Change (int i, int j, int k, int l, const double& val) 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1HHHH::Change (int i,int j,int k,int l,const double& val) \n";
			  Sortie(1);
			};
   #endif
   *t  = val;
  };

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur1HHHH::ChangePlus (int i, int j, int k, int l, const double& val)
  {
   #ifdef MISE_AU_POINT
     if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
   { cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
     cout << "Tenseur1HHHH::Changeplus (int i,int j,int k,int l,const double& val) \n";
     Sortie(1);
   };
   #endif
   *t  += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur1HHHH::operator () (int i, int j, int k, int l) const 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1HHHH::OPERATOR() (int i,int j,int k,int l) \n";
			  Sortie(1);
			};
   #endif
   return (*t);
  };

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur1HHHH::MaxiComposante() const
  { return Dabs(*t) ;
  };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur1HHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1HHHH")
      { Sortie(1);
        return entree;
      };
    // lecture des coordonnées    
    entree >> *(this->t);
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur1HHHH::Ecriture(ostream & sort) const 
  {  // écriture du type
     sort << "Tenseur1HHHH ";
     // puis les datas
     sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(this->t) << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur1HHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 11) A.Message(1,"operator >> (istream & entree, Tenseur1HHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(A.t);
    return entree;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur1HHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur1HHHH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(A.t) << " ";
    return sort;
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur1HHHH::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 1)
           Message(1,"Tenseur1HHHH::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur1HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
   
     res->Coor(1,1) = a1BB(1,1) * (*t) ;
     return *res ;
  };
//=========== fin fonction protected ======================	
 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::ChangementIndex::ChangementIndex() :
  idx_i(1),idx_j(1),odVect(2)
  { idx_i(1)=1;idx_j(1)=1;
    odVect(1,1)=1;
  };
// variables globales
//Tenseur1BBBB::ChangementIndex  Tenseur1BBBB::cdex1BBBB;


    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::Tenseur1BBBB() :  
 ipointe() // par défaut
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::Tenseur1BBBB( const double val)  :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=val;
  };
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::Tenseur1BBBB(bool normal, const TenseurBB & aBB, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
    const Tenseur1BB & b1BB = *((Tenseur1BB*) &bBB); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1BB.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un premier tenseur non symetriques \n")+
                   "Tenseur1BBBB::Tenseur1BBBB(bool normal, const"
                   + " TenseurBB & aBB, const TenseurBB & bBB);");
       if (Dabs(b1BB.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un second tenseur non symétriques \n")+
                  "Tenseur1BBBB::Tenseur1BBBB(bool normal, const"
                   + " TenseurBB & aBB, const TenseurBB & bBB);");
    #endif
    *t =   a1BB(1,1) * b1BB(1,1);
  };
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::~Tenseur1BBBB() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::Tenseur1BBBB ( const TenseurBBBB & B) :  
 ipointe()
  { dimension = 11;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 11)
//      { cout << "\n erreur de dimension, elle devrait etre = 11 ";
//        cout << "\n Tenseur1BBBB::Tenseur1BBBB ( TenseurBBBB &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 11 ) // cas d'un tenseur du même type
      { *t = *B.t;
      }
    else
      {// cas d'un tenseur quelconque, on récupère uniquement le premier terme
       // on va mettre un message car j'ai peur que l'on fasse des conversions non voulues
       Message(1,string("\n conversion d'un tenseur de dimension ")
                 + ChangeEntierSTring(B.dimension)
                 + "Tenseur1BBBB::Tenseur1BBBB ( const TenseurBBBB & B)");
       Sortie(1);
       ///*t = B(1,1,1,1);
      };
  };
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBBB::Tenseur1BBBB (  const Tenseur1BBBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t = *B.t;
  };
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur1BBBB::Inita(double val)   
	{ *t = val; };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 11) Message(1,"Tenseur1BBBB::operator + ( etc..");
     #endif
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) + *(B.t); //somme des données
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur1BBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBBB::operator += ( etc..");
     #endif
     *(this->t) += *(B.t);
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = - *(this->t); //oppose
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBBB::operator - ( etc..");
     #endif
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) - *(B.t);
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBBB::operator -= ( etc..");
     #endif
     *(this->t) -= *(B.t);
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBBB::operator = ( etc..");
     #endif
     *(this->t) = *(B.t);
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) * b;
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBBB::operator *= ( const double & b)
  {*(this->t) *= b;
  }; //multiplication des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1BBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(res->t) = *(this->t) / b;
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1BBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(this->t) /= b;
  }; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur1BBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 1)
           Message(1,"Tenseur1BBBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurBB * res;
     res =  new Tenseur1BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
     res->Coor(1,1) = *t * a1HH(1,1);
     return *res ;
  };
  
    
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  Tenseur1BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new Tenseur1BBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
    const Tenseur1BB & b1BB = *((Tenseur1BB*) &bBB); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(1);
           }      
       if (Dabs(b1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(1);
           }      
    #endif
    *(res->t) = a1BB(1,1) * b1BB(1,1);
    return *res;
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  Tenseur1BBBB::Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new Tenseur1BBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
    const Tenseur1BB & b1BB = *((Tenseur1BB*) &bBB); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(1);
           }      
       if (Dabs(b1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(1);
           }      
    #endif
    *(res->t) = a1BB(1,1) * b1BB(1,1);
    return *res;
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::Transpose1et2avec3et4() const
   { TenseurBBBB * res;
     res =  new Tenseur1BBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur1BBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur1BBBB::Affectation_trans_dimension(");
       };
   };


#ifndef MISE_AU_POINT
  inline 
#endif
    // transférer un tenseur général accessible en indice, dans un tenseur 9 Tenseur1HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
void Tenseur1BBBB::TransfertDunTenseurGeneral(const TenseurBBBB & aBBBB)
   {  *t = aBBBB(1,1,1,1);
   };

    // ---- manipulation d'indice ----
    // on monte les deux premiers indices -> création d'un tenseur TenseurHHBB 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB& Tenseur1BBBB::Monte2premiersIndices()
   { TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   }; 
    // on monte les deux derniers indices -> création d'un tenseur TenseurBBHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH& Tenseur1BBBB::Monte2derniersIndices()
   { TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   };
   
// on monte les 4 indices -> création d'un tenseur TenseurHHHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH& Tenseur1BBBB::Monte4Indices()    
   { TenseurHHHH * res;
     res =  new Tenseur1HHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   };
              
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::Baselocale(TenseurBBBB & A,const BaseB & gi) const
   {  return produit44_1(A,*this,gi);
   };
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur1BBBB::ChangeBase(TenseurBBBB & A,const BaseH & gi) const
   {  return produit55_1(A,*this,gi);
   };
             
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur1BBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBBB::operator == ( etc..");
     #endif
     if (*t != *(B.t)) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur1BBBB::Change (int i, int j, int k, int l,const double& val) 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1BBBB::Change (int i,int j,int k,int l,const double& val) \n";
			  Sortie(1);
			};
   #endif
   *t  = val;
  };

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur1BBBB::ChangePlus (int i, int j, int k, int l,const double& val)
  {
   #ifdef MISE_AU_POINT
     if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
   { cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
     cout << "Tenseur1BBBB::ChangePlus (int i,int j,int k,int l,const double& val) \n";
     Sortie(1);
   };
   #endif
   *t  += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur1BBBB::operator () (int i, int j, int k, int l) const 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1BBBB::OPERATOR() (int i,int j,int k,int l) \n";
			  Sortie(1);
			};
   #endif
   return (*t);
  };

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur1BBBB::MaxiComposante() const
  { return Dabs(*t) ;
  };


    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur1BBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(this->t);
    return entree;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur1BBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur1BBBB ";
    // puis les datas
     sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(this->t) << " ";
     return sort;
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur1BBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 11) A.Message(1,"operator >> (istream & entree, Tenseur1BBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(A.t);
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur1BBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur1BBBB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(A.t) << " ";
    return sort;
  };

  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur1BBBB::Prod_gauche( const TenseurHH & aHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 1)
           Message(1,"Tenseur1BBBB::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur1BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
   
     res->Coor(1,1) = a1HH(1,1) * (*t) ;
     return *res ;
  };
//=========== fin fonction protected ======================	

  
  
#endif
