

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ-1.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur1.h"
#include "CharUtil.h"

#ifndef  TenseurQ1_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes mixte 2BBHH
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::ChangementIndex::ChangementIndex() :
  idx_i(1),idx_j(1),odVect(2)
  { idx_i(1)=1;idx_j(1)=1;
    odVect(1,1)=1;
  };

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::Tenseur1BBHH() :
 ipointe() // par défaut
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=0.;
  };


// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::Tenseur1BBHH( const double val)  :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::Tenseur1BBHH(const TenseurBB & aBB, const TenseurHH & bHH) :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
    const Tenseur1HH & b1HH = *((Tenseur1HH*) &bHH); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1BB.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur1BBHH::Tenseur1BBHH(bool normal, const"
                   + " TenseurBB & aBB, const TenseurHH & bHH);");
       if (Dabs(b1HH.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur1BBHH::Tenseur1BBHH(bool normal, const"
                   + " TenseurBB & aBB, const TenseurHH & bHH);");
    #endif
    *t =   a1BB(1,1) * b1HH(1,1);
  };
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::~Tenseur1BBHH() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::Tenseur1BBHH ( const TenseurBBHH & B) :  
 ipointe()
  { dimension = 11;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 11)
//      { cout << "\n erreur de dimension, elle devrait etre = 11 ";
//        cout << "\n Tenseur1BBHH::Tenseur1BBHH ( TenseurBBHH &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 11 ) // cas d'un tenseur du même type
      { *t = *B.t;
      }
    else
      {// cas d'un tenseur quelconque, on récupère uniquement le premier terme
       // on va mettre un message car j'ai peur que l'on fasse des conversions non voulues
       Message(1,string("\n conversion d'un tenseur de dimension ")
                 + ChangeEntierSTring(B.dimension)
                 + "Tenseur1HHHH::Tenseur1BBHH ( const TenseurBBHH & B)");
       Sortie(1);
       ///*t = B(1,1,1,1);
      };
  };

// constructeur de copie
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1BBHH::Tenseur1BBHH (  const Tenseur1BBHH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t = *B.t;
  };
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur1BBHH::Inita(double val)   
	{ *t = val; };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator + ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 11) Message(1,"Tenseur1BBHH::operator + ( etc..");
     #endif
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) + *(B.t); //somme des données
     return *res ;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur1BBHH::operator += ( const TenseurBBHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBHH::operator += ( etc..");
     #endif
     *(this->t) += *(B.t);
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator - () const 
  {  TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = - *(this->t); //oppose
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator - ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBHH::operator - ( etc..");
     #endif
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) - *(B.t);
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBHH::operator -= ( const TenseurBBHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBHH::operator -= ( etc..");
     #endif
     *(this->t) -= *(B.t);
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator = ( const TenseurBBHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBHH::operator = ( etc..");
     #endif
     *(this->t) = *(B.t);
    LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator * ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) * b;
     return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBHH::operator *= ( const double & b)
    {*(this->t) *= b;
    }; //multiplication des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1BBHH::operator / ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1BBHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(res->t) = *(this->t) / b;
    return *res ;};

#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1BBHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1BBHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(this->t) /= b;
  };
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur1BBHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 1)
           Message(1,"Tenseur1BBHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurBB * res;
     res =  new Tenseur1BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
     (*res).Coor(1,1) = *t * a1BB(1,1);
     return *res ;
  };
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH &  Tenseur1BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) 
  { TenseurBBHH * res;
    res =  new Tenseur1BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
    const Tenseur1HH & b1HH = *((Tenseur1HH*) &bHH); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(1);
           }      
       if (Dabs(b1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(1);
           }      
    #endif
    *(res->t) = a1BB(1,1) * b1HH(1,1);
    return *res;
  };

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1BBHH::Transpose1et2avec3et4() const
   { TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur1BBHH::Affectation_trans_dimension(const TenseurBBHH & aBBHH,bool plusZero)
   { switch (abs(aBBHH.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aBBHH(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBHH.Dimension()))
                    +"n'est pas prise en compte \n Tenseur1BBHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur1BBHH::operator == ( const TenseurBBHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1BBHH::operator == ( etc..");
     #endif
     if (*t != *(B.t)) res = 0 ;
     return res;
  };
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur1BBHH::Change (int i, int j, int k, int l, const double& val) 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1BBHH::Change (int i,int j,int k,int l,const double& val) \n";
			  Sortie(1);
			};
   #endif
   *t  = val;
  };

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur1BBHH::ChangePlus (int i, int j, int k, int l, const double& val)
  {
   #ifdef MISE_AU_POINT
     if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
   { cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
     cout << "Tenseur1BBHH::ChangePlus (int i,int j,int k,int l,const double& val) \n";
     Sortie(1);
   };
   #endif
   *t  += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur1BBHH::operator () (int i, int j, int k, int l) const 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1BBHH::OPERATOR() (int i,int j,int k,int l) \n";
			  Sortie(1);
			};
   #endif
   return (*t);
  };

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur1BBHH::MaxiComposante() const
  { return Dabs(*t) ;
  };

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur1BBHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1BBHH")
      { Sortie(1);
        return entree;
      };
    // lecture des coordonnées    
    entree >> *(this->t);
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur1BBHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur1BBHH ";
     // puis les datas
     sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(this->t) << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur1BBHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 11) A.Message(1,"operator >> (istream & entree, Tenseur1BBHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1BBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(A.t);
    return entree;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur1BBHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur1BBHH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(A.t) << " ";
    return sort;
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur1BBHH::Prod_gauche( const TenseurHH & aHH) const
 {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 1)
           Message(1,"Tenseur1BBHH::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur1HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
  
     res->Coor(1,1) = a1HH(1,1) * (*t) ;

     return *res ;
  };
//=========== fin fonction protected ======================	


 
//------------------------------------------------------------------
//          cas des composantes mixte 1HHBB
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::ChangementIndex::ChangementIndex() :
  idx_i(1),idx_j(1),odVect(2)
  { idx_i(1)=1;idx_j(1)=1;
    odVect(1,1)=1;
  };
// variables globales
//Tenseur1HHBB::ChangementIndex  Tenseur1HHBB::cdex2HHBB;

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::Tenseur1HHBB() :  
 ipointe() // par défaut
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::Tenseur1HHBB( const double val)  :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::Tenseur1HHBB(const TenseurHH & aHH, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 11;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
    const Tenseur1BB & b1BB = *((Tenseur1BB*) &bBB); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1HH.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur1HHBB::Tenseur1HHBB(bool normal, const"
                   + " TenseurHH & aHH, const TenseurBB & bBB);");
       if (Dabs(b1BB.Dimension()) != 1)
          Message(1,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur1HHBB::Tenseur1HHBB(bool normal, const"
                   + " TenseurHH & aHH, const TenseurBB & bBB);");
    #endif
    *t =   a1HH(1,1) * b1BB(1,1);
  };
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::~Tenseur1HHBB() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::Tenseur1HHBB ( const TenseurHHBB & B) :  
 ipointe()
  { dimension = 11;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 11)
//      { cout << "\n erreur de dimension, elle devrait etre = 11 ";
//        cout << "\n Tenseur1HHBB::Tenseur1HHBB ( TenseurHHBB &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 11 ) // cas d'un tenseur du même type
      { *t = *B.t;
      }
    else
      {// cas d'un tenseur quelconque, on récupère uniquement le premier terme
       // on va mettre un message car j'ai peur que l'on fasse des conversions non voulues
       Message(1,string("\n conversion d'un tenseur de dimension ")
                 + ChangeEntierSTring(B.dimension)
                 + "Tenseur1HHBB::Tenseur1HHBB ( const TenseurHHBB & B)");
       Sortie(1);
       ///*t = B(1,1,1,1);
      };
  };
// constructeur de copie
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur1HHBB::Tenseur1HHBB (  const Tenseur1HHBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    *t = *B.t;
  };
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur1HHBB::Inita(double val)   
	{ *t = val; };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator + ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 11) Message(1,"Tenseur1HHBB::operator + ( etc..");
     #endif
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) + *(B.t); //somme des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur1HHBB::operator += ( const TenseurHHBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 1111) Message(1,"Tenseur1HHBB::operator += ( etc..");
     #endif
     *(this->t) += *(B.t);
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator - () const 
  {  TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = - *(this->t); //oppose
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator - ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHBB::operator - ( etc..");
     #endif
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) - *(B.t);
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHBB::operator -= ( const TenseurHHBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHBB::operator -= ( etc..");
     #endif
     *(this->t) -= *(B.t);
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator = ( const TenseurHHBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHBB::operator = ( etc..");
     #endif
     *(this->t) = *(B.t);
    LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator * ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     *(res->t) = *(this->t) * b;
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHBB::operator *= ( const double & b)
  {*(this->t) *= b;
  }; //multiplication des données

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur1HHBB::operator / ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur1HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1HHBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(res->t) = *(this->t) / b;
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur1HHBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur1HHBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     *(this->t) /= b;
   };
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur1HHBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 1)
           Message(1,"Tenseur1HHBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurHH * res;
     res =  new Tenseur1HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
     (*res).Coor(1,1) = *t * a1HH(1,1);
   return *res ;
  };
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB &  Tenseur1HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) 
  { TenseurHHBB * res;
    res =  new Tenseur1HHBB;
    LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
    const Tenseur1BB & b1BB = *((Tenseur1BB*) &bBB); // passage en dim 1
    #ifdef MISE_AU_POINT
       if (Dabs(a1HH.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur1HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(1);
           }      
       if (Dabs(b1BB.Dimension()) != 1)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur1HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(1);
           }      
    #endif
    *(res->t) = a1HH(1,1) * b1BB(1,1);
    return *res;
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur1HHBB::Transpose1et2avec3et4() const   
   { TenseurBBHH * res;
     res =  new Tenseur1BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     *(res->t) = *t;
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur1HHBB::Affectation_trans_dimension(const TenseurHHBB & aHHBB,bool plusZero)
   { switch (abs(aHHBB.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aHHBB(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur1HHBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur1HHBB::operator == ( const TenseurHHBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 11) Message(1,"Tenseur1HHBB::operator == ( etc..");
     #endif
     if (*t != *(B.t)) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur1HHBB::Change (int i, int j, int k, int l,const double& val) 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1HHBB::Change (int i,int j,int k,int l,const double& val) \n";
			  Sortie(1);
			};
   #endif
   *t  = val;
  };

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur1HHBB::ChangePlus (int i, int j, int k, int l,const double& val)
  {
   #ifdef MISE_AU_POINT
     if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
   { cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
     cout << "Tenseur1HHBB::ChangePlus (int i,int j,int k,int l,const double& val) \n";
     Sortie(1);
   };
   #endif
   *t  += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur1HHBB::operator () (int i, int j, int k, int l) const 
  {
   #ifdef MISE_AU_POINT
	 	  if ( (i!=1) || (j!=1) || (k != 1) || (l!=1))
			{ cout << "\nErreur : composante " << i <<"," << j <<"," << k <<"," << l <<"  inexistante !\n";
			  cout << "Tenseur1HHBB::OPERATOR() (int i,int j,int k,int l) \n";
			  Sortie(1);
			};
   #endif
   return (*t);
  };

// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur1HHBB::MaxiComposante() const
  { return Dabs(*t) ;
  };


    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur1HHBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(this->t);
    return entree;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur1HHBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur1HHBB ";
    // puis les datas
     sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(this->t) << " ";
     return sort;
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur1HHBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 11) A.Message(1,"operator >> (istream & entree, Tenseur1HHBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur1HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> *(A.t);
    return entree;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur1HHBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur1HHBB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   *(A.t) << " ";
    return sort;
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur1HHBB::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 1)
           Message(1,"Tenseur1HHBB::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur1BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
   
     res->Coor(1,1) = a1BB(1,1) * (*t) ;
     return *res ;
  };
//=========== fin fonction protected ======================	
  
#endif
