

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "ConstMath.h" 
#include "MathUtil.h"
//#include "Debug.h"
#include "TenseurQ1gene.h"
#include "Tenseur1.h"
#include "CharUtil.h"


#ifndef  TenseurQ1gene_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes BHBH
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHBH::TenseurQ1geneBHBH() :  
 ipointe() // par défaut
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHBH::TenseurQ1geneBHBH( const double& x1111) :  
 ipointe() 
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
   
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHBH::~TenseurQ1geneBHBH() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHBH::TenseurQ1geneBHBH ( const TenseurBHBH & B) :  
 ipointe()
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHBH::TenseurQ1geneBHBH (  const TenseurQ1geneBHBH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1geneBHBH::Inita(double val)   
	{  t[0] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator + ( const TenseurBHBH & B) const 
  {  TenseurBHBH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 10) Message(1,"TenseurQ1geneBHBH::operator + ( etc..");
     #endif
     res =  new TenseurQ1geneBHBH;
     LesMaillonsBHBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1geneBHBH::operator += ( const TenseurBHBH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHBH::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsBHBH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator - () const 
  {  TenseurBHBH * res;
     res =  new TenseurQ1geneBHBH;
     LesMaillonsBHBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator - ( const TenseurBHBH & B) const 
  {  TenseurBHBH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHBH::operator - ( etc..");
     #endif
     res =  new TenseurQ1geneBHBH;
     LesMaillonsBHBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
     return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHBH::operator -= ( const TenseurBHBH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHBH::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsBHBH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator = ( const TenseurBHBH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHBH::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsBHBH::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator * ( const double & b) const 
  {  TenseurBHBH * res;
     res =  new TenseurQ1geneBHBH;
     LesMaillonsBHBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHBH::operator *= ( const double & b)
  {this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneBHBH::operator / ( const double & b) const 
  {  TenseurBHBH * res;
     res =  new TenseurQ1geneBHBH;
     LesMaillonsBHBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneBHBH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHBH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneBHBH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& TenseurQ1geneBHBH::operator && ( const TenseurBH & aBH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBH.Dimension()) != 1) 
           Message(1,"TenseurQ1geneBHBH::operator && ( const TenseurBH & aBH)");
     #endif
     TenseurBH * res;
     res =  new Tenseur1BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BH & a1BH = *((Tenseur1BH*) &aBH); // passage en dim 1
     res->Coor(1,1)=t[0] * a1BH(1,1);
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneBHBH::Transpose1et2avec3et4() const   
   { TenseurHBHB * res;
     res =  new TenseurQ1geneHBHB; res->t[0]=this->t[0];
     LesMaillonsHBHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1geneBHBH::Affectation_trans_dimension(const TenseurBHBH & aBHBH,bool plusZero)
   { switch (abs(aBHBH.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aBHBH(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBHBH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ1geneBHBH::Affectation_trans_dimension(");
       };
   };


// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1geneBHBH::operator == ( const TenseurBHBH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHBH::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1geneBHBH::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneBHBH::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1geneBHBH::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneBHBH::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1geneBHBH::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1geneBHBH::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1geneBHBH::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1geneBHBH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneBHBH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1geneBHBH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1geneBHBH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// fonction pour le produit contracté à gauche
TenseurBH& TenseurQ1geneBHBH::Prod_gauche( const TenseurBH & aBH) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(aBH.Dimension()) != 1) 
           Message(1,"TenseurQ1geneBHBH::Prod_gauche( const TenseurBH & aBH)");
     #endif
     TenseurBH * res;
     res =  new Tenseur1BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BH & a1BH = *((Tenseur1BH*) &aBH); // passage en dim 1
     res->Coor(1,1)= a1BH(1,1) * t[0];
     return *res ;
    };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1geneBHBH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 10) A.Message(1,"operator >> (istream & entree, TenseurQ1geneBHBH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneBHBH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1geneBHBH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1geneBHBH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes HBHB
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBHB::TenseurQ1geneHBHB() :  
 ipointe() // par défaut
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBHB::TenseurQ1geneHBHB( const double& x1111) :  
 ipointe() 
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
   
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBHB::~TenseurQ1geneHBHB() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBHB::TenseurQ1geneHBHB ( const TenseurHBHB & B) :  
 ipointe()
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBHB::TenseurQ1geneHBHB (  const TenseurQ1geneHBHB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1geneHBHB::Inita(double val)   
	{  t[0] = val;
     };

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator + ( const TenseurHBHB & B) const 
  {  TenseurHBHB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 10) Message(1,"TenseurQ1geneHBHB::operator + ( etc..");
     #endif
     res =  new TenseurQ1geneHBHB;
     LesMaillonsHBHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1geneHBHB::operator += ( const TenseurHBHB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBHB::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsHBHB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator - () const 
  {  TenseurHBHB * res;
     res =  new TenseurQ1geneHBHB;
     LesMaillonsHBHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator - ( const TenseurHBHB & B) const 
  {  TenseurHBHB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBHB::operator - ( etc..");
     #endif
     res =  new TenseurQ1geneHBHB;
     LesMaillonsHBHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
     return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBHB::operator -= ( const TenseurHBHB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBHB::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsHBHB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator = ( const TenseurHBHB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBHB::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsHBHB::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator * ( const double & b) const 
  {  TenseurHBHB * res;
     res =  new TenseurQ1geneHBHB;
     LesMaillonsHBHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBHB::operator *= ( const double & b)
  {this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBHB & TenseurQ1geneHBHB::operator / ( const double & b) const 
  {  TenseurHBHB * res;
     res =  new TenseurQ1geneHBHB;
     LesMaillonsHBHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneHBHB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBHB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneHBHB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& TenseurQ1geneHBHB::operator && ( const TenseurHB & aHB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHB.Dimension()) != 1) 
           Message(1,"TenseurQ1geneHBHB::operator && ( const TenseurHB & aHB)");
     #endif
     TenseurHB * res;
     res =  new Tenseur1HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HB & a1HB = *((Tenseur1HB*) &aHB); // passage en dim 1
     res->Coor(1,1)=t[0] * a1HB(1,1);
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHBH & TenseurQ1geneHBHB::Transpose1et2avec3et4() const   
   { TenseurBHBH * res;
     res =  new TenseurQ1geneBHBH; res->t[0]=this->t[0];
     LesMaillonsBHBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1geneHBHB::Affectation_trans_dimension(const TenseurHBHB & aHBHB,bool plusZero)
   { switch (abs(aHBHB.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aHBHB(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHBHB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ1_troisSym_HBHB::Affectation_trans_dimension(");
       };
   };


// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1geneHBHB::operator == ( const TenseurHBHB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBHB::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1geneHBHB::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneHBHB::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1geneHBHB::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneHBHB::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1geneHBHB::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1geneHBHB::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1geneHBHB::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1geneHBHB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneHBHB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1geneHBHB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1geneHBHB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// fonction pour le produit contracté à gauche
TenseurHB& TenseurQ1geneHBHB::Prod_gauche( const TenseurHB & aHB) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(aHB.Dimension()) != 1) 
           Message(1,"TenseurQ1geneHBHB::Prod_gauche( const TenseurHB & aHB)");
     #endif
     TenseurHB * res;
     res =  new Tenseur1HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HB & a1HB = *((Tenseur1HB*) &aHB); // passage en dim 1
     res->Coor(1,1)= a1HB(1,1) * t[0];
     return *res ;
    };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1geneHBHB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 10) A.Message(1,"operator >> (istream & entree, TenseurQ1geneHBHB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneHBHB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1geneHBHB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1geneHBHB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };


//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes HBBH
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBBH::TenseurQ1geneHBBH() :  
 ipointe() // par défaut
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBBH::TenseurQ1geneHBBH( const double& x1111) :  
 ipointe() 
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
   
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBBH::~TenseurQ1geneHBBH() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBBH::TenseurQ1geneHBBH ( const TenseurHBBH & B) :  
 ipointe()
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneHBBH::TenseurQ1geneHBBH (  const TenseurQ1geneHBBH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1geneHBBH::Inita(double val)   
	{  t[0] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator + ( const TenseurHBBH & B) const 
  {  TenseurHBBH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 10) Message(1,"TenseurQ1geneHBBH::operator + ( etc..");
     #endif
     res =  new TenseurQ1geneHBBH;
     LesMaillonsHBBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1geneHBBH::operator += ( const TenseurHBBH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBBH::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsHBBH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator - () const 
  {  TenseurHBBH * res;
     res =  new TenseurQ1geneHBBH;
     LesMaillonsHBBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator - ( const TenseurHBBH & B) const 
  {  TenseurHBBH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBBH::operator - ( etc..");
     #endif
     res =  new TenseurQ1geneHBBH;
     LesMaillonsHBBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
     return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBBH::operator -= ( const TenseurHBBH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBBH::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsHBBH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator = ( const TenseurHBBH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBBH::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsHBBH::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator * ( const double & b) const 
  {  TenseurHBBH * res;
     res =  new TenseurQ1geneHBBH;
     LesMaillonsHBBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBBH::operator *= ( const double & b)
  {this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneHBBH::operator / ( const double & b) const 
  {  TenseurHBBH * res;
     res =  new TenseurQ1geneHBBH;
     LesMaillonsHBBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneHBBH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneHBBH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneHBBH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHB& TenseurQ1geneHBBH::operator && ( const TenseurBH & aBH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBH.Dimension()) != 1) 
           Message(1,"TenseurQ1geneHBBH::operator && ( const TenseurBH & aBH)");
     #endif
     TenseurHB * res;
     res =  new Tenseur1HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BH & a1BH = *((Tenseur1BH*) &aBH); // passage en dim 1
     res->Coor(1,1)=t[0] * a1BH(1,1);
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneHBBH::Transpose1et2avec3et4() const   
   { TenseurBHHB * res;
     res =  new TenseurQ1geneBHHB; res->t[0]=this->t[0];
     LesMaillonsBHHB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1geneHBBH::Affectation_trans_dimension(const TenseurHBBH & aHBBH,bool plusZero)
   { switch (abs(aHBBH.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aHBBH(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHBBH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ1geneHBBH::Affectation_trans_dimension(");
       };
   };


// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1geneHBBH::operator == ( const TenseurHBBH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneHBBH::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1geneHBBH::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneHBBH::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1geneHBBH::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneHBBH::Changeplus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1geneHBBH::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1geneHBBH::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1geneHBBH::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1geneHBBH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneHBBH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1geneHBBH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1geneHBBH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// fonction pour le produit contracté à gauche
TenseurBH& TenseurQ1geneHBBH::Prod_gauche( const TenseurHB & aHB) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(aHB.Dimension()) != 1) 
           Message(1,"TenseurQ1geneHBBH::Prod_gauche( const TenseurHB & aHB)");
     #endif
     TenseurBH * res;
     res =  new Tenseur1BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HB & a1HB = *((Tenseur1HB*) &aHB); // passage en dim 1
     res->Coor(1,1)= a1HB(1,1) * t[0];
     return *res ;
    };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1geneHBBH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 10) A.Message(1,"operator >> (istream & entree, TenseurQ1geneHBBH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneHBBH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1geneHBBH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1geneHBBH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };
 
//  
//------------------------------------------------------------------
//          cas des composantes 2 fois mixtes en inverse
//------------------------------------------------------------------

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHHB::TenseurQ1geneBHHB() :  
 ipointe() // par défaut
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHHB::TenseurQ1geneBHHB( const double& x1111) :  
 ipointe() 
  { dimension = 10;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
  
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHHB::~TenseurQ1geneBHHB() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHHB::TenseurQ1geneBHHB ( const TenseurBHHB & B) :  
 ipointe()
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1geneBHHB::TenseurQ1geneBHHB (  const TenseurQ1geneBHHB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1geneBHHB::Inita(double val)   
	{  t[0] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator + ( const TenseurBHHB & B) const 
  {  TenseurBHHB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 10) Message(1,"TenseurQ1geneBHHB::operator + ( etc..");
     #endif
     res =  new TenseurQ1geneBHHB;
     LesMaillonsBHHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
     return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1geneBHHB::operator += ( const TenseurBHHB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHHB::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsBHHB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator - () const 
  {  TenseurBHHB * res;
     res =  new TenseurQ1geneBHHB;
     LesMaillonsBHHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
     return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator - ( const TenseurBHHB & B) const 
  {  TenseurBHHB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHHB::operator - ( etc..");
     #endif
     res =  new TenseurQ1geneBHHB;
     LesMaillonsBHHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHHB::operator -= ( const TenseurBHHB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHHB::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsBHHB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator = ( const TenseurBHHB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHHB::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsBHHB::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator * ( const double & b) const 
  {  TenseurBHHB * res;
     res =  new TenseurQ1geneBHHB;
     LesMaillonsBHHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHHB::operator *= ( const double & b)
  { this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBHHB & TenseurQ1geneBHHB::operator / ( const double & b) const 
  {  TenseurBHHB * res;
     res =  new TenseurQ1geneBHHB;
     LesMaillonsBHHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneBHHB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1geneBHHB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1geneBHHB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBH& TenseurQ1geneBHHB::operator && ( const TenseurHB & aHB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHB.Dimension()) != 1) 
           Message(1,"TenseurQ1geneBHHB::operator && ( const TenseurHB & aHB)");
     #endif
     TenseurBH * res;
     // deux cas suivant que le tenseur aHH est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur1BH;
     LesMaillonsBH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HB & a1HB = *((Tenseur1HB*) &aHB); // passage en dim 1
     res->Coor(1,1)=t[0] * a1HB (1,1);
     return *res ;
  };
  

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHBBH & TenseurQ1geneBHHB::Transpose1et2avec3et4() const   
   { TenseurHBBH * res;
     res =  new TenseurQ1geneHBBH; res->t[0]=this->t[0];
     LesMaillonsHBBH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1geneBHHB::Affectation_trans_dimension(const TenseurBHHB & aBHHB,bool plusZero)
   { switch (abs(aBHHB.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aBHHB(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBHHB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ1geneBHHB::Affectation_trans_dimension(");
       };
   };


// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1geneBHHB::operator == ( const TenseurBHHB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 10) Message(1,"TenseurQ1geneBHHB::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1geneBHHB::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneBHHB::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1geneBHHB::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1geneBHHB::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1geneBHHB::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1geneBHHB::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1geneBHHB::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1geneBHHB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneBHHB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1geneBHHB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1geneBHHB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// fonction pour le produit contracté à gauche
TenseurHB& TenseurQ1geneBHHB::Prod_gauche( const TenseurBH & aBH) const 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(aBH.Dimension()) != 1) 
           Message(1,"TenseurQ1geneBHHB::Prod_gauche( const TenseurHB & aHB)");
     #endif
     TenseurHB * res;
     res =  new Tenseur1HB;
     LesMaillonsHB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BH & a1BH = *((Tenseur1BH*) &aBH); // passage en dim 1
     res->Coor(1,1)= a1BH(1,1) * t[0];
     return *res ;
    };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1geneBHHB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 10) A.Message(1,"operator >> (istream & entree, TenseurQ1geneBHHB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1geneBHHB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1geneBHHB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1geneBHHB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };

 
  
#endif
