

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ-2.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur2.h"
#include "CharUtil.h"

#ifndef  TenseurQ2_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes mixte 2BBHH
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::ChangementIndex::ChangementIndex() :
  idx_i(3),idx_j(3),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=1;             idx_j(3)=2;
    odVect(1,1)=1;odVect(1,2)=3;
    odVect(2,1)=3;odVect(2,2)=2;
  };

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::Tenseur2BBHH() :  
 ipointe() // par défaut
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<9;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::Tenseur2BBHH( const double val)  :  
 ipointe()
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<9;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::Tenseur2BBHH(const TenseurBB & aBB, const TenseurHH & bHH) :  
 ipointe()
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          Message(2,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur2BBHH::Tenseur2BBHH(bool normal, const"
                   + " TenseurBB & aBB, const TenseurHH & bHH);");
       if (Dabs(b2HH.Dimension()) != 2)
          Message(2,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur2BBHH::Tenseur2BBHH(bool normal, const"
                   + " TenseurBB & aBB, const TenseurHH & bHH);");
    #endif
    for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          t[(ij-1)*3+kl-1] =   a2BB(cdex2BBHH.idx_i(ij),cdex2BBHH.idx_j(ij))
                             * b2HH(cdex2BBHH.idx_i(kl),cdex2BBHH.idx_j(kl));
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::~Tenseur2BBHH() 
{ listdouble9.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::Tenseur2BBHH ( const TenseurBBHH & B) :  
 ipointe()
  { dimension = 22;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 22)
//      { cout << "\n erreur de dimension, elle devrait etre = 22 ";
//        cout << "\n Tenseur2BBHH::Tenseur2BBHH ( TenseurBBHH &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 22 ) // cas d'un tenseur du même type
      { for (int i=0;i< 9;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 3;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 3;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurBBHH (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur2BBHH::Tenseur2BBHH(const TenseurBBHH & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2BBHH::Tenseur2BBHH (  const Tenseur2BBHH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 9;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur2BBHH::Inita(double val)   
	{ for (int i=0;i< 9;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator + ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 22) Message(2,"Tenseur2BBHH::operator + ( etc..");
     #endif
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2BBHH::operator += ( const TenseurBBHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2BBHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator - () const 
  {  TenseurBBHH * res;
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator - ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2BBHH::operator - ( etc..");
     #endif
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BBHH::operator -= ( const TenseurBBHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2BBHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator = ( const TenseurBBHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2BBHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator * ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BBHH::operator *= ( const double & b)
  {for (int i = 0; i< 9; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2BBHH::operator / ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur2BBHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2BBHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur2BBHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 9; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur2BBHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"Tenseur2BBHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
     for (int ij=1;ij < 4;ij++)
      {for (int kl=1;kl < 3;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex2BBHH.idx_i(ij),cdex2BBHH.idx_j(ij)) +=  t[(ij-1)*3+kl-1]
                                             * a2BB(cdex2BBHH.idx_i(kl),cdex2BBHH.idx_j(kl));
       int kl = 3; // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       res->Coor(cdex2BBHH.idx_i(ij),cdex2BBHH.idx_j(ij)) +=  t[(ij-1)*3+kl-1]
                                  * (a2BB(cdex2BBHH.idx_i(kl),cdex2BBHH.idx_j(kl))
                                     +a2BB(cdex2BBHH.idx_j(kl),cdex2BBHH.idx_i(kl)));
       };                                     
   return *res ;
  };
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH &  Tenseur2BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) 
  { TenseurBBHH * res;
    res =  new Tenseur2BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur2BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur2BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          res->t[(ij-1)*3+kl-1] =   a2BB(cdex2BBHH.idx_i(ij),cdex2BBHH.idx_j(ij))
                                  * b2HH(cdex2BBHH.idx_i(kl),cdex2BBHH.idx_j(kl));
    return *res;                         
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2BBHH::Transpose1et2avec3et4() const
   { TenseurHHBB * res;
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          res->t[(kl-1)*3+ij-1] = t[(ij-1)*3+kl-1] ;
     return *res;
   };           

    // il s'agit ici de calculer la variation d'un tenseur dans une nouvelle base
    //  --> variation par rapport aux composantes covariantes d'un tenseur (ex: composantes eps_ij)

    //  d sigma_ij /  d eps_kl = d beta_i^{.a} / d eps_kl . sigma_ab . beta_j^{.b}
    //                          + beta_i^{.a} . d sigma_ab / d eps_kl. beta_j^{.b}
    //                          + beta_i^{.a} . sigma_ab . d beta_j^{.b} / d eps_kl

    // connaissant sa variation dans la base actuelle
    // var_tensBBHH : en entrée: la variation du tenseur dans la base initiale qu'on appelle g^i
    //   ex: var_tensBBHH(i,j,k,l) = d sigma_ij / d eps_kl
    //            : en sortie: la variation du tenseur dans la base finale qu'on appelle gp^i
    // beta       : en entrée gpB(i) = beta(i,j) * gB(j)
    // var_beta   : en entrée : la variation de beta
    //   ex: var_beta(i,j,k,l) = d beta_i^{.j} / d eps_kl
    // tensBB     : le tenseur dont on cherche la variation
    /// -- pour mémoire ---
        // changement de base (cf. théorie) : la matrice beta est telle que:
    // gpB(i) = beta(i,j) * gB(j) <==> gp_i = beta_i^j * g_j
    // et la matrice gamma telle que:
    // gamma(i,j) represente les coordonnees de la nouvelle base duale gpH dans l'ancienne gH
    // gpH(i) = gamma(i,j) * gH(j), i indice de ligne, j indice de colonne
    //   c-a-d= gp^i = gamma^i_j * g^j
    // rappel des différentes relations entre beta et gamma
    // [beta]^{-1} = [gamma]^T      ; [beta]^{-1T} = [gamma]
    // [beta] = [gamma]^{-1T}      ; [beta]^{T} = [gamma]^{-1}
    // changement de base pour un tenseur en  deux fois covariants:
    // [Ap_kl] = [beta] * [A_ij] * [beta]^T

#ifndef MISE_AU_POINT
  inline
#endif
TenseurBBHH &  Tenseur2BBHH::Var_tenseur_dans_nouvelle_base
              (const Mat_pleine& beta,Tenseur2BBHH& var_tensBBHH, const Tableau2 <Tenseur2HH>& var_beta
              ,const Tenseur2BB& tensBB)
 {
 
    TenseurBBHH * res;
    res =  new Tenseur2BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    //  d sigma_ij /  d eps_kl = d beta_i^{.a} / d eps_kl . sigma_ab . beta_j^{.b}
    //                          + beta_i^{.a} . d sigma_ab / d eps_kl. beta_j^{.b}
    //                          + beta_i^{.a} . sigma_ab . d beta_j^{.b} / d eps_kl
    for (int i=1;i<3;i++)
     for (int j=1;j<3;j++)
      for (int k=1;k<3;k++)
       for (int l=1;l<3;l++)
        { double d_sig_ij_d_eps_kl = 0.;
          for (int a=1;a<3;a++)
           for (int b=1;b<3;b++)
             d_sig_ij_d_eps_kl += var_beta(i,a)(k,l) * tensBB(a,b) * beta(j,b)
                                  + beta(i,a) *  var_tensBBHH(a,b,k,l) * beta(j,b)
                                  + beta(i,a) * tensBB(a,b) * var_beta(j,b)(k,l);
          res->Change(i,j,k,l,d_sig_ij_d_eps_kl);
        };

    return *res;
};

// test
#ifndef MISE_AU_POINT
  inline
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur2BBHH::Affectation_trans_dimension(const TenseurBBHH & aBBHH,bool plusZero)
   { switch (abs(aBBHH.Dimension()))
       { case 33 : case 22 :
           for (int ij=1;ij < 4;ij++)
              for (int kl=1;kl < 4;kl++)
                t[(ij-1)*3+kl-1] = aBBHH(cdex2BBHH.idx_i(ij),cdex2BBHH.idx_j(ij)
                                         ,cdex2BBHH.idx_i(kl),cdex2BBHH.idx_j(kl));
           break;
         case 11 :
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBHH(1,1,1,1);
           break;
         default:
          Message(2,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBHH.Dimension()))
                    +"n'est pas prise en compte \n Tenseur2BBHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2BBHH::operator == ( const TenseurBBHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2BBHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur2BBHH::Change (int i, int j, int k, int l, const double& val) 
  { t[(cdex2BBHH.odVect(i,j)-1)*3+cdex2BBHH.odVect(k,l)-1]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur2BBHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[(cdex2BBHH.odVect(i,j)-1)*3+cdex2BBHH.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur2BBHH::operator () (int i, int j, int k, int l) const 
 { return t[(cdex2BBHH.odVect(i,j)-1)*3+cdex2BBHH.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2BBHH::MaxiComposante() const
  { return DabsMaxiTab(t,  9) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2BBHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2BBHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2BBHH ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur2BBHH::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 4;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 4;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 4;kl++)
             { int i= cdex2BBHH.idx_i(ij); int j= cdex2BBHH.idx_j(ij);
               int k= cdex2BBHH.idx_i(kl); int l= cdex2BBHH.idx_j(kl);
               sort << setw(15) << setprecision(7)
                    << t[(cdex2BBHH.odVect(i,j)-1)*3+cdex2BBHH.odVect(k,l)-1];
             }
       sort << '\n';
      }
    cout << endl ;
 };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2BBHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 22) A.Message(2,"operator >> (istream & entree, Tenseur2BBHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2BBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2BBHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2BBHH ";
    // puis les datas
    for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur2BBHH::Prod_gauche( const TenseurHH & aHH) const
 {  cout << "\n fonction non implanté pour l'instant";
     Sortie(1);
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"Tenseur2BBHH::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i < 3;i++) for (int j=1;j<=i;j++) for (int k=1;k < 3;k++) for (int l=1;l < 3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(k,l) +=  a2HH(i,j) * t[8*i+4*j+2*k+l-15] ;
     return *res ;
  };
//=========== fin fonction protected ======================	


 
//------------------------------------------------------------------
//          cas des composantes mixte 2HHBB
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::ChangementIndex::ChangementIndex() :
  idx_i(3),idx_j(3),odVect(2)
  { idx_i(1)=1;idx_i(2)=2;  idx_j(1)=1;idx_j(2)=2;
    idx_i(3)=1;             idx_j(3)=2;
    odVect(1,1)=1;odVect(1,2)=3;
    odVect(2,1)=3;odVect(2,2)=2;
  };
// variables globales
//Tenseur2HHBB::ChangementIndex  Tenseur2HHBB::cdex2HHBB;

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::Tenseur2HHBB() :  
 ipointe() // par défaut
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<9;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::Tenseur2HHBB( const double val)  :  
 ipointe()
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<9;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::Tenseur2HHBB(const TenseurHH & aHH, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 22;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          Message(2,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur2HHBB::Tenseur2HHBB(bool normal, const"
                   + " TenseurHH & aHH, const TenseurBB & bBB);");
       if (Dabs(b2BB.Dimension()) != 2)
          Message(2,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur2HHBB::Tenseur2HHBB(bool normal, const"
                   + " TenseurHH & aHH, const TenseurBB & bBB);");
    #endif
    for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          t[(ij-1)*3+kl-1] =   a2HH(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij))
                             * b2BB(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl));
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::~Tenseur2HHBB() 
{ listdouble9.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::Tenseur2HHBB ( const TenseurHHBB & B) :  
 ipointe()
  { dimension = 22;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 22)
//      { cout << "\n erreur de dimension, elle devrait etre = 22 ";
//        cout << "\n Tenseur2HHBB::Tenseur2HHBB ( TenseurHHBB &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 22 ) // cas d'un tenseur du même type
      { for (int i=0;i< 9;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 3;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 3;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurHHBB (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur2HHBB::Tenseur2HHBB(const TenseurHHBB & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur2HHBB::Tenseur2HHBB (  const Tenseur2HHBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble9.push_front(Reels9());  // allocation
    ipointe = listdouble9.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 9;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur2HHBB::Inita(double val)   
	{ for (int i=0;i< 9;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator + ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 22) Message(2,"Tenseur2HHBB::operator + ( etc..");
     #endif
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur2HHBB::operator += ( const TenseurHHBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2HHBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator - () const 
  {  TenseurHHBB * res;
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator - ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2HHBB::operator - ( etc..");
     #endif
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HHBB::operator -= ( const TenseurHHBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2HHBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator = ( const TenseurHHBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2HHBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator * ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HHBB::operator *= ( const double & b)
  {for (int i = 0; i< 9; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur2HHBB::operator / ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur2HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur2HHBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 9; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur2HHBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur2HHBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 9; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur2HHBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"Tenseur2HHBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     for (int ij=1;ij < 4;ij++)
      {for (int kl=1;kl < 3;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij)) +=  t[(ij-1)*3+kl-1]
                                             * a2HH(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl));
       int kl = 3; // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       res->Coor(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij)) +=  t[(ij-1)*3+kl-1]
                                  * (a2HH(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl))
                                     +a2HH(cdex2HHBB.idx_j(kl),cdex2HHBB.idx_i(kl)));
       };                                     
   return *res ;
  };
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB &  Tenseur2HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) 
  { TenseurHHBB * res;
    res =  new Tenseur2HHBB;
    LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur2HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur2HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          res->t[(ij-1)*3+kl-1] =   a2HH(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij))
                                  * b2BB(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl));
    return *res;                         
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur2HHBB::Transpose1et2avec3et4() const   
   { TenseurBBHH * res;
     res =  new Tenseur2BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 4;ij++)
        for (int kl=1;kl < 4;kl++)
          res->t[(kl-1)*3+ij-1] = t[(ij-1)*3+kl-1] ;
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur2HHBB::Affectation_trans_dimension(const TenseurHHBB & aHHBB,bool plusZero)
   { switch (abs(aHHBB.Dimension()))
       { case 33 : case 22 :
           for (int ij=1;ij < 4;ij++)
              for (int kl=1;kl < 4;kl++)
                t[(ij-1)*3+kl-1] = aHHBB(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij)
                                         ,cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl));
           break;
         case 11 :
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHBB(1,1,1,1);
           break;
         default:
          Message(2,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur2HHBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur2HHBB::operator == ( const TenseurHHBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 22) Message(2,"Tenseur2HHBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 9; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur2HHBB::Change (int i, int j, int k, int l,const double& val) 
  { t[(cdex2HHBB.odVect(i,j)-1)*3+cdex2HHBB.odVect(k,l)-1]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur2HHBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[(cdex2HHBB.odVect(i,j)-1)*3+cdex2HHBB.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur2HHBB::operator () (int i, int j, int k, int l) const 
 { return t[(cdex2HHBB.odVect(i,j)-1)*3+cdex2HHBB.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur2HHBB::MaxiComposante() const
  { return DabsMaxiTab(t,  9) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur2HHBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur2HHBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur2HHBB ";
    // puis les datas
     for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur2HHBB::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 4;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 4;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 4;kl++)
             { int i= cdex2HHBB.idx_i(ij); int j= cdex2HHBB.idx_j(ij);
               int k= cdex2HHBB.idx_i(kl); int l= cdex2HHBB.idx_j(kl);
               sort << setw(15) << setprecision(7)
                    << t[(cdex2HHBB.odVect(i,j)-1)*3+cdex2HHBB.odVect(k,l)-1];
             }
       sort << '\n';
      }
    cout << endl ;
 };


#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur2HHBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 22) A.Message(2,"operator >> (istream & entree, Tenseur2HHBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur2HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 9; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur2HHBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur2HHBB ";
    // puis les datas
    for (int i = 0; i< 9; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur2HHBB::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"Tenseur2HHBB::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
   
     for (int kl=1;kl < 4;kl++)
      {for (int ij=1;ij < 3;ij++) // partie simple produit : la partie diagonale
         (*res).Coor(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl)) +=
                a2BB(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij)) * t[(ij-1)*3+kl-1];
       int ij = 3; // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       (*res).Coor(cdex2HHBB.idx_i(kl),cdex2HHBB.idx_j(kl)) +=
                                    (a2BB(cdex2HHBB.idx_i(ij),cdex2HHBB.idx_j(ij))
                                     +a2BB(cdex2HHBB.idx_j(ij),cdex2HHBB.idx_i(ij))
                                    )
                                  *  t[(ij-1)*3+kl-1] ;
       };

//     // pour vérif
//     for (int i=1;i<3;i++) for (int j=1;j<3;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<3;k++) for (int l=1;l<3;l++)
//         res->Coor(i,j) += a2BB(i,j) * (*this)(i,j,k,l) ;
//      };

     return *res ;
  };
//=========== fin fonction protected ======================	
  
#endif
