
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        2/02/2005                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Calcul des differentes grandeurs liee a la deformation      *
 *     Dans le cas des grandeurs relatives aux procédures Umat.         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef DEF_UMAT_H
#define DEF_UMAT_H

#include "Deformation.h"
#include "Met_ElemPoint.h"

/// @addtogroup groupe_des_deformations
///  @{
///

///     BUT: Calcul des differentes grandeurs liee a la deformation     
///     Dans le cas des grandeurs relatives aux procédures Umat.        
///
///
/// \author    Gérard Rio
/// \version   1.0
/// \date       2/02/2005


class  Def_Umat : public Deformation
{
  public :

    
    // CONSTRUCTEURS :
    Def_Umat () ; // par defaut ne doit pas etre utilise -> message
                       // d'erreur en phase de mise au point
    // constructeur normal dans le cas d'un ou de plusieurs pt d'integration             	  
    Def_Umat (Met_abstraite & ,Tableau<Noeud *>& tabnoeud,     
           Tableau <Mat_pleine> const & tabDphi,Tableau <Vecteur>const  & tabPhi, 
           Enum_type_deformation type_de_deformation = DEFORMATION_STANDART); 
    // constructeur de copie       
    Def_Umat (const Def_Umat &);  
    // DESTRUCTEUR :
    virtual ~Def_Umat () {}; 

    // METHODES PUBLIQUES :
    
    // Surcharge de l'operateur = : realise l'affectation
	   virtual Deformation& operator= (const Deformation& def)
	      {((Deformation*) this)->operator=(def); return *this;};
		
		      
    // définition d'une déformation du même type, permet d'utiliser des types dérivée surchargé
    virtual Def_Umat * Nevez_deformation(Tableau <Noeud *> & tabN) const 
      { Def_Umat* def = new Def_Umat(*this);def->PointeurTableauNoeud(tabN);return def;} ; 
            

    // ------------------------- cas d'une procédure Umat ------------------------
	   const Met_ElemPoint::Umat_cont& Cal_defUmat(bool premier_calcul)
	        {    return  ((Met_ElemPoint*) metrique)->Calcul_grandeurs_Umat(premier_calcul);};
	   // mise à jour de la métrique
	   void Mise_a_jourUmat(const UmatAbaqus& umat) 
	        {((Met_ElemPoint*) metrique)->Mise_a_jour(umat);};
    //   - calcul des déformations équivalentes
    void CalDefEqui(const Tableau <double>& def_equi_t,TenseurBB & epsBB_tdt
                 ,Tableau <double>& def_equi,TenseurBB& DepsBB,TenseurBB& delta_epsBB_tdt
                 ,const Met_abstraite::Umat_cont& ex,bool premier_calcul);

  protected :
    // VARIABLES PROTEGEES :

 };
 /// @}  // end of group

#endif  
