

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        6/04/2008                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Concernant un potentiel hyperélastique composé uniquement   *
 *        d'une fonction bulk, avec une écriture comme les potentiels . *
 *         grenoblois E = K(V)                                          *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                              
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef ISOHYPERBULK_GENE_H
#define ISOHYPERBULK_GENE_H

#include "Hyper3D.h"



/// @addtogroup Les_lois_hyperelastiques
///  @{
///

class IsoHyperBulk_gene  :
  public  Hyper3D
{
  public :
    // VARIABLES PUBLIQUES :
    
  // CONSTRUCTEURS :
  // Constructeur par defaut
  IsoHyperBulk_gene ();
  // Constructeur de copie
  IsoHyperBulk_gene (const IsoHyperBulk_gene& loi) ;
  // DESTRUCTEUR :
  ~IsoHyperBulk_gene ();
   
  // Lecture des donnees de la classe sur fichier
  void LectureDonneesParticulieres
     (UtilLecture * ,LesCourbes1D& lesCourbes1D,LesFonctions_nD& lesFonctionsnD);

  // affichage de la loi
  void Affiche() const ;
  // test si la loi est complete
  // = 1 tout est ok, =0 loi incomplete
  int TestComplet();

   
 //----- lecture écriture de restart -----
  // cas donne le niveau de la récupération
  // = 1 : on récupère tout
  // = 2 : on récupère uniquement les données variables (supposées comme telles)
  void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
  // cas donne le niveau de sauvegarde
  // = 1 : on sauvegarde tout
  // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
  void Ecriture_base_info_loi(ofstream& sort,const int cas);
           
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);

  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new IsoHyperBulk_gene(*this)); };
             
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  
		  	       
//  METHODES internes spécifiques à l'hyperélasticité isotrope découlant de 
//  méthodes virtuelles de Hyper3D


  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0
  double PoGrenoble
           (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel tout seul avec la phase donc dans le cas où Qeps est non nul
  double PoGrenoble
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0, et de sa variation suivant V uniquement
  PoGrenoble_V PoGrenoble_et_V
           (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel et de sa variation suivant V uniquement
  PoGrenoble_V PoGrenoble_et_V
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel tout seul sans la phase car Qeps est  nul
  // ou très proche de 0, et de ses variations première et seconde suivant V uniquement
  PoGrenoble_VV PoGrenoble_et_VV
           (const double & Qeps,const Invariant & invariant);
  // calcul du potentiel et de sa variation première et seconde suivant V uniquement
  PoGrenoble_VV PoGrenoble_et_VV
           (const Invariant0QepsCosphi & inv,const Invariant & invariant);
  // calcul du potentiel et de ses dérivées non compris la phase
  PoGrenobleSansPhaseSansVar PoGrenoble
           (const InvariantQeps & inv,const Invariant & invariant);
  // calcul du potentiel et de ses dérivées avec la phase
  PoGrenobleAvecPhaseSansVar PoGrenoblePhase
           (const InvariantQepsCosphi& inv,const Invariant & invariant); 
  // calcul  du potentiel sans phase et dérivées avec  ses variations par rapport aux invariants
  PoGrenobleSansPhaseAvecVar PoGrenoble_et_var
           (const Invariant2Qeps& inv,const Invariant & invariant);
  // calcul  du potentiel avec phase et dérivées avec  ses variations par rapport aux invariants
  PoGrenobleAvecPhaseAvecVar PoGrenoblePhase_et_var
           (const Invariant2QepsCosphi& inv,const Invariant & invariant);

 
protected :

  // VARIABLES PROTEGEES :
  //fonction de dépendance éventuelle 
  Courbe1D* F_w_V, * F_w_T; // dépendance à V, dépendance à la température
    
///=============== fonctions pour la vérification et la mise au point ===============
    
    // vérif des dérivées du potentiels par rapport aux invariants, ceci par différences finies
void Verif_PoGrenoble_et_var(const double & Qeps,const Invariant & inv
                     ,const PoGrenobleSansPhaseAvecVar& potret );
    // idem mais avec la phase
void Verif_PoGrenoble_et_var(const double & Qeps,const Invariant & inv,const double& cos3phi 
                     ,const PoGrenobleAvecPhaseAvecVar& potret );

static int indic_Verif_PoGrenoble_et_var; // indicateur utilisé par Verif_Potentiel_et_var                           

 };
 /// @}  // end of group

#endif  
